      BLOCK DATA EQOS
C*******************************************************************
c: verification
C     .                                                          .
C     .TOUGH_LGM, MODULE LGMEOS3, VERSION 1.0, MARCH 1993     
C     .                                                          .
C     .FROM     TOUGH_AMD, MODULE EOS3_AMD, VERSION 1.1, SEPT 1993
C     .   and   TOUGH2,    MODULE EOS3,     VERSION 1.0,MARCH 1991     
C*******************************************************************
C       EOS3 WAS MODIFIED FOR LANDFILL GAS MIGRATION (LGM) 
C       MODIFIED BY MIRO NASTEV, LAVAL UNIVERSITY, QUEBEC.
C*******************************************************************
C
C     *FOUR COMPONENTS ARE CONSIDERED:
C       - WATER, METHANE, CARBON DIOXIDE, NITROGEN - AIR
C
C     * FIVE (5) PRIMARY THERMODYNAMIC VARIABLES (DEFAULT):
C       - SINGLE PHASE CONDITIONS: water saturated zone conditions
C            (1) P    (PRESSURE), 
C            (2) X    (AIR MASS FRACTION IN PHASE kg/kg)X=0.00015 below water; 
C            (3) XCH4 (METHANE MASS FRACTION IN THE AIR),
C            (4) XCO2 (CARBON DIOXIDE MASS FRACTION IN THE AIR),
C            (4) T    (TEMPERATURE oC).
C       - TWO-PHASE CONDITIONS: unsaturated zone conditions
C            (1) PG   (GAS PRESSURE), 
C            (2) SG+10(Sg+10=10.579; gas saturation=58%),
C            (3) XCH4 (METHANE MASS FRACTION IN THE AIR; 50% of these 58%),
C            (4) XCO2 (CARBON DIOXIDE MASS FRACTION IN THE AIR; same above),
C            (5) T    (TEMPERATURE oC).
C
C     * OPTIONS FOR PARAMETERS (NK,NEQ,NPH,NB):
C       - (4,5,2,6): WATER, THREE-COMPONENT AIR, NON-ISOTHERMAL.
C       - (4,4,2,6): WATER, THREE-COMPONENT AIR, ISOTHERMAL NEQ=NK.
C       - (4,3,2,6): WATER, CONSTANT COMPOSITION AIR, ISOTHERMAL.
C
C     * DOES NOT ALLOW TOUGH-STYLE VARIABLES SINCE FIVE PRIMARY
C       VARIABLES ARE CONSIDERED.
C
C     * SUBROUTINE QLOSS WAS ADAPTED TO EOS3 FROM EOS1 AND EOS2
C       TO ALLOW HEAT EXCHANGE WITH CONFINING BEDS.
C
C
C     NOTE: ALL OF THE MODIFIED SECTIONS ARE WRITTEN BY small letters  
C
C
C     *** BEGINNING MODIFICATIONS ***
C
C     MODIFIED SECTION ...
C
C     *** END MODIFICATIONS ***
C
C**************************************************************************
	implicit real*8 (a-h,o-z)
C NEL: No of eleme; NCON: No of connections; NOGN: No of generation times; 
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
C 
C*** BEGINNING MODIFICATIONS *** WATER, THREE-COMPONENT AIR,NON-ISOTHERMAL.
C
      DATA NK,NEQ,NPH,NB/4,4,2,6/
C
C     *** END MODIFICATIONS ***
C
C----- NK IS THE NUMBER OF FLUID COMPONENTS =4 (H2O, CH4, CO2, N2-AIR)
C----- NEQ IS THE NUMBER OF EQUATIONS PER GRID BLOCK.
C      USUALLY NEQ = NK+1 = 5, FOR NK MASS- AND ONE ENERGY-BALANCE.
C      if NEQ=NK no temperature changes are considered
C----- NPH IS THE NUMBER OF PHASES WHICH CAN BE PRESENT = 2 (liquid, gas)
C----- NB IS THE NUMBER OF SECONDARY PARAMETERS OTHER THAN MASS FRACT.=6
C      THE TOTAL NUMBER OF SECONDARY PARAMETERS IS NBK = NB+NK =10 
C NK=4; NK1=5; NEQ=5; NEQ1=6; NFLUX=11; NBK=10; NSEC=22; NPH=2; NB=6; NEQ1=6
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 1 xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE EOS
      implicit real*8 (a-h,o-z) 
C
C-----THIS ROUTINE PERFORMS PHASE TRANSITION DIAGNOSTIC, AND COMPUTES
C     ALL THERMOPHYSICAL PROPERTIES OF GASEOUS, LIQUID, OR
C     TWO-PHASE MIXTURES OF WATER AND AIR.
C
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
      COMMON/P3/DELX(100000)
      COMMON/E1/ELEM(20000)
      COMMON/E2/MATX(20000)
      COMMON/E5/P(20000)
      COMMON/E6/T(20000)
      COMMON/SECPAR/PAR(2640000)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
C      COMMON/KONIT/KON,DELT,IGOOD                                        JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                  JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/KC/KC
      COMMON/SVZ/NOITE,MOP(24)
C     *** BEGINNING MODIFICATIONS ***
C     ADDING AMCH4 and AMCO2 - MOLECULAR MASS TO THIS COMMON BLOCK:
      COMMON/GASLAW/R,AMH2O,AMN2,AMCH4,AMCO2,CVAIR
C
C     *** END MODIFICATIONS ***
      COMMON/FAIL/IHALVE
C      COMMON/SOLID/NM,DROK(27),POR(27),PER(3,27),CWET(27),SPHT(27)       JPF 03 2005
      COMMON/SOLID/DROK(27),POR(27),PER(3,27),CWET(27),SPHT(27),NM
      COMMON/SOCH/MAT(27)
C      COMMON/RPCAP/IRP(27),RP(7,27),ICP(27),CP(7,27),IRPD,RPD(7),         JPF 03 2005
C     1  ICPD,CPD(7)
      COMMON/RPCAP/RP(7,27),RPD(7),IRP(27),ICP(27),CP(7,27),IRPD,
     XICPD,CPD(7)
      COMMON/BC/NELA
      common/ff/h1
      character*1 h1
      CHARACTER*5 ELEM,MAT
C
      DIMENSION XX(5),DP(18)
C
      SAVE ICALL,ZERO,ZA,HC
      DATA ZERO,ZA,HC/1.E-6,1.,1.E-10/
C
C-HC IS THE SOLUBILITY CONSTANT OF AIR IN WATER, DEFINED AS
C HC = (DISSOLVED AIR MOL FRACTION) / (AIR PARTIAL PRESSURE).
C STRICTLY SPEAKING, HC DEPENDS UPON TEMPERATURE, VARYING FROM
C 2.3E-10 AT 0 DEG-C TO .91E-10 AT 100 DEG-C. THIS TEMPERATURE
C DEPENDENCE HAS BEEN NEGLECTED, WHICH IS CONSIDERED PERMISSIBLE
C FOR MOST APPLICATIONS, AS AIR SOLUBILITY ITSELF IS SMALL.
C Tchoban dava P=XMOLE*(4.32E+09)       1./(4.32E+09)=(2.31E-10)
C Pruess  dava P=XMOLE/(2.31E-10)
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(/6X,'EOS      1.0         MARCH     1995',6X,
     X'*LGMEOS3 * ... THERMOPHYSICAL PROPERTIES MODULE FOR ',
     X'WATER/THREE-COMPONENT AIR')
C
      IF (MOP(5).GE.4) PRINT 32,KCYC,ITER
   32 FORMAT(/' %%%%%%%%%% E O S %%%%%%%%%% E O S %%%%%%%%%%',
     1  '  [KCYC =',I6,']  and  [ITER =',I4,']'/)
      IF (KC.NE.0) GO TO 40
C ##################################################### INITIALIZATION BELOW
C KC: is the time step No, if KC>0 go to 40 and SKIP THE INITIALIZATION
      h1=char(12)
C     INITIALIZE SOME DATA
      R=8314.56
      AMH2O=18.016
      AMN2=28.96 
      AMCH4=16.043
      AMCO2=44.010
C
C     R   IS THE GAS CONSTANT.  IT INCLUDES A CONVERSION FACTOR FROM G/MOL
C         TO kg/MOL FOR THE MOLECULAR MASSES.  (PA m3/MOL K  OR  J/MOL K).
C     AMH2O   -IS THE STEAM (WATER) MOLECULAR MASS (G/m3).
C     AMN2    -IS THE N2-AIR MOLECULAR MASS (G/m3)
C     AMCH4   -IS THE METHANE MOLECULAR MASS (G/m3)
C     AMCO2   -IS THE CARBON DIOXIDE MOLECULAR MASS (G/m3)
C     CVAIR IS THE DRY AIR SPECIFIC HEAT (OR MASSIC HEAT CAPACITY) (J/kg K).
C     *** END MODIFICATIONS ***
      CVAIR=733.0
      CVCH4=1680.0
      CVCO2=643.0
C-----COME HERE TO ASSIGN DEFAULT RELATIVE PERMEABILITY AND CAPILLARY
C     PRESSURE PARAMETERS TO DOMAINS WITHOUT SPECIAL ASSIGNMENTS.
C     NM: No of materials; RP: relative permeability; CP: capillary pressure
C
      DO 33 N=1,NM
        IF (IRP(N).NE.0) GO TO 33
        IRP(N)=IRPD
        ICP(N)=ICPD
        DO 34 M=1,7
          RP(M,N)=RPD(M)
          CP(M,N)=CPD(M)
   34     CONTINUE
   33   CONTINUE
C
C     *** BEGINNING MODIFICATIONS ***
C
C     *** MODIFIED PRINTOUT - CHANGES WITH RESPECT TO EOS3
C
C-----GENERATE SOME PRINTOUT CONCERNING THE EQUATION-OF-STATE PACKAGE.
      PRINT 300,h1
  300 FORMAT(A1/' ',131('*')/' *                LGMEOS3 : EQUATION',
     X' OF STATE FOR WATER AND THREE-COMPONENT AIR MIXTURES',43X,'*'/
     X' ',131('*'))
      PRINT 301,NK,NEQ,NPH,NB
  301 FORMAT(//'    OPTIONS SELECTED ARE: (NK,NEQ,NPH,NB) = (',
     X3(I1,','),I2,')'/)
      PRINT 302,NK,NEQ,NPH,NB
  302 FORMAT(27X,'NK  = ',I2,'   - NUMBER OF FLUID COMPONENTS '/,
     X27X,'NEQ = ',I2,'   - NUMBER OF EQUATIONS PER GRID BLOCK'/,
     X27X,'NPH = ',I2,'   - NUMBER OF PHASES THAT CAN BE PRESENT'/,
     X27X,'NB  = ',I2,'   - NUMBER OF SECONDARY PARAMETERS (OTHER THAN',
     X' COMPONENT MASS FRACTIONS)'/)
      PRINT 303
  303 FORMAT(/'   AVAILABLE OPTIONS ARE: (NK,NEQ,NPH,NB) = ',
     X'(4,5,2,6) - WATER, THREE-COMP.AIR (CH4, CO2 and N2), NON-',
     X'ISOTHERMAL '/
     X44X,'(4,4,2,6) - WATER AND THREE-COMP.AIR (CH4, CO2 and N2)',
     X'ISOTHERMAL',/,56X,'CAN USE WITH OPTION MOP(24)=1'/,
     X44X,'(4,3,2,6) - WATER AND THREE-COMP.AIR (CONS.CH4&CO2',
     X'FRACTION), ISOTHERMAL',/,56X,'USE WITH THE OPTION MOP(24)=0')
      PRINT 306
      PRINT 304
  304 FORMAT(//26H THE PRIMARY VARIABLES ARE/' 1) P: PRESSURE  ',
     A'2) X: AIR MASS FRACTION  ',
     B'3) XCH4: CH4 MASS FRACTION IN AIR','  4) XCO2: CO2 ', 
     C'MASS FRACTION IN AIR ',' 5) T: TEMPERATURE ')
      PRINT 3041
 3041 FORMAT(17X,'2) Sg+10.-(GAS SAT+10.)')
      PRINT 305
  305 FORMAT(/' ',55('_')//
     X'     PHASE CONDITION               PRIMARY VARIABLES'/
     X' ',55('_')//
     X'     SINGLE-PHASE GAS              P,     X, XCH4, XCO2, T'//
     X'     SINGLE-PHASE LIQUID           P,     X, XCH4, XCO2, T'//
     X'     TWO-PHASE                     P, S+10., XCH4, XCO2, T'//
     X' ',55('_')//)
      PRINT 299
  299 FORMAT(' ',131('*')//
     X' CHOICES BY WAY OF THE MOP-ARRAY ARE NOT ALLOWED IN LGMEOS3.')
C
      PRINT 298,MOP(19)
  298 FORMAT(/,
     X' MOP(19) =',I2,' *** DOES NOT ALLOW INITIALIZATION WITH', 
     X' DIFFERENT SETS OF PRIMARY VARIABLES.',
     X/,'         = 0: (P,X,XCH4,XCO2,T) FOR SINGLE PHASE, (P,S+10,XCH4',
     X'XCO2,T)','FOR TWO-XPHASE',
     X/,'         = 1: SAME PRIMARY VARIABLES BUT WARNING ISSUED',/)
C
C     *** END MODIFICATIONS ***
C
      PRINT 21
   21 FORMAT(44H ..... NEGLECT VAPOR PRESSURE LOWERING .....)
      PRINT 306
  306 FORMAT(/' ',131('*'))
C###########################################################################
C################################first time step KC=0. starts initialization
C###########################################################################
C-----COME HERE TO CONVERT INITIAL CONDITIONS. NEL: TOTAL No of elements
C ovde doaga samo ako KC=0, toa e samo vo prviot time step
C---------------------------------------------------------------------------
      DO 41 N=1,NEL
C---------------------------------------------------------------------------
C allows NLOC=5 places for primary variables. and NLOC2=132=6*22 places 
C per element for secondary variables. Zema 6 posledovatelni stepovi
        NLOC=(N-1)*NK1
        NLOC2=(N-1)*NSEC*NEQ1
        IF (X(NLOC+1).GT.374.15) GO TO 42
C this will usually not happen; press. DEP(1) was stared as X(1)
C if x(1) < 374.15 then x(1) represents  temperature and one has
C to change the order of variables so that x(1)=vap.sat.pressure
C     *** BEGINNING MODIFICATIONS ******************will not use
C     CHANGE FROM (T,...,...,...,...) TO (PSAT,...,...,...,T)
        TX=X(NLOC+1)
        CALL SAT(TX,PX)
        IF (IGOOD.NE.0) GO TO 200
        X(NLOC+1)=PX
        X(NLOC+5)=TX
C
   42   CONTINUE
C
C-----NO OPTIONAL CONVERSION FROM TOUGH-TYPE INITIALIZATION in LGMEOS3
C mop(19)=0
      IF(MOP(19).EQ.0) GOTO 142
      PRINT 600
  600 FORMAT(' *** OPTION MOP(19)=1 NO LONGER ACTIVE *** ')
C
C     *** END MODIFICATIONS ***
C
  142 CONTINUE
C x(1)>374.15 then x(1) initial phase pressure, go zapisuva vo /E5/P(NEL)
C     *** BEGINNING MODIFICATIONS *** mass frac. in air X3=XCH4 and X4=XCO2
        P(N)=X(NLOC+1)
        X3=X(NLOC+3)
        X4=X(NLOC+4)
C     *** END MODIFICATIONS ***
C-----INITIALIZE GAS SATURATION FOR PHASE CHOICE.
C gas sat. Sg=X(NLOC+2) if <5. this means single phase conditions
        IF (X(NLOC+2).LT.5.) GO TO 44
C #initialization###################################### two phase conditions
C-COME HERE FOR TWO-PHASE POINTS. PAR(1)=Sg=X(NLOC+2)=S+10(Gas Satur.+10)
C     *** BEGINNING MODIFICATIONS ***
        if(mop(7).ne.0) PRINT *,'ELEMENT=',ELEM(N),
     x' INITIALIZATION: TWO PHASE COND.'
        PAR(NLOC2+1)=X(NLOC+2)-10.
        TX=X(NLOC+5)
        T(N)=TX
        CALL SAT(TX,PSAT)
C     *** END MODIFICATIONS ***
C if TOTAL gas press.X(1)>PSAT sat.vap.press.INITIALIZATION.FINISHED.for elem.
        IF (X(NLOC+1).GE.PSAT*(1.-1.E-12)) GO TO 41
        PRINT 204,ELEM(N),TX,PSAT,X(NLOC+1)
  204   FORMAT(' TWO-PHASE CONDITIONS SPECIFIED AT ELEMENT *',A5,
     1    '* ARE ERRONEOUS'/' AT T = ',E12.6,' DEG-C SATURATION',
     2    ' PRESSURE PSAT = ',E20.13,' PA EXCEEDS TOTAL PRESSURE P = ',
     3    E20.13,' PA'/
     4    ' ACTION TAKEN: SET P = PSAT   ---   EXECUTION CONTINUES'/)
c if P<PSAT  =>>  P=PSAT and INITIALIZATION FINISHED for that element
      X(NLOC+1)=PSAT
        GO TO 41
C #initialization###################################### one phase conditions
C now X2 is air mass fraction (kg/kg) or (g/g) in the phase. which? it doesn't know.
C 
   44   X2=X(NLOC+2)
C----COME HERE FOR SINGLE PHASE POINTS. X2=X(2) air mass fraction in a phase
C                                                *** BEGINNING MODIFICATIONS
        TX=X(NLOC+5)
        DENOM=X2*(1.-X3-X4)/AMN2+(1-X2)/AMH2O+X2*X3/AMCH4+X2*X4/AMCO2
        XMOLH2O=((1.-X2)/AMH2O)/DENOM
        XMOLN2=(X2*(1.-X3-X4)/AMN2)/DENOM
        XMOLCH4=(X2*X3/AMCH4)/DENOM
        XMOLCO2=(X2*X4/AMCO2)/DENOM
        CALL HENRYC(TX,HCN2,HCCH4,HCCO2)
        if(mop(7).ne.0) print *,'t=',tx,' HCN2=',
     x  hcn2,' HCCH4=',hcch4,' HCCO2=',hcco2
        PN2=XMOLN2/HCN2
        PCH4=XMOLCH4/HCCH4
        PCO2=XMOLCO2/HCCO2
        PAIR=PN2+PCH4+PCO2
C                                                      *** END MODIFICATIONS
        T(N)=TX
        CALL SAT(TX,PSAT)
        IF (IGOOD.NE.0) GO TO 200
C 
        PS=X(NLOC+1)-PAIR
        IF (PS.LT.PSAT) GO TO 45
C if vapor pres. PS<PSAT sat.vap.pres.; gaseous phase PAR(NLOC2+1)=Sg=1.
C if vapor pres. PS>PSAT sat.vap.pres.; liquid  phase PAR(NLOC2+1)=Sg=0.
        PAR(NLOC2+1)=0.
        if(mop(7).ne.0) PRINT *,'ELEMENT=',ELEM(N),
     x' INITIALIZATION: ONE PHASE - LIQUID'
        if(mop(7).ne.0) print *,'XMOLH2O=',xmolh2o,
     x' XMOLN2=',xmoln2,' XMOLCH4=',xmolch4,' XMOLCO2=',
     x  xmolco2,' HCN2=',hcn2,' HCCH4=',hcch4,' HCCO2=',hcco2
        if(mop(7).ne.0) print *,'pn2',pn2,' pch4',
     x  pch4,' pco2',pco2,' pair',
     1          pair,' T',tx,' ps',ps,' psat',psat
        GO TO 41
C
   45   PAR(NLOC2+1)=1.
        PRINT *,'ELEMENT=',ELEM(N),' INITIALIZATION: ONE PHASE - GAS'
        print *,'pn2',pn2,' pch4',pch4,' pco2',pco2,' pair',
     1          pair,' T',tx,' ps',ps,' psat',psat
C
   41   CONTINUE
C###########################################################################
C################################################# (initialization finished)
C###########################################################################
   40  CONTINUE
C                            come here if it is not the first time step KC>0
      IF (MOP(5).GE.7) PRINT 31
   31 FORMAT(/' PRIMARY VARIABLES'/)
C
C ##########################################################################
C ############################################### succesive time steps; KC>0
C ##########################################################################
C
      N1=NEL
C if this is not the first time step KC>0 N1=NELA No of active elements
      IF(KC.GT.0) N1=NELA
C---------------------------------------------------------------------------
C---------------------------------------------------------------------------
C----------------------------------------glavna do-loop po site vol.elementi
      DO 1 N=1,N1
C---------------------------------------------------------------------------
C---------------------------------------------------------------------------
C---------------------------------------------------------------------------
        NMAT=MATX(N)
        NLOC=(N-1)*NK1
        NLOC2=(N-1)*NSEC*NEQ1
C-allows NLOC=5 places and NLOC2=132 places per element 
C-gi modificira X(M) vo XX(M) so dodavanje DX od prethodnata iteracija
c X(M) se od prethodnata iteracija (ili originalnite sto jas go zadavam)
        DO 2 M=1,NK1
          XINCR=0.
C                                                       ITER is iteration No
          IF (ITER.NE.0) XINCR=DX(NLOC+M)
          XX(M)=X(NLOC+M)+XINCR
    2     CONTINUE
cc        IF(KC.EQ.0) PRINT 35,ELEM(N),(XX(M),M=1,NK1)
C
        IF (ABS(XX(1)).GT.1.E8) IGOOD=5
C
        IF (MOP(5).GE.7) PRINT 35,ELEM(N),(XX(M),M=1,NK1)
C                                                      *** BEG MODIFICATIONS
   35   FORMAT(' AT ELEMENT *',A5,'* --- ',(5(2X,E12.6)))
C                                                      *** END MODIFICATIONS
C
        IF (IGOOD.NE.0) GO TO 200
        IF (IHALVE.EQ.0) GO TO 36
C if not
C #succ.time.steps########################## MAKE CHOICE OF PHASE CONDITIONS
C-----COME HERE FOR PHASE CHOICE AFTER TIME STEP REDUCTION FOR
C     LACK OF CONVERGENCE (IHALVE.ne.0).
        IF (XX(2).GT.5.) GO TO 10
C-COME HERE FOR SINGLE-PHASE POINTS. XX(2): gas.mass.frac** BEG MODIFICATIONS
C AMM: average air(air+CH4+CO2) molecular mass~25
C XMOLE: air mole fraction ~.25
c        AMM=1./((XX(3)/AMCH4)+(XX(4)/AMCO2)+((1.-XX(3)-XX(4))/AMN2))
c        XMOLE=(XX(2)/AMM)/((XX(2)/AMM)+(1.-XX(2))/AMH2O)
c        PA=XMOLE/HC 
         DENOM=XX(2)*(1.-XX(3)-XX(4))/AMN2+(1-XX(2))/AMH2O+
     1         XX(2)*XX(3)/AMCH4+XX(2)*XX(4)/AMCO2
        XMOLH2O=((1.-XX(2))/AMH2O)/DENOM
        XMOLN2=(XX(2)*(1.-XX(3)-XX(4))/AMN2)/DENOM
        XMOLCH4=(XX(2)*XX(3)/AMCH4)/DENOM
        XMOLCO2=(XX(2)*XX(4)/AMCO2)/DENOM
        XMOLL=XMOLN2+XMOLCH4+XMOLCO2
        CALL HENRYC(XX(5),HCN2,HCCH4,HCCO2)
        PN2=XMOLN2/HCN2
        PCH4=XMOLCH4/HCCH4
        PCO2=XMOLCO2/HCCO2
        PA=PN2+PCH4+PCO2
        CALL SAT(XX(5),PSAT)
C                                                      *** END MODIFICATIONS
        IF (IGOOD.NE.0) GO TO 200
        PG=PA+PSAT
c        PRINT *
c        PRINT *,'XX(1),XMOLE,PA,PG,PSAT',XX(1),XMOLL,PA,PG,PSAT
        IF (XX(1).GT.PG) GO TO 8
        GO TO 9
C go to 8 liquid phase only Sg=0.; go to 9 gaseous phase only Sg=1.
   36   CONTINUE
C-COME HERE FOR PHASE CHOICE IN "NORMAL" CASE (converg.vo preth time step)
C sega gi usmeruva spored Sg vo 8) liquid 9) gas and 10) two phases
        SG=PAR(NLOC2+1)
        IF (SG.EQ.1.) GO TO 9
        IF (SG.EQ.0.) GO TO 8
        GO TO 10
C ##succesive time steps####################################################
C ##################################################### ONE phase conditions
C #######################LIQUID-----LIQUID-----LIQUID#######################
C
    8   CONTINUE
c        PRINT *,'-'
c        PRINT *,'ELEMENT=',ELEM(N),'   ONE PHASE CONDITION LIQUID'
C                        main DO-loop 100 across No of equations + 1; NEQ1=6
        DO 100 K=1,NEQ1
C                                           NLOC2=(1-N)*132; K=1,6; NSEC=22;
C                          NLK2  start sec.variables gaseous PAR( 1)~PAR(10)
C                          NLK2L start sec.variables liquid  PAR(11)~PAR(20)
          NLK2=NLOC2+(K-1)*NSEC
          NLK2L=NLK2+NBK
C                                     TOTAL pressure of the liquid phase PX
C                                     x2 is air mass frac. in liquid phase
          PX=XX(1)
          X2=XX(2)
          X3=XX(3)
          X4=XX(4)
          TX=XX(5)
c          if(k.eq.1) PRINT 453,ELEM(N),PX,X2,X3,X4,TX
          IF (K.NE.1) GO TO 103
          IF (X2.GE.0.) GO TO 104
C dojdi ovde samo za prvata eq. K=1 first equation mass constraints
C-----COME HERE TO ENFORCE MASS FRACTION CONSTRAINT for AIR,CH4 & CO2.
          XX(2)=0.
          X2=0.
          DX(NLOC+2)=-X(NLOC+2)
C
  104     CONTINUE
C                                                      *** BEG MODIFICATIONS
          IF (X3.GE.0.) GO TO 603
          XX(3)=0.
          X3=0
          DX(NLOC+3)=-X(NLOC+3)
  603     CONTINUE
          IF (X4.GE.0.) GO TO 604
          XX(4)=0.
          X4=0
          DX(NLOC+4)=-X(NLOC+4)
  604     CONTINUE
C
C-----DO PHASE CHECK. COMPUTE AIR MOLE FRACTION.                                               
c          AMM=1./((X3/AMCH4)+(X4/AMCO2)+((1.-X3-X4)/AMN2))
c          XMOLE=(X2/AMM)/(X2/AMM+(1.-X2)/AMH2O)
C provereno
        DENOM=X2*(1.-X3-X4)/AMN2+(1-X2)/AMH2O+X2*X3/AMCH4+X2*X4/AMCO2
        XMOLH2O=((1.-X2)/AMH2O)/DENOM
        XMOLN2=(X2*(1.-X3-X4)/AMN2)/DENOM
        XMOLCH4=(X2*X3/AMCH4)/DENOM
        XMOLCO2=(X2*X4/AMCO2)/DENOM
        XMOLL=XMOLN2+XMOLCH4+XMOLCO2
        CALL HENRYC(TX,HCN2,HCCH4,HCCO2)
        PN2=XMOLN2/HCN2
        PCH4=XMOLCH4/HCCH4
        PCO2=XMOLCO2/HCCO2
        PAIR=PN2+PCH4+PCO2
C                                                      *** END MODIFICATIONS
          CALL SAT(TX,PSAT)
          IF (IGOOD.NE.0) GO TO 200
          PG=PAIR+PSAT
c        PRINT *,'XMOLE,PA,PG,PS',XMOLL,PAIR,PG,PSAT
          IF (PX.GE.PG) GO TO 101
C liquid phase pres.PX<PG gaseous phase pres. gas phase saturation starts 
C XX(2) stanuva gas.sat.(ne e vise gas frac) Sg=zero+10 at the element
C-----COME HERE FOR TRANSITION TO TWO PHASE CONDITIONS.
          IF (MOP(5).GE.3.OR.KC.EQ.0) PRINT 50,ELEM(N),X2,PX,PG
   50     FORMAT(' $$$$$$$$$$$$$$$$ GAS PHASE EVOLVES AT ELEMENT *',A5,
     1'*  $$$$$',3X,' XAIR = ',E12.6,'    PX = ',E12.6,'   PG = ',E12.6)
          IF(MOP(5).GE.3.OR.KC.EQ.0)PRINT 55,X3,X4,HCN2,HCCH4,HCCO2
   55     FORMAT(5(E10.6,5X))
          IF (KC.EQ.0) GO TO 202
          XX(2)=ZERO+10.
          DX(NLOC+2)=XX(2)-X(NLOC+2)
c go to two phase conditions
          GO TO 10
C 
  103     CONTINUE
C dojdi ovde za equations K=2,NEQ1; DFAC-increment, jas mu go davam~1.10-8
C K=2 pres.X(1); K=3 gas frac.X(2); K=4 CH4 X(3); K=5 CO2 X(4); K=6 Temp.X(5
C ova e vazno za iteraciite im dodava incrementi DELX na primary variables
C za pressure and temp. DELX=DFAC*X, a za mass fractions DELX=DFAC
C vo EOS2 ima: za CO2 partial pressure DELX(NLOC+3)=MAX(0.1,DELX(NLOC+3))
C                                      PCX=XX(3)+DELX(NLOC+3)
          DELX(NLOC+K-1)=DFAC*XX(K-1)+1.E-10
          IF (K.EQ.2) PX=XX(1)+DELX(NLOC+1)
          IF (K.EQ.3) DELX(NLOC+2)=DFAC
          IF (K.EQ.3) X2=XX(2)+DELX(NLOC+2)
C                                                      *** BEG MODIFICATIONS
          IF (K.EQ.4) DELX(NLOC+3)=DFAC
          IF (K.EQ.4) X3=XX(3)+DELX(NLOC+3)
          IF (K.EQ.5) DELX(NLOC+4)=DFAC
          IF (K.EQ.5) X4=XX(4)+DELX(NLOC+4)
          IF (K.EQ.6) TX=XX(5)+DELX(NLOC+5)
C                                                      *** END MODIFICATIONS
  101     CONTINUE
C naoga wat.visc.VW selon temp TX, tot.phase.pres.PX & w.vap.part.press PSAT
          CALL VISW(TX,PX,PSAT,VW)
c naoga water density D and internal energy U
          CALL COWAT(TX,PX,D,U)
          IF (IGOOD.NE.0) GO TO 200
c naoga capillary press.PC za Sw=1
          CALL PCAP(1.,TX,PC,NMAT)
C II) LIQUID PHASE: PAR(11): Sw=1 saturation; PAR(12): Rw rel.permeability
C     PAR(13): Vw viscosity; PAR(14): Dw density; PAR(15): Hw spec.enthalpy
C     PAR(16): capillary pressure; PAR(17=NB+1) H2O mass fraction
C     PAR(18=NB+2): N2 mass fraction in liquid phase;
C     PAR(19=NB+3): CH4 mass in liquid phase; PAR(20=NB+4): CO2 mass in liq.
          PAR(NLK2L+1)=1.
          PAR(NLK2L+2)=1.
          PAR(NLK2L+3)=VW
          PAR(NLK2L+4)=D
          PAR(NLK2L+5)=U+PX/D
          PAR(NLK2L+6)=PC
          PAR(NLK2L+NB+1)=1.-X2
C                                                      *** BEG MODIFICATIONS
          PAR(NLK2L+NB+2)=X2*(1.-X3-X4)
          PAR(NLK2L+NB+3)=X2*X3
          PAR(NLK2L+NB+4)=X2*X4
C                                                      *** END MODIFICATIONS
C I) GASEOUS PHASE: PAR(1): Sg=0 saturation; PAR(2): Rg rel.permeability
C    PAR(3): Vg viscosity; PAR(4): Dg density; PAR(5): Hg spec.enthalpy
C    PAR(6): capillary pressure; PAR(7=NB+1) H2O mass fraction
C    PAR(8=NB+2): N2 mass fraction in gaseous phase;
C    PAR(9=NB+3): CH4 mass in gaseous phase; PAR(20=B+4): CO2 mass
          PAR(NLK2+1)=0.
          PAR(NLK2+2)=0.
          PAR(NLK2+3)=1.
          PAR(NLK2+4)=0.
          PAR(NLK2+5)=0.
          PAR(NLK2+6)=0.
          PAR(NLK2+NB+1)=0.
          PAR(NLK2+NB+2)=0.
C                                                      *** BEG MODIFICATIONS
          PAR(NLK2+NB+3)=0.
          PAR(NLK2+NB+4)=0.
C                                                      *** END MODIFICATIONS
          PAR(NLK2+NSEC-1)=TX
          PAR(NLK2+NSEC)=0.
C
  100     CONTINUE
C                                               go to 1 : idi na nov element
        GO TO  1
C
    9   CONTINUE
c        PRINT *,'ELEMENT=',ELEM(N),'   ONE PHASE CONDITION GAS'
C ##succesive time steps####################################################
C ##################################################### ONE phase conditions
C #########################GAS----GAS----GAS----GAS#########################
C main DO-loop 110 across No of equations + 1; NEQ1=6
C NLOC2=(1-N)*132; K=1,6; NSEC=22; NBK=10
C NLK2  pocetok na sec.variables gaseous PAR( 1)~PAR(10)
C NLK2L pocetok na sec.variables liquid  PAR(11)~PAR(20)
        DO 110 K=1,NEQ1
          NLK2=NLOC2+(K-1)*NSEC
          NLK2L=NLK2+NBK
C  PX: TOTAL gas phase pressure PX; x2 is air mass fraction in phase
          PX=XX(1)
          X2=XX(2)
          X3=XX(3) 
          X4=XX(4)
          TX=XX(5)
C 
          IF (K.NE.1) GO TO 115
          IF (X2.GE.0.) GO TO 105
C-COME HERE K=1 first equation TO ENFORCE MASS FRACTION CONSTRAINT.
          XX(2)=0.
          X2=0.
          DX(NLOC+2)=-X(NLOC+2)
  105     CONTINUE
C                                                      *** BEG MODIFICATIONS
          IF (X3.GE.0.) GO TO 602
          XX(3)=0.
          X3=0.
          DX(NLOC+3)=-X(NLOC+3)
  602     CONTINUE
          IF (X4.GE.0.) GO TO 605
          XX(4)=0.
          X4=0.
          DX(NLOC+4)=-X(NLOC+4)
  605     CONTINUE
C                                                      *** END MODIFICATIONS
C-----DO PHASE CHECK.
C calculates vapor saturated pressure PSAT for given temp TX
          CALL SAT(TX,PSAT)
          IF (IGOOD.NE.0) GO TO 200
C puts liquid sat. SL=zero and checks capillary pressure PC at temp. TX
          SL=ZERO
          CALL PCAP(SL,TX,PC,NMAT)
          PS=PX
C if air mass frac.=0. and TOTAL gas pressure<sat. vapor press. PSAT
          IF (X2.EQ.0..AND.PX.LT.PSAT) GO TO 111
C
C-----DETERMINE PARTIAL PRESSURE OF VAPOR IN GAS PHASE.
C
C          IF (KC.NE.0) GO TO 116
C                                   KC>0 means this is not the 1st time step
C-----INITIAL GUESS FOR vapor part.press. PS IS BASED ON IDEAL GAS LAW.
C                                                      *** BEG MODIFICATIONS
          AMM=1./((X3/AMCH4)+(X4/AMCO2)+((1.-X3-X4)/AMN2))
          IF (KC.NE.0) GO TO 116
          PS=(1.-X2)*AMM*PX/((1.-X2)*AMM+X2*AMH2O)
C                                   *** END MODIFICATIONS ***+++++++PROVERIV
          GO TO 113
C-----SUBSEQUENT GUESSES for vap.part.press.PS are TAKEN FROM LAST ITERATION
  116     PS=PAR(NLK2L+5)
C calculates vapor pressure PS(VLEZ&IZLEZ), density DS and int.energy US
C for given TOTAL gas pressure PX, temp. TX and air mass fract. X2
  113     CALL PP(PX,TX,X2,AMM,PS,DS,US)
          IF (IGOOD.NE.0) GO TO 200
C if vapor partial pressure PS < saturated vapor pressure PSAT
          IF (PS.LT.PSAT) GO TO 111
C-----COME HERE FOR TRANSITION TO TWO-PHASE CONDITIONS.liquid phase starts
          IF (MOP(5).GE.3.OR.KC.EQ.0) PRINT 51,ELEM(N),PS,PSAT
   51     FORMAT(' $$$$$$$$$$$$$ LIQUID PHASE EVOLVES AT ELEMENT *',A5,
     1      '*  $$$$$      PS = ',E12.6,'  PSAT = ',E12.6)
          IF (KC.EQ.0) GO TO 202
          XX(2)=1.-ZERO+10.
          DX(NLOC+2)=XX(2)-X(NLOC+2)
c                       go to two phase conditions XX(2) becomes gas.sat. Sg
          GO TO 10
C                                                   dojdi ovde ako e K>1
  115     CONTINUE
C ova e vazno za iteraciite im dodava incrementi DELX na primary variables
C za pressure and temp. DELX=DFAC*X, a za mass fractions DELX=DFAC
C vo EOS2 ima: za CO2 partial pressure DELX(NLOC+3)=MAX(0.1,DELX(NLOC+3))
C                                      PCX=XX(3)+DELX(NLOC+3)
          DELX(NLOC+K-1)=DFAC*XX(K-1)+1.E-10
          IF (K.EQ.2) PX=XX(1)+DELX(NLOC+1)
          IF (K.EQ.3) DELX(NLOC+2)=DFAC
          IF (K.EQ.3) X2=XX(2)+DELX(NLOC+2)
C                                                      *** BEG MODIFICATIONS
          IF (K.EQ.4) DELX(NLOC+3)=DFAC
          IF (K.EQ.4) X3=XX(3)+DELX(NLOC+3)
          IF (K.EQ.5) DELX(NLOC+4)=DFAC
          IF (K.EQ.5) X4=XX(4)+DELX(NLOC+4)
          IF (K.EQ.6) TX=XX(5)+DELX(NLOC+5)
C                                                      *** END MODIFICATIONS
C
  111     CONTINUE
      IF(K.NE.1) CALL PCAP(SL,TX,PC,NMAT)
C
          AMM=1./((X3/AMCH4)+(X4/AMCO2)+((1.-X3-X4)/AMN2))
          CALL PP(PX,TX,X2,AMM,PS,DS,US)
          IF (IGOOD.NE.0) GO TO 200
C          UAIR=CVAIR*TX
C
C-----NOW FOR THE REAL GAS LAW FOR AIR.
C                                                      *** BEG MODIFICATIONS
          AMM=1./((X3/AMCH4)+(X4/AMCO2)+((1.-X3-X4)/AMN2))
          DAIR=(PX-PS)*AMM/(ZA*R*(TX+273.15))         
C                                                      *** END MODIFICATIONS
          DGAS=DAIR+DS
C
          CALL VISCOSITY(TX,DS,X2,X3,X4,VISMIX)
          CALL SHEAT(TX,CVAIR,CVCH4,CVCO2)
C          TXk=TX+273.15
C          CALL VISCO(TX,PX,X2,DS,VISMIX)
C I) GASEOUS PHASE: PAR(1): Sg=1 saturation; PAR(2): Rg rel.permeability
C    PAR(3): Vg viscosity; PAR(4): Dg density; PAR(5): Hg spec.enthalpy
C    PAR(6): capillary pressure; PAR(7=NB+1) H2O mass fraction
C    PAR(8=NB+2): N2 mass fraction in gaseous phase;
C    PAR(9=NB+3): CH4 mass; PAR(20=B+4): CO2 mass
          PAR(NLK2+1)=1.
          PAR(NLK2+2)=1.
          PAR(NLK2+3)=VISMIX
          PAR(NLK2+4)=DGAS
c          PAR5=X2*UAIR+(1.-X2)*US+PX/DGAS
          PAR(NLK2+5)=X2*(1.-X3-X4)*CVAIR*TX+X2*X3*CVCH4*TX+
     x                X2*X4*CVCO2*TX+(1.-X2)*US+PX/DGAS
          PAR(NLK2+6)=0.
          PAR(NLK2+NB+1)=1.-X2
C                                                      *** BEG MODIFICATIONS
          PAR(NLK2+NB+2)=X2*(1.-X3-X4)
          PAR(NLK2+NB+3)=X2*X3
          PAR(NLK2+NB+4)=X2*X4
C                                                      *** END MODIFICATIONS
C
C II) LIQUID PHASE: PAR(11): Sw=0 saturation; PAR(12): Rw rel.permeability
C     PAR(13): Vw viscosity; PAR(14): Dw density; PAR(15): Hw spec.enthalpy
C     PAR(16): capillary pressure; PAR(17=NB+1) H2O mass fraction
C     PAR(18=NB+2): N2 mass fraction in liquid phase;
C     PAR(19=NB+3): CH4 mass; PAR(20=NB+4): CO2 mass
          PAR(NLK2L+1)=0.
          PAR(NLK2L+2)=0.
          PAR(NLK2L+3)=1.
          PAR(NLK2L+4)=0.
          PAR(NLK2L+5)=PS
          PAR(NLK2L+6)=PC
          PAR(NLK2L+NB+1)=0.
          PAR(NLK2L+NB+2)=0.
C                                                      *** BEG MODIFICATIONS
          PAR(NLK2L+NB+3)=0.
          PAR(NLK2L+NB+4)=0.
C                                                      *** END MODIFICATIONS
          PAR(NLK2+NSEC-1)=TX
          PAR(NLK2+NSEC)=0.
C
  110     CONTINUE
C
        GO TO  1
C
   10   CONTINUE
c        PRINT *,'ELEMENT=',N,'   TWO PHASE CONDITION '
C ##succesive time steps####################################################
C ##################################################### TWO phase conditions
C ##########################################################################
C main DO-loop 120 across No of equations + 1; NEQ1=NEQ+1=NK+1+1=6
C NLOC2=(1-N)*132; K=1,6; NSEC=NPH*NBK+2=22; NBK=NB+NK=10; NK=4 mass compon.
C NLK2  pocetok na sec.variables in gaseous phase PAR( 1)~PAR(10)
C NLK2L pocetok na sec.variables in liquid  phase PAR(11)~PAR(20)
C XX(I) se primary variables updated so dx ( XX(M)=X(NLOC+M)+DX(NLOC+M)
          DO 120 K=1,NEQ1
          NLK2=NLOC2+(K-1)*NSEC
          NLK2L=NLK2+NBK
          PX=XX(1)
          SX=XX(2)-10.
          X3=XX(3)
          X4=XX(4)
          TX=XX(5)
c          if (k.eq.1 .and. n.eq.1)PRINT *,'----------------'
c          if (k.eq.1)PRINT 453,ELEM(N),PX,SX,X3,X4,TX
453       format(A5,2X,5f20.10)
C                                                      *** END MODIFICATIONS
C                                        K=1 first equation mass constraints
          IF (K.NE.1) GO TO 123
C
C-----DO PHASE CHECK.
C
          IF (0.LT.SX.AND.SX.LT.1.) GO TO 121
C 121 - pure two phase conditions
C #succ.time.steps#####################TRANSITION TO SINGLE PHASE CONDITIONS
          CALL SAT(TX,PSAT)
C
          IF (SX.GT.1.) GO TO 124
C dojdi ovde if Sg < 0.
C a) #succ.time.steps******************PHASE TRANSITION TO LIQUID CONDITIONS
c          PRINT *,'PHASE TRANSITION TO LIQUID CONDITIONS'
          IF (MOP(5).GE.3.OR.KC.EQ.0) PRINT 52,ELEM(N),SX
   52     FORMAT(' $$$$$$$$$$$$$ GAS PHASE DISAPPEARS AT ELEMENT *',A5,
     1      '*  $$$$$      SG = ',E12.6)
          IF (KC.EQ.0) GO TO 202
C ** BEGINNING MODIFICATIONS ** XX(2) is not air mass fract. tuku Sg. ZATOA:
C air mass fract. N2+CH4+CO2 in liquid ph=xx(2)=x2*(1-x3-x4)+x2*x3+x2*x4
C gi stavam site XX(I) da se = na par(nbk+nb+i),
          XX(2)=PAR(NLOC2+NBK+NB+2)+PAR(NLOC2+NBK+NB+3)+
     1          PAR(NLOC2+NBK+NB+4)
          XX(3)=PAR(NLOC2+NBK+NB+3)/XX(2)
          XX(4)=PAR(NLOC2+NBK+NB+4)/XX(2)
C     *** END MODIFICATIONS ***
          DX(NLOC+2)=XX(2)-X(NLOC+2)
c ovie dve jas gi stavam RENE? gi nema+++++++++++++++++
          DX(NLOC+3)=XX(3)-X(NLOC+3)
          DX(NLOC+4)=XX(4)-X(NLOC+4)
C-----INITIALIZE PRESSURE AS SUM OF SATURATED VAPOR AND AIR PRESSURES.
C *** BEGINNING MODIFICATIONS **PAZI XX(3)&XX(4) se mass.fract. vo gas ph???
C     AMM=1./((XX(3)/AMCH4)+(XX(4)/AMCO2)+((1.-XX(3)-XX(4))/AMN2))
C     XMOLE=(XX(2)/AMM)/((XX(2)/AMM)+(1.-XX(2))/AMH2O)
        DENOM=XX(2)*(1.-XX(3)-XX(4))/AMN2+(1-XX(2))/AMH2O+
     1        XX(2)*XX(3)/AMCH4+XX(2)*XX(4)/AMCO2
        XMOLH2O=((1.-XX(2))/AMH2O)/DENOM
        XMOLN2=(XX(2)*(1.-XX(3)-XX(4))/AMN2)/DENOM
        XMOLCH4=(XX(2)*XX(3)/AMCH4)/DENOM
        XMOLCO2=(XX(2)*XX(4)/AMCO2)/DENOM
        CALL HENRYC(TX,HCN2,HCCH4,HCCO2)
        PN2=XMOLN2/HCN2
        PCH4=XMOLCH4/HCCH4
        PCO2=XMOLCO2/HCCO2
        PAIR=PN2+PCH4+PCO2
C *** END MODIFICATIONS ***
      XX(1)=(1.+ZERO)*(PSAT+PAIR)
      DX(NLOC+1)=XX(1)-X(NLOC+1)
          PRINT *,'go to 8 :  ONE PHASE LIQUID CONDITIONS ',ELEM(N)
          GO TO 8
C go upwards out of 120 do-loop to one phase liquid conditions
  124     CONTINUE
C                                                      dojdi ovde if Sg > 1.
C b) #succ.time.steps********************PHASE TRANSITION TO GASEOUS CONDITIONS
c          PRINT *,'PHASE TRANSITION TO GASEOUS CONDITIONS'
          IF (MOP(5).GE.3.OR.KC.EQ.0) PRINT 53,ELEM(N),SX
   53     FORMAT(' $$$$$$$$$$ LIQUID PHASE DISAPPEARS AT ELEMENT *',A5,
     1      '*  $$$$$      SG = ',E12.6)
          IF (KC.EQ.0) GO TO 202
C ** BEGINNING MODIFICATIONS ** XX(2) is not air mass fract. tuku Sg. ZATOA:
C air mass fract. N2+CH4+CO2 in liquid ph=xx(2)=x2*(1-x3-x4)+x2*x3+x2*x4
C ovde se xx(3)&xx(4) mass fractions vo gaseous phase, a xx(2) mora da go
C zeme od PAR zasto e inace xx(2)=Sg
          XX(2)=PAR(NLOC2+NB+2)+PAR(NLOC2+NB+3)+PAR(NLOC2+NB+4)
          XX(3)=PAR(NLOC2+NB+3)/XX(2)
          XX(4)=PAR(NLOC2+NB+4)/XX(2)
C                                                      *** END MODIFICATIONS
          IF (XX(2).GE.1.) XX(2)=1.-ZERO
          DX(NLOC+2)=XX(2)-X(NLOC+2)
c ovie jas gi staviv RENE? gi nema+on mi kaza deka mu plante ako menja phase
          DX(NLOC+3)=XX(3)-X(NLOC+3)
          DX(NLOC+4)=XX(4)-X(NLOC+4)
C-----RESET PRESSURE TO BE SLIGHTLY SMALLER THAN THE SUM OF AIR
C     PARTIAL PRESSURE AND SATURATED VAPOR PRESSURE.
          CALL SUPST(TX,PSAT,DS,US)
C     *** BEGINNING MODIFICATIONS ***XX(3)&XX(4) se mass frac.vo gaseous ph.
          AMM=1./((XX(3)/AMCH4)+(XX(4)/AMCO2)+((1.-XX(3)-XX(4))/AMN2))
          PAIR=DS*R*(TX+273.15)*XX(2)/(AMM*(1.-XX(2)))
C                                                      *** END MODIFICATIONS
          XX(1)=(1.-ZERO)*(PSAT+PAIR)
          DX(NLOC+1)=XX(1)-X(NLOC+1)
c         PRINT *,'go to 9 :  ONE PHASE GASEOUS CONDITIONS ',ELEM(N)
          GO TO 9
C go upwards out of do-loop to one phase gaseous conditions
C dojdi ovde za equations povisoki od 2
  123     CONTINUE
C K=2,NEQ1-toa se equat.; K=2 pression; K=3 air; K=4 CH4; K=5 CO2; K=6 Temp
C ?RENE: factor for iterations
          DELX(NLOC+K-1)=DFAC*XX(K-1)+1.E-10
          IF (K.EQ.2) PX=XX(1)+DELX(NLOC+1)
          IF (K.EQ.3) DELX(NLOC+2)=DFAC
          IF (K.EQ.3) SX=XX(2)+DELX(NLOC+2)-10.
C     *** BEGINNING MODIFICATIONS ***
          IF (K.EQ.4) DELX(NLOC+3)=DFAC
          IF (K.EQ.4) X3=XX(3)+DELX(NLOC+3)
          IF (K.EQ.5) DELX(NLOC+4)=DFAC
          IF (K.EQ.5) X4=XX(4)+DELX(NLOC+4)
          IF (K.EQ.6) TX=XX(5)+DELX(NLOC+5)
c          PRINT *,N,'P ',K,PX,SX,X3,X4,TX
C     *** END MODIFICATIONS ***
C
  121     CONTINUE
c###########################################################################
C #succ.time.steps################################ PURE TWO PHASE CONDITIONS
c###########################################################################
c          PRINT *,'PURE TWO PHASE CONDITIONS'
          CALL SAT(TX,PS)
          IF (IGOOD.NE.0) GO TO 200
C ja vika SAT da dobie sat.vapor pressure PS. Ako e PS>PX (gas pression)
C togas stava deka e XX(1)=PS, zasto ne moze da e gas pres.pomal od vapor
C-----NOW A SECTION TO DEAL WITH LARGE TEMPERATURE CHANGES.
          IF (PS.LE.PX .OR. K.NE.1) GO TO 125
C     COME HERE TO ADJUST PRESSURE.
          PX=PS
          XX(1)=PX
          DX(NLOC+1)=PX-X(NLOC+1)
  125     CONTINUE
C ja vika COWAT da dobie water density DW i int.energy UW za dadeni temp TX
C and gas pressure PX
          CALL COWAT(TX,PX,DW,UW)
          IF (IGOOD.NE.0) GO TO 200
C ja vika SUPST da dobie water vapor density DS and int.energy US za dadeni
C temp. Tx i vapor pression PS
          CALL SUPST(TX,PS,DS,US)
C          UAIR=CVAIR*TX
          PAIR=PX-PS
c mole fractions in gas phase
            AMMG=1./((X3/AMCH4)+(X4/AMCO2)+((1.-X3-X4)/AMN2))
            XMOLEAIR=PAIR/PX
            XMOLEH2O=PS/PX
c mass fractions in gas phase
            XAIRG=PAIR*AMMG/(PAIR*AMMG+PS*AMH2O)
            XCH4G=X3*XAIRG
            XCO2G=X4*XAIRG
            XN2G=(1-X3-X4)*XAIRG
            XH2OG=1-XAIRG
c partial pressures in gas phase
            PCH4G=XCH4G*(PAIR*AMMG+PS*AMH2O)/AMCH4
            PCO2G=XCO2G*(PAIR*AMMG+PS*AMH2O)/AMCO2
            PN2G=XN2G*(PAIR*AMMG+PS*AMH2O)/AMN2
            PH2OG=XH2OG*(PAIR*AMMG+PS*AMH2O)/AMH2O
            CALL HENRYC(TX,HCN2,HCCH4,HCCO2)
C mole fractions in liquid phase
            XMOLEN2L=HCN2*PN2G
            XMOLECH4L=HCCH4*PCH4G
            XMOLECO2L=HCCO2*PCO2G
            XMOLEAIRL=XMOLEN2L+XMOLECH4L+XMOLECO2L
            XMOLEH2OL=1-XMOLEAIRL
c mass fractions in liquid phase
c 7 fev 05           DENOM=XMOLEAIRL*AMML+XMOLEH2OL*AMH2O
	      DENOM=XMOLEH2OL*AMH2O
            XN2L=XMOLEN2L*AMN2/DENOM
            XCH4L=XMOLECH4L*AMCH4/DENOM
            XCO2L=XMOLECO2L*AMCO2/DENOM
            XH2OL=XMOLEH2OL*AMH2O/DENOM
c          print 30,elem(n),xh2og,xn2g,xch4g,xco2g,xh2ol,xn2l,
c    xxch4l,xco2l
C 
C-----REAL GAS LAW FOR AIR.
C     *** BEGINNING MODIFICATIONS ***
          DAIR=PAIR*AMMG/(ZA*R*(TX+273.15))
C     *** END MODIFICATIONS ***
          DGAS=DAIR+DS
          XAIRG=DAIR/DGAS
C
          CALL VISW(TX,PX,PS,VW)
          CALL VISCOSITY(TX,DS,XAIRG,X3,X4,VISMIX)
C          CALL VISCO(TX,PX,XAIRG,DS,VISMIX)
          PHZX=SX
      SX0=XX(2)-10.
      KX=K
C KX: mu e code za prviot type function
          CALL RELP(PHZX,REPW,REPS,NMAT,KX,NLOC,SX0)
          CALL PCAP(1.-SX,TX,PC,NMAT)
          CALL SHEAT(TX,CVAIR,CVCH4,CVCO2)
C          TXk=TX+273.15
C I) GASEOUS PHASE: PAR(1): SX gas saturation; PAR(2): REPS gas rel.perm.
C    PAR(3): VISMIX visc. of gas mixture; PAR(4): DGAS dens. of gas mixt; 
C    PAR(5): spec.enthalpy
C    PAR(6): capillary pressure; PAR(7=NB+1) H2O mass fraction
C    PAR(8=NB+2): N2 mass fraction in gaseous phase;
C    PAR(9=NB+3): CH4 mass; PAR(20=B+4): CO2 mass
          PAR(NLK2+1)=SX
          PAR(NLK2+2)=REPS
          PAR(NLK2+3)=VISMIX
          PAR(NLK2+4)=DGAS
c          PAR5=XAIRG*UAIR+(1.-XAIRG)*US+PX/DGAS
          PAR(NLK2+5)=XAIRG*(1.-X3-X4)*CVAIR*TX+XAIRG*X3*CVCH4*TX+
     A                XAIRG*X4*CVCO2*TX+(1.-XAIRG)*US+PX/DGAS
          PAR(NLK2+6)=0.
          PAR(NLK2+NB+1)=1.-XAIRG
C     *** BEGINNING MODIFICATIONS ***
          PAR(NLK2+NB+2)=XAIRG*(1.-X3-x4)
          PAR(NLK2+NB+3)=XAIRG*X3
          PAR(NLK2+NB+4)=XAIRG*X4
C     *** END MODIFICATIONS ***
C
C II) LIQUID PHASE: PAR(11): Sw=1-SX saturation; PAR(12): REPW wat.rel.perm.
C     PAR(13): Vw viscosity; PAR(14): Dw density; PAR(15): Hw spec.enthalpy
C     PAR(16): capillary pressure; PAR(17=NB+1) H2O mass fraction
C     PAR(18=NB+2): N2 mass fraction in liquid phase;
C     PAR(19=NB+3): CH4 mass; PAR(20=NB+4): CO2 mass
          PAR(NLK2L+1)=1.-SX
          PAR(NLK2L+2)=REPW
          PAR(NLK2L+3)=VW
          PAR(NLK2L+4)=DW
          PAR(NLK2L+5)=UW+PX/DW
          PAR(NLK2L+6)=PC
          PAR(NLK2L+NB+1)=1.-(XN2L+XCH4L+XCO2L)
C     *** BEGINNING MODIFICATIONS ***
          PAR(NLK2L+NB+2)=XN2L
          PAR(NLK2L+NB+3)=XCH4L
          PAR(NLK2L+NB+4)=XCO2L
C     *** END MODIFICATIONS ***
C
          PAR(NLK2+NSEC-1)=TX
          PAR(NLK2+NSEC)=0.
C
  120     CONTINUE
C ##################################################################### END
C #########################################################################
    1   CONTINUE
C mop(5)=3 ZNACI KRAJ
      IF (MOP(5).LT.8) GO TO 29
      PRINT 27
   27 FORMAT(/' SECONDARY PARAMETERS')
      DO 28 N=1,NEL
        NLOC2=(N-1)*NSEC*NEQ1
C-----PRINT PARAMETERS AT STATE POINT.
        PRINT 30,ELEM(N),(PAR(NLOC2+M),M=1,NSEC)
C
        IF (MOP(5).EQ.8) GO TO 28
C-----COME HERE TO PRINT INCREMENTED PARAMETERS.
        DO 25 K=2,NEQ1
          PRINT 37,(PAR(NLOC2+(K-1)*NSEC+M),M=1,NSEC)
   25     CONTINUE
   30   FORMAT(/' ELEMENT ',A5/(10(1X,E12.6)))
   37   FORMAT(/(10(1X,E12.6)))
        GO TO 28
C
C-----COME HERE TO PRINT DERIVATIVES.
   26   CONTINUE
        NLOC=(N-1)*NK1
        DO 24 K=2,NEQ1
          DO 23 M=1,NSEC
            DP(M)=(PAR(NLOC2+(K-1)*NSEC+M)-PAR(NLOC2+M))/DELX(NLOC+K-1)
   23       CONTINUE
          PRINT 37,(DP(M),M=1,NSEC)
   24     CONTINUE
   28   CONTINUE
C
   29 CONTINUE
C
      RETURN
C
  200 CONTINUE
      PRINT 201,ELEM(N),(XX(M),M=1,NK1)
  201 FORMAT(' +++++++++   CANNOT FIND PARAMETERS AT ELEMENT *',A5,
     1  '*          XX(M) =',4(1X,E12.6))
      IF(KC.EQ.0) GOTO 202
      RETURN
C
  202 PRINT 203
  203 FORMAT(' !!!!!!!!!!! ERRONEOUS DATA INITIALIZATION !!!!!!!!!!',
     1  11X,'STOP EXECUTION----------')
      STOP
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 2 xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx no changes
       SUBROUTINE PP(P,T,X,AMM,PS,D,U)
      implicit real*8 (a-h,o-z)
C-----THIS ROUTINE COMPUTES VAPOR PARTIAL PRESSURE PS, DENSITY D, AND
C     INTERNAL ENERGY U,  AS FUNCTION OF P,T,X,AMM
C P: TOTAL gas pressure; T: gas temperature;  
C X(X2):air mass fraction. AMM: average air molecular mass
C PS:  vapor partial pressure; ova e i VLEZ & IZLEZ
C D: vapor density; U: vapor internal energy
C     IDEAL GAS LAW IS ASSUMED FOR AIR.
C
C***** ASSUMING THAT SLIGHT CHANGES IN AIR MOLECULAR MASS WILL NOT
C      CHANGE THE COMPUTED STEAM PROPERTIES SIGNIFICANTLY  (R.L.) ***
C
      COMMON/GASLAW/R,AMH2O,AMN2,AMCH4,AMCO2,CVAIR
C
C     *** END MODIFICATIONS ***
C      COMMON/KONIT/KON,DELT,IGOOD                                  JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      SAVE ICALL,ZA,DEL
      DATA ZA,DEL/1.,1.E-9/
C
      DATA ICALL/0/
C samo definira funkcija mora da e vo nacalo F(pression,density)
      F(PX,RHO)=R*(T+273.15)*RHO*ZA*X - (P-PX)*(1.-X)*AMM
C
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
c      PRINT *,'SUBROUTINE PP'
  899 FORMAT(6X,'PP       1.0       1 FEBRUARY  1990',6X,
     X'CALCULATE VAPOR PRESS,DENS,INT.ENERGY=function of F(P,T,X)')
C ako e air mass fraction>.9999
      IF (X.GE..9999) GO TO 2
      PS=MAX(PS,7.E2)
C
      IC=0
    1 CALL SUPST(T,PS,D,U)
      F0=F(PS,D)
C
      IF (ABS(F0).LT.1.E-11*PS.AND.IC.GE.2) RETURN
      DPS=DEL*PS
C calls SUPST for wat.vap.dens.D1&int. energy U1 for given Temp i Press
      CALL SUPST(T,PS+DPS,D1,U1)
      F1=F(PS+DPS,D1)
      DPS=-DPS*F0/(F1-F0)
      PS=PS+DPS
C
      IC=IC+1
      IF (IC.GT.20) GO TO 3
      GO TO 1
C
    2 CONTINUE
      DGAS=P*AMM*AMH2O/(R*(T+273.15)*(X*AMH2O+(1.-X)*AMM))
      D=(1.-X)*DGAS
      U=0.
      PS=D*R*(T+273.15)/AMH2O
C
      RETURN
C
    3 CONTINUE
C-----COME HERE WHEN ITERATION FAILS.
      PRINT 4
    4 FORMAT(' NO CONVERGENCE IN SUBROUTINE PP')
      IGOOD=2
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 3 xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx no changes
      SUBROUTINE VISCO(T,P,XAIR,D,VISMIX)
      implicit real*8 (a-h,o-z)
C-----THIS ROUTINE COMPUTES THE VISCOSITY OF VAPOR-AIR MIXTURES.
C     IT USES A MODIFIED VERSION OF A FORMULATION BASED ON KINETIC
C     GAS THEORY, AS GIVEN BY J.O. HIRSCHFELDER, C.F. CURTISS, AND
C     R.B. BIRD, MOLECULAR THEORY OF GASES AND LIQUIDS, JOHN WILEY
C     & SONS, 1954, PP. 528-530.
C
C     THE MODIFICATION MADE TO THE HIRSCHFELDER ET AL. EXPRESSIONS IS
C     THAT FOR VAPOR VISCOSITY ACCURATE (EMPIRICAL) VALUES ARE USED,
C     RATHER THAN THE FIRST ORDER EXPRESSION OF KINETIC THEORY.
C
C     THE FORMULATION MATCHES EXPERIMENTAL DATA ON VISCOSITIES OF
C     VAPOR-AIR MIXTURES IN THE TEMPERATURE RANGE FROM 100 TO 150
C     DEG. C, FOR ALL COMPOSITIONS, TO BETTER THAN 4%.
C
C***** ASSUMING THAT SLIGHT CHANGES IN AIR MOLECULAR MASS WILL NOT
C      CHANGE THE COMPUTED STEAM PROPERTIES SIGNIFICANTLY  (R.L.) ***
C
      REAL M 
      DIMENSION TRD(3),OME(3),M(3),VIS(3),Z(3)
      SAVE ICALL,M
      DATA M(1),M(2)/28.96,18.016/
C
      DATA ICALL/0/
      ICALL=ICALL+1
c      IF(ICALL.EQ.1) WRITE(11,899)
c      PRINT *,'SUBROUTINE VISCO'
c  899 FORMAT(6X,'VISCO    1.0       1 FEBRUARY  1990',6X,
c     X'CALCULATE VISCOSITY OF VAPOR-AIR MIXTURES')
C
      FAIR=97.0
      FWAT=363.0
      CAIR=3.617
      CWAT=2.655
      FMIX=SQRT(FAIR*FWAT)
      CMIX=(CAIR+CWAT)/2.0
C     PRINT 7
    7 FORMAT(6X,'TEMP(C)',10X,'X1(PERCENT)',10X,'X2(PERCENT)',10X,
     1  'VISMIX(GM/CM.SEC)'//)
C   3 READ 1,T,X1
C
      X1=(XAIR/M(1))/((XAIR/M(1))+(1.-XAIR)/M(2))
C     X1 IS AIR MOLE FRACTION
    1 FORMAT(3E10.4)
      IF (T.EQ.0.) STOP
      TK=T+273.15
      X2=1.-X1
      TRD(1)=TK/FAIR
      TRD(2)=TK/FWAT
      TRD(3)=TK/FMIX
      OME(1)=(1.188-0.051*TRD(1))/TRD(1)
      OME(3)=(1.480-0.412*LOG(TRD(3)))/TRD(3)
      ARD=1.095/TRD(3)
      M(3)=2.0*M(1)*M(2)/(M(1)+M(2))
      VIS(1)=COVIS(TRD(1),CAIR,OME(1),M(1),FAIR)
C
      CALL VISS(T,P,D,VS)
      VIS(2)=10.*VS
      VIS(3)=COVIS(TRD(3),CMIX,OME(3),M(3),FMIX)
      Z(1)=X1*X1/VIS(1)+2.0*X2*X1/VIS(3)+X2*X2/VIS(2)
      G=X1*X1*M(1)/M(2)
      H=X2*X2*M(2)/M(1)
      E=(2.0*X1*X2*M(1)*M(2)/M(3)**2)*VIS(3)/(VIS(1)*VIS(2))
      Z(2)=0.6*ARD*(G/VIS(1)+E+H/VIS(2))
      Z(3)=0.6*ARD*(G+E*(VIS(1)+VIS(2))-2.0*X1*X2+H)
      VISMIX=(1.0+Z(3))/(Z(1)+Z(2))
      VISMIX=VISMIX/10.
c      PRINT 2,T,P,X1,X2,D,VISMIX
c    2 FORMAT(1X,'VISCO    ',6X,2(G10.5),10X,2(G10.5),20X,2(G10.5))
C     GO TO 3
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 35xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C calculates viscosity VISMIX of VH2O,VN2,VCH4&VCO2 for given temperature
C TX and water vapor density D, according to WILKE formula for gas mixtures
C at low pressures. From Bird pp.24
C  TX      0         5        10        15        20         30       40
C VH2O  7.863E-6  8.070E-6  8.276E-6  8.482E-6  8.689E-5  9.101E-6  9.514E-6
C VN2   1.671E-5  1.695E-5  1.718E-5  1.741E-5  1.764E-5  1.810E-5  1.854E-5
C VCH4  1.026E-5  1.042E-5  1.057E-5  1.072E-5  1.088E-5  1.118E-5  1.149E-5
C VCO2  1.360E-5  1.391E-5  1.423E-5  1.449E-5  1.476E-5  1.523E-5  1.563E-5
C  TX     50         60        70        80        90       100 
C VH2O  9.926E-6  1.034E-5  1.075E-5  1.116E-5  1.158E-5  1.199E-5
C VCO2  1.596E-5  1.623E-5  1.642E-5  1.654E-5  1.660E-5  1.658E-5  
C VCH4  1.179E-5  1.210E-5  1.240E-5  1.271E-5  1.301E-5  1.332E-5  
C VN2   1.898E-5  1.941E-5  1.983E-5  2.024E-5  2.064E-5  2.103E-5  
C
      SUBROUTINE VISCOSITY(TX,D,X2,X3,X4,VISMIX)
      implicit real*8 (a-h,o-z)
      DIMENSION AM(4),XMLL(4),VISC(4)
      SAVE ICALL
C
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'VISCOSITY1.0      26 MARCH     1995',6X,'VISCOSITY',
     X' OF GAS MIXTURE H2O, CO2, CH4 & N2, FOR GIVEN TEMPERATURE')
      CODE=0.
C 1: H2O; 2: N2; 3:CH4; 4: CO2
      AM(1)=18.016
      AM(2)=28.96
      AM(3)=16.05
      AM(4)=44.01
        DENOM=X2*(1.-X3-X4)/AM(2)+(1-X2)/AM(1)+X2*X3/AM(3)+X2*X4/AM(4)
        XMLL(1)=((1.-X2)/AM(1))/DENOM
        XMLL(2)=(X2*(1.-X3-X4)/AM(2))/DENOM
        XMLL(3)=(X2*X3/AM(3))/DENOM
        XMLL(4)=(X2*X4/AM(4))/DENOM
C
       T=TX+273.15
c WATER-VAPOR (STEAM) VISCOSITY [(N s)/m2]
       VISC(1)=1.E-7*(0.407*TX+80.4-D*(1858.-5.9*TX)*1.E-3)
c Reid 1966 cited by Findicacis 1979
c      VISC(2)=(0.526+0.071*T-0.000043*T*T)*1.0E-06
c      VISC(3)=(1.935+0.0305*T)*1.0E-6
c      VISC(4)=(-30.212+0.256*T-0.00035*T*T)*1.0E-06
c Reid 1977 cited by McBean 1995
c      VISC(2)=(14.956*T**1.5)/(121.99+T)*1.0E-07
c      VISC(3)=(9.1935*T**1.5)/(131.18+T)*1.0E-07
c      VISC(4)=(14.841*T**1.5)/(208.61+T)*1.0E-07
c   DYNAMIC  VISCOSITIES ARE CALCULATED ACCORDING TO  IRVINE &
C   LILEY 1983: "STEAM AND GAS TABLES WITH COMPUTER EQUATIONS". 
C   UNITS ARE [(N s)/m2]*10**6
C AIR VISCOSITY  [250.le.T.lt.600.]
       Vair=-9.8601e-01+9.080125e-02*t-1.17635575e-04*t*t+
     A 1.2349703e-07*t**3-5.7971299e-011*t**4
       if(t.ge.600. .and. t.le.1050.) Vair=4.8856745+
     A 5.43232e-02*t-2.4261775e-05*t*t+7.9306e-09*t**3-
     A 1.10398e-12*t**4
       VISC(2)=Vair*1.0E-06
C METHANE CH4 VISCOSITY  [200.le.T.le.1000.]
       Vch4=2.968267e-01+3.711201e-02*t+1.218298e-05*t*t-
     B 7.02426e-08*t**3+7.543269e-11*t**4-2.7237166e-14*t**5
       VISC(3)=Vch4*1.0E-06
C CARBON DIOXIDE CO2 VISCOSITY  [200.le.T.le.1000.]
       Vco2=-8.095191e-01+6.0395329e-02*t-2.824853e-05*t*t+
     C 9.843776e-09*t**3-1.47315277e-12*t**4
       VISC(4)=Vco2*1.0E-06
       if(code.eq.0.) GO TO 54
C CARBON MONOXIDE CO VISCOSITY  [250.le.T.le.1050.]
       Vco=-5.24575e-01+7.9606e-02*t-7.82295e-05*t*t+
     D 6.2821488e-08*t**3-2.83747e-11*t**4+5.317831e-15*t**5
C ETHANE C2H6 VISCOSITY  [200.le.T.le.1000.
       Vc2h6=-5.107728e-01+3.76582e-02*t-1.59412113e-05*t*t+
     E 3.906e-09*t**3
C HELIUM He VISCOSITY  [250.le.T.lt.500.]
       Vhe=3.9414e-01+1.7213335e-01*t-1.38733e-03*t*t+
     F 8.020045e-06*t**3-2.4278655e-08*t**4+3.641644e-11*t**5-
     F 2.14117e-14*t**6
       if(t.gt.500..and.t.le.1050.)Vhe=7.442412+4.6649873e-02*t-
     F 1.0385665e-05*t*t+1.35269e-09*t**3
C HIDROGEN H2 VISCOSITY  [250.le.T.lt.500.]
       Vh=-1.35666e-01+6.84115878e-02*t-3.928747e-04*t*t+
     G 1.8996e-06*t**3-5.23104e-09*t**4+7.4490972e-12*t**5-
     G 4.250937e-15*t**6
       if(t.ge.500..and.t.le.1050.)Vhe=2.72941+2.3224377e-02*t-
     G 7.6287854e-06*t*t+2.92585e-09*t**3-5.2889938e-13*t**4
C NITROGEN N2 VISCOSITY  [250.le.T.le.1050.]
       Vn2=2.5465e-02+7.5336535e-02*t-6.51566245e-05*t*t+
     H 4.34945e-08*t**3-1.5622457e-11*t**4+2.249666e-15*t**5
C OXYGEN O2 VISCOSITY  [250.le.T.le.1050.]
       Vo2=-3.97863+8.7605894e-02*t-7.064124e-05*t*t+
     I 4.6287e-08*t**3-1.690435e-11*t**4+2.534147e-15*t**5
c PROPANE C3H8 VISCOSITY  [270.le.T.le.600.]
       Vc3h8=-3.543711e-01+3.080096e-02*t-6.99723e-06*t*t
C SULFUR DIOXIDE SO2 VISCOSITY  300.le.T.le.1100.
       Vso2=-1.141748+5.1281456e-02*t-1.73644245e-06*t*t+
     J 5.09847985e-09*t**3-7.53585825e-12*t**4+
     J 5.48078289e-15*t**5-1.56355469e-18*t**6
C ARGON Ar VISCOSITY  [200.le.T.lt.540.]
       Var=1.22573+5.9456964e-02*t+1.897011e-04*t*t-
     K 8.171242e-07*t**3+1.2939183e-09*t**4-7.5027442e-13*t**5
       if(t.ge.540..and.t.le.1000.)Var=4.03764+7.3665688e-02*t-
     K 3.3867e-05*t*t+1.127158e-08*t**3-1.585569e-12*t**4
C n-BUTANE C4H10 VISCOSITY  [270.le.T.le.520.]
       Vbut=-1.099487e-02+2.634504e-02*t-3.54700854e-06*t*t
C Wilke formula cited by Bird 1960, pp.24
54    CONTINUE
      VISMIX=0.
      DO 1 I=1,4
      XIMI=XMLL(I)*VISC(I)
      XJFIJ=0.
      DO 2 J=1,4
      FIJ=((1.+((VISC(I)/VISC(J))**0.5)*((AM(J)/AM(I))**0.25))**2.)*
     1    0.3535534/SQRT(1.+AM(I)/AM(J))
      XJFIJ=XJFIJ+XMLL(J)*FIJ
2     CONTINUE
      VISMIX=VISMIX+XIMI/XJFIJ
1     CONTINUE
      X1=1.-X2
c      PRINT 22,T,X2,X1,x3,x4,D,VISMIX
c   22 FORMAT(1X,'VISCOSITY',6X,G10.5,20X,6(G10.5))
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW FUNCTION xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx no changes
      FUNCTION COVIS(TRD,C,OME,M,F)
      implicit real*8 (a-h,o-z)
C
C     THIS FUNCTION COMPUTES THE COEFFICIENT OF
C     VISCOSITY TO THE FIRST APPROXIMATION

      REAL M
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'COVIS    1.0       1 FEBRUARY  1990',6X,
     X'COEFFICIENT FOR GAS PHASE VISCOSITY CALCULATION')
C
      FOX=SQRT(M*TRD*F)/(C*C*OME*TRD)
      COVIS=(266.93 E -7)*FOX                                   
c
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 4 xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx 
      SUBROUTINE OUT
      implicit real*8 (a-h,o-z)
C-----THIS SUBROUTINE GENERATES PRINTOUT.
C
C
C$$$$$$$$$ COMMON BLOCKS FOR ELEMENTS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL (= NUMBER OF ELEMENTS)
C
      COMMON/E1/ELEM(20000)
      COMMON/E2/MATX(20000)
      COMMON/E3/EVOL(20000)
      COMMON/E4/PHI(20000)
      COMMON/E5/P(20000)
      COMMON/E6/T(20000)
      common/e7/gch4(20000)
      common/e8/gco2(20000)
C     *** END MODIFICATIONS ***
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR PRIMARY VARIABLES $$$$$$$$$$$$$$$$$$$$$$$$$
C
C     *** BEGINNING MODIFICATIONS ***
C     THESE BLOCKS HAVE A LENGTH OF 4*NEL
C     *** END MODIFICATIONS ***
C
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
      COMMON/P3/DELX(100000)
      COMMON/P4/R(100000)
      COMMON/P5/DOLD(100000)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$C
C
      COMMON/C1/NEX1(44000)
      COMMON/C2/NEX2(44000)
      COMMON/C3/DEL1(44000)
      COMMON/C4/DEL2(44000)
      COMMON/C5/AREA(44000)
      COMMON/C6/BETA(44000)
      COMMON/C7/ISOX(44000)
      COMMON/C8/GLO(44000)
      COMMON/C9/ELEM1(44000)
      COMMON/C10/ELEM2(44000)
C  
      common/C11/fvd(44000)
      common/c12/fvt(44000)
      common/c13/fn2d(44000)
      common/c14/fn2t(44000)
      common/c15/fch4d(44000)
      common/c16/fch4t(44000)
      common/c17/fco2d(44000)
      common/c18/fco2t(44000)
      common/c19/fhd(44000)
      common/c20/fht(44000)
      common/c21/fhc(44000)
      common/c22/fha(44000)
      common/c23/fn2l(44000)
      common/c24/fch4l(44000)
      common/c25/fco2l(44000)
      common/c26/fh2o(44000)
C
      common/t1/theat(44000)
      common/t2/tliquid(44000)
      common/t3/tvapor(44000)
      common/t4/tn2(44000)
      common/t5/tch4(44000)
      common/t6/tco2(44000)
      common/t7/tn2l(44000)
      common/t8/tch4l(44000)
      common/t9/tco2l(44000)
      common/t10/th2o(44000)
      common/t11/tgas(44000)
C
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G4/ELEG(10000)
      COMMON/G5/SOURCE(10000)
      COMMON/G6/LTABG(10000)
      COMMON/G7/G(10000)
      COMMON/G8/EG(10000)
      COMMON/G9/NEXG(10000)
      COMMON/G10/ITABG(10000)
      COMMON/G11/NGIND(10000)
      COMMON/G12/LCOM(10000)
      COMMON/G13/PI(10000)
      COMMON/G14/PWB(10000)
      COMMON/G15/HG(10000)
      COMMON/G16/GPO(10000)
      COMMON/G17/SDENS(10000)
      COMMON/G18/SSAT(10000)
      COMMON/G19/GVOL(10000)
      COMMON/G20/HL(10000)
      COMMON/G21/HS(10000)
      COMMON/G22/QVGC(10000)
      COMMON/G23/QVWC(10000)
      COMMON/G24/QVOC(10000)
      COMMON/G25/GRAD(10000)
      COMMON/G26/FF(10000)
      common/g27/fmvap(10000)
      common/g28/fmn2(10000)
      common/g29/fmch4(10000)
      common/g30/fmco2(10000)
      common/g31/gtotg(10000)
      common/g32/fmn2l(10000)
      common/g33/fmch4l(10000)
      common/g34/fmco2l(10000)
      common/g35/fmwat(10000)
      common/g36/fmheat(10000)
      common/g37/gtotl(10000)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
      COMMON/COMPO/FLO(88000)
      COMMON/PORVEL/VEL(88000)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
C      COMMON/KONIT/KON,DELT,IGOOD                                                   JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      COMMON/TITLE/ TITLE
c      COMMON/DOP/ENTH,KDATA,QUAL                                                    JPF 03 2005
      COMMON/DOP/QUAL,ENTH,KDATA
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/POV6/ TSTART
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
      COMMON/SECPAR/PAR(2640000)
C      COMMON/SOLID/NM,DROK(27),POR(27),PER(3,27),CWET(27),SPHT(27)                   JPF 03 2005
      COMMON/SOLID/DROK(27),POR(27),PER(3,27),CWET(27),SPHT(27),NM
      COMMON/SOCH/MAT(27)
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                               JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      common/ff/h1
c
C      common/bal2/heatin,gasin,n2gin,ch4gin,co2gin,h2ogin,                             JPF 03 2005
C     x            liqin,n2lin,ch4lin,co2lin,h2olin
      common/bal2/heatin,gasin,ch4gin,co2gin,h2ogin,ch4lin,
     x            co2lin,h2olin,n2gin,liqin,n2lin
C      common/bal3/heatout,gasout,n2gout,ch4gout,co2gout,h2ogout,                        JPF 03 2005
C     x            liqout,n2lout,ch4lout,co2lout,h2olout
      common/bal3/heatout,gasout,ch4gout,co2gout,h2ogout,ch4lout,
     x            co2lout,h2olout,n2gout,liqout,n2lout
      common/bal4/welgas,weln2,welch4,welco2,welvap,welheat
      common/bal5/welliq,weln2l,welch4l,welco2l,welwat
c
      character*1 h1
      real n2gin,n2gout,n2lin,n2lout,liqin,liqout
C      DIMENSION DXM(3)                                                                   JPF 03 2005
      DIMENSION DXM(NK1)
      CHARACTER*80 TITLE
      CHARACTER*5 ELEM,ELEM1,ELEM2,ELEG,SOURCE,MAT
      CHARACTER*1 ITABG,HB,H0
C
      SAVE ICALL,HB,H0
      DATA HB,H0/' ',' '/
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C     *** BEGINNING MODIFICATIONS ***
  899 FORMAT(/6X,'OUT      1.2      29 SEPTEMBER 1993',6X,
     X'PRINT RESULTS FOR ELEMENTS, CONNECTIONS, AND SINKS/SOURCES')
      AMH2O=18.016
      AMN2=28.96 
      AMCH4=16.043
      AMCO2=44.010
C     *** END MODIFICATIONS ***
C
C-----COMPUTE MAXIMUM CHANGES.
C
      DO 1 I=1,NK1
        DXM(I)=0.
    1   CONTINUE
C
      DO 2 N=1,NEL
        NLOC=(N-1)*NK1
C ja mesto 5 staviv NEQ                                *** BEG MODIFICATIONS
        DO 3 I=1,NEQ
C                                                      *** END MODIFICATIONS
          ADX=ABS(DX(NLOC+I))
          IF (ADX.GT.DXM(I)) DXM(I)=ADX
    3     CONTINUE
    2   CONTINUE
C
C-----PRINT HEADER INFORMATION.
C
      DAY=SUMTIM/8.64E4
      YEARS=DAY/365.25
      PRINT 5010,H1,KCYC,ITER,KON,DAY,YEARS
      PRINT 5140,HB
C ja mesto 5 staviv NEQ                               *** BEG MODIFICATIONS
      PRINT 9010,SUMTIM,KCYC,ITER,ITERC,KON,(DXM(I),I=1,NEQ),
     1  RERM,NER,KER,DELTEX
C                                                      *** END MODIFICATIONS
C
C-----PRINT PRIMARY VARIABLES AND FLUID COMPOSITION:
C
      PRINT 5140,HB
      PRINT 1000,H0
      PRINT 1001
 1000 FORMAT(A1,'ELEM.  No       P',8X,'T',7X,'SG',5X,
     1  'X-AIRG',4X,'X-AIRL',3x,'M-N2G ',
     3   2X,'M-CH4G',3X,'M-CO2G',3x,'M-H2OG',5X,'N2',6X,'CH4',6X,
     1'CO2',6X,'Dgas',5X,
     2  'PCAP',5X,'Krelg',5X,'Krell')
 1001 FORMAT(12X,'   (PA)    (DEG-C)',5x,'%',3x,'(N2&CH4&CO2 MASS',
     1 ' FR)',2X,'(MOLE FRACT IN',
     1' GASEOUS PHASE)',5X,'(MASS FRACT IN X-AIRL)',5x,
     1'(kg/m3)',3x,'(Pa)'/)
C
      DO 3021 N=1,NEL
        NLOC=(N-1)*NK1
        NLOC2=(N-1)*NSEC*NEQ1
        NLOC2L=NLOC2+NBK
C                                                      *** BEG MODIFICATIONS
        TX=PAR(NLOC2+NSEC-1)
        SG=PAR(NLOC2+1)
        SL=PAR(NLOC2L+1)
        
      XAIRG=PAR(NLOC2+NB+1)+PAR(NLOC2+NB+2)+
     xPAR(NLOC2+NB+3)+PAR(NLOC2+NB+4)
      XAIRL=PAR(NLOC2L+NB+2)+PAR(NLOC2L+NB+3)+PAR(NLOC2L+NB+4)
      XH2OG=PAR(NLOC2+NB+1)
      XH2OL=PAR(NLOC2L+NB+1)
        IF(XAIRG.EQ.0.) THEN
          XN2AIRG=0.
          XCH4AIRG=0.
          XCO2AIRG=0.
          XH2OAIRG=0.
        ELSE
          XH2OAIRG=PAR(NLOC2+NB+1)/XAIRG
          XN2AIRG=PAR(NLOC2+NB+2)/XAIRG
          XCH4AIRG=PAR(NLOC2+NB+3)/XAIRG
          XCO2AIRG=PAR(NLOC2+NB+4)/XAIRG
        ENDIF
ccc
        IF(XAIRL.EQ.0.) THEN
          XN2AIRL=0.
          XCH4AIRL=0.
          XCO2AIRL=0.
        ELSE
          XN2AIRL=PAR(NLOC2L+NB+2)/XAIRL
          XCH4AIRL=PAR(NLOC2L+NB+3)/XAIRL
          XCO2AIRL=PAR(NLOC2L+NB+4)/XAIRL
C
        ENDIF
c AIR (N2, Ch4 et CO2) mass fraction in gas phase
        XAIRG=XAIRG-PAR(NLOC2+NB+1)
C obtaining mole fractions in air in gas phase (no water wapor)
        DOWN=XH2OAIRG/AMH2O+XN2AIRG/AMN2+XCH4AIRG/AMCH4+XCO2AIRG/AMCO2
        if(sg.eq.0.) then
        xh2omg=0.
        xn2mg=0.
        xch4mg=0.
        xco2mg=0.
        else
        XH2OMG=(XH2OAIRG/AMH2O)/DOWN
        XN2MG=(XN2AIRG/AMN2)/DOWN
        XCH4MG=(XCH4AIRG/AMCH4)/DOWN
        XCO2MG=(XCO2AIRG/AMCO2)/DOWN
        endif
        PC=PAR(NLOC2L+6)
        CALL SAT(TX,PS)
        PRES=X(NLOC+1)
        IF (KON.NE.2) PRES=X(NLOC+1)+DX(NLOC+1)
c       PRINT 50401,ELEM(N),N,PRES,TX,SG,XAIRG,XAIRL,
c    1  XN2MG,XCH4MG,XCO2MG,XH2OMG,PAR(NLOC2+4),
c    2  PAR(NLOC2+3),PC,PAR(NLOC2+2),PAR(NLOC2L+2)
        PRINT 50401,ELEM(N),N,PRES,TX,SG,XAIRG,XAIRL,
     1  XN2MG,XCH4MG,XCO2MG,XH2OMG,xn2airl,xch4airl,
     2  xco2airl,PAR(NLOC2+4),PC,PAR(NLOC2+2),PAR(NLOC2L+2)
 3021   CONTINUE
c50401 FORMAT(1X,A5,I6,F15.6,1x,1f7.4,12(1X,E9.3))
50401 FORMAT(1X,A5,I4,F12.2,1x,1f7.4,3(E9.3),4(E9.3),7(E9.3))
C     *** END MODIFICATIONS ***
      PRINT 5140,HB
C-------------------------------------------------------------------------
      IF (MOD(KDATA,10).LT.2) GO TO 3045
      if(mop(23).lt.9) go to 30201
      go to 33030
C
C-----NOW PRINT FLOW TERMS.
C
C      PRINT 5000,h1,TITLE,KCYC,ITER,SUMTIM
      PRINT 5010,H1,KCYC,ITER,KON,DAY,YEARS
      PRINT 5060,H0,HB
      PRINT 5061
 5060 FORMAT(A1,/,X,' ELEM1  ELEM2  INDEX    FLOH',A6,'FLOH/FLOF',4X,
     1  'FLOF',6X,'FLO(GASconv) FLO(LIQ.)   VEL(GAS)    VEL(LIQ.)')
 5061 FORMAT(26X,'(W)',8X,'(J/kg)',4X,'(kg/s)',7X,'(kg/s)',6X,
     1  '(kg/s)      (m/s)',8X,'(m/s)'/)
C
      DO 3030 N=1,NCON
        IF (NEX1(N).EQ.0.OR.NEX2(N).EQ.0) GO TO 3030
        NNP=(N-1)*NPH
        FLOF=0.
C
        DO 3031 NP=1,NPH
          FLOF=FLOF+FLO(NNP+NP)
 3031     CONTINUE
        H=0.
        IF (FLOF.NE.0.) H=FHT(N)/FLOF
C
C     *** BEGINNING MODIFICATIONS ***
C     SECTION REMOVED
C     *** END MODIFICATIONS ***
C
        PRINT 5070,ELEM1(N),ELEM2(N),N,FHT(N),H,FLOF,
     1    (FLO(NNP+NP),NP=1,NPH),(VEL(NNP+NP),NP=1,NPH)
C
C
 3030   CONTINUE
33030   continue
C
      PRINT 5140,HB
C
C     *** BEGINNING MODIFICATIONS ***
C
C-----NOW PRINT NEW DETAILLED FLOW TERMS.
C
C      PRINT 5000,h1,TITLE,KCYC,ITER,SUMTIM
      PRINT 5010,H1,KCYC,ITER,KON,DAY,YEARS
      PRINT 5080,H0
      PRINT 5081
c 5080 FORMAT(A1,/,X,' ELEM1  ELEM2  INDEX',4X,'TOTAL',3X,
c     1'CONDUCTIVE',3X,'DIFFUSIVE',4(5X,'TOTAL',5X,'DIFFUSIVE'),/,
c     220X,3('  HEAT FLUX '),2(' VAPOR-FLUX '),2x,2(' N2-FLUX    '),
c     32(' CH4-FLUX   '),2(' CO2-FLUX   '))
 5080 FORMAT(A1,/,X,' ELEM1  ELEM2  INDEX',4X,'TOTAL',6X,
     x' TOTAL',6X,'TOTAL',5x,'[            TOTAL GAS:  CONVECTIVE + ',
     x'DIFFUSIVE',5x,'                 ]  [LIQUID CONV. ONLY',2x,']',
     x/,20X,'  HEAT FLUX    GAS FLUX  LIQUID FLUX ',
     x'  N2T-FLUX    N2D-FLUX    CH4T-FLUX  ',
     x' CH4D-FLUX    CO2T-FLUX   CO2D-FLUX   CH4L-FLUX',
     X'  CO2L-FLUX')
 5081 FORMAT(20X,5X,'(W)',4X,10(4X,'(kg/s)',2X),/)
C 
      DO 3060 N=1,NCON
        IF (NEX1(N).EQ.0.OR.NEX2(N).EQ.0) GO TO 3060
C
c        PRINT 5070,ELEM1(N),ELEM2(N),N,FHT(N),FHC(N),FHD(N),FVT(N),
c     1    FVD(N),FN2T(N),FN2D(N),FCH4T(N),FCH4D(N),FCO2T(N),FCO2D(N)
        fgas= fvt(n)+fn2t(n)+fch4t(n)+fco2t(n)
        fliq=fh2o(n)+fn2l(n)+fch4l(n)+fco2l(n)
        PRINT 5070,ELEM1(N),ELEM2(N),N,FHT(N),fgas,fliq,FN2T(N),
     1FN2D(N),FCH4T(N),FCH4D(N),FCO2T(N),FCO2D(N),fch4l(N),fco2l(n)
C
C
 3060   CONTINUE
30201   continue
c      if(mop(24).lt.4) go to 30202
c---------------------------------------------------------------------
c
C-----NOW PRINT TOTAL (UP TO DATE) FLOW TERMS 
c
      PRINT 111
  111 FORMAT(' @@@@@@@@@@@@@ FLUX FOR MASS-BALANCES @@',116('@')/)
      print 5010,h1,kcyc,iter,kon,day,years
      print 5066,h0
      print 5067
 5066 FORMAT(A1,/,X,' ELEM1  ELEM2  INDEX',30x,
     1'@@@@@@@@@@@@@ TOTAL UP TO DATE MASS QUANTITY TO/FROM',
     1' THE BOUNDARIES @@@@@@@@@@@@',/,'               ',
     23X,' TOT. HEAT FLUX  GAS-FLUX   ',
     3' N2G-FLUX    CH4G-FLUX   CO2G-FLUX  VAPOR-FLUX',
     3' LIQUID-FLUX    N2L-FLUX   CH4L-FLUX   CO2L-FLUX   H2O-FLUX   ')
 5067 FORMAT(25X,'(J)',10(10x,'kg'),/)
C
c  initialisation of the sums of the up to date total mass in/out
c
        heatin=0.
        heatout=0.
        gasin=0.
        gasout=0.
        n2gin=0.
        n2gout=0.
        ch4gin=0.
        ch4gout=0.
        co2gin=0.
        co2gout=0.
        h2ogin=0.
        h2ogout=0.
        liqin=0.
        liqout=0.
        n2lin=0.
        n2lout=0.
        ch4lin=0.
        ch4lout=0.
        co2lin=0.
        co2lout=0.
        h2olin=0.
        h2olout=0.
c
c summing the total up to date mass in/out
c
      do 3065 n=1,ncon
        n1=nex1(n)
        n2=nex2(n)
        nloc21=(n1-1)*nsec*neq1+1
        nloc22=(n2-1)*nsec*neq1+1
        if (NEX1(N).EQ.0.OR.NEX2(N).EQ.0) GO TO 3065
c flux with bc elements only
        if(evol(n1).ne.0 .and. evol(n2).ne.0) go to 3065
        print 5070,elem1(n),elem2(n),n,theat(n),tgas(n),
     X  tn2(n),tch4(n),tco2(n),tvapor(n),tliquid(n),tn2l(n),
     x  tch4l(n),tco2l(n),th2o(n)
C
c in the connections BC element should allways be the second element
c
c        if(evol(n1).eq.0.) theat(n)=-theat(n)
c        if(evol(n1).eq.0.) tgas(n)=-tgas(n)
c        if(evol(n1).eq.0.) tn2(n)=-tn2(n)
c        if(evol(n1).eq.0.) tch4(n)=-tch4(n)
c        if(evol(n1).eq.0.) tco2(n)=-tco2(n)
c        if(evol(n1).eq.0.) tvapor(n)=-tvapor(n)
c        if(evol(n1).eq.0.) tliquid(n)=-tliquid(n)
c        if(evol(n1).eq.0.) tn2l(n)=-tn2l(n)
c        if(evol(n1).eq.0.) tch4l(n)=-tch4l(n)
c        if(evol(n1).eq.0.) tco2l(n)=-tco2l(n)
c        if(evol(n1).eq.0.) th2o(n)=-th2o(n)
        thn   = theat(n)
        tgasn = tgas(n)
        tn2n  = tn2(n)
        tch4n = tch4(n)
        tco2n = tco2(n)
        tvapn = tvapor(n)
        tln   = tliquid(n)
        tn2ln = tn2l(n)
        tch4ln= tch4l(n)
        tco2ln= tco2l(n)
        th2on = th2o(n)
c
        if(evol(n1).eq.0.) then
        thn   =-theat(n)
        tgasn =-tgas(n)
        tn2n  =-tn2(n)
        tch4n =-tch4(n)
        tco2n =-tco2(n)
        tvapn =-tvapor(n)
        tln   =-tliquid(n)
        tn2ln =-tn2l(n)
        tch4ln=-tch4l(n)
        tco2ln=-tco2l(n)
        th2on =-th2o(n)
        end if
c
        if(thn.gt.0.) heatin=heatin+thn
        if(thn.lt.0.) heatout=heatout+thn
        if(tgasn.gt.0.) gasin=gasin+tgasn
        if(tgasn.lt.0.) gasout=gasout+tgasn
        if(tn2n.gt.0.) n2gin=n2gin+tn2n
        if(tn2n.lt.0.) n2gout=n2gout+tn2n
        if(tch4n.gt.0.) ch4gin=ch4gin+tch4n
        if(tch4n.lt.0.) ch4gout=ch4gout+tch4n
        if(tco2n.gt.0.) co2gin=co2gin+tco2n
        if(tco2n.lt.0.) co2gout=co2gout+tco2n
        if(tvapn.gt.0.) h2ogin=h2ogin+tvapn
        if(tvapn.lt.0.) h2ogout=h2ogout+tvapn
c
        if(tln.gt.0.) liqin=liqin+tln
        if(tln.lt.0.) liqout=liqout+tln
        if(tn2ln.gt.0.) n2lin=n2lin+tn2ln
        if(tn2ln.lt.0.) n2lout=n2lout+tn2ln
        if(tch4ln.gt.0.) ch4lin=ch4lin+tch4ln
        if(tch4ln.lt.0.) ch4lout=ch4lout+tch4ln
        if(tco2ln.gt.0.) co2lin=co2lin+tco2ln
        if(tco2ln.lt.0.) co2lout=co2lout+tco2ln
        if(th2on.gt.0.) h2olin=h2olin+th2on
        if(th2on.lt.0.) h2olout=h2olout+th2on
C
C
3065    continue
        print 5071,heatin,gasin,n2gin,ch4gin,co2gin,h2ogin,
     x  liqin,n2lin,ch4lin,co2lin,h2olin
 5071 FORMAT(/,'  @ TOTAL MASS IN @',1x,(11(1X,E11.5)))
        print 5072,heatout,gasout,n2gout,ch4gout,co2gout,h2ogout,
     x  liqout,n2lout,ch4lout,co2lout,h2olout
 5072 FORMAT('  @ TOTAL MASS OUT@',1x,(11(1X,E11.5)))
30202   continue
C
C-----PRINTING AMD COMPUTATION RESULTS FOR ACTIVE ELEMENTS
C
C
c      IF (MOP(24).EQ.0) GOTO 30201
C
CP      PRINT 5000,h1,TITLE,KCYC,ITER,SUMTIM
CP      PRINT 4030,H0
CP      PRINT 4031
 4030 FORMAT(A1,/,10X,' ELEM. INDEX  TOTAL GCH4   GCH4/VOL  ',  
     1'TOTAL GCO2   GCO2/VOL    TOTAL GHT    GHT/VOL')
 4031 FORMAT(26X,'(kg/s)',3X,'(kg/s m3)',3x,'(kg/s)',4X,
     1 '(kg/s m3)',5X,'(W)',6X,'(W/m3)',/)
C
c     CFHT=-12580357.
c        CFHT=2528000.
c        DO 4000 N = 1,NEL
c        GHT=CFHT*GCH4(N)
c        IF(NEQ.LT.NK1) GHT=0.
c        IF(EVOL(N).LE.0.) THEN
c        VGHT=0.
c        VGCH4=0.
c        VGCO2=0.
c        ELSE
c        VGHT=GHT/EVOL(N)
c        VGCH4=GCH4(N)/EVOL(N)
c        VGCO2=GCO2(N)/EVOL(N)
c        ENDIF
CP        PRINT 4001,ELEM(N),N,GCH4(N),VGCH4,GCO2(N),VGCO2,GHT,VGHT
C
c 4000   CONTINUE
C
C     *** END MODIFICATIONS ***
C
c
      if(mop(23).lt.9) go to 3045
C
      IF (MOD(KDATA,10).LT.3) GO TO 3045
C
C-----PRINTING PRIMARY VARIABLES AND THEIR CHANGES
C
CP      PRINT 5000,h1,TITLE,KCYC,ITER,SUMTIM
CP      PRINT 5030,H0
CP      PRINT 5031
C
      DO 3020 N = 1,NEL
        NLOC=(N-1)*NK1
        NLOC2=(N-1)*NSEC*NEQ1
        NLOC2L=NLOC2+NBK
C     *** BEGINNING MODIFICATIONS ***
CP        PRINT 5041,ELEM(N),N,(X(NLOC+I),I=1,NK1),(DX(NLOC+I),I=1,NK1)
C     *** END MODIFICATIONS ***
C
 3020   CONTINUE
c      PRINT 5140,HB
C
C     *** BEGINNING MODIFICATIONS ***
C
      IF (MOD(KDATA,10).LT.4) GO TO 3045
C
C-----PRINTING SECONDARY PARAMETERS - FLUID PROPERTIES & KR
C
CP      PRINT 5000,h1,TITLE,KCYC,ITER,SUMTIM
CP      PRINT 6030,H0
CP      PRINT 6031
 6030 FORMAT(A1,/,10X,' ELEM.  INDEX    DG',10X,'DL',8X,'VISG',8X,
     1'VISL',10X,  'HG',10X,'HL',9X,'KRG',9X,'KRL')
 6031 FORMAT(24X,'(kg/m3)   (kg/m3)    (PA S)',6X,'(PA S)',
     1 6X,'(J/kg)',6X,'(J/kg)'/)
C
      DO 3040 N = 1,NEL
        NLOC=(N-1)*NK1
        NLOC2=(N-1)*NSEC*NEQ1
        NLOC2L=NLOC2+NBK
        DG=PAR(NLOC2+4)
        DL=PAR(NLOC2L+4)
        VISG=PAR(NLOC2+3)
        VISL=PAR(NLOC2L+3)
        HTG=PAR(NLOC2+5)
        HTL=PAR(NLOC2L+5)
        RPG=PAR(NLOC2+2)
        RPL=PAR(NLOC2L+2)
CP        PRINT 5041,ELEM(N),N,DG,DL,VISG,VISL,HTG,HTL,RPG,RPL
C
 3040   CONTINUE
c      PRINT 5140,HB
C
C     *** END MODIFICATIONS ***
C
 3045 CONTINUE
c
C-----NOW PRINT NEW SOURCE/SINK FLOW TERMS.
c
      IF (NOGN.EQ.0) RETURN
C      PRINT 5000,h1,TITLE,KCYC,ITER,SUMTIM
      PRINT 4
4     FORMAT(//' @@@@@@@@@@ MASS EXTRACTED/INJECTED BY WELLS ',
     1 111('@')/)
      PRINT 5010,H1,KCYC,ITER,KON,DAY,YEARS
      PRINT 5120,H0
 5000 FORMAT(A1/10X,A80//80X,'KCYC =',I5,'  -  ITER =',I5,
     1  '  -  TIME =',E12.5)
 5010 FORMAT(A1/' ',10X,'OUTPUT DATA AFTER (',I6,',',I4,')-',I1,
     1  '-TIME STEPS',52(' '),'THE TIME IS ',E12.5,' D  OR  ',
     2F15.1,' Y')
c
      PRINT 5121
 5120 FORMAT(A1/'ELEMENT SOURCE  INDEX  GENERATION RATE',
     1  '     ENTHALPY       FRACTIONAL  FLOWS       P (WB)',4X,
     2' @@@@@@@@@ UP TO DATE EXTRACTED/INJECTED MASS IN (kg) @@@@',
     3'@@@@@')
 5121 FORMAT(23X,' (kg/s) OR (W)       (J/kg)',6X,'FF(gas)',
     x6X,'FF(liq)',6X,' (Pa)         TOTAL        N2           CH4   ',
     x'       CO2          H2O  ',/)
 5122 FORMAT(A1/' ELEMENT SOURCE INDEX  GENERATION RATE',
c     1  '     ENTHALPY    FF(GAS)      FF(LIQ.)      P (WB)')
     1  '     ENTHALPY      FRACTIONAL  FLOWS        P (WB)',
     2'  TOTAL EXTRACTION RATES IN (kg)')
C
      GC=0.
      HGC=0.
C
      welheat=0.
      welgas=0.
      welliq=0.
      weln2=0.
      welch4=0.
      welco2=0.
      welvap=0.
      weln2l=0.
      welch4l=0.
      welco2l=0.
      welwat=0.
c
      DO 3050 N=1,NOGN
        J=NEXG(N)
        IF (J.EQ.0) GO TO 3050
c
c        if(abs(fmch4(n)).lt.1.e-17)  fmch4(n)=0.
c        if(abs(fmch4l(n)).lt.1.e-17) fmch4l(n)=0.
c        if(abs(fmco2(n)).lt.1.e-17)  fmco2(n)=0.
c        if(abs(fmco2l(n)).lt.1.e-17) fmco2l(n)=0.
        welgas=welgas+fmn2(n)+fmch4(n)+
     x                fmco2(n)+fmvap(n)
        weln2=weln2+fmn2(n)
        welch4=welch4+fmch4(n)
        welco2=welco2+fmco2(n)
        welvap=welvap+fmvap(n)
        welheat=welheat+fmheat(n)
        welliq=welliq+fmn2l(n)+fmch4l(n)+
     x                fmco2l(n)+fmwat(n)
        weln2l=weln2l+fmn2l(n)
        welch4l=welch4l+fmch4l(n)
        welco2l=welco2l+fmco2l(n)
        welwat=welwat+fmwat(n)
c
        IF (MOD(N,58).EQ.54) PRINT 5120,H1
        IF (MOD(N,58).EQ.54) PRINT 5121
        if(lcom(n).gt.2) go to 397
c
C mass injection GPO.gt.0. or pumping GPO.lt.0., of  WATER  lcom(n).eq.1
        IF (lcom(n).eq.1) 
     x     PRINT 5130,ELEG(N),SOURCE(N),
     1     N,GPO(N),EG(N),(FF((N-1)*NPH+NP),NP=1,NPH),pwb(n),
     2     gtotl(n),fmn2l(n),fmch4l(n),fmco2l(n),fmwat(n)
c
C mass injection GPO.gt.0. or pumping GPO.lt.0., of AIRatm(N2) lcom(n).eq.2
        IF (lcom(n).eq.2) 
     x     PRINT 5130,ELEG(N),SOURCE(N),
     1     N,GPO(N),EG(N),(FF((N-1)*NPH+NP),NP=1,NPH),pwb(n),
     2     gtotg(n),fmn2(n),fmch4(n),fmco2(n),fmvap(n)
        go to 399
397     continue
c
c pumping GPO.lt.0, of phase gaseuse
        if(FF((N-1)*NPH+1).eq.0.) go to 398
        IF (GPO(N).LE.0..AND.LCOM(N).NE.4) 
     x     PRINT 5131,ELEG(N),SOURCE(N),
     1     N,GPO(N),EG(N),(FF((N-1)*NPH+NP),NP=1,NPH),pwb(n),
     2     gtotg(n),fmn2(n),fmch4(n),fmco2(n),fmvap(n)
 5131 FORMAT(2X,A5,3X,A5,3X,I2,1X,'Gas',E12.5,2X,11(1X,E12.5),'G')
c
c pumping GPO.lt.0, of phase liquide
398     if(FF((N-1)*NPH+2).eq.0.) go to 399
        IF (GPO(N).LE.0..AND.LCOM(N).NE.4) 
     x     PRINT 5132,ELEG(N),SOURCE(N),
     1     N,GPO(N),EG(N),(FF((N-1)*NPH+NP),NP=1,NPH),pwb(n),
     2     gtotl(n),fmn2l(n),fmch4l(n),fmco2l(n),fmwat(n)
 5132 FORMAT(2X,A5,3X,A5,3X,I2,1X,'Liq',E12.5,2X,11(1X,E12.5),'L')
c
399     IF (GPO(N).LE.0..AND.LCOM(N).EQ.4) 
     x    PRINT 5130,ELEG(N),SOURCE(N),
     1    N,G(N),EG(N),(FF((N-1)*NPH+NP),NP=1,NPH),PWB(N)
C
        IF (LCOM(N).NE.4) GO TO 102
C
C-----NOW FOR A LITTLE SECTION WHICH COMPUTES TOTAL FLOWRATE
C     AND FLOWING ENTHALPY FOR WELLS ON DELIVERABLITY WITH COMPLE-
C     TIONS IN DIFFERENT LAYERS (OR, GENERALLY, ELEMENTS)
C
C     ALL OPEN INTERVALS OF A WELL MUST BE KNOWN BY THE SAME SOURCE
C     NAME, AND MUST BE GIVEN IN UNINTERRUPTED SEQUENCE.
C
        IF (NOGN.EQ.1) GO TO 102
        IF (N.EQ.1.AND.SOURCE(N+1).NE.SOURCE(N)) GO TO 102
        IF (N.EQ.1) GO TO 100
        IF (SOURCE(N-1).EQ.SOURCE(N).OR.SOURCE(N).EQ.SOURCE(N+1))
     1    GO TO 100
C
        GO TO 103
C
C-----COME HERE FOR SOURCE IN A CHAIN AND ACCUMULATE TERMS.
  100   GC=GC+GPO(N)
        HGC=HGC+GPO(N)*EG(N)
C
C-----FIND OUT WHETHER CHAIN TERMINATES.
        IF (N.EQ.NOGN) GO TO 101
        IF (SOURCE(N+1).EQ.SOURCE(N)) GO TO 102
C
C-----COME HERE FOR END OF CHAIN.
  101   IF (GC.NE.0.) HGC=HGC/GC
        PRINT 110,SOURCE(N),GC,HGC
  110   FORMAT(' ***** SOURCE  $',A5,'$     RATE =',E12.5,
     1    ' kg/s     FLOWING ENTHALPY =',E12.5,' J/kg     *****'/)
C
C-----COME HERE FOR SOURCES OUTSIDE CHAINS, AND AT END OF CHAIN.
  103   GC=0.
        HGC=0.
C
  102   CONTINUE
C
C-----END OF SECTION FOR MULTI-LAYER WELLS ON DELIVERABILITY
C
C
3050   CONTINUE
c     
       print 5133,welheat,welgas,weln2,welch4,welco2,welvap
5133   format(/,' @@@@@@ TOTAL (IN/OUT) ENERGY (in J):',
     x2x,e12.5,'    TOTAL (IN/OUT) GAS     MASS (in',
     x ' kg):',5(E12.5,1x))
       print 5532,welliq,weln2l,welch4l,welco2l,welwat
5532   format(  '                                     ',
     x14x,     '    TOTAL (IN/OUT) LIQUID  MASS (in',
     x ' kg):',5(E12.5,1x))
C
c      PRINT 5140,HB
C
C
      RETURN
C
C---*----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8
C     *** BEGINNING MODIFICATIONS ***
 9010 FORMAT('   TOTAL TIME  KCYC ITER ITERC KON         DX1M',
     1  9X,'DX2M',9X,'DX3M',9X,'DXM4',9x,'DXM5',
     2  10X,'MAX. RES.',4X,'NER KER        DELTEX'/
     3  1X,E12.5,I6,2I5,I4,5X,5E13.5,2X,E13.5,2X,I4,I4,5X,E13.5)
 4001 FORMAT(11X,A5,I6,6(1X,E11.5))
 5030 FORMAT(A1,/,10X,' ELEM.  INDEX    X1',10X,'X2',10X,'X3',10X,
     1'X4',10X,'X5',10X,'DX1',9X,'DX2',9X,'DX3',9X,'DX4',9X,'DX5')
 5031 FORMAT(110X,/)
c 1000 FORMAT(A1,'ELEM.  INDEX     P',12X,'T',8X,'SG',8X,'SL',6X,
c     1  'X-AIRG',4X,'X-AIRL',4x,'X-N2G '4X,'X-N2L',
c     3   3X,'X-CH4G',4X,'X-CH4L',4X,'X-CO2G',4X,
c     2  'X-CO2L',5X,'Dgas',6X,'Dliq')
c 1001 FORMAT(15X,'  (PA)       (DEG-C)',52X,'(MASS FRACTION IN AIR',
c     1' - NOT IN PHASE)',11X,'(kg/m3)',4X,'(kg/m3)'/) 
 5041 FORMAT(11X,A5,I6,10(1X,E11.5))
 5040 FORMAT(1X,A5,I6,1x,F11.2,12(1X,E11.5))
 5070 FORMAT(2A7,1X,I4,1x,(11(1X,E11.5)))
 5130 FORMAT(2X,A5,3X,A5,3X,I2,4X,E12.5,2X,11(1X,E12.5))
 5140 FORMAT(/' ',155('@')/A1)
C
      END
C     *** BEGINNING MODIFICATIONS (NEW SUBROUTINE) ***
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 8 xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx baca
       SUBROUTINE SHEAT(TX,CVair,CVch4,CVco2)
      implicit real*8 (a-h,o-z)
C-----THIS SUBROUTINE CALCULATES HEAT  CAPACITY (SPECIFIC HEAT) OF
C     GASES AT CONSTANT PRESSURE Cp in[kJ/kg oK] and HEAT CAPACITY
C     OF GASES AT CONSTANT VOLUME Cv=Cp-R in [J/kg oK].REF: 'STEAM 
C     AND GAS TABLES WITH COMPUTER EQUATIONS' IRVINE & LILEY, 1983
C                   DOMAINS OF VALIDITY:
C     AIR: 250~2000oK; CH4: 280~755oK; CO2: 200~1000oK; CO: 250~1050oK
C     ETHANE: 280~755oK; HELIUM: 250~1050oK; HYDROGEN: 250~1050oK
c     NITROGEN: 280~1080; OXYGEN: 250~1050 oK; PROPANE: 280~755 oK
C     SULFURDIOXIDE: 300~1100 oK; Arg: 200~1600oK; n-BUTANE: 280~755oK; 
C     R is in [kJ/kg oK] and differs for each gas (equal in moles)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(/6X,'SHEAT    1.0      13   JUNE    1995',6X,
     X'CALCULATES SPECIFIC HEAT Cv VALUES FOR GASES FOR GIVEN TEMP.')
      CODE=0.
c
       T=TX+273.15
c-AIR: air MNOZAM SO 1000 DA DOBIJAM VO Jouli (inace e vo kJ)
       CPair=1.03409-.284887E-03*t+.7816818E-06*t*t-
     A .4970786E-09*t**3+.1077024E-12*t**4
       CVair=(CPair-.28704)*1000.
c-CH4: methane
       CPch4=1.9165258-1.09269E-03*t+8.696605E-06*t*t-
     B 5.2291144E-09*t**3
       CVch4=(CPch4-.518251)*1000.
c-CO2: carbon dioxide
       CPco2=4.5386462E-01+1.5334795E-03*t-4.195556E-07*t*t-
     C 1.871946E-09*t**3+2.862388E-12*t**4-1.6962E-15*t**5-
     D 3.717285E-19*t**6
       CVco2=(CPco2-.188919)*1000.
       IF(CODE.EQ.0.) GO TO 54
c-CO: carbon monoxide
       CPco=1.020802+3.82075E-04*t-2.4945E-06*t*t+
     E 6.81145E-09*t**3-7.93722E-12*t**4+4.291972E-15*t**5-
     F 8.903274E-19*t**6
       CVco=(CPco-.296828)*1000.
c-C2H6: ethane
       CPc2h6=5.319795E-01+3.755877E-03*T+1.7892896E-06*T*T-
     G 2.13225E-09*T**3
       CVc2h6=(CPc2h6-.276498)*1000.
c-He: helium
       CPhe=5.1931
       CVhe=(CPhe-2.077022)*1000.
c-H: hydrogen
       CPh=5.0066253+1.01569422E-01*T-6.02891517E-04*T*T+
     H 2.7375894E-06*T**3-8.4758275E-09*T**4+1.43800374e-11*T**5-
     I 9.8072403E-15*t**6
       if(t.ge.425. .and. t.lt.490.) CPh=1.44947e+01
       if(t.ge.490. .and. t.le.1050.) CPh=1.4920082e+01-
     J 1.996917584e-03*t+2.540615e-06*t*t-4.7588954e-10*t**3
       CVh=(CPh-4.124289)*1000.
c-N2: nitrogen
       CPn2=1.088047-3.55968E-04*T+7.2907605E-07*T*T-
     J 2.8861556E-10*T**3
       IF(T.GE.590. .and. T.LE.1080.) CPn2=1.4055077-
     K 2.1894566E-03*T+4.7852898E-06*T*T-4.540166E-09*T**3+
     L 2.08491259E-12*T**4-3.7903033E-16*T**5
       CVn2=(CPn2-.296798)*1000.
c-O2: oxygen
       CPo2=9.29247E-01-3.220603E-04*T+1.166523E-06*T*T-
     M 7.1157865E-10*T**3
       if(t.ge.590. .and. t.le.1050.) CPo2=5.977293e-01+
     N 1.183704e-03*t-1.156226e-06*t**2+5.82171e-10*t**3-
     O 1.1772692e-13*t**4
       CVo2=(CPo2-.259832)*1000.
c-C3H8: propane
       CPc3h8=8.41607e-02+5.7701407e-03*t-1.292127e-06*t*t-
     P 6.9945925e-10*t**3
       CVc3h8=(CPc3h8-.188545)*1000.
c-SO2: sulfur dioxide
       CPso2=4.32805e-01+5.9994156e-04*t+4.593367e-07*t*t-
     Q 1.433024e-09*t**3+1.0409341e-12*t**4-2.5313735e-16*t**5
       CVso2=(CPso2-.129784)*1000.
c-AR: argon
       CPar=0.52034
       CVar=(CPar-.208129)*1000.
c-n-BUTANE
       CPbut=2.3665134e-01+5.10573e-03*t-4.16089e-07*t*t-
     R 1.1450804e-09*t**3
       CVbut=(CPbut-.143044)*1000.
54      CONTINUE
c       print 90
c       print 89,tx,CPair,CPch4,CPco2,CPco,CPc2h6,CPh,CPhe,CPn2,
c     S CPo2,CPc3h8,CPso2,CPar,CPbut
c       print 88,tx,CVair,CVch4,CVco2,CVco,CVc2h6,CVhe,CVh,CVn2,
c     T CVo2,CVc3h8,CVso2,CVar,CVbut
89     format('Cp',1x,f7.2,13f11.4)
88     format('Cv',1x,f7.2,13f11.4)
90     format(6x,'t',8x,'AIR',9x,'CH4',8x,'CO2',9x,'CO',
     U        9x,'C2H6',9x,'H',8x,'HE',9x,'N2',9x,
     V        'O2',8x,'C3H8',7x,'SO2',8x,'AR',8x,'BUT',/)
C
       RETURN
       END
C
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 9 xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx rene 
C      SUBROUTINE PLOT  liminated by JPF 03 2005
C
C      No longeur usefull and causing problems in the execution 
C
c
C
c
c
C     *** END MODIFICATIONS (NEW SUBROUTINE) ***
C
C
C     *** BEGINNING GLOBAL MODIFICATIONS (NEW SUBROUTINE) ***
C
C
C
C
C
C
C
C
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx 
      SUBROUTINE LGMBAL
      implicit real*8 (a-h,o-z)
C-----THIS SUBROUTINE COMPUTES TOTAL FLOWRATES
C-----IT IS CALLED AFTER EACH SUCCESFULL ITERATION
C
C$$$$$$$$$ COMMON BLOCKS FOR ELEMENTS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL (= NUMBER OF ELEMENTS)
C
      COMMON/E1/ELEM(20000)
      COMMON/E2/MATX(20000)
      COMMON/E3/EVOL(20000)
      COMMON/E4/PHI(20000)
      COMMON/E5/P(20000)
      COMMON/E6/T(20000)
C     *** BEGINNING MODIFICATIONS ***
      COMMON/E7/GCH4(20000)
      COMMON/E8/GCO2(20000)
C     *** END MODIFICATIONS ***
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR PRIMARY VARIABLES $$$$$$$$$$$$$$$$$$$$$$$$$
C
C     *** BEGINNING MODIFICATIONS ***
C     THESE BLOCKS HAVE A LENGTH OF 4*NEL
C     *** END MODIFICATIONS ***
C
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
      COMMON/P3/DELX(100000)
      COMMON/P4/R(100000)
      COMMON/P5/DOLD(100000)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$C
C
      COMMON/C1/NEX1(44000)
      COMMON/C2/NEX2(44000)
      COMMON/C3/DEL1(44000)
      COMMON/C4/DEL2(44000)
      COMMON/C5/AREA(44000)
      COMMON/C6/BETA(44000)
      COMMON/C7/ISOX(44000)
      COMMON/C8/GLO(44000)
      COMMON/C9/ELEM1(44000)
      COMMON/C10/ELEM2(44000)
C     *** BEGINNING MODIFICATIONS ***
      common/C11/fvd(44000)
      common/c12/fvt(44000)
      common/c13/fn2d(44000)
      common/c14/fn2t(44000)
      common/c15/fch4d(44000)
      common/c16/fch4t(44000)
      common/c17/fco2d(44000)
      common/c18/fco2t(44000)
      common/c19/fhd(44000)
      common/c20/fht(44000)
      common/c21/fhc(44000)
      common/c22/fha(44000)
      common/c23/fn2l(44000)
      common/c24/fch4l(44000)
      common/c25/fco2l(44000)
      common/c26/fh2o(44000)
C
      common/t1/theat(44000)
      common/t2/tliquid(44000)
      common/t3/tvapor(44000)
      common/t4/tn2(44000)
      common/t5/tch4(44000)
      common/t6/tco2(44000)
      common/t7/tn2l(44000)
      common/t8/tch4l(44000)
      common/t9/tco2l(44000)
      common/t10/th2o(44000)
      common/t11/tgas(44000)
C
C     *** END MODIFICATIONS ***
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G4/ELEG(10000)
      COMMON/G5/SOURCE(10000)
      COMMON/G6/LTABG(10000)
      COMMON/G7/G(10000)
      COMMON/G8/EG(10000)
      COMMON/G9/NEXG(10000)
      COMMON/G10/ITABG(10000)
      COMMON/G11/NGIND(10000)
      COMMON/G12/LCOM(10000)
      COMMON/G13/PI(10000)
      COMMON/G14/PWB(10000)
      COMMON/G15/HG(10000)
      COMMON/G16/GPO(10000)
      COMMON/G17/SDENS(10000)
      COMMON/G18/SSAT(10000)
      COMMON/G19/GVOL(10000)
      COMMON/G20/HL(10000)
      COMMON/G21/HS(10000)
      COMMON/G22/QVGC(10000)
      COMMON/G23/QVWC(10000)
      COMMON/G24/QVOC(10000)
      COMMON/G25/GRAD(10000)
      COMMON/G26/FF(10000)
      common/g27/fmvap(10000)
      common/g28/fmn2(10000)
      common/g29/fmch4(10000)
      common/g30/fmco2(10000)
      common/g31/gtotg(10000)
      common/g32/fmn2l(10000)
      common/g33/fmch4l(10000)
      common/g34/fmco2l(10000)
      common/g35/fmwat(10000)
      common/g36/fmheat(10000)
      common/g37/gtotl(10000)
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
      COMMON/COMPO/FLO(88000)
      COMMON/PORVEL/VEL(88000)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
C      COMMON/KONIT/KON,DELT,IGOOD                                               JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      COMMON/TITLE/ TITLE
C      COMMON/DOP/ENTH,KDATA,QUAL                                                JPF 03 2005
      COMMON/DOP/QUAL,ENTH,KDATA
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/POV6/ TSTART
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
      COMMON/SECPAR/PAR(2640000)
C      COMMON/SOLID/NM,DROK(27),POR(27),PER(3,27),CWET(27),SPHT(27)              JPF 03 2005
      COMMON/SOLID/DROK(27),POR(27),PER(3,27),CWET(27),SPHT(27),NM
      COMMON/SOCH/MAT(27)
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                          JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      common/ff/h1
      COMMON/MADIM/MNEL,MNCON,MNEQ,MNPH,MNBK,MNOGN,MGTAB,MNK
      character*1 h1
      CHARACTER*80 TITLE
      CHARACTER*5 ELEM,ELEM1,ELEM2,ELEG,SOURCE,MAT
      CHARACTER*1 ITABG,HB,H0
C
      SAVE ICALL,HB,H0
      DATA HB,H0/' ',' '/
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C     *** BEGINNING MODIFICATIONS ***
  899 FORMAT(/6X,'LGMBAL   1.0      29 DECEMBER 1995',6X,
     X'PRINT RESULTS FOR ELEMENTS, CONNECTIONS, AND SINKS/SOURCES')
      if(icall.ne.1) go to 3
c initialization of TOTAL flux
      do 1 n=1,ncon
        if (nex1(n).eq.0.OR.nex2(n).eq.0) GO TO 1
        theat(n)=0.
        tliquid(n)=0.
        tgas(n)=0.
        tvapor(n)=0.
        tn2(n)=0.
        tch4(n)=0.
        tco2(n)=0.
        tn2l(n)=0.
        tch4l(n)=0.
        tco2l(n)=0.
        th2o(n)=0.
1     continue
C initialization of TOTAL mass fraction extracted at the well
      do 2 n=1,mnogn
        fmheat(n)=0.
        gtotg(n)=0.
        fmvap(n)=0.
        fmn2(n)=0.
        fmch4(n)=0.
        fmco2(n)=0.
        fmn2l(n)=0.
        fmch4l(n)=0.
        fmco2l(n)=0.
        fmwat(n)=0.
        gtotl(n)=0.
2     continue
3     continue
c COMPUTING OF TOTAL FLUX TOWARDS BOUNDARY CONDITION ELEMENTS
C ELEM1; ELEM2; INDEX; TOTAL HEAT FLUX; CONDUCTIVE HEAT FLUX; DIFFUSIVE
C HEAT FLUX; VAPOR TOTAL&DIFFUSIVE; N2 TOTAL&DIFFUSIVE; CH4 TOTAL&DIFFU
C CO2 TOTAL&DIFFUSIVE FLUX
      do 4 n=1,ncon
        IF (nex1(n).eq.0. or. nex2(n).eq.0) go to 4
        n1=nex1(n)
        n2=nex2(n)
        if(evol(n1).ne.0 .and. evol(n2).ne.0) go to 4
        nnp=(n-1)*2
cc        if(n.le.4) print 88,n,tn2(n),tch4(n),th2o(n),deltex
88      format(1i5,4e15.7)
        theat(n)=theat(n)+fht(n)*deltex
        tliquid(n)=tliquid(n)+flo(nnp+2)*deltex
        tgas(n)=tgas(n)+flo(nnp+1)*deltex
        tvapor(n)=tvapor(n)+fvt(n)*deltex
        tn2(n)=tn2(n)+fn2t(n)*deltex
        tch4(n)=tch4(n)+fch4t(n)*deltex
        tco2(n)=tco2(n)+fco2t(n)*deltex
c
        th2o(n)=th2o(n)+fh2o(n)*deltex
c       if(n.eq.1) print 89,n,theat(n),tliquid(n),th2o(n)
89      format(1i5,3e15.7)
        tn2l(n)=tn2l(n)+fn2l(n)*deltex
        tch4l(n)=tch4l(n)+fch4l(n)*deltex
        tco2l(n)=tco2l(n)+fco2l(n)*deltex
4      continue
C
C SECTION FOR COMPUTING TOTAL MASS COMPONENTS AT EXTRACTION WELL
C
      do 5 n=1,nogn
c fractions of gas and liquid phase
        fgas=ff((n-1)*2+1)
        fliq=ff((n-1)*2+2)
        j=nexg(n)
        if (j.eq.0) go to 5
        j2loc=(j-1)*nsec*neq1
        fmheat(n)=fmheat(n)+g(n)*deltex*eg(n)
        if(lcom(n).le.2) then
        if(lcom(n).eq.1) fmwat(n)=fmwat(n)+g(n)*deltex*fliq
        if(lcom(n).eq.2) fmn2(n)=fmn2(n)+g(n)*deltex*fgas
        else
c  gas mass fraction to the well
        gtotg(n)=gtotg(n)+g(n)*deltex*fgas
        fmvap(n)=fmvap(n)+par(j2loc+nb+1)*g(n)*deltex*fgas
        fmn2(n)=fmn2(n)+par(j2loc+nb+2)*g(n)*deltex*fgas
        fmch4(n)=fmch4(n)+par(j2loc+nb+3)*g(n)*deltex*fgas
        fmco2(n)=fmco2(n)+par(j2loc+nb+4)*g(n)*deltex*fgas
c  liquid mass fraction to the well
362     gtotl(n)=gtotl(n)+g(n)*deltex*fliq
        fmwat(n)=fmwat(n)+par(j2loc+nb+nbk+1)*g(n)*deltex*fliq
        fmn2l(n)=fmn2l(n)+par(j2loc+nb+nbk+2)*g(n)*deltex*fliq
        fmch4l(n)=fmch4l(n)+par(j2loc+nb+nbk+3)*g(n)*deltex*fliq
        fmco2l(n)=fmco2l(n)+par(j2loc+nb+nbk+4)*g(n)*deltex*fliq
        endif
5     continue
C
      RETURN
      END
c
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 5 xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx 
      SUBROUTINE BALLAN
      implicit real*8 (a-h,o-z)
C-----THIS SUBROUTINE PERFORMS VOLUME- AND MASS-BALANCES.
C
C
      COMMON/E2/MATX(20000)
      COMMON/E3/EVOL(20000)
      COMMON/E4/PHI(20000)
      COMMON/P1/X(100000)
      COMMON/SECPAR/PAR(2640000)
c      COMMON/SOLID/NM,DROK(27),POR(27),PER(3,27),CWET(27),SPHT(27)                JPF 03 2005
      COMMON/SOLID/DROK(27),POR(27),PER(3,27),CWET(27),SPHT(27),NM
      COMMON/SOCH/MAT(27)
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                            JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/BC/NELA
      COMMON/GASLAW/R,BMH2O,BMN2,BMCH4,BMCO2,CVAIR
      common/ff/h1
C
      common/bal1/sumch4,sumco2,sumght
C      common/bal2/heatin,gasin,n2gin,ch4gin,co2gin,h2ogin,                          JPF 03 2005
C     x            liqin,n2lin,ch4lin,co2lin,h2olin
      common/bal2/heatin,gasin,ch4gin,co2gin,h2ogin,ch4lin,
     x            co2lin,h2olin,n2gin,liqin,n2lin
C      common/bal3/heatout,gasout,n2gout,ch4gout,co2gout,h2ogout,                     JPF 03 2005
C     x            liqout,n2lout,ch4lout,co2lout,h2olout
      common/bal3/heatout,gasout,ch4gout,co2gout,h2ogout,ch4lout,
     x            co2lout,h2olout,n2gout,liqout,n2lout
      common/bal4/welgas,weln2,welch4,welco2,welvap,welheat
      common/bal5/welliq,weln2l,welch4l,welco2l,welwat
      character*1 h1
      CHARACTER*5 MAT
      real n2gin,n2gout,n2lin,n2lout,liqin,liqout
      dimension dx(10)
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(/6X,'BALLAN   1.0         AVRIL      1995',6X,
     X'PERFORM SUMMARY BALANCES FOR VOLUME, MASS, AND ENERGY')
C
      DAY=SUMTIM/8.64E4
      YEARS=DAY/365.25
C
      PRINT 1,h1
    1 FORMAT(A1/' @@@@@@@@@@ VOLUME- AND MASS-BALANCES @@',116('@')/)
C
      PRINT 2,KCYC,ITER,SUMTIM,DAY,YEARS
    2 FORMAT('            [KCYC,ITER] = [',I4,',',I3,']     ',19X,
     1  'THE TIME IS ',E12.5,' SECONDS,  ',E12.5,' DAYS,  ',F12.5,
     2' YEARS'/)
C
C     *** BEGINNING MODIFICATIONS ***
C
C     VOLG: gas phase volume,     VOLL: liquid phase volume, 
C     AMG:  gas phase mass,       AML:  liquid phase mass,
C     AMV:  water vapor mass,     AMW:  liquid water mass
C     AMN2: N2 mass (gas&liquid), AMCH4:methane mass (gas&liquid)
C     AMCO2:carbon dioxide mass (gas&liquid) 
C     *** END MODIFICATIONS ***
C
      VOLG=0.
      VOLL=0.
      AMG=0.
      AML=0.
      AMV=0.
      AMW=0.
      AMN2G=0.
      AMN2L=0.
      AMCH4G=0.
      AMCO2G=0.
      AMCH4L=0.
      AMCO2L=0.
C
      DO 10 N=1,NELA
        NMAT=MATX(N)
        IF (SPHT(NMAT).GE.1.E4) GO TO 10
        NLOC2=(N-1)*NSEC*NEQ1
        PHIV=PHI(N)*EVOL(N)
c PAR(1) phase saturation
        DVOLG=PHIV*PAR(NLOC2+1)
        DVOLL=PHIV*PAR(NLOC2+NBK+1)
        VOLG=VOLG+DVOLG
        VOLL=VOLL+DVOLL
C PAR(4) phase density
        DAMG=DVOLG*PAR(NLOC2+4)
        AMG=AMG+DAMG
        DAML=DVOLL*PAR(NLOC2+NBK+4)
        AML=AML+DAML
        DAMV=DAMG*PAR(NLOC2+NB+1)
        AMV=AMV+DAMV
C                                                      *** BEG MODIFICATIONS
        AMN2G=AMN2G+DAMG*(PAR(NLOC2+NB+2))
        AMN2L=AMN2L+DAML*(PAR(NLOC2+NBK+NB+2))
        DAMW=DAML*PAR(NLOC2+NBK+NB+1)
        AMW=AMW+DAMW
C
        AMCH4G=AMCH4G+DAMG*PAR(NLOC2+NB+3)
        AMCH4L=AMCH4L+DAML*PAR(NLOC2+NBK+NB+3)
C
        AMCO2G=AMCO2G+DAMG*PAR(NLOC2+NB+4)
        AMCO2L=AMCO2L+DAML*PAR(NLOC2+NBK+NB+4)
C
   10   CONTINUE
C MOLL (VOLUME) FRACTIONS (N2; CH4; CO2 & H2O) IN GAS PHASE in %
          XMH2O=AMV/(BMH2O/22.414)
          XMN2=AMN2G/(BMN2/22.414)
          XMCH4=AMCH4G/(BMCH4/22.414)
          XMCO2=AMCO2G/(BMCO2/22.414)
c
      if(icall.eq.1) then
      dx(1)=amg
      dx(2)=amn2g
      dx(3)=amch4g
      dx(4)=amco2g
      dx(5)=amv
      dx(6)=aml
      dx(7)=amn2l
      dx(8)=amch4l
      dx(9)=amco2l
      dx(10)=amw
      end if
      d1=dx(1)+dx(6)
      d2=dx(2)+dx(7)
      d3=dx(3)+dx(8)
      d4=dx(4)+dx(9)
      d5=dx(5)+dx(10)
c
      PRINT 97,(dx(i),i=1,10),d1,d2,d3,d4,d5
97    FORMAT(' INITIAL MASS IN PLACE         : ',
     x'  gas    ',
     x E13.6,' kg;   [( N2G',
     1    E13.6,' kg  CH4G',E13.6,' kg  CO2G',E13.6,' kg',
     2'  VAPOR',E13.6,' kg )]',/,34X,' liquid ',E13.6,' kg;',
     3'   [( N2L',E13.6,' kg  CH4L',E13.6,' kg  CO2L',E13.6,
     4' kg  WATER',E13.6,' kg )]',/,33x,'  TOTAL  ',E13.6,
     5' kg;   [( N2 ',E13.6,' kg  CH4 ',E13.6,' kg  CO2 ',
     6E13.6,' kg  WATER',E13.6,' kg )]'/)
C
c      PRINT 6,VOLG,XMN2,XMCH4,XMCO2,XMH2O,VOLL
6     FORMAT(/' CURRENT PHASE VOLUMES IN PLACE: ',
     x'  gas    ',E13.6,' m3;',3X,'[( N2G',E13.6,
     2' m3  CH4G',E13.6,' m3  CO2G',E13.6,' m3  VAPOR',
     3E13.6,' m3 )]',/,34X,' liquid ',E13.6,' m3;')
c      if(abs(amch4g).lt.1.e-13) amch4g=0.
c      if(abs(amch4l).lt.1.e-13) amch4l=0.
c      if(abs(amco2g).lt.1.e-13) amco2g=0.
c      if(abs(amco2l).lt.1.e-13) amco2l=0.
      d11=amg+aml
      d12=amn2g+amn2l
      d13=amch4g+amch4l
      d14=amco2g+amco2l
      d15=amv+amw
      PRINT 4,AMG,AMN2G,AMCH4G,AMCO2G,AMV,
     1AML,AMN2L,AMCH4L,AMCO2L,AMW,d11,d12,d13,d14,d15
4     FORMAT(/' CURRENT MASS IN PLACE         : ',
     x'  gas    ',E13.6,' kg;   [( N2G',E13.6,' kg  CH4G',
     1E13.6,' kg  CO2G',E13.6,' kg  VAPOR',E13.6,
     2' kg )]',/,34X,' liquid ',E13.6,' kg;','   [( N2L',
     3E13.6,' kg  CH4L',E13.6,' kg  CO2L',E13.6,
     4' kg  WATER',E13.6,' kg )]',/,33x,'  TOTAL  ',E13.6,
     5' kg;   [( N2 ',E13.6,' kg  CH4 ',E13.6,' kg  CO2 ',
     6E13.6,' kg  WATER',E13.6,' kg )]'/)
c
      if(icall.eq.1) go to 98
      sumprod=sumch4+sumco2
      print 7,sumprod,sumn2,sumch4,sumco2,sumvap
7     format(/' MASS PRODUCED     (IN)        : ',
     x'  gas    ',E13.6,' kg;',3x,
     1'[( N2G',E13.6,' kg  CH4G',E13.6,' kg  CO2G',E13.6,
     2' kg  VAPOR',E13.6,' kg )]'/)
c
c      if(abs(welch4).lt.1.e-13)  welch4=0.
c      if(abs(welch4l).lt.1.e-13) welch4l=0.
c      if(abs(welco2).lt.1.e-13)  welco2=0.
c      if(abs(welco2l).lt.1.e-13) welco2l=0.
      print 99,welgas,weln2,welch4,welco2,welvap
99    FORMAT(' MASS TO WELLS (IN/OUT)        : ',
     x'  gas    ',E13.6,' kg;',3x,
     1'[( N2G',E13.6,' kg  CH4G',E13.6,' kg  CO2G',E13.6,' kg',
     2'  VAPOR',E13.6,' kg )]')
      print 999,welliq,weln2l,welch4l,welco2l,welwat
999   FORMAT('                               : ',
     x'  liquid ',E13.6,' kg;',3x,
     1'[( N2L',E13.6,' kg  CH4L',E13.6,' kg  CO2L',E13.6,' kg',
     2'  WATER',E13.6,' kg )]'/)
c
c      if(abs(ch4gin).lt.1.e-13)  ch4gin=0.
c      if(abs(ch4lin).lt.1.e-13)  ch4lin=0.
c      if(abs(co2gin).lt.1.e-13)  co2gin=0.
c      if(abs(co2lin).lt.1.e-13)  co2lin=0.
      ting=n2gin+ch4gin+co2gin+h2ogin
      tinl=n2lin+ch4lin+co2lin+h2olin
c1      d21=gasin+liqin
      d21=ting+tinl
      d22=n2gin+n2lin
      d23=ch4gin+ch4lin
      d24=co2gin+co2lin
      d25=h2ogin+h2olin
      print 8,ting,n2gin,ch4gin,co2gin,h2ogin,
     x        tinl,n2lin,ch4lin,co2lin,h2olin,
     x        d21,d22,d23,d24,d25
8     FORMAT(' MASS FROM LIMITS  (IN)        : ',
     x'  gas    ',E13.6,' kg;',3x,
     1'[( N2G',E13.6,' kg  CH4G',E13.6,' kg  CO2G',E13.6,' kg',
     2'  VAPOR',E13.6,' kg )]',/,34X,' liquid ',E13.6,' kg;',
     3'   [( N2L',E13.6,' kg  CH4L',E13.6,' kg  CO2L',E13.6,
     4' kg  WATER',E13.6,' kg )]',/,33x,'  TOTAL  ',E13.6,
     5' kg;   [( N2 ',E13.6,' kg  CH4 ',E13.6,' kg  CO2 ',
     6E13.6,' kg  WATER',E13.6,' kg )]'/)
c
c      if(abs(ch4gout).lt.1.e-13)  ch4gout=0.
c      if(abs(ch4lout).lt.1.e-13)  ch4lout=0.
c      if(abs(co2gout).lt.1.e-13)  co2gout=0.
c      if(abs(co2lout).lt.1.e-13)  co2lout=0.
      toutg=n2gout+ch4gout+co2gout+h2ogout
      toutl=n2lout+ch4lout+co2lout+h2olout
c1      d31=gasout+liqout
      d31=toutg+toutl
      d32=n2gout+n2lout
      d33=ch4gout+ch4lout
      d34=co2gout+co2lout
      d35=h2ogout+h2olout
      print 9,toutg,n2gout,ch4gout,co2gout,h2ogout,
     x        toutl,n2lout,ch4lout,co2lout,h2olout,
     x        d31,d32,d33,d34,d35
c1      print 9,gasout,n2gout,ch4gout,co2gout,h2ogout,
c1     x        liqout,n2lout,ch4lout,co2lout,h2olout,
c1     x        d31,d32,d33,d34,d35
9     FORMAT(' MASS  TO  LIMITS (OUT)        : ',
     x'  gas    ',E13.6,' kg;',3x,
     1'[( N2G',E13.6,' kg  CH4G',E13.6,' kg  CO2G',E13.6,' kg',
     2'  VAPOR',E13.6,' kg )]',/,34X,' liquid ',E13.6,' kg;',
     3'   [( N2L',E13.6,' kg  CH4L',E13.6,' kg  CO2L',E13.6,
     4' kg  WATER',E13.6,' kg )]',/,33x,'  TOTAL  ',E13.6,
     5' kg;   [( N2 ',E13.6,' kg  CH4 ',E13.6,' kg  CO2 ',
     6E13.6,' kg  WATER',E13.6,' kg )]'/)
c
c mass differences: (initial+produced+in+out+wells)-current
      t1 = (d1+sumprod+d21+d31+welgas+welliq ) -d11
      t2 = (d2+sumn2  +d22+d32+weln2 + weln2l) -d12
      t3 = (d3+sumch4 +d23+d33+welch4+welch4l) -d13
      t4 = (d4+sumco2 +d24+d34+welco2+welco2l) -d14
      t5 = (d5+sumvap +d25+d35+welvap+ welwat) -d15
c
c  % differnce: difference/(initial+produced+in)
      t1p = 100.*t1/(d1+sumprod+d21)
      t2p = 100.*t2/(d2+sumn2  +d22)
      t3p=0.
      t4p=0.
      if((d3+sumch4 +d23).gt.0.) 
     xt3p = 100.*t3/(d3+sumch4 +d23)
      if((d4+sumco2 +d24).gt.0.) 
     xt4p = 100.*t4/(d4+sumco2 +d24)
      t5p = 100.*t5/(d5+sumvap +d25)
c if air injection
      if(welgas.gt.0.)
     xt1p = 100.*t1/(d1+sumprod+d21+welgas)
      if(weln2 .gt.0.)
     xt2p = 100.*t2/(d2+sumn2  +d22+weln2 )
      if(welch4.gt.0.)
     xt3p = 100.*t3/(d3+sumch4 +d23+welch4)
      if(welco2.gt.0.)
     xt4p = 100.*t4/(d4+sumco2 +d24+welco2)
      if(welvap.gt.0.)
     xt5p = 100.*t5/(d5+sumvap +d25+welvap)
      print 96,t1,t2,t3,t4,t5,t1p,t2p,t3p,t4p,t5p
96    FORMAT(/' DIFFERENCE IN MASS            : ',
     x'  TOTAL  ',E13.6,' kg;   [( N2 ',E13.6,
     1' kg  CH4 ',E13.6,' kg  CO2 ',E13.6,' kg  WATER',
     2E13.6,' kg )]',/,' DIFFERENCE IN  %              : ',
     3'  TOTAL  ',f13.6,'  %;   [( N2 ',
     4f13.6,'  %  CH4 ',f13.6,'  %  CO2 ',f13.6,'  %',
     5'  WATER',f13.6,'  % )]'/)
c
      dh=heatin+sumght+heatout+welheat
      dp=100.*dh/(heatin+sumght)
      print 17,heatin,heatout,sumght,welheat,dh,dp
17    format(/' ENERGY (HEAT) :   in: ',E12.6,' J ',
     x' out: ',E12.6,' J ',
     x' produced: ',E12.6,' J ',
     X' to wells: ',E12.6,' J ',
     x' BALLANCE: ',E12.6,' J  (',E12.6,' %)',/)
C                                                      *** END MODIFICATIONS
98    PRINT 5
      PRINT 16
    5 FORMAT(/1X,155('@'))
   16 FORMAT(1X,155('@')//)
      RETURN
      END
C
C
C
C
C
C
C
C
C
C
C
