      PROGRAM LGM
      implicit real*8 (a-h,o-z)
C     **************************************************************
C     .        TOUGH_LGM, MODULE LGMT2M , version 1.0, MARCH  95.
C     .from:   TOUGH_AMD, MODULE T2M_AMD, version 1.1, SEPT   93.
C     .        TOUGH2   , MODULE T2M    , version 1.0, MARCH  91.
C      *************************************************************
C
C     **************************************************************
C     T2M and T2M_AMD WERE MODIFIED FOR Landfill Gas Migration-LGM 
C     modified by:   MIRO NASTEV,  LAVAL UNIVERSITY, 1995, QUEBEC
C     **************************************************************
C
C     THE MAIN CHANGES FROM T2M ARE:
C
C     * FOUR COMPONENTS ARE CONSIDERED:
C       - WATER, METHANE, CARBON DIOXIDE, ATMOSPHERIC AIR (NITROGEN)
C
C     * NEW ARRAYS ARE CREATED FOR LGM-RELATED COMPUTATIONS:
C       - MCDPR:        METHANE and CARBON DIOXIDE PRODUCTION RATE
C       - OMF:          ORGANIC MASS FRACTION REMAINING IN REFUSE
C       - DOMF:         ORGANIC MASS FRACTION CHANGE DURING TIME STEP
C
C     * NEW TABLES ARE CREATED TO STORE DATA OF INTEREST FOR LGM:
C       - FVD:          DIFFUSIVE WATER VAPOR FLUX BETWEEN ELEMENTS
C       - FVDSP:        DISPERSIVE WATER VAPOR FLUX BETWEEN ELEMENTS
C       - FVT:          TOTAL WATER VAPOR FLUX BETWEEN ELEMENTS
C       - FN2D:         DIFFUSIVE N2 FLUX BETWEEN ELEMENTS
C       - FN2DSP:       DISPERSIVE N2 FLUX BETWEEN ELEMENTS
C       - FN2T:         TOTAL N2 FLUX BETWEEN ELEMENTS
C       - FCH4D:        DIFFUSIVE METHANE FLUX BETWEEN ELEMENTS
C       - FCH4DSP:      DISPERSIVE METHANE FLUX BETWEEN ELEMENTS
C       - FCH4T:        TOTAL METHANE FLUX BETWEEN ELEMENTS
C       - FCO2D:        DIFFUSIVE CARBON DIOXIDE FLUX BETWEEN ELEMENTS
C       - FCO2DSP:      DISPERSIVE CARBON DIOXIDE FLUX BETWEEN ELEMENTS
C       - FCO2T:        TOTAL CARBON DIOXIDE FLUX BETWEEN ELEMENTS
C       - FHC:          CONDUCTIVE HEAT FLUX BETWEEN ELEMENTS
C       - FHA:          ADVECTIVE/CONVECTIVE HEAT FLUX BETWEEN ELEMENTS
C       - FHD:          DIFFUSIVE HEAT FLUX BETWEEN ELEMENTS
C       - FHT:          TOTAL HEAT FLUX BETWEEN ELEMENTS = FHC+FHA+FHD
C       - FN2L:         CONVECTIVE  N2 FLUX IN LIQUID PHASE
C       - FCH4L:        CONVECTIVE CH4 FLUX IN LIQUID PHASE
C       - FCO2L:        CONVECTIVE CO2 FLUX IN LIQUID PHASE
C       - FH2O:         CONVECTIVE H2O FLUX IN LIQUID PHASE
C
C     NOTE: MODIFIED SECTIONS INDICATED BY  
C
C     *** BEGINNING MODIFICATIONS ***
C     MODIFIED SECTION ...
C     *** END MODIFICATIONS ***
C
C
C-----THIS IS AN EXPERIMENTAL VERSION OF MULKOM, KNOWN AS TOUGH2 !!!!!!!
C
C
C     ******************************************************************
C
C     MULKOM WAS DEVELOPED BY K. PRUESS
C                             AT LAWRENCE BERKELEY LABORATORY.
C
C     ******************************************************************
C
C
C-----PROGRAM MAIN IS A HIGH-LEVEL EXECUTIVE ROUTINE.
C
C     MAIN CALLS SEVERAL LOWER-LEVEL EXECUTIVE ROUTINES, WHICH EXECUTE
C     THE ACTUAL CALCULATIONS.
C
C     ALL LARGE ARRAYS ARE DIMENSIONED IN PROGRAM MAIN. IF MODIFICATIONS
C     IN ARRAY DIMENSIONS ARE DESIRED, THESE NEED TO BE DONE ONLY IN
C     MAIN, BUT NOT IN THE SUBROUTINES.
C
C     THE ARRAY DIMENSIONS ARE SET IN A PARAMETER STATEMENT PRECEDING
C     THE COMMON BLOCKS.
C
C=======================================================================
C
C               GLOSSARY OF MAJOR ARRAYS, IN ORDER OF APPEARANCE BELOW
C
C=======================================================================
C
C     ELEMENTS:
C                  *ELEM* HOLDS ELEMENT CODE NAMES (IDENTIFIERS)
C                  *MATX* HOLDS MATERIAL DOMAIN IDENTIFIER
C                  *EVOL* HOLDS VOLUME
C                  *PHI*  IS POROSITY
C                  *P*    IS PRESSURE AT LAST CONVERGED TIME STEP
C                  *T*    IS TEMPERATURE AT LAST CONVERGED TIME STEP
C     *** BEGINNING MODIFICATIONS ***
C                  *MCDPR* METHANE AND CARBON DIOXIDE PRODUCTION RATE
C                  *OMF*   ORGANIC MASS FRACTION REMAINING IN REFUSE
C                  *DOMF*  OMF CHANGE IN REFUSE DURING TIME STEP
C                  *XYZ*   CONTAINS ELEMENTS COORDINATES
C     *** END MODIFICATIONS ***
C                  *AI*   IS FOR HEAT EXCHANGE WITH CONFINING LAYERS
C                  *AHT*  ARE HEAT TRANSFER AREAS
C
C     PRIMARY VARIABLES:
C                  *X*    LATEST CONVERGED VALUES OF PRIMARY VARIABLES
C                  *DX*   LATEST INCREMENTS OF PRIMARY VARIABLES
C                  *DELX* SMALL INCREMENTS OF PRIMARY VARIABLES
C                         FOR NUMERICALLY COMPUTING DERIVATIVES
C                  *R*    LATEST RESIDUALS
C                  *DOLD* ACCUMULATION TERMS AT BEGINNING OF TIME STEP
C                  *ROW*  SCALING FACTORS FOR MATRIX ROWS
C                  *COL*  SCALING FACTORS FOR MATRIX COLUMNS
C
C     CONNECTIONS (INTERFACES):
C                  *NEX1*  INDICES OF FIRST ELEMENTS IN A CONNECTION
C                  *NEX2*  INDICES OF SECOND ELEMENTS IN A CONNECTION
C                  *DEL1*  FIRST ELEMENT NODAL DISTANCES
C                  *DEL2*  SECOND ELEMENT NODAL DISTANCES
C                  *AREA*  INTERFACE AREAS
C                  *BETA*  COSINE OF ANGLE BETWEEN NODAL LINE AND
C                          THE VERTICAL
C                  *ISOX*  ISOTROPY INDICES
C                  *GLO*   HEAT FLOW RATES
C                  *ELEM1* CODE NAMES OF FIRST ELEMENT IN A CONNECTION
C                  *ELEM2* CODE NAMES OF SECOND ELEMENT
C
C
C     LINEAR EQUATION SOLVER:
C                  *IRN*    ROW INDICES OF NON-ZERO MATRIX ELEMENTS
C                  *ICN*    COLUMN INDICES OF NON-ZERO MATRIX ELEMENTS
C                  *WKAREA* WORKSPACE USED IN MA28
C                  *IKEEP*  INTERNAL STORAGE SPACE FOR MA28
C                  *JVECT*  COLUMN INDICES OF NON-ZERO MATRIX ELEMENTS
C                  *IW*     INTERNAL STORAGE SPACE FOR MA28
C                  *W19A*   WORKSPACE FOR SCALING ROUTINE (MC19A)
C
C     SINKS AND SOURCES:
C                  *F1* TABLE OF GENERATION TIMES
C                  *F2* TABLE OF GENERATION RATES
C                  *F3* TABLE OF INJECTION ENTHALPIES
C
C     OTHER LARGE ARRAYS:
C                  *PAR* SECONDARY (THERMOPHYSICAL) PARAMETERS FOR
C                        ALL ELEMENTS
C                  *FLO* RATES OF GAS AND LIQUID PHASE FLOWS ACROSS
C                        ALL INTERFACES
C                  *VEL* PORE VELOCITIES OF GAS AND LIQUID PHASE FLOW
C
C=======================================================================
C
C$$$$$$$$$ ASSIGN PARAMETERS FOR FLEXIBLE DIMENSIONING OF LARGE ARRAYS $
C
C     *** BEGINNING MODIFICATIONS ***
C
C      PARAMETER(MNEL=4000,MNCON=10000,MNEQ=5,MNK=4,MNPH=2,MNB=6)        JPF fevr.2005   MNEQ=5 pour isothermal, 
C      PARAMETER(MNEL=20000,MNCON=44000,MNEQ=4,MNK=4,MNPH=2,MNB=6)                           RECAHNGER JPF avril
      PARAMETER(MNEL=20000,MNCON=44000,MNEQ=5,MNK=4,MNPH=2,MNB=6)
C MNEL: max No of elements; MNCON: max No of connections; MNEQ: No of equations 
C (MNK+energy); MNK: No of elements (H2O, CH4, CO2, N2); MNPH: No of phases
C (liquid+gaseous); MNB: No of secondary variables other than mass
C     *** END MODIFICATIONS ***
C
c     PARAMETER(MNOGN=100,MGTAB=5000)                                    JPF fevr.2005
      PARAMETER(MNOGN=10000,MGTAB=25000)
C MNOGN: No of generation siks/sources; 
C MGTAB: No of time dependent generation data 
      COMMON/MADIM/M1,M2,M3,M4,M5,M6,M7,M8
C
C
C************************ COMMON BLOCKS FOR ELEMENTS ***********************
C
C     THESE BLOCKS HAVE A LENGTH OF NEL (= NUMBER OF ELEMENTS)
C
      COMMON/E1/ELEM(MNEL)
      COMMON/E2/MATX(MNEL)
      COMMON/E3/EVOL(MNEL)
      COMMON/E4/PHI(MNEL)
      COMMON/E5/P(MNEL)
      COMMON/E6/T(MNEL)
C ELEM: element ident.; MATX: material identifiers; EVOL: volume; PHI: porosity
C P: pressure at last converged time step; T: temp. at last converged time step
C     *** BEGINNING MODIFICATIONS ***
      COMMON/E7/GCH4(MNEL)
      COMMON/E8/GCO2(MNEL)
      COMMON/E9/TDEPOS(MNEL)
      COMMON/E10/DXSO4(MNEL)
      COMMON/E11/XSO4(MNEL)
      COMMON/E12/XPY(MNEL)
      COMMON/E13/DXPY(MNEL)
      COMMON/E14/XYZ(MNEL*3)
C     *** END MODIFICATIONS ***
      COMMON/VINWES/AI(MNEL)
      COMMON/AHTRAN/AHT(MNEL)
C
C
C************** COMMON BLOCKS FOR PRIMARY VARIABLES **********************
C
C     DATA BLOCKS /P1/, /P2/, /P3/ HAVE A LENGTH OF (NK+1)*NEL.
C     NK IS THE NUMBER OF MASS BALANCE EQUATIONS PER GRID BLOCK.
C     DATA BLOCKS /P4/ THROUGH /P7/ HAVE A LENGTH OF NEQ*NEL.
C     NEQ IS THE NUMBER OF BALANCE EQUATIONS PER GRID BLOCK.
C X: latest converged values of primary variables; DX: latest increments of primary variables
C DELX: increments of primary variables; R: latest residuals; DOLD: accumulation terms at the
C beginning of time step; ROW: scaling factors of matrix rows; COL: scaling factor of matrix columns
      COMMON/P1/X((MNK+1)*MNEL)
      COMMON/P2/DX((MNK+1)*MNEL)
      COMMON/P3/DELX((MNK+1)*MNEL)
      COMMON/P4/R(MNEQ*MNEL)
      COMMON/P5/DOLD(MNEQ*MNEL)
      COMMON/P6/ROW(MNEQ*MNEL)
      COMMON/P7/COL(MNEQ*MNEL)
C
C
C**************** COMMON BLOCKS FOR CONNECTIONS ************************
C
C     THESE BLOCKS HAVE A LENGTH OF NCON (=NUMBER OF CONNECTIONS).
C NEX1: first element in a link; NEX2: second element in a link; DEL1: first element nodal
C distances; DEL2: second element nodal distances; AREA: interface areas; BETA: cosine 
C of angle between nodal line and the vertical; ISOX: isotropy; GLO: heat flow rates
C ELEM1: code names of first element in a link; ELEM2: code names of second element in a link
      COMMON/C1/NEX1(MNCON)
      COMMON/C2/NEX2(MNCON)
      COMMON/C3/DEL1(MNCON)
      COMMON/C4/DEL2(MNCON)
      COMMON/C5/AREA(MNCON)
      COMMON/C6/BETA(MNCON)
      COMMON/C7/ISOX(MNCON)
      COMMON/C8/GLO(MNCON)
      COMMON/C9/ELEM1(MNCON)
      COMMON/C10/ELEM2(MNCON)
C     *** BEGINNING MODIFICATIONS ***
      common/c11/fvd(mncon)
      common/c12/fvt(mncon)
      common/c13/fn2d(mncon)
      common/c14/fn2t(mncon)
      common/c15/fch4d(mncon)
      common/c16/fch4t(mncon)
      common/c17/fco2d(mncon)
      common/c18/fco2t(mncon)
      common/c19/fhd(mncon)
      common/c20/fht(mncon)
      common/c21/fhc(mncon)
      common/c22/fha(mncon)
      common/c23/fn2l(mncon)
      common/c24/fch4l(mncon)
      common/c25/fco2l(mncon)
      common/c26/fh2o(mncon)
C
      common/t1/theat(mncon)
      common/t2/tliquid(mncon)
      common/t3/tvapor(mncon)
      common/t4/tn2(mncon)
      common/t5/tch4(mncon)
      common/t6/tco2(mncon)
      common/t7/tn2l(mncon)
      common/t8/tch4l(mncon)
      common/t9/tco2l(mncon)
      common/t10/th2o(mncon)
      common/t11/tgas(mncon)
C     *** END MODIFICATIONS ***
C
C************* COMMON BLOCKS FOR LINEAR EQUATIONS **********************
C IRN: row indices of non-zero matrix elements; ICN: column indices of non-zero matrix elements
C WKAREA: workspace used in MA28: IKEEP: inetrnal storage space for MA28
C JVECT: column indices of non-zero matrix elements; IW: internal storage space for MA28
C W19A: workspace for scaling routine MC19A
      COMMON/L1/IRN(2*(MNEL+2*MNCON)*MNEQ*MNEQ)
C     IRN HAS A LENGTH OF LIRN, WHICH NEEDS TO BE SOMEWHAT LARGER
C     THAN NZ = (NEL + 2*NCON)*NEQ**2.
C
      COMMON/L2/ICN(4*(MNEL+2*MNCON)*MNEQ*MNEQ)
      COMMON/L3/CO(4*(MNEL+2*MNCON)*MNEQ*MNEQ)
C     ICN AND CO HAVE A LENGTH OF LICN = (2-4) * NZ.
C
      COMMON/L4/WKAREA(MNEQ*MNEL)
C     WKAREA HAS A LENGTH OF NEQ*NEL.
C
      COMMON/L5/IKEEP(5*MNEQ*MNEL)
C     IKEEP HAS A LENGTH OF 5*NEQ*NEL.
      COMMON/L6/IW(8*MNEQ*MNEL)
C     IW HAS A LENGTH OF 8*NEQ*NEL.
C
      COMMON/L7/JVECT((MNEL+2*MNCON)*MNEQ*MNEQ)
C     JVECT HAS A LENGTH OF NZ.
C
      COMMON/L8/W19A(5*MNEQ*MNEL)
C     W19A HAS A LENGTH OF 5*NEQ*NEL.
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
C
C************ COMMON BLOCKS FOR SINKS/SOURCES **************************
C
C F1: table of generation times; F2: table of generation rates; F3: table of injection enthalpies
C
      COMMON/G1/F1(MGTAB)
      COMMON/G2/F2(MGTAB)
      COMMON/G3/F3(MGTAB)
C     ARRAYS F1, F2, AND F3 HOLD, RESPECTIVELY, TABLES OF TIMES, RATES,
C     AND ENTHALPIES OF TIME-DEPENDENT SINKS OR SOURCES. THEIR DIMENSIONS
C     MUST NOT BE SMALLER THAN THE TOTAL NUMBER OF SUCH DATA.
C
      COMMON/G4/ELEG(MNOGN)
      COMMON/G5/SOURCE(MNOGN)
      COMMON/G6/LTABG(MNOGN)
      COMMON/G7/G(MNOGN)
      COMMON/G8/EG(MNOGN)
      COMMON/G9/NEXG(MNOGN)
      COMMON/G10/ITABG(MNOGN)
      COMMON/G11/NGIND(MNOGN)
      COMMON/G12/LCOM(MNOGN)
      COMMON/G13/PI(MNOGN)
      COMMON/G14/PWB(MNOGN)
      COMMON/G15/HG(MNOGN)
      COMMON/G16/GPO(MNOGN)
      COMMON/G17/SDENS(MNOGN)
      COMMON/G18/SSAT(MNOGN)
      COMMON/G19/GVOL(MNOGN)
      COMMON/G20/HL(MNOGN)
      COMMON/G21/HS(MNOGN)
      COMMON/G22/QVGC(MNOGN)
      COMMON/G23/QVWC(MNOGN)
      COMMON/G24/QVOC(MNOGN)
      COMMON/G25/GRAD(MNOGN)
C     THE DIMENSION OF ALL ARRAYS IN COMMON G4 THROUGH G25 MUST BE
C     EQUAL TO OR LARGER THAN THE TOTAL NUMBER OF SINKS AND SOURCES.
C
      COMMON/G26/FF(MNPH*MNOGN)
C     THE DIMENSION OF ARRAY FF MUST BE EQUAL TO OR LARGER THAN THE
C     PRODUCT OF NUMBER OF PHASES AND TOTAL NUMBER OF SINKS AND SOURCES.
      common/g27/fmvap(mnogn)
      common/g28/fmn2(mnogn)
      common/g29/fmch4(mnogn)
      common/g30/fmco2(mnogn)
      common/g31/gtot(mnogn)
      common/g32/fmn2l(mnogn)
      common/g33/fmch4l(mnogn)
      common/g34/fmco2l(mnogn)
      common/g35/fmh2o(mnogn)
      common/g36/fmheat(mnogn)
      common/g37/gtotl(mnogn)
C
C*****************COMMON BLOCKS FOR OTHER LARGE ARRAYS ***************
C
C PAR: secondary (thermophysical) parameters for all elements; FLO: rates of gas and liquid phase
C flows across all interfaces; VEL: pore velocities of gas and liquid phase flow
      COMMON/SECPAR/PAR((MNEQ+1)*(MNPH*(MNB+MNK)+2)*MNEL)
C     ARRAY PAR HAS A LENGTH OF (NEQ+1)*NSEC*NEL.
C     NSEC = NPH*(NB+NK)+2 IS THE total NUMBER OF SECONDARY PARAMETERS.
C     NPH IS THE NUMBER OF PHASES.
C     NB IS THE NUMBER OF THERMOPHYSICAL PARAMETERS (USUALLY 6).
C     NK IS THE NUMBER OF COMPONENTS (SPECIES). H2O, CH4, CO2, air-N2
C
      COMMON/COMPO/FLO(MNPH*MNCON)
      COMMON/PORVEL/VEL(MNPH*MNCON)
C
      COMMON/TITLE/TITLE
      CHARACTER*80 TITLE
      CHARACTER*5 ELEM,ELEM1,ELEM2,ELEG,SOURCE,VV
C     LOGICAL EX
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
C      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1                     JPF 03 2005
      COMMON/DMN/TZERO,TIMP1,INUM,IPRINT,MCYC,MCYPR,MSEC
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/LDIM/LICN,LIRN
      COMMON/BC/NELA
      COMMON/V/IV,IS
      common/ff/h1
      character*1 h1
      DIMENSION VV(26)
C
CRAY-SPECIFIC: CONNECT TO FILE "INPUT" AS DEFAULT INPUT, FILE "OUTPUT"
C     AS DEFAULT OUTPUT.
C     CALL LINK('UNIT5=(INPUT,OPEN,TEXT),READ5,
C    X           UNIT6=(OUTPUT,CREATE,HC),PRINT6 //')
C
C
      h1=char(12)
      PRINT 1,h1
c    1 FORMAT(A1/7X,'@@@@@   @@   @  @   @@@  @  @   @@       @@@  @ ',
c     A  ' @   @  ',
c     1  '@  @  @      @@   @@@@@  @   @@   @   @     @@@   @  @  @   @'/
c     2  7X,'  @    @  @  @  @  @     @  @  @  @     @     @  @@ @@',
c     B  '  @  @',
c     3  '  @     @  @    @    @  @  @  @@  @     @  @  @  @  @@  @'/
c     4  7X,'  @    @  @  @  @  @ @@  @@@@    @       @@   @  @ @ @',
c     C  '  @  @',
c     5  '  @     @@@@    @    @  @  @  @ @ @     @@@   @  @  @ @ @'/
c     6  7X,'  @    @  @  @  @  @  @  @  @   @          @  @  @   @',
c     D  '  @  @',
c     71X,  ' @     @  @    @    @  @  @  @  @@     @ @   @  @  @  @@'/
c     8  7X,'  @     @@    @@    @@@  @  @  @@@@     @@@   @  @   @',
c     E  '   @@ ',
c     9  '  @@@@  @  @    @    @   @@   @   @     @  @   @@   @   @'//)
C2345
1     FORMAT(A1/4X,
     1   '@      @@   @   @  @@@   @@@@  @  @     @        @@@@ ',
     1' @     ',
     1'@  @  @  @@@    @@@     @   @  @   @@@  @@@    @@   @@@@@  ',
     1'@   @@   @   @'/
     24X,'@     @  @  @@  @  @  @  @     @  @     @        @     @   ',
     2'  @  @  @  @  @  @      ',
     2'  @@ @@  @  @     @  @  @  @    @    @  @  @  @@  @'/
     34X,'@     @@@@  @ @ @  @  @  @@@@  @  @     @        @@@@  @   ',
     3'  @  @  @  @  @   @@    ',
     3'  @ @ @  @  @  @  @@@   @@@@    @    @  @  @  @ @ @'/
     44X,'@     @  @  @  @@  @  @  @     @  @     @        @     @   ',
     4'  @  @  @  @  @     @    ',
     4   ' @   @  @  @  @  @ @   @  @    @    @  @  @  @  @@'/
     54X,'@@@@  @  @  @   @  @@@   @     @  @@@@  @@@@     @     @@@@',
     5'   @@   @  @@@   @@@    ',
     5'  @   @  @   @@   @  @  @  @    @    @   @@   @   @'//)
C
      PRINT 4
    4 FORMAT(12X,'LFM (MODIFIED VERSION OF TOUGH2) ',
     X'IS A PROGRAM FOR TWO-PHASE FOUR-COMPONENT',
     X' FLOW IN PERMEABLE MEDIA, INCLUDING HEAT FLOW.'/16X,
     X' TOUGH2 IS A MEMBER OF THE MULKOM FAMILY OF CODES, DEVELOPED',
     X' BY KARSTEN PRUESS AT LAWRENCE BERKELEY LABORATORY.')
C
C     *** BEGINNING MODIFICATIONS ***
      PRINT 900
  900 FORMAT(//24X,'@     @@@@  @   @      @@@  @ ',
     1' @   @  ',
     1'@  @  @      @@   @@@@@  @   @@   @   @     @@@   @  @  @   @',
     2        /24X,'@     @     @@ @@     @     @  @@ @@',
     2  '  @  @',
     2  '  @     @  @    @    @  @  @  @@  @     @  @  @  @  @@  @',
     3        /24X,'@     @@@@  @ @ @      @@   @  @ @ @',
     3  '  @  @',
     3  '  @     @@@@    @    @  @  @  @ @ @     @@@   @  @  @ @ @',
     4        /24X,'@     @     @   @        @  @  @   @',
     4  '  @  @ ',
     4  ' @     @  @    @    @  @  @  @  @@     @ @   @  @  @  @@',
     5        /24X,'@@@@  @     @   @     @@@   @  @   @',
     5  '   @@ ',
     5  '  @@@@  @  @    @    @   @@   @   @     @  @   @@   @   @'//)
C
      PRINT 901
  901 FORMAT(15X,'   LFM IS A MODIFIED VERSION OF TOUGH2 (VERSION 1.0'
     A,', AP.91) ADAPTED TO LANDFILL FLUIDS MIGRATION (LFM) ',
     A'MODELLING.',/,
     B29X,'    MODIFICATIONS DONE BY: BY MIRO NASTEV AT LAVAL ',
     C'UNIVERSITY, QUEBEC, CANADA.')
C
      PRINT 5
    5 FORMAT(/30X,80('*')/
     X30X,21('*'),'  LFM - VERSION 3.2  (MARCH 1996)   ',
     X23('*')/30X,80('*'))
C     *** END MODIFICATIONS ***
C
      M1=MNEL
      M2=MNCON
      M3=MNEQ
      M4=MNPH
      M5=MNB+MNK
      M6=MNOGN
      M7=MGTAB
      M8=MNK
      MNZ=(MNEL+2*MNCON)*MNEQ*MNEQ
      LIRN=2*MNZ
      LICN=4*MNZ
C
      MPRIM=(MNK+1)*MNEL
      MSEC=(MNEQ+1)*(MNPH*(MNB+MNK)+2)*MNEL
      PRINT 6,MNEL,MNCON,MNEQ,MNK,MNPH,MNB,MNOGN,MGTAB
    6 FORMAT(//' PARAMETERS FOR FLEXIBLE DIMENSIONING OF MAJOR ARRAYS ',
     X'(MAIN PROGRAM) ARE AS FOLLOWS'//
     X' MNEL =',I5,'   MNCON =',I5,'   MNEQ =',I3,'   MNK =',I3,
     X'   MNPH =',I3,
     X'   MNB =',I3,'   MNOGN =',I4,'   MGTAB =',I5/' ',131('='))
C
      PRINT 7,MNEL,MNCON,MPRIM,MNOGN
    7 FORMAT(/' MAXIMUM NUMBER OF VOLUME ELEMENTS (GRID BLOCKS):',12X,
     X'MNEL  =',I10/
     X' MAXIMUM NUMBER OF CONNECTIONS (INTERFACES):',17X,'MNCON =',I10/
     X' MAXIMUM LENGTH OF PRIMARY VARIABLE ARRAYS:',18X,'MPRIM =',I10/
     X' MAXIMUM NUMBER OF GENERATION ITEMS (SINKS/SOURCES):',9X,
     X'MNOGN =',I10)
      PRINT 9,MGTAB,MSEC,MNZ,LIRN,LICN
    9 FORMAT(
     X' MAXIMUM NUMBER OF TABULAR (TIME-DEPENDENT) GENERATION DATA',
     X': MGTAB =',I10/
     X' LENGTH OF SECONDARY PARAMETER ARRAY:',24X,'MSEC  =',I10/
     X' MAXIMUM NUMBER OF JACOBIAN MATRIX ELEMENTS:',17X,'MNZ   =',I10/
     X/' LARGE LINEAR EQUATION ARRAYS: LENGTH OF IRN IS',14X,
     X'LIRN  =',I10/
     X'                               LENGTH OF ICN AND CO IS',7X,
     X'LICN  =',I10//' ',131('='))
C
      CALL IO
c      CALL SECOND(TZERO)
C
      WRITE(11,8)
    8 FORMAT(/'      TOUGH-LGM1.1      31 OCTOBER   1993',6X,
     X'MAIN PROGRAM FROM TOUGH2 VERSION 1.0 (29 MARCH 1991)')
C
      READ 2,TITLE
    2 FORMAT(A80)
      PRINT 3,TITLE
    3 FORMAT(/1X,131('=')//5X,'PROBLEM TITLE:  ',A80//)
C############################################################ FETCH INPUT DATA
      CALL INPUT
C     EVALUATE FLOATING POINT PROCESSOR.
      CALL FLOP
C
      IF(IS.NE.0) GOTO 100
C###################################################### FETCH FINAL INPUT DATA
      CALL RFILE
      PRINT 10,NEL,NELA,NCON,NOGN
   10 FORMAT(/' MESH HAS',I5,' ELEMENTS (',I5,' ACTIVE) AND',I5,
     A' CONNECTIONS (INTERFACES)',
     A' BETWEEN THEM'/' GENER HAS',I4,' SINKS/SOURCES')
C IN LGMT2F.F: PROVIDES PRINTOUT OF MOST OF THE INPUT DATA
C###################################################### PRINT FINAL INPUT DATA
      IF(MOP(7).NE.0) CALL INDATA
C
c      CALL SECOND(ELT1)
C      ELT1=ELT1-TZERO
C      PRINT 11,ELT1
C     *** BEGINNING MODIFICATIONS ***
C   11 FORMAT(//' END OF TOUGH2 INPUT JOB --- ELAPSED TIME = ',E10.3,
C     A'SECONDS'/)
C     *** END MODIFICATIONS ***
      PRINT 79,h1
   79 FORMAT(A1/' ',131('*'))
      PRINT 80
   80 FORMAT(' *          ARRAY *MOP*',
     X' ALLOWS TO GENERATE MORE PRINTOUT IN VARIOUS SUBROUTINES, AND',
     X' TO MAKE SOME CALCULATIONAL CHOICES.           *')
      PRINT 78
   78 FORMAT(' ',131('*')/)
      PRINT 81,MOP(1)
   81 FORMAT(' ',
     X'  MOP(1)  =',I2,' *** ALLOWS TO GENERATE A SHORT PRINTOUT FOR'
     X,' EACH NEWTON-RAPHSON ITERATION'/
     X'           = 0, 1, OR 2: GENERATE 0, 1, OR 2 LINES OF PRINTOUT'//
     X12X,'MORE PRINTOUT IS GENERATED FOR MOP(I) > 0 IN THE FOLLOWING',
     X' SUBROUTINES (THE LARGER MOP IS, THE MORE WILL BE PRINTED).'/)
      PRINT 82,MOP(2),MOP(3),MOP(4),MOP(5),MOP(6)
   82 FORMAT(' ',
     X'  MOP(2) =',I2,' *** CYCIT    ',
     X'   MOP(3) =',I2,' *** MULTI    ',
     X'   MOP(4) =',I2,' *** QU    ',
     X'   MOP(5) =',I2,' *** EOS    ',
     X'   MOP(6) =',I2,' *** LINEQ    '/)
      PRINT 83,MOP(7)
   83 FORMAT(' ',
     X'  MOP(7)  =',I2,' *** IF UNEQUAL ZERO, WILL GENERATE A PRINTOUT',
     X' OF INPUT DATA'/)
C
      PRINT 84,MOP(9),MOP(10)
   84 FORMAT(' ',
     X8X,'   CALCULATIONAL CHOICES OFFERED BY MOP ARE AS FOLLOWS:'//
     X'   MOP(9)  =',I2,' *** CHOOSES FLUID COMPOSITION ON WITHDRAWAL',
     X' (PRODUCTION).'/
     X'           = 0: ACCORDING TO RELATIVE MOBILITIES.'/
     X'           = 1: ACCORDING TO COMPOSITION IN PRODUCING ELEMENT.'//
     X'   MOP(10) =',I2,' *** CHOOSES INTERPOLATION FORMULA FOR DEPEN',
     X'DENCE OF THERMAL CONDUCTIVITY ON LIQUID SATURATION (SL).'/
     X'           = 0: K = KDRY + SQRT(SL)*(KWET-KDRY)'/
     X'           = 1: K = KDRY + SL*(KWET-KDRY)'/)
      PRINT 85,MOP(11)
   85 FORMAT(' ',
     X'  MOP(11) =',I2,' *** CHOOSES EVALUATION OF MOBILITY',
     X' AND ABSOLUTE PERMEABILITY AT INTERFACES.'/
     X'           = 0: MOBILITIES ARE UPSTREAM WEIGHTED WITH WUP.',
     X' (DEFAULT IS WUP = 1.0). PERMEABILITY IS UPSTREAM WEIGHTED.'/
     X'           = 1: MOBILITIES ARE AVERAGED BETWEEN',
     X' ADJACENT ELEMENTS. PERMEABILITY IS UPSTREAM WEIGHTED.'/
     X'           = 2: MOBILITIES ARE UPSTREAM WEIGHTED WITH WUP.',
     X' (DEFAULT IS WUP = 1.0). PERMEABILITY IS HARMONIC WEIGHTED.'/
     X'           = 3: MOBILITIES ARE AVERAGED BETWEEN',
     X' ADJACENT ELEMENTS. PERMEABILITY IS HARMONIC WEIGHTED.'/
     X'           = 4: MOBILITY * PERMEABILITY PRODUCT IS HARMONIC',
     X' WEIGHTED.'/)
      PRINT 86,MOP(12),MOP(14),MOP(15)
   86 FORMAT(
     X'   MOP(12) = NOT VALID (MODIFFIED TO A REAL STEP FUNCTION)'/
     X'           =',I2,' *** CHOOSES PROCEDURE FOR INTERPOLATING ',
     X'GENERATION RATES FROM A TIME TABLE.'/
     X'           = 0: TRIPLE LINEAR INTERPOLATION.'/
     X'           = 1: "STEP FUNCTION" OPTION.'//
     X'   MOP(14) =',I2,' *** SPECIFIES THE HANDLING OF PIVOT FAILURES',
     X' IN THE LINEAR EQUATION SOLUTION'/
     X'           = 0: PERFORM NEW MATRIX DECOMPOSITION AFTER PIVOT',
     X' FAILURE'/
     X'           > 0: IGNORE PIVOT FAILURE AND PROCEED',//
     X'   MOP(15) =',I2,' *** ALLOWS TO SELECT A SEMI-ANALYTICAL HEAT',
     X' EXCHANGE CALCULATION WITH CONFINING BEDS.'/
     X'           = 0: NO SEMI-ANALYTICAL HEAT EXCHANGE'/
     X'           > 0: SEMI-ANALYTICAL HEAT EXCHANGE ENGAGED (WHEN A',
     X' SPECIAL SUBROUTINE MODULE *QLOSS* IS PRESENT)'/)
      PRINT 87,MOP(16),MOP(17),MOP(18)
   87 FORMAT(' ',
     X'  MOP(16) =',I2,' *** PERMITS TO CHOOSE TIME STEP SELECTION ',
     X'OPTION'/
     X'           = 0: USE TIME STEPS EXPLICITLY PROVIDED AS INPUT.'/
     X'           > 0: INCREASE TIME STEP BY AT LEAST A FACTOR 2, IF',
     X' CONVERGENCE OCCURS IN .LE. MOP(16) ITERATIONS.'//
     X'   MOP(17) =',I2,' *** HANDLES SCALING OPTIONS.'/
     X'           = 0: NO SCALING.'/
     X'           = 7: SCALING.'//
     X'   MOP(18) =',I2,' *** ALLOWS TO SELECT HANDLING OF INTERFACE ',
     X'DENSITY.'/
     X'           = 0: PERFORM UPSTREAM WEIGHTING FOR INTERFACE DENSITY.
     X'/'           > 0: COMPUTE INTERFACE DENSITY AS AVERAGE OF',
     X' THE TWO GRID BLOCK DENSITIES.'/
     X'                HOWEVER, WHEN ONE OF THE TWO PHASE SATURATIONS',
     X' IS ZERO, DO UPSTREAM WEIGHTING.'/)
C
C     *** BEGINNING MODIFICATIONS ***
C
      PRINT 987,MOP(22),MOP(23),MOP(24)
  987 FORMAT(' ',
     X'  MOP(22) =',I2,' *** WRITE LGM PARAMETERS ONTO DISK',
     X' FILE *TRACE* AFTER EACH TIME STEP. ',/
     X'           = 0: NO DATA PRINTED ONTO *TRACE*. ',/
     X'           > 0: DATA PRINTED ONTO FILE *TRACE*. ',//
     X'   MOP(23) =',I2,' *** SCREEN DATA PRINTING. ',/
     X'           = 0: NO DATA PRINTED ON THE SCREEN. ',/
     X'           = 9: MORE PRINTOUT ',//
C     X'COORDINATES IN THE PLOT FILE. ',/
C     X'                 HIGHER VALUES OF MOP(23) GENERATE MORE ',
C     X'PRINTOUT (SEE SUBROUTINE PLOT LISTING).',//
     X'   MOP(24) =',I2,' *** LFM COMPUTATION OPTIONS. ',/
     X'           = 0: NO METHANE, CARBON DIOXIDE OR HEAT PRODUCTION', 
     X'(NO CALLS TO CCHPR).',/
     X'           = 4: LANDFILL GAS AND HEAT PRODUCTION (CCHPR).',//
     X' ',131('*'))
C     X'           > 1: SAME AS OPTION 1 PLUS THE INTERNAL GEN. ',
C     X'OF FILES XAMD AND XAMDF.',/
C     X'           > 2: (ESPEC. >8) MORE PRINTOUT.',//' ',131('*'))
C     *** END MODIFICATIONS ***
C
C-----SOLVE MASS- AND ENERGY-TRANSPORT EQUATIONS.
C IN LGMT2F.F: EXECUTIVE ROUTINE FOR MARCHING IN TIME-   MOST IMPORTANT
c################################################################# CYCIT
      IF(IS.EQ.0) CALL CYCIT
C     IF IS.NE.0 (KEYWORD "ENDFI" IN INPUT FILE, OR PROBLEM IN RFILE),
C     NO FLOW SIMULATION WILL BE PERFORMED.
C
  100 CONTINUE
      IF(IV.EQ.0) GOTO 15
      PRINT 20,h1
   20 FORMAT(A1/
     X' ',131('*')/' *',129X,'*'/' *',49X,'SUMMARY OF PROGRAM UNITS',
     X' USED',51X,'*'/' *',129X,'*'/' ',131('*')//
     X'       UNIT    VERSION     DATE                 COMMENTS'/
     X1X,131('_')/)
C
      ENDFILE 11
      REWIND 11
C
   16 CONTINUE
      DO17 I=1,26
   17 VV (I)='     '
C
      READ(11,14,END=15) (VV (I),I=1,26)
   14 FORMAT(26A5)
      PRINT 14,(VV(I),I=1,26)
      GOTO 16
C
   15 CONTINUE
      PRINT 19
   19 FORMAT(//1X,131('*'))
C      PRINT *,'8: main +++++++++++++++++++++++++++++++++CALL SECOND'
c      CALL SECOND(ELT)
C      ELT=ELT-TZERO
C      ELTC=ELT-ELT1
C      PRINT 12,ELT,ELTC,ELT1,h1
C     *** BEGINNING MODIFICATIONS ***
C   12 FORMAT(//' END OF TOUGH2 SIMULATION RUN --- ELAPSED TIME = ',
C     AE10.3,' SEC-- CALCULATION TIME = ',E10.3,' SEC-- DATA INPUT ',
C     B'TIME = ',E10.3,' SEC'/A1)
C     ***END MODIFICATIONS ***
      STOP
      END
c#######################################################################
      BLOCK DATA VV
      COMMON/V/IV,IS
      DATA IV/1/
      END
c#######################################################################
      SUBROUTINE IO
      implicit real*8 (a-h,o-z)
      COMMON/SVZ/NOITE,MOP(24)
      LOGICAL EX
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      PRINT 1
    1 FORMAT(//' SUMMARY OF DISK FILES'/)
C
      INQUIRE(FILE='VERS',EXIST=EX)
      IF(EX) GOTO 62
      PRINT 63
   63 FORMAT(' FILE *VERS*  DOES NOT EXIST -11- OPEN AS A NEW FILE')
      OPEN(11,FILE='VERS',STATUS='NEW')
      ENDFILE 11
      GOTO 70
C
   62 PRINT 64
   64 FORMAT(' FILE *VERS*      EXISTS -11- OPEN AS AN OLD FILE')
      OPEN(11,FILE='VERS',STATUS='OLD')
C
   70 CONTINUE
      REWIND 11
C
      IF(ICALL.EQ.1) WRITE(11,899)
C     *** BEGINNING MODIFICATIONS ***
  899 FORMAT(6X,'IO       1.0         MARCH     1995',6X,'OPEN',
     X' FILES VERS, MESH, INCON, GENER, SAVE, LINEQ,',
     X' TABLE, XAMD, XAMDF, PLOT, TRACE')
C      XAMD: remaining pyrite; XAMDF: 
C     *** END MODIFICATIONS ***
C
      INQUIRE(FILE='MESH',EXIST=EX)
      IF(EX) GOTO 2
      PRINT 3
    3 FORMAT(' FILE *MESH*  DOES NOT EXIST --4- OPEN AS A NEW FILE')
      OPEN(4,FILE='MESH',STATUS='NEW')
      GOTO 10
C
    2 PRINT 4
    4 FORMAT(' FILE *MESH*      EXISTS --4- OPEN AS AN OLD FILE')
      OPEN(4,FILE='MESH',STATUS='OLD')
C
   10 INQUIRE(FILE='INCON',EXIST=EX)
      IF(EX) GOTO 12
      PRINT 13
   13 FORMAT(' FILE *INCON* DOES NOT EXIST --1- OPEN AS A NEW FILE')
      OPEN(1,FILE='INCON',STATUS='NEW')
      ENDFILE 1
      GOTO 20
C
   12 PRINT 14
   14 FORMAT(' FILE *INCON*     EXISTS --1- OPEN AS AN OLD FILE')
      OPEN(1,FILE='INCON',STATUS='OLD')
C
   20 INQUIRE(FILE='GENER',EXIST=EX)
      IF(EX) GOTO 22
      PRINT 23
   23 FORMAT(' FILE *GENER* DOES NOT EXIST --3- OPEN AS A NEW FILE')
      OPEN(3,FILE='GENER',STATUS='NEW')
      ENDFILE 3
      GOTO 30
C
   22 PRINT 24
   24 FORMAT(' FILE *GENER*     EXISTS --3- OPEN AS AN OLD FILE')
      OPEN(3,FILE='GENER',STATUS='OLD')
C
   30 INQUIRE(FILE='SAVE',EXIST=EX)
      IF(EX) GOTO 32
      PRINT 33
   33 FORMAT(' FILE *SAVE*  DOES NOT EXIST --7- OPEN AS A NEW FILE')
      OPEN(7,FILE='SAVE',STATUS='NEW')
      GOTO 40
C
   32 PRINT 34
   34 FORMAT(' FILE *SAVE*      EXISTS --7- OPEN AS AN OLD FILE')
      OPEN(7,FILE='SAVE',STATUS='OLD')
C
   40 INQUIRE(FILE='LINEQ',EXIST=EX)
      IF(EX) GOTO 42
      PRINT 43
   43 FORMAT(' FILE *LINEQ* DOES NOT EXIST -15- OPEN AS A NEW FILE')
      OPEN(15,FILE='LINEQ',STATUS='NEW')
      GOTO 50
C
   42 PRINT 44
   44 FORMAT(' FILE *LINEQ*     EXISTS -15- OPEN AS AN OLD FILE')
      OPEN(15,FILE='LINEQ',STATUS='OLD')
      REWIND 15
C
   50 CONTINUE
      INQUIRE(FILE='TABLE',EXIST=EX)
      IF(EX) GOTO 52
      PRINT 53
   53 FORMAT(' FILE *TABLE* DOES NOT EXIST --8- OPEN AS A NEW FILE')
      OPEN(8,FILE='TABLE',STATUS='NEW')
      ENDFILE 8
      GOTO 60
C
   52 PRINT 54
   54 FORMAT(' FILE *TABLE*     EXISTS --8- OPEN AS AN OLD FILE')
      OPEN(8,FILE='TABLE',STATUS='OLD')
C
      INQUIRE(FILE='TRACE',EXIST=EX)
      IF(EX) GOTO 92
      PRINT 93
   93 FORMAT(' FILE *TRACE* DOES NOT EXIST -55- OPEN AS A NEW FILE')
      OPEN(55,FILE='TRACE',STATUS='NEW')
      ENDFILE 55
      GOTO 90
   92 PRINT 94
   94 FORMAT(' FILE *TRACE*     EXISTS -55- OPEN AS AN OLD FILE')
      OPEN(55,FILE='TRACE',STATUS='OLD')
   90 CONTINUE
      REWIND 55
C
C                                                  *** BEG MODIFICATIONS ***
   60 CONTINUE 
      INQUIRE(FILE='cont.dat',EXIST=EX)
      IF(EX) GOTO 262
      PRINT 263
  263 FORMAT(' FILE *cont.dat*   NOT EXIST -20- OPEN AS A NEW FILE')
c     OPEN(20,FILE='cont.dat',STATUS='NEW',form='unformatted')
      OPEN(20,FILE='cont.dat',STATUS='NEW')
      ENDFILE 20
      GOTO 260
  262 PRINT 264
  264 FORMAT(' FILE *cont.dat*  EXISTS -20- OPEN AS AN OLD FILE')
      OPEN(20,FILE='cont.dat',STATUS='OLD')
  260 CONTINUE
      REWIND 20
C
      INQUIRE(FILE='flux.dat',EXIST=EX)
      IF(EX) GOTO 322
      PRINT 323
  323 FORMAT(' FILE *flux.dat*   NOT EXIST -30- OPEN AS A NEW FILE')
      OPEN(30,FILE='flux.dat',STATUS='NEW')
      ENDFILE 30
      GOTO 320
  322 PRINT 324
  324 FORMAT(' FILE *flux.dat*  EXISTS -30- OPEN AS AN OLD FILE')
      OPEN(30,FILE='flux.dat',STATUS='OLD')
  320 CONTINUE
      REWIND 30
C     *** END MODIFICATIONS ***
      RETURN
      END
c############################################################################
      SUBROUTINE FLOP
      implicit real*8 (a-h,o-z)
C-----CALCULATE NUMBER OF SIGNIFICANT DIGITS FOR FLOATING POINT
C     PROCESSING. ASSIGN DEFAULT FOR DFAC, AND PRINT APPROPRIATE
C     WARNING WHEN MACHINE ACCURACY IS INSUFFICIENT.
C
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'FLOP     1.0      11 APRIL     1991',6X,
     X'CALCULATE NUMBER OF SIGNIFICANT DIGITS FOR FLOATING POINT',
     X' ARITHMETIC')
C
      A=SQRT(.99)
      B=A
      DO1 N=1,260
      B=B/2.
      C=A+B
      D=C-A
      IF(D.EQ.0.) GOTO 2
    1 CONTINUE
C
    2 B2=B*2.
      N10=-INT(LOG10(B2))
      DF=SQRT(B2)
C
      PRINT 3,N10,DF
    3 FORMAT(/24X,84('*')/24X,'*',23X,'EVALUATE FLOATING POINT',
     X' ARITHMETIC',25X,'*'/24X,84('*')/24X,'*',82X,'*'/24X,'*',
     X' FLOATING POINT PROCESSOR HAS APPROXIMATELY',I3,
     X' SIGNIFICANT DIGITS',17X,'*'/24X,'*',82X,'*'/24X,'*',
     X' DEFAULT VALUE OF INCREMENT FACTOR FOR NUMERICAL DERIVATIVES',
     X' IS DFAC = ',E10.4,' *')
C
      IF(DFAC.EQ.0.) THEN
      DFAC=DF
      PRINT 10
   10 FORMAT(24X,'* DEFAULT VALUE FOR DFAC WILL BE USED',46X,'*')
      ELSE
      PRINT 11,DFAC
   11 FORMAT(24X,'*',
     X' USER-SPECIFIED VALUE DFAC = ',E10.4,' WILL BE USED',30X,'*')
      ENDIF
      PRINT 6
    6 FORMAT(24X,'*',82X,'*'/24X,84('*')/)
C
      IF(N10.LE.12.AND.N10.GT.8) PRINT 4
    4 FORMAT(' WWWWWWWWWW WARNING WWWWWWWWWW: NUMBER OF SIGNIFICANT',
     X' DIGITS IS MARGINAL; EXPECT DETERIORATED CONVERGENCE BEHAVIOR'/)
      IF(N10.LE.8) PRINT 5
    5 FORMAT(' WWWWWWWWWW WARNING WWWWWWWWWW: NUMBER OF SIGNIFICANT',
     X' DIGITS IS INSUFFICIENT; CONVERGENCE WILL BE POOR OR FAIL'/
     X' WWWWWWWWWWWWWWWWWWWWWWWWWWWWW: CODE SHOULD BE RUN IN DOUBLE'
     X,' PRECISION!'/)
C
      RETURN
      END
C#############################################################################
C     THIS SUBROUTINE IS ADAPTED TO THE SUN FORTRAN:
C out of use by Rene
      SUBROUTINE SECOND(SEC)
      implicit real*8 (a-h,o-z)
      INTEGER TIME
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'SECOND   1.1      17 AUGUST    1993',6X,
     X'SUN SUBROUTINE FOR CPU TIMING FUNCTION')
      SEC=FLOAT(TIME())
C     THIS SUBROUTINE HAS TO BE ADAPTED TO THE TIMING
C     FUNCTIONS AVAILABLE ON THE HARDWARE USED.
C     MAKE IT A DUMMY SUBROUTINE BY CHANGING THE ABOVE
C     LINE TO THE FOLLOWING LINE :
C     SEC=0.
      RETURN
      END
