      SUBROUTINE INPUT
      implicit real*8 (a-h,o-z)
C     **********************************************************
c: verification
C     .   this is a preprocessor reads and writes data
C     .
C     .   TOUGH_LGM, MODULE LGMT2F, VERSION 1.0, MARCH 1995 
C     .
C     .   FROM TOUGH2,  MODULE T2F, VERSION 1.0, MARCH 1991 
C     **********************************************************
C
C     **********************************************************
C       T2F.F  WAS  MODIFIED  FOR LANDFILL GAS MIGRATION - LGM
C       MODIFIED BY MIRO NASTEV, LAVAL UNIVERSITY, 1995, QUEBEC.
C     **********************************************************
C
C     THE MAIN CHANGES FROM T2F ARE:
C
C     * FOUR COMPONENTS ARE CONSIDERED:
C           a)WATER    b)N2-AIR     c)METHANE  d)CARBON DIOXIDE
C     * their mass fractions are stored in secondary variables: 
C           PAR(NB+1)  PAR(NB+2)    PAR(NB+3)  PAR(NB+4)
C
C     * A NEW  LGM MODULE ADDED TO COMPUTE METHANE AND CARBON 
C       DIOXIDE PRODUCTION FROM ORGANIC MATTER BIODEGRADATION
C       THE CONTRIBUTION OF THE  MASS SINK AND HEAT SOURCE ON 
C       THE LINEAR EQUATION TERMS IS COMPUTED.
C
C     * THE AND OUTPUT MODULES ARE MODIFIED TO TAKE INTO
C       ACCOUNT THE NEW DISK FILES AND ARRAYS REQUIRED FOR LGM
C       MODELLING.
C
C
C     NOTE: MODIFIED SECTIONS INDICATED BY  
C
C     *** BEGINNING MODIFICATIONS ***
C     MODIFIED SECTION ...
C     *** END MODIFICATIONS ***
C
C
C-----READS ALL DATA PROVIDED THROUGH THE INPUT FILE &
C-----WRITES COMPLET DATA FILES (this is preprocessor)
C
C****************** COMMON BLOCKS FOR CONNECTIONS *************************
C NEX1: indices first element in link; NEX2: indices second element in link
C DEL1: first element nodal distance;  DEL2: second  element nodal distance
C AREA: interface areas; BETA: cosine of angle betwee nodal line & vertical
C ISOX: isotropy; GLO: heat flow rates; ELEM1: code name first elem in link
C ELEM2: code name of second element in a link. ELEM is a text
      COMMON/C1/NEX1(44000)
      COMMON/C2/NEX2(44000)
      COMMON/C3/DEL1(44000)
      COMMON/C4/DEL2(44000)
      COMMON/C5/AREA(44000)
      COMMON/C6/BETA(44000)
      COMMON/C7/ISOX(44000)
      COMMON/C8/GLO(44000)
      COMMON/C9/ELEM1(44000)
      COMMON/C10/ELEM2(44000)
C**************************************************************************
C NEL=No of volume elem.; NCON=No of connections; NPH=No of phases present
C NK=4, No of fluid components H2O, N2, CH4 & CO2; NK1=NK+1=5
C NEQ= No of ballance eqs. per volume element. Usually NEQ=NK+1=5, 4mass
C comp. and 1energy; {However, if small temp. changes consider just mass
C ballance equation, e.g., NEQ=NK=4, no energy eqs. are solved}
C NEQ1=NEQ+1=6(5); NFLUX=2*NEQ+1=11; NBK=NB+NK=6+4=10
C NB=No of secondary param. other than mass=6; NOGN=No of generation times
C NSEC=No of sec. param. per volume element NPH*NBK+2=22 
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
C
C******** COMMON BLOCKS FOR SINKS/SOURCES *********************************
C used as injection or production wells. NEXG: number of wells (s/s)
C Tables: F1-generation times; F2-generation rates; F3-inj. enthalpies
      COMMON/G1/F1(25000)
      COMMON/G2/F2(25000)
      COMMON/G3/F3(25000)
      COMMON/G9/NEXG(10000)
C
C**************************************************************************
C
      COMMON/SVZ/NOITE,MOP(24)
C      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)                     JPF 03 2005
      COMMON/SOLID/DM(27),POR(27),PER(3,27),CWET(27),SH(27),NM
      COMMON/SOCH/MAT(27)
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/SOIN/YIN(4,27)
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
      COMMON/STEP/ELST
      COMMON/STE1/NST
      COMMON/DEFINI/DEP(10)
      COMMON/DG/WUP,WNR
      COMMON/DFM/TIMAX,REDLT
c      COMMON/DLT/NDLT,DLT(100)                                            JPF 03 2005
      COMMON/DLT/DLT(100),NDLT
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
C      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1                  JPF 03 2005
      COMMON/DMN/TZERO,TIMP1,INUM,IPRINT,MCYC,MCYPR,MSEC
C      COMMON/DOP/ENTH,KDATA,QUAL                                          JPF 03 2005
      COMMON/DOP/QUAL,ENTH,KDATA
      COMMON/DX/K,NI,SCALE
      COMMON/POV6/TSTART
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                    JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/PATCH/SING
C      COMMON/TIMES/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX              JPF 03 2005
      COMMON/TIMES/DELAF,ITI,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
C**************************************************************************
C-----NOW FOR A COMMON BLOCK WITH EOS-SELECTION PARAMETERS.
      COMMON/EOSEL/IE(16),FE(512)
C      COMMON/RPCAP/IRP(27),RP(7,27),ICP(27),CP(7,27),IRPD,RPD(7),            JPF 03 2005
C     AICPD,CPD(7)
      COMMON/RPCAP/RP(7,27),RPD(7),IRP(27),ICP(27),CP(7,27),IRPD,
     XICPD,CPD(7)
      COMMON/BIND/DIFF0,TEXP,BE
      COMMON/V/IV,IS
c
      CHARACTER*5 ELEM1,ELEM2,MAT,ELST,VER,WORD,MA12,NAM
      CHARACTER*3 EL,MA1,EL1,EL2,SL
      CHARACTER MA2*2,TYPE*4,ITAB*1
C
      DIMENSION VER(17),XIN(4,27),NAM(27)
C
      SAVE ICALL
      DATA ICALL/0/
      DATA VER /'PARAM','ELEME','CONNE','GENER','INCON','START','RESTA',
     A'ENDCY','ENDFI','ROCKS','MULTI','TIMES','SELEC','RPCAP','MESHM',
     X'INDOM','NOVER'/
C
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(/6X,'INPUT    1.0         MARCH     1995',6X,
     X'READ ALL DATA PROVIDED THROUGH FILE *INPUT*')
C
      NI=0
C     NI = 1 FOR START; NI = 0 OTHERWISE.
C
      DELTMX=0.
      ITI=0
C-----ITI  IS THE FLAG FOR THE *TIMES* DATA BLOCK.  IF THIS BLOCK IS
C     PRESENT, ITI WILL BE SET EQUAL TO NUMBER OF TIME DATA PROVIDED.
      KIN=0
C-----KIN IS THE FLAG FOR THE  *INDOM* DATA BLOCK FOR DOMAIN-SPECIFIC
C     INITIALIZATION. IF THIS BLOCK IS PRESENT, KIN WILL BE SET EQUAL
C     TO THE NUMBER OF DOMAINS IN IT.
      IS=0
C     WHEN KEYWORD  "ENDFI"  IS  ENCOUNTERED IN THE INPUT FILE,
C     RESET  IS=IS+1 ;  THIS WILL IMMEDIATELY RETURN CONTROL TO
C     MAIN PROGRAM TOUGH2, AND FLOW SIMULATION WILL BE BYPASSED
C
C     ASSIGN DEFAULTS FOR RELATIVE PERMEABILITY & CAPILLARY PRESSURE.
C     DEFAULTS ARE KREL=1, PCAP=0, APPROPRIATE FOR SINGLE PHASE FLOW
      IRPD=5
      ICPD=1
      CPD(1)=0.
      CPD(2)=0.
      CPD(3)=1.
Cyes***********************************************************************
C*****************************************************************  'RESTA'
C**************************************************************************
 5019 READ 5020,WORD
 5020 FORMAT(A5)
C
      DO900 K=1,17
  900 IF(WORD.EQ.VER(K)) GOTO920
      PRINT 901,WORD
  901 FORMAT(' HAVE READ UNKNOWN BLOCK LABEL "',A5,
     X'" --- IGNORE THIS, AND CONTINUE 
     
     ING INPUT DATA')
      GOTO 5019
C
  920 GOTO(1000,1100,1200,1300,1900,1450,5019,1500,1600,1700,1800,
     A2000,2100,2200,2300,2400,2500),K
Cyes***********************************************************************
C******************* READ ROCK PROPERTIES **************************'ROCKS'
C**************************************************************************
C MAT: material name; NAD: flag 0,1,2; DM: mat. density; POR: mat. porosity
C PER: abs. permeabi in principal axis X,Y,Z; CWET: saturated heat conduct.
C SH: rock grain specific heat; NM: No materials (internal counter)
 1700 NM=1
    2 READ 1,MAT(NM),NAD,DM(NM),POR(NM),(PER(I,NM),I=1,3),
     ACWET(NM),SH(NM)
    1 FORMAT(A5,I5,7E10.4)
C ako vece nemame nov materijal go to 3 (end of rock properties)
      IF(MAT(NM).EQ.'     ') GOTO 3
C
      YIN(1,NM)=0.
      COM(NM)=0.
      EXPAN(NM)=0.
      CDRY(NM)=0.
      TORT(NM)=0.
      GK(NM)=0.
      IRP(NM)=0
      ICP(NM)=0
C                                                      *** BEG MODIFICATIONS
C COM: compressibility of porous medium; EXPAN: expansivity of por.med.;  
C CDRY: heat condu. (desaturated); TORT:tortuosity ; GK: KLINKENBERG  
C
      IF(NAD.GE.1) READ 7,COM(NM),EXPAN(NM),CDRY(NM),TORT(NM),GK(NM)
C                                                      *** END MODIFICATIONS
    7 FORMAT(8E10.4)
C if CDRY (desat. heat cond.=0) by default CDRY=CWET (saturated heat cond.)
      IF(CDRY(NM).EQ.0.) CDRY(NM)=CWET(NM)
      IF(NAD.LE.1) GOTO 1701
C-----READ PARAMETERS FOR RELATIVE PERMEABILITY AND CAPILLARY PRESSURE.
C IRP: No of rel. perm. funct.; RP(I): param. of chosen rel.perm. funct.
C CP:  No of cap. pres. funct.; CP(I): param. of chosen capillary funct.
      READ 1702,IRP(NM),(RP(I,NM),I=1,7)
      READ 1702,ICP(NM),(CP(I,NM),I=1,7)
 1702 FORMAT(I5,5X,7E10.4)
 1701 CONTINUE
C
      NAD=0
C
      PRINT 5,NM,MAT(NM)
    5 FORMAT(/11H DOMAIN NO.,I3,22H     MATERIAL NAME -- ,A5)
      NM=NM+1
      GOTO2
C
    3 CONTINUE
      NM=NM-1
      GOTO5019
C-----END OF ROCK PROPERTIES.----------------------------------------------
C
Cno************************************************************************
C******* READ REL PERMEABILITY AND CAPILLARY PRES ****************  'RPCAP'
C**************************************************************************
 2200 CONTINUE
C-----COME HERE TO READ DEFAULT ASSIGNMENTS FOR RELATIVE PERMEABILITY
C     AND CAPILLARY PRESSURE PARAMETERS.
      READ 1702,IRPD,(RPD(I),I=1,7)
      READ 1702,ICPD,(CPD(I),I=1,7)
      GOTO 5019
C
Cno************************************************************************
C****************************************************************** 'MESHM'
C**************************************************************************
 2300 CONTINUE
C-----COME HERE TO CALL INTERNAL MESH GENERATION MODULE.
      CALL MESHM
      GOTO 5019
C
C**************************************************************************
C******* SUPPLEMENT INCON WITH DEFAULTS***************************  'START'
C**************************************************************************
 1450 CONTINUE
      NI=1
      PRINT *,'ROCKS'
      GOTO5019
C**************************************************************************
C*******  READ COMPUTATION PARAMETERS.****************************  'PARAM'
C**************************************************************************
C NOITE: N0 of iterat=8;  KDATA: print flag 0,1,2,3=2; MCYC: max No of time
C steps to be calc.=9999; MSEC: max duration of simul=0. (default.infinite)
C MCYPR: printout=9999; MOP: ;  DIFF0: strength parameter for diffusive
C vapor flux; TEXP: temperature dependence of binary diffusion
 1000 CONTINUE
C                                                     *** BEG MODIFICATIONS
      READ 200,NOITE,KDATA,MCYC,MSEC,MCYPR,(MOP(I),I=1,24),
     XDIFF0,TEXP,BE
C                                                     *** END MODIFICATIONS
  200 FORMAT(2I2,3I4,24I1,4E10.4)
      MCYC=999999
C
C TSTART: start time of simul.=0.; TIMAX: finish time in.sec=315576000.=
C 365.25 days=1 year; DELTEN: time steps length in seconds=86400=24h=1day; 
C DELTMX: upper limit  for DELTEN=0.; ELST:=0.; GF: gravity.force=9.81m/s2;
C FOR CONVERGENCY: REDLT: reduction of time step=2(default 4)
C SCALE: factor to change the size of the mesh=0.(default=1.)
      READ 8,TSTART,TIMAX,DELTEN,DELTMX,ELST,GF,REDLT,SCALE
    8 FORMAT(4E10.4,A5,5X,3E10.4)
      NDLT=0
      IF(DELTEN.GE.0.) GOTO4
      NDLT=-INT(DELTEN)
      DO6 N=1,NDLT
    6 READ 7,(DLT(J+8*(N-1)),J=1,8)
      DELTEN=DLT(1)
    4 CONTINUE
C
C RE1: convergence criterion for relative error (default =1.E-5); 
C RE2: convergence criterion for absolute error (default =1);
C U: pivoting parameter (=0.1); WUP: weighting factor for mobilities. and
C enthalpies=1.0; WNR: weighting factor for increments in Newton/Raphson 
C iteration =1.0; DFAC: increment =1.E-8; FOR:       
      READ 7,RE1,RE2,U,WUP,WNR,DFAC,FOR
C
C-----ASSIGN DEFAULT VALUES.
C
      IF(NOITE.EQ.0) NOITE=8
      IF(KDATA.EQ.0) KDATA=1
      IF(MCYPR.EQ.0) MCYPR=1
      IF(TEXP.EQ.0.) TEXP=1.80
      IF(REDLT.EQ.0.) REDLT=4.
      IF(SCALE.EQ.0.) SCALE=1.
      IF(RE1.EQ.0.) RE1=1.E-5
      IF(RE2.EQ.0.) RE2=1.
      IF(FOR.EQ.0.) FOR=1.
      IF(WUP.LE.0.) WUP=1.
      IF(WNR.EQ.0.) WNR=1.
C     IF(DFAC.EQ.0.) DFAC=1.E-8
      IF(U.EQ.0.) U=0.1
C
C-----INITIALIZE SOME PARAMETERS.
C
      KCYC=0
      ITER=0
      ITERC=0
      TIMIN=TSTART
C reads PRIMARY VARIABLES allready written in INCON; DEP(1):PRESSURE 
C =101325.2Pa; DEP(2):GAS SATUR. IN TWO PHASE S+10=10.58 (Sg=58%) or 
C Sg=0.0000159 gas saturation=0 in water saturated zone (one phase);
C DEP(3): CH4 MASS FRACTION IN GAS PHASE XCO4=0.5 (50%); DEP(4): CO2 
C MASS FRACTION IN GAS PHASE XCO2=0.5 ;   DEP(5): TEMPERATURE T=20oC
C     *** BEGINNING MODIFICATIONS ***
      READ 1908,DEP(1),DEP(2),DEP(3),DEP(4),DEP(5)
C     *** END MODIFICATIONS ***
C-----END OF COMPUTATION PARAMETERS.-----------------------------------
C
      GOTO5019
C**************************************************************************
C******* READ ELEMENT DATA. **************************************  'ELEME'
C**************************************************************************
 1100 CONTINUE
      PRINT 41
      REWIND 4
      PRINT 1103
 1103 FORMAT(' WRITE FILE *MESH* FROM INPUT DATA')
      WRITE(4,1101)
 1101 FORMAT(5HELEME)
C EL-NE: code name of an element; NADD : increment between code numbers
C NSEQ: additional elements having same characteristics(same reservoir)
C MA1-MA2:identifier of the reservoir domain; VOLX: element volume (m3)
C AHTX: area of the interface with the confining beds for heat exchange
C X,Y,Z: coordinates. Coordinate 0.,0.& 0. is at the bottom left corner
C TDEP: time from deposition of waste el. to the start of simulation
 1102 READ 10,EL,NE,NSEQ,NADD,MA1,MA2,VOLX,AHTX,X,Y,Z,TDEP
   10 FORMAT(A3,I2,2I5,A3,A2,2E10.4,10X,4E10.4)
      IF(EL.EQ.'   '.AND.NE.EQ.0) GOTO40
C
      NSEQ1=NSEQ+1
C
      IF(MA1.EQ.'   ') GOTO15
C
C-----FIND MATERIAL INDEX.
      MA12=MA1//MA2
      DO11 M=1,NM
      IF(MA12.EQ.MAT(M)) GOTO 12
   11 CONTINUE
      PRINT 9,MA1,MA2,EL,NE
    9 FORMAT(31H REFERENCE TO UNKNOWN MATERIAL ,A3,A2,12H AT ELEMENT ,A2
     A,I3,24H --- WILL IGNORE ELEMENT)
      GOTO1102
C
   12 CONTINUE
C
C-----COME HERE FOR ELEMENTS WITH MATERIAL NAME.
C
      DO13 I=1,NSEQ1
      N=NE+(I-1)*NADD
   13 WRITE(4,14) EL,N,M,VOLX,AHTX,X,Y,Z,TDEP
   14 FORMAT(A3,I2,10X,I5,2E10.4,10X,4E10.4)
      GOTO1102
C
C-----COME HERE FOR ELEMENTS WITH MATERIAL NUMBER.
C
   15 CONTINUE
      DO113 I=1,NSEQ1
      N=NE+(I-1)*NADD
  113 WRITE(4,114) EL,N,MA2,VOLX,AHTX,X,Y,Z,TDEP
  114 FORMAT(A3,I2,13X,A2,2E10.4,10X,4E10.4)
      GOTO1102
C-----END OF ELEMENT DATA.----------------------------------------------
C
   40 WRITE(4,41)
   41 FORMAT(5H     )
      GOTO5019
C**************************************************************************
C****** READ CONNECTION DATA. ***********************************  'CONNE '
C**************************************************************************
 1200 CONTINUE
      WRITE(4,1201)
 1201 FORMAT(5HCONNE)
      NCON=0
C ELNE1-ELNE2 elements in the connection; NSEQ: No of addit. connections
C NAD1: increment of code number for the first element in the link; NAD2
C ISOT: 1,2 or 3, which of the three given permeab. will be used in link
C D1: distance from mass center to the interface; AREAX: contact area
c BETA: angle that the link makes with the vertical axis
 1202 READ 20,EL1,NE1,EL2,NE2,NSEQ,NAD1,NAD2,ISOT,D1,D2,AREAX,BETAX
   20 FORMAT(A3,I2,A3,I2,4I5,4E10.4)
      IF(EL1.EQ.'   '.AND.NE1.EQ.0) GOTO50
      IF(EL1.EQ.'+++') GOTO51
C
      NSEQ1=NSEQ+1
C
      DO23 I=1,NSEQ1
      NCON=NCON+1
      N1=NE1+(I-1)*NAD1
      N2=NE2+(I-1)*NAD2
   23 WRITE(4,24) EL1,N1,EL2,N2,ISOT,D1,D2,AREAX,BETAX
   24 FORMAT(A3,I2,A3,I2,15X,I5,4E10.4)
      GOTO1202
C-----END OF CONNECTION DATA.-------------------------------------------
C
   50 WRITE(4,41)
      GOTO5019
   51 WRITE(4,42)
   42 FORMAT(5H+++  )
      READ 1505,(NEX1(N),NEX2(N),N=1,NCON)
      WRITE(4,1505) (NEX1(N),NEX2(N),N=1,NCON)
 1505 FORMAT(20I4)
      GOTO5019
C**************************************************************************
C****** READ SINK/SOURCE DATA. **********************************  'GENER '
C**************************************************************************
 1300 CONTINUE
      PRINT 41
      REWIND 3
      PRINT 1302
 1302 FORMAT(' WRITE FILE *GENER* FROM INPUT DATA')
      WRITE(3,1301)
 1301 FORMAT(5HGENER)
      NOGN=0
   35 READ 30,EL,NE,SL,NS,NSEQ,NADD,NADS,LTAB,TYPE,ITAB,GX,EX,HX
   30 FORMAT(A3,I2,A3,I2,4I5,5X,A4,A1,3E10.4)
C if '   ' : first run;           if '+++' : consecutive run 
      IF(EL.EQ.'   '.AND.NE.EQ.0) GOTO60
      IF(EL.EQ.'+++') GOTO61
      NSEQ1=NSEQ+1
      IF(LTAB.EQ.0) LTAB=1
C
      DO 33 I=1,NSEQ1
        NOGN=NOGN+1
        N1=NE+(I-1)*NADD
        N2=NS+(I-1)*NADS
        WRITE(3,34) EL,N1,SL,N2,LTAB,TYPE,ITAB,GX,EX,HX
        print 34,EL,N1,SL,N2,LTAB,TYPE,ITAB,GX,EX,HX
   34   FORMAT(A3,I2,A3,I2,15X,I5,5X,A4,A1,3G10.4)
        LTABA=IABS(LTAB)
        IF (LTABA.LE.1.OR.TYPE.EQ.'DELV') GO TO 33
        IF (I.GE.2) GO TO 133
C-----COME HERE TO READ TABLES OF TIMES AND RATES.
        READ 36,(F1(L),L=1,LTABA)
        READ 36,(F2(L),L=1,LTABA)
        IF (ITAB.NE.' ') READ 36,(F3(L),L=1,LTABA)
  337   FORMAT(4E14.7)
   36   FORMAT(10F10.0)
  133   CONTINUE
        WRITE(3,337) (F1(L),L=1,LTABA)
        WRITE(3,337) (F2(L),L=1,LTABA)
        IF (ITAB.NE.' ') WRITE(3,337) (F3(L),L=1,LTABA)
   33   CONTINUE
      GO TO 35
C-----END OF SINK/SOURCE DATA.-----------------------------------------
C
   60 WRITE(3,41)
      print *,'GENER ZAVRSI '
      GOTO5019
   61 WRITE(3,42)
      READ 1505,(NEXG(N),N=1,NOGN)
      WRITE(3,1505) (NEXG(N),N=1,NOGN)
      GOTO5019
C
C**************************************************************************
C****** READ MULTICOMPONENT PARAMETERS ***************************  'MULTI'
C**************************************************************************
 1800 CONTINUE
      READ 1801,NK,NEQ,NPH,NB
 1801 FORMAT(16I5)
C
      GOTO 5019
C-----END OF MULTICOMPONENT PARAMETERS----------------------------------
C
C*************************************************************************
C***** READ MULTI-COMPONENT INITIAL CONDITIONS *******************  'INCON'
C**************************************************************************
 1900 CONTINUE
      PRINT 41
      REWIND 1
      PRINT 1909
 1909 FORMAT(' WRITE FILE *INCON* FROM INPUT DATA')
      WRITE(1,1901)
 1901 FORMAT(5HINCON)
C
 1906 READ 1902,EL,NE,NSEQ,NADD,PORX
 1902 FORMAT(A3,I2,2I5,E15.8,5G10.4)
C
      IF(EL.EQ.'   '.AND.NE.EQ.0) GOTO 1903
      IF(EL.EQ.'+++') GOTO 1904
C READ PRIMARY VARIABLES P=X1 press; X=X2 gas saturation/air mass fraction 
C T=X5 temp.; XCH4=X3 frac of CH4 in gas phase; XCO2=X4 frac of CO2 in gas 
C                                                     *** BEG MODIFICATIONS
      READ 1908,X1,X2,X3,X4,X5
 1908 FORMAT(5E20.13)
C                                                     *** END MODIFICATIONS
C writes the other elements with ident.initial conditions (there are NSEQ)
      NSEQ1=NSEQ+1
      DO 1905 I=1,NSEQ1
      N1=NE+(I-1)*NADD
      WRITE(1,1907) EL,N1,PORX
 1907 FORMAT(A3,I2,10X,E15.8,5E10.4)
C                                                     *** BEG MODIFICATIONS
      WRITE(1,1908) X1,X2,X3,X4,X5
C                                                     *** END MODIFICATIONS
 1905 CONTINUE
C
      GOTO 1906
 1904 WRITE(1,42)
C ovde samo ako e posledovatelen run +++
      READ 74,KCYCX,ITERCX,NMX,TSTX,TIMINX
   74 FORMAT(3I5,2E15.8)
      WRITE(1,74) KCYCX,ITERCX,NMX,TSTX,TIMINX
C
      GOTO 5019
C
 1903 WRITE(1,1403)
 1403 FORMAT(5H     )
      GOTO 5019
C-----END OF MULTICOMPONENT INITIAL CONDITIONS--------------------------
C
Cno************************************************************************
C***** READ DOMAIN-SPECIFIC INITIAL CONDITIONS *******************  'INDOM'
C**************************************************************************
 2400 CONTINUE
      READ 1,NAM(KIN+1)
      IF(NAM(KIN+1).EQ.'     ') GOTO 5019
      KIN=KIN+1
      READ 1908,(XIN(I,KIN),I=1,4)
      GOTO 2400
C-----END OF DOMAIN-SPECIFIC INITIAL CONDITIONS------------------------
C
 2500 CONTINUE
Cno************************************************************************
C*****SET FLAG FOR SUPPRESSING VERSION-PRINTOUT AT END OF RUN.***** 'NOVER'
C**************************************************************************
      IV=0
      GOTO 5019
C
Cyes***********************************************************************
C****** READ DATA BLOCK WITH TIMES. ******************************* 'TIMES'
C**************************************************************************
 2000 CONTINUE
      READ 2001,ITI,ITE,DELAF,TINTER
C
C     ITI       IS THE NUMBER OF TIMES PROVIDED.
C     ITE       IS THE TOTAL NUMBER OF TIMES DESIRED.
C     DELAF     IS THE MAXIMUM TIME STEP TO BE TAKEN AFTER ANY OF THE
C               PRESCRIBED TIMES HAVE BEEN REACHED.
C     TINTER    IS THE TIME INCREMENT TO BE APPLIED FOR THE TIME VALUES
C               WITH INDEX ITI+1,ITI+2, ... ,ITE
C
 2001 FORMAT(2I5,2E10.4)
      READ 2002,(TIS(I),I=1,ITI)
 2002 FORMAT(10F10.0)
      IF(ITE.LE.ITI.OR.TINTER.LE.0.) GOTO 5019
C
C-----COME HERE TO ASSIGN INCREMENTED TIMES.
      ITI1=ITI+1
      DO2003 I=ITI1,ITE
 2003 TIS(I)=TIS(I-1)+TINTER
      ITI=ITE
C
      GOTO 5019
C-----END OF TIMES BLOCK------------------------------------------------
C
Cno************************************************************************
C***** READ DATA BLOCK WITH SELECTION PARAMETERS *****************  'SELEC'
C**************************************************************************
 2100 CONTINUE
      READ 1801,(IE(I),I=1,16)
      IE1=1
      IF(IE(1).GT.1) IE1=IE(1)
      IF(IE1.GT.64) PRINT 2101,IE1
 2101 FORMAT(' WWWWW WARNING WWWWW:',
     X' IE(1) =',I5,' IN BLOCK *SELEC* IS TOO LARGE; CAN READ AT MOST',
     X' 64 RECORDS'/)
      IE18=MIN(IE1*8,512)
      READ 2002,(FE(I),I=1,IE18)
      GOTO 5019
C-----END OF SELECTION PARAMETERS---------------------------------------
C
Cno************************************************************************
C*****************************************************************  'ENDFI'
C**************************************************************************
 1600 PRINT 1601
 1601 FORMAT(/' HAVE READ KEYWORD *ENDFI* IN FILE INPUT ---',
     X' BYPASS FLOW SIMULATION   ',62('*')/
     X' OPTIONAL  PRINTOUT OF INPUT DATA (FOR MOP(7).NE.0) IS',
     X' AVAILABLE')
      IS=IS+1
Cyes***********************************************************************
C***** CLOSE TOUGH DATA DECK AND INITIATE SIMULATION *************  'ENDCY'
C**************************************************************************
 1500 CONTINUE
      NEQ1=NEQ+1
      NK1=NK+1
      NFLUX=2*NEQ+1
      NBK=NB+NK
      NSEC=NPH*NBK+2
C NK=4; NK1=5; NEQ=5; NEQ1=6; NFLUX=11; NBK=10; NSEC=22; NPH=2; NB=6; NEQ1=6
      IF(KIN.EQ.0) GOTO 2410
C***** COME HERE TO PROCESS DOMAIN-SPECIFIC INITIAL CONDITIONS *******
      DO 2401 J=1,KIN
      DO 2402 I=1,NM
      IF(MAT(I).EQ.NAM(J)) GOTO 2403
 2402 CONTINUE
      GOTO 2401
 2403 DO 2404 L=1,NK1
 2404 YIN(L,I)=XIN(L,J)
 2401 CONTINUE
C
 2410 CONTINUE
C-----END OF DOMAIN-SPECIFIC INITIAL CONDITIONS ----------------------
C
      RETURN
C
      END
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx verified
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 1xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE RFILE
      implicit real*8 (a-h,o-z)
C
C-----READ INPUT DATA FROM DISK FILES, WHICH ARE EITHER PROVIDED
C     BY THE USER, OR ARE INTERNALLY GENERATED IN SUBROUTINE
C     INPUT (preprocessor) FROM DATA GIVEN IN THE JOB DECK.
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/C1/NEX1(44000)
      COMMON/C2/NEX2(44000)
      COMMON/C3/DEL1(44000)
      COMMON/C4/DEL2(44000)
      COMMON/C5/AREA(44000)
      COMMON/C6/BETA(44000)
      COMMON/C7/ISOX(44000)
      COMMON/C8/GLO(44000)
      COMMON/C9/ELEM1(44000)
      COMMON/C10/ELEM2(44000)
C                                                 *** BEG MODIFICATIONS
      common/C11/fvd(44000)
      common/c12/fvt(44000)
      common/c13/fn2d(44000)
      common/c14/fn2t(44000)
      common/c15/fch4d(44000)
      common/c16/fch4t(44000)
      common/c17/fco2d(44000)
      common/c18/fco2t(44000)
      common/c19/fhd(44000)
      common/c20/fht(44000)
      common/c21/fhc(44000)
      common/c22/fha(44000)
      common/c23/fn2l(44000)
      common/c24/fch4l(44000)
      common/c25/fco2l(44000)
      common/c26/fh2o(44000)
C                                                *** END MODIFICATIONS
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G1/F1(25000)
      COMMON/G2/F2(25000)
      COMMON/G3/F3(25000)
      COMMON/G4/ELEG(10000)
      COMMON/G5/SOURCE(10000)
      COMMON/G6/LTABG(10000)
      COMMON/G7/G(10000)
      COMMON/G8/EG(10000)
      COMMON/G9/NEXG(10000)
      COMMON/G10/ITABG(10000)
      COMMON/G11/NGIND(10000)
      COMMON/G12/LCOM(10000)
      COMMON/G13/PI(10000)
      COMMON/G14/PWB(10000)
      COMMON/G15/HG(10000)
      COMMON/G16/GPO(10000)
      COMMON/G17/SDENS(10000)
      COMMON/G18/SSAT(10000)
      COMMON/G19/GVOL(10000)
      COMMON/G20/HL(10000)
      COMMON/G21/HS(10000)
      COMMON/G22/QVGC(10000)
      COMMON/G23/QVWC(10000)
      COMMON/G24/QVOC(10000)
      COMMON/G25/GRAD(10000)
      COMMON/G26/FF(10000)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/P1/X(100000)
      COMMON/E1/ELEM(20000)
      COMMON/E2/MATX(20000)
      COMMON/E3/EVOL(20000)
      COMMON/E4/PHI(20000)
      COMMON/E5/P(20000)
      COMMON/E6/T(20000)
C                                                 *** BEG MODIFICATIONS
      common/e7/gch4(20000)
      common/e8/gco2(20000)
      common/e9/tdepos(20000)
      common/e14/XYZ(60000)
C                                                 *** END MODIFICATIONS
      COMMON/AHTRAN/AHT(20000)
C      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)           JPF 03 2005
      COMMON/SOLID/DM(27),POR(27),PER(3,27),CWET(27),SH(27),NM
      COMMON/SOCH/MAT(27)
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/SOIN/YIN(4,27)
      COMMON/STEP/ELST
      COMMON/STE1/NST
      COMMON/DEFINI/DEP(10)
      COMMON/DFM/TIMAX,REDLT
C      COMMON/DLT/NDLT,DLT(100)                                            JPF  03 2005
      COMMON/DLT/DLT(100),NDLT
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
C      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1                  JPF 03 2005
      COMMON/DMN/TZERO,TIMP1,INUM,IPRINT,MCYC,MCYPR,MSEC
c      COMMON/DOP/ENTH,KDATA,QUAL                                          JPF 03 2005
      COMMON/DOP/QUAL,ENTH,KDATA
      COMMON/BALA/TFM0(27),TVM0(27),TFE0(27),TVE0(27),TSE0(27)
      COMMON/DX/K,NI,SCALE
      COMMON/POV6/TSTART
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                    JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
C      COMMON/KONIT/KON,DELT,IGOOD                                           JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      COMMON/BC/NELA
      COMMON/MMC/MM,MC
      COMMON/MADIM/MNEL,MNCON,MNEQ,MNPH,MNBK,MNOGN,MGTAB,MNK
      COMMON/V/IV,IS
      CHARACTER*5 ELEM1,ELEM2,ELEG,SOURCE,ELEM,MAT,ELST,DENT,EL1,EL2,EL
      CHARACTER ITABG*1,TYPE*4,MA1*3,MA2*2,MA12*5
C
      DIMENSION DEPU(10)
      SAVE ICALL
      DATA ICALL/0/
C
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C                                                     *** BEG MODIFICATIONS
  899 FORMAT(6X,'RFILE    1.2         MARCH     1995',6X,
     X'INITIALIZE DATA FROM FILES *MESH* OR *MINC*, *GENER*,',
     X' *INCON* AND *XAMD*')
C                                                     *** END MODIFICATIONS
C
      SCALEA=SCALE*SCALE
      SCALEV=SCALEA*SCALE
C
      PRINT 901
  901 FORMAT(1H )
C
C*****READ GEOMETRY DATA FROM FILE MESH OR MINC.***************************
C
C
      IF(MC.EQ.0) THEN
      IM=4
      ELSE
      IM=10
      ENDIF
C
      REWIND IM
      READ(IM,5020) DENT
      N=0
      N0=0
 1492 CONTINUE
      N=N+1
      IF(N.LE.MNEL) GOTO 40
      PRINT 41,MNEL
   41 FORMAT(' NUMBER OF ELEMENTS SPECIFIED IN DATA BLOCK "ELEME"',
     X' EXCEEDS ALLOWABLE MAXIMUM OF ',I5/
     X' INCREASE PARAMETER *MNEL* IN MAIN PROGRAM, AND RECOMPILE'//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
   40 CONTINUE
C
      READ(IM,1499) ELEM(N),MA1,MA2,EVOL(N),AHT(N),XYZ(N*3-2),
     1XYZ(N*3-1),XYZ(N*3),TDEPOS(N)
 1499 FORMAT(A5,10X,A3,A2,2E10.4,10X,4E10.4)
      IF(ELEM(N).EQ.'     ') GOTO1502
      IF(MA1.EQ.'   ') GOTO15
C-----FIND MATERIAL INDEX
      MA12=MA1//MA2
      DO16 M=1,NM
      IF(MA12.EQ.MAT(M)) GOTO 17
   16 CONTINUE
      PRINT 18,MA12,ELEM(N)
   18 FORMAT(' REFERENCE TO UNKNOWN MATERIAL ',A5,' AT ELEMENT *',A5,
     X'* --- WILL IGNORE ELEMENT')
      GOTO 40
C
   17 MATX(N)=M
      GOTO 19
   15 CONTINUE
C-----COME HERE FOR ELEMENTS WITH DOMAIN NUMBER
      READ(MA2,'(I2)') MATX(N)
   19 CONTINUE
      IF(MATX(N).NE.0) GOTO 1498
C     COME HERE FOR ELEMENTS WITHOUT DOMAIN ASSIGNMENT, AND ASSIGN
C     THEM TO DOMAIN # 1.
      MATX(N)=1
      PRINT 1497,ELEM(N),MAT(1)
 1497 FORMAT(' WWWWW WARNING WWWWW:',
     X' ELEMENT *',A5,'* HAS NO DOMAIN ASSIGNMENT; ASSIGN TO',
     X' DOMAIN # 1, *',A5,'*')
 1498 CONTINUE
      IF(EVOL(N).LE.0..AND.N0.EQ.0) N0=N-1
C     N0 IS THE INDEX PRECEDING THE FIRST ELEMENT WITH V = 0.
      IF(SCALE.EQ.1.) GOTO1490
      EVOL(N)=EVOL(N)*SCALEV
 1490 CONTINUE
      GOTO 1492
 1502 NEL=N-1
      NELA=NEL
      IF(N0.NE.0) NELA=N0
C     NELA IS THE NUMBER OF "ACTIVE" ELEMENTS; IT IS ASSUMED THAT
C     ALL ELEMENTS WITH INDEX N > NELA ARE BOUNDARY ELEMENTS.
C
      NST=0
      IF(ELST.EQ.'     ') GOTO80
C
      DO81 N=1,NEL
      IF(ELEM(N).EQ.ELST) GOTO82
   81 CONTINUE
      GOTO 80
C
   82 NST=N
   80 CONTINUE
C
      READ(IM,5020) DENT
      NCON=0
 1493 CONTINUE
      IF(NCON.LT.MNCON) GOTO 42
      PRINT 43,MNCON
   43 FORMAT('NUMBER OF CONNECTIONS SPECIFIED IN DATA BLOCK "CONNE"',
     X' EXCEEDS ALLOWABLE MAXIMUM OF ',I5/
     X' INCREASE PARAMETER *MNCON* IN MAIN PROGRAM, AND RECOMPILE'//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
   42 CONTINUE
C
      N=NCON+1
      READ(IM,1503) ELEM1(N),ELEM2(N),ISOX(N),DEL1(N),
     B                        DEL2(N),AREA(N),BETA(N)
 1503 FORMAT(2A5,15X,I5,4E10.4)
      IF(ELEM1(N).EQ.'+++  ') GOTO1504
      IF(ELEM1(N).EQ.'     ') GOTO 1507
      NCON=NCON+1
      IF(SCALE.EQ.1.) GOTO1491
      DEL1(N)=DEL1(N)*SCALE
      DEL2(N)=DEL2(N)*SCALE
      AREA(N)=AREA(N)*SCALEA
 1491 CONTINUE
      GOTO 1493
C
 1504 READ(IM,1505) (NEX1(N),NEX2(N),N=1,NCON)
 1505 FORMAT(20I4)
      GOTO1479
C
 1507 BACKSPACE IM
      WRITE(IM,1508)
 1508 FORMAT('+++  ')
      DO26 J=1,NCON
      EL1=ELEM1(J)
      EL2=ELEM2(J)
      DO27 N=1,NEL
      IF(ELEM(N).NE.EL1) GOTO27
      NEX1(J)=N
      GOTO28
   27 CONTINUE
      NEX1(J)=0
      PRINT 127,EL1,J
  127 FORMAT(30H REFERENCE TO UNKNOWN ELEMENT ,A5,15H AT CONNECTION ,I4,
     A27H --- WILL IGNORE CONNECTION)
C
   28 DO29 N=1,NEL
      IF(ELEM(N).NE.EL2) GOTO29
      NEX2(J)=N
      GOTO300
   29 CONTINUE
      NEX2(J)=0
      PRINT 127,EL2,J
  300 CONTINUE
   26 CONTINUE
C
   22 WRITE(IM,1505) (NEX1(N),NEX2(N),N=1,NCON)
      ENDFILE IM
 1479 CONTINUE
C
C-----END OF FILE MESH.------------------------------------------------
C
C                                                     *** BEG MODIFICATIONS
C  INITIALIZE VAPOR, N2, CH4, CO2 AND HEAT FLUXES AS ZERO.
C  D: diffusive flux  ;  T: total flux
      DO25 J=1,NCON
      FVD(J)=0.
      FVT(J)=0.
      FHC(J)=0.
      FHD(J)=0.
      FHT(J)=0.
      FHA(J)=0.
      FN2D(J)=0.
      FN2T(J)=0
      FCH4D(J)=0.
      FCH4T(J)=0.
      FCO2D(J)=0.
      FCO2T(J)=0.
C                                                     *** END MODIFICATIONS
   25 CONTINUE
C*****READ SINKS/SOURCES FROM FILE GENER.******************************
C NOGN: brojac na generation sinks/sources; MNOGN: max broj na s/s =50
C (NEXG(N),N=1,NOGN) numbers of the elements containing wells-s/s
      REWIND 3
      NOGN=0
      READ(3,5020,END=1415) DENT
      NGL=0
C
 1483 CONTINUE
      IF(NOGN.LT.MNOGN) GOTO 44
      PRINT 45,MNOGN
   45 FORMAT(' NUMBER OF SINKS/SOURCES SPECIFIED IN DATA BLOCK "GENER"',
     X' EXCEEDS ALLOWABLE MAXIMUM OF ',I4/
     X' INCREASE PARAMETER *MNOGN* IN MAIN PROGRAM, AND RECOMPILE'//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
   44 CONTINUE
C
      N=NOGN+1
      READ(3,1481) ELEG(N),SOURCE(N),LTABG(N),TYPE,ITABG(N)
     A,G(N),EG(N),HG(N)
 1481 FORMAT(2A5,15X,I5,5X,A4,A1,3G10.4)
C if '   ' : first run; if '+++' : consecutive run
      IF(ELEG(N).EQ.'+++  ') GOTO 1482
      IF(ELEG(N).EQ.'     ') GOTO 1406
      NOGN=NOGN+1
C
      NGIND(N)=NGL
      LCOM(N)=0
      IF(TYPE.EQ.'WATR') TYPE='COM2'
      IF(TYPE.EQ.'NACL') TYPE='COM1'
      IF(TYPE.EQ.'HEAT') LCOM(N)=NK1
      IF(TYPE.EQ.'MASS') LCOM(N)=1
      IF(TYPE.EQ.'WATE') LCOM(N)=1
      IF(TYPE.EQ.'AIR ') LCOM(N)=2
      IF(TYPE.EQ.'COM1') LCOM(N)=1
      IF(TYPE.EQ.'COM2') LCOM(N)=2
      IF(TYPE.EQ.'COM3') LCOM(N)=3
      IF(TYPE.EQ.'COM4') LCOM(N)=4
      IF(TYPE.EQ.'VOL.') LCOM(N)=NK1+2
      IF(TYPE.EQ.'DELV') LCOM(N)=NK1+1
C
C DELIVERABILITY WELLS: PI(N)  =  G(N) productivity index PI(m3)
C                       PWB(N) = EG(N) wellbore pressure; LCOM=6
C                              = HG(N) thickness of layer (m)
C VOLUMETRIC WELLS    : GVOL(N)=  G(N) production rate  ; LCOM=7
C WATER INJECTION WATE:
C AIR INJECTION   AIR :
C
      IF (LCOM(N).NE.0) GO TO 13
      PRINT 14,TYPE,ELEG(N),SOURCE(N)
   14 FORMAT(' IGNORE UNKNOWN GENERATION OPTION *',A4,'* AT ELEMENT *',
     1  A5,'* SOURCE *',A5,'*')
      NOGN=NOGN-1
      GO TO 1483
   13 CONTINUE
C
      IF(LCOM(N).EQ.NK1+1) GOTO 11
      GOTO 10
   11 CONTINUE
C-----COME HERE FOR TYPE = DELV, IN WHICH CASE A PRODUCTIVITY INDEX
C     (PI) AND A FLOWING WELLBORE PRESSURE (PWB) ARE SPECIFIED.
      PI(N)=G(N)
      PWB(N)=EG(N)
   10 CONTINUE
      IF(TYPE.EQ.'VOL.') GOTO 12
      GOTO 20
C
   12 CONTINUE
C-----COME HERE IF VOLUMETRIC PRODUCTION RATE IS SPECIFIED.
      GVOL(N)=G(N)
   20 CONTINUE
C
      LTABA=IABS(LTABG(N))
      IF(LTABA.LE.1.OR.TYPE.EQ.'DELV') GOTO1483
      IF(NGL+LTABA.LE.MGTAB) GOTO 46
      PRINT 47,MGTAB
   47 FORMAT(' NUMBER OF TABULAR GENERATION DATA SPECIFIED IN DATA',
     X' BLOCK "GENER" EXCEEDS ALLOWABLE MAXIMUM OF ',I5/
     X' INCREASE PARAMETER *MGTAB* IN MAIN PROGRAM, AND RECOMPILE'//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
   46 CONTINUE
C
      READ(3,36) (F1(NGL+L),L=1,LTABA)
      READ(3,36) (F2(NGL+L),L=1,LTABA)
      IF(ITABG(N).NE.' ') READ(3,36) (F3(NGL+L),L=1,LTABA)
      NGL=NGL+LTABA
      GOTO 1483
 1482 CONTINUE
      PRINT *,'END OF GENER   ',NOGN
      IF(NOGN.EQ.0) GOTO 1415
C
      READ(3,1505) (NEXG(N),N=1,NOGN)
      GOTO1415
 1406 BACKSPACE 3
C now writes '+++  ' if used latter as consecutive RUN; 
C 1508 FORMAT('+++  ')
      WRITE(3,1508)
      DO37 N=1,NOGN
      EL=ELEG(N)
      DO38 J=1,NEL
      IF(ELEM(J).NE.EL) GOTO38
C NEXG(N) is number of the element (from 1 to NEL) containing well-s/s
      NEXG(N)=J
      GOTO37
   38 CONTINUE
      NEXG(N)=0
      PRINT 128,EL,N
c  128 FORMAT(30H REFERENCE UNKNOWN ELEMENT ,A5,11H AT SOURCE ,I2,23H
c     1--- WILL IGNORE SOURCE)
  128 FORMAT('REFERENCE UNKNOWN ELEMENT ',A5,' AT SOURCE ',I2,'
     1 WILL IGNORE SOURCE')
   37 CONTINUE
C writes elements' numbers containing s/s 
   39 WRITE(3,1505) (NEXG(N),N=1,NOGN)
      ENDFILE 3
 1415 CONTINUE
C
C-----END OF FILE GENER.-----------------------------------------------
C
C*****READ INITIAL CONDITIONS FROM FILE INCON.*************************
C
 1300 REWIND 1
      IF(NI.EQ.0) GOTO1412
C
C-----COME HERE FOR START.
      DO1409 N=1,NEL
      PHI(N)=POR(MATX(N))
      NLOC=(N-1)*NK1
C
      IF(YIN(1,MATX(N)).NE.0.) GOTO 1410
C writes primary variables DEP(I) into X array  used later in EOS
      DO 2000 I=1,NK1
 2000 X(NLOC+I)=DEP(I)
      GOTO 1409
C
 1410 CONTINUE
      DO 1411 I=1,NK1
 1411 X(NLOC+I)=YIN(I,MATX(N))
C
C
 1409 CONTINUE
      READ(1,5020,END=1703) DENT
C
C-----COME HERE FOR ASSIGNING INITIAL CONDITIONS.
C
 2002 CONTINUE
      READ(1,1403,END=1700) EL,PORX
 1403 FORMAT(A5,10X,E15.8,5E10.4)
C
      IF(EL.EQ.'     ') RETURN
      IF(EL.EQ.'+++  ') GOTO 2004
C
      READ(1,1402,END=1700) (DEPU(I),I=1,NK1)
c                                                     *** BEG MODIFICATIONS
 1402 FORMAT(5E20.13)
C                                                     *** END MODIFICATIONS
      DO 2005 J=1,NEL
      IF(ELEM(J).NE.EL) GOTO 2005
      JLOC=(J-1)*NK1
      IF(PORX.NE.0.) PHI(J)=PORX
C
      DO 2006 I=1,NK1
 2006 X(JLOC+I)=DEPU(I)
      GOTO 2002
 2005 CONTINUE
C
      PRINT 1408,EL
 1408 FORMAT(50H WILL IGNORE INITIAL CONDITION AT UNKNOWN ELEMENT ,A5)
C
      GOTO 2002
C
 2004 READ(1,74) KCYC,ITERC,NM,TSTART,TIMIN
      IF(TIMAX.NE.0..AND.TIMIN.GE.TIMAX) TIMIN=0.
c mop(19)=0
      IF(MOP(19).EQ.0) RETURN
      PRINT 2007,MOP(19)
 2007 FORMAT(//49H SUBROUTINE RFILE: HAVE ENCOUNTERED '+++' ON FILE,
     X' *INCON* --- RESET MOP(19) =',I2,' TO DEFAULT VALUE ZERO.'/)
      MOP(19)=0
      RETURN
C
 1412 CONTINUE
C-----COME HERE TO READ RESTART INFORMATION.
      READ(1,5020,END=1700) DENT
      IF(DENT.NE.'INCON') GOTO 1700
      DO 1702 N=1,NEL
      NLOC=(N-1)*NK1
      READ(1,1403,END=1700) EL,PHI(N)
      IF(PHI(N).EQ.0.) PHI(N)=POR(MATX(N))
      READ(1,1402,END=1700) (X(NLOC+I),I=1,NK1)
 1702 CONTINUE
C
      READ(1,5020) DENT
      IF(DENT.EQ.'     ') RETURN
      READ(1,74) KCYC,ITERC,NM,TSTART,TIMIN
   74 FORMAT(3I5,2E15.8)
      IF(TIMAX.NE.0..AND.TIMIN.GE.TIMAX) TIMIN=0.
      IF(MOP(19).EQ.0) RETURN
      PRINT 2007,MOP(19)
      MOP(19)=0
   75 FORMAT(5E15.8)
C
C-----END OF FILE INCON.-----------------------------------------------
C
 1703 RETURN
 5020 FORMAT(A5)
   36 FORMAT(4E14.7)
C
 1700 PRINT 1701,DENT
 1701 FORMAT(' IMPROPER FORMAT ON FILE INCON --- IDENTIFIER "',A5,'"',//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
      END
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 2xxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE INDATA
      implicit real*8 (a-h,o-z)
C-----THIS ROUTINE PROVIDES A PRINTOUT OF MOST OF THE INPUT DATA.
C     IT IS CALLED ONLY IF MOP(7).NE.0.
C
C
C
C$$$$$$$$$ COMMON BLOCKS FOR ELEMENTS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C     THESE BLOCKS HAVE A LENGTH OF NEL (= NUMBER OF ELEMENTS)
C
      COMMON/E1/ELEM(20000)
      COMMON/E2/MATX(20000)
      COMMON/E3/EVOL(20000)
      COMMON/E4/PHI(20000)
      COMMON/E5/P(20000)
      COMMON/E6/T(20000)
C     *** BEGINNING MODIFICATIONS ***
      COMMON/E7/GCH4(20000)
      COMMON/E8/GCO2(20000)
      COMMON/E9/TDEPOS(20000)
C     *** END MODIFICATIONS ***
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C
C$$$$$$$$$ COMMON BLOCKS FOR PRIMARY VARIABLES $$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL*NEQ
C
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
      COMMON/P3/DELX(100000)
      COMMON/P4/R(100000)
      COMMON/P5/DOLD(100000)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/C1/NEX1(44000)
      COMMON/C2/NEX2(44000)
      COMMON/C3/DEL1(44000)
      COMMON/C4/DEL2(44000)
      COMMON/C5/AREA(44000)
      COMMON/C6/BETA(44000)
      COMMON/C7/ISOX(44000)
      COMMON/C8/GLO(44000)
      COMMON/C9/ELEM1(44000)
      COMMON/C10/ELEM2(44000)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/TITLE/TITLE
      CHARACTER*80 TITLE
C
C
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G1/F1(25000)
      COMMON/G2/F2(25000)
      COMMON/G3/F3(25000)
      COMMON/G4/ELEG(10000)
      COMMON/G5/SOURCE(10000)
      COMMON/G6/LTABG(10000)
      COMMON/G7/G(10000)
      COMMON/G8/EG(10000)
      COMMON/G9/NEXG(10000)
      COMMON/G10/ITABG(10000)
      COMMON/G11/NGIND(10000)
      COMMON/G12/LCOM(10000)
      COMMON/G13/PI(10000)
      COMMON/G14/PWB(10000)
      COMMON/G15/HG(10000)
      COMMON/G16/GPO(10000)
      COMMON/G17/SDENS(10000)
      COMMON/G18/SSAT(10000)
      COMMON/G19/GVOL(10000)
      COMMON/G20/HL(10000)
      COMMON/G21/HS(10000)
      COMMON/G22/QVGC(10000)
      COMMON/G23/QVWC(10000)
      COMMON/G24/QVOC(10000)
      COMMON/G25/GRAD(10000)
      COMMON/G26/FF(10000)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/SVZ/NOITE,MOP(24)
C      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)        JPF 03 2005
      COMMON/SOLID/DM(27),POR(27),PER(3,27),CWET(27),SH(27),NM
      COMMON/SOCH/MAT(27)
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
      COMMON/STEP/ELST
      COMMON/STE1/NST
      COMMON/DEFINI/DEP(10)
      COMMON/DG/WUP,WNR
      COMMON/DFM/TIMAX,REDLT
C      COMMON/DLT/NDLT,DLT(100)                                         JPF 03 2005
      COMMON/DLT/DLT(100),NDLT
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
c      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1              JPF 03 2005
      COMMON/DMN/TZERO,TIMP1,INUM,IPRINT,MCYC,MCYPR,MSEC
C      COMMON/DOP/ENTH,KDATA,QUAL                                      JPF 03 2005
      COMMON/DOP/QUAL,ENTH,KDATA
      COMMON/DX/K,NI,SCALE
      COMMON/POV6/TSTART
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/PATCH/SING
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/BIND/DIFF0,TEXP,BE
C      COMMON/TIMES/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX           JPF 03 2005
      COMMON/TIMES/DELAF,ITI,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
C     *** BEGINNING MODIFICATIONS ***
      COMMON/AHTRAN/AHT(20000)
C     *** END MODIFICATIONS ***
      common/ff/h1
      character*1 h1
      CHARACTER*5 ELEM1,ELEM2,ELEM,ELEG,SOURCE,MAT,ELST
      CHARACTER ITABG*1
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'INDATA   1.2      29 SEPTEMBER 1993',6X,
     X'PROVIDE PRINTOUT OF MOST DATA PROVIDED THROUGH FILE *INPUT*')
C
      h1=char(12)
      PRINT 100,h1
      PRINT 150,TITLE
      PRINT 1000
      PRINT 1000
      PRINT 200
      PRINT 250
C     *** BEGINNING MODIFICATIONS ***
      PRINT 300,NOITE,KDATA,MCYC,MSEC,MCYPR,(MOP(I),I=1,24),
     XDIFF0,TEXP,BE
      if (DIFF0.gt.0.) print *,'diffusion: LAI'
      if (DIFF0.lt.0.) print *,'diffusion: Millington & Quirq'
      if (DIFF0.eq.0.) print *,'diffusion: aucune'
C     *** END MODIFICATIONS ***
      PRINT 350
      PRINT 401,TSTART,TIMAX,DELTEN,DELTMX,ELST,GF,REDLT,SCALE
  401 FORMAT(4(5X,E10.4),10X,A5,3(5X,E10.4))
      IF(NDLT.EQ.0) GOTO 10
      PRINT 450
      PRINT 451,((DLT(J+8*(N-1)),J=1,8),N=1,NDLT)
  451 FORMAT(5X,8(5X,E10.4))
      GO TO 20
   10 PRINT 500
   20 PRINT 550
      PRINT 400,RE1,RE2,U,WUP,WNR,DFAC,FOR
      PRINT 600
C     *** BEGINNING MODIFICATIONS ***
      PRINT 650,DEP(1),DEP(2),DEP(3),DEP(4),DEP(5)
C     *** END MODIFICATIONS ***
      PRINT 1000
      PRINT 700
      DO 1 I=1,NM
      PRINT 750
      PRINT 800,
     AI,MAT(I),DM(I),POR(I),CWET(I),SH(I),COM(I),EXPAN(I)
      PRINT 810
      PRINT 820,PER(1,I),PER(2,I),PER(3,I)
C
    1 CONTINUE
      PRINT 1000
      PRINT 830
      PRINT 840
      DO 2 I=1,NEL
C     *** BEGINNING MODIFICATIONS ***
      PRINT 850,ELEM(I),MATX(I),EVOL(I),AHT(I),TDEPOS(I)
C     *** END MODIFICATIONS ***
    2 CONTINUE
      PRINT 1000
      PRINT 860
      PRINT 870
      DO 3 I=1,NCON
      PRINT 880,ELEM1(I),ELEM2(I),ISOX(I),DEL1(I),DEL2(I),AREA(I),
     ABETA(I)
    3 CONTINUE
      IF(NOGN .LE. 0 )GO TO 30
      PRINT 1000
      PRINT 890
      IDELV=0
      DO 4 I=1,NOGN
      IF(LCOM(I).EQ.NK1+1) GOTO 6
      PRINT 895
      PRINT 900,ELEG(I),SOURCE(I),G(I),EG(I)
      IF(LTABG(I) .GT. 1)GO TO 40
      PRINT 896
      GO TO 50
   40 PRINT 897
   50 CONTINUE
      GOTO 4
    6 CONTINUE
      IDELV=IDELV+1
      IF(IDELV.EQ.1) LAY=LTABG(I)
      IF(IDELV.EQ.1) PRINT 885,LAY
  885 FORMAT(5X,43HWELL ON DELIVERABILITY   $$$$$$$$   OPEN IN,I3,19H  L
     AAYERS   $$$$$$$$//5X,'   ELEMENT         SOURCE           PI',
     B'             PWB            DEL(Z)'/)
      PRINT 900,ELEG(I),SOURCE(I),PI(I),PWB(I),HG(I)
    4 CONTINUE
   30 PRINT 1000
      PRINT 910
      PRINT 920
      DO 5 I=1,NEL
      PRINT 930,ELEM(I),PHI(I),(X((I-1)*NK1+J),J=1,NK1)
    5 CONTINUE
      PRINT 1000
      PRINT 940
      PRINT 1000
      PRINT 1000
      RETURN
  100 FORMAT(A1/'     TOUGH2 INPUT DATA'/)
  150 FORMAT(5X,15HPROBLEM TITLE: ,A80,/)
  200 FORMAT(/,5X,22HPROBLEM SPECIFICATIONS,/)
C     *** BEGINNING MODIFICATIONS ***
  250 FORMAT('     NOITE  KDATA   MCYC   MSEC  MCYPR',
     A12X,'MOP',15X,'DIFF0',8X,'TEXP',9X,'BE')
  300 FORMAT(3X,5(I6,1X),2X,24I1,4(2X,E10.4),/)
C     *** END MODIFICATIONS ***
  350 FORMAT(117H       TSTART         TIMAX          DELTEN         DEL
     ATMX            ELST        GF             REDLT         SCALE )
  400 FORMAT(5X,8(5X,E10.4))
  450 FORMAT(/,5X,35H VARIABLE TIME STEPS ARE PRESCRIBED)
  500 FORMAT(/,5X,45H A CONSTANT TIME STEP OF DELTEN IS PRESCRIBED,/)
  550 FORMAT('            RE1            RE2            U              W
     AUP            WNR            DFAC           FOR')
C     *** BEGINNING MODIFICATIONS ***
  600 FORMAT(/,5X,'DEFAULT VALUES FOR PRIMARY VARIABLES',/,
     A2X,'           P=DEP(1)            X=DEP(2)         XCH4=',
     B'DEP(3)         XCO2=DEP(4)            T=DEP(5)')
  650 FORMAT(5X,5(5X,E15.8))
C     *** END MODIFICATIONS ***
  700 FORMAT(/,5X,15HROCK PROPERTIES,/)
  750 FORMAT(5X,102HDOMAIN     MAT        DENSITY        POROSITY     CO
     ANDUCTIVITY     HEAT CAP       COMPR          EXPAN)
  800 FORMAT(5X,I4,6X,A5,6(5X,E10.4),/)
  810 FORMAT(2X,45H          PERM1          PERM2          PERM3)
  820 FORMAT(5X,3(5X,E10.4),/)
  830 FORMAT(/,5X,8HELEMENTS,/)
C     *** BEGINNING MODIFICATIONS ***
  840 FORMAT(5X,'        ELEMENT       MATERIAL         VOLU',
     A'ME          HEAT LOSS AREA      TIME OF DEPOS.')
  850 FORMAT(14X,A5,8X,I5,2(10X,E10.4),12x,1f6.1)
C     *** END MODIFICATIONS ***
  860 FORMAT(/,5X,11HCONNECTIONS,/)
  870 FORMAT(5X,'          ELEM1          ELEM2           ISOT    ',
     A'        DEL1           DEL2           AREA           BETA'/)
  880 FORMAT(5X,2(10X,A5),9X,I5,6X,4(5X,E10.4))
  890 FORMAT(/,5X,15HGENERATION DATA)
  895 FORMAT('        ELEMENT         SOURCE           RATE',
     A'           ENTHALPY'/)
  896 FORMAT(5X,26HCONSTANT RATE IS SPECIFIED,/)
  897 FORMAT(5X,26HVARIABLE RATE IS SPECIFIED,/)
  900 FORMAT(5X,5X,A5,10X,A5,4X,4(5X,E10.4))
  910 FORMAT(/,5X,18HINITIAL CONDITIONS,/)
  920 FORMAT('ELEMENT    POROSITY      X1-Pres.      X',
     A'2-Sg      X3-CH4      X4-CO2      X5-Temp.',/)
  930 FORMAT(1X,A5,4x,E10.4,2x,E12.6,4(2X,E10.4))
  940 FORMAT(/,5X,17HEND OF INPUT DATA,/)
C     *** BEGINNING MODIFICATIONS ***
  980 FORMAT(10X,26HAMD COMPUTATION PROPERTIES,/,
     15X,42H       PYRITE MASS FRAC.    RATE CONSTANT ,
     219H       RATIO TD/TC )
  990 FORMAT(5X,3(10X,E10.4),/)
C     *** END MODIFICATIONS ***
 1000 FORMAT(5X,117(1H*))
      END
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 3xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE CYCIT
      implicit real*8 (a-h,o-z)
C-----THIS ROUTINE IS THE EXECUTIVE ROUTINE FOR MARCHING IN TIME.
C-----IS CALLED FROM THE MAIN PROGRAM
C
C
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
      COMMON/E1/ELEM(20000)
      COMMON/VINWES/AI(20000)
      COMMON/SECPAR/PAR(2640000)
C                                                     *** BEG MODIFICATIONS
      COMMON/E7/GCH4(20000)
      COMMON/E8/GCO2(20000)
      COMMON/E9/TDEPOS(20000)
C                                                     *** END MODIFICATIONS
      COMMON/STEP/ELST
      COMMON/STE1/NST
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
      COMMON/KC/KC
      COMMON/DFM/TIMAX,REDLT
C      COMMON/DOP/ENTH,KDATA,QUAL                                        JPF 03 2005
      COMMON/DOP/QUAL,ENTH,KDATA
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
C      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1                JPF 03 2005
      COMMON/DMN/TZERO,TIMP1,INUM,IPRINT,MCYC,MCYPR,MSEC
C      COMMON/KONIT/KON,DELT,IGOOD                                        JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
C DELT is updated dT, SUMTIM is elapsed time
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                  JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
c      COMMON/TIMES/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX             JPF 03 2005
      COMMON/TIMES/DELAF,ITI,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
      COMMON/FAIL/IHLV
      COMMON/BC/NELA
      COMMON SGCH4,SGCO2,SGHT
      common/bal1/sumch4,sumco2,sumght
      CHARACTER*5 ELEM,ELST
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C                                                     *** BEG MODIFICATIONS
  899 FORMAT(/6X,'CYCIT    1.0         MARCH     1995',6X,
     X'EXECUTIVE ROUTINE FOR MARCHING IN TIME')
C                                                     *** END MODIFICATIONS
C Initializira: KC=0, total time elapsed up to last iterat.SUMTIM=TSTART
      nbrojac=0
      KC=0
      SUMTIM=TIMIN
      TIMOUT=SUMTIM
      KON=1
      DELT=DELTEN
      MA=0
      IAB=0
      IPIV=0
      IGOOD=0
      NSTL=(NST-1)*NK1
      NSTL2=(NST-1)*NEQ1*NSEC
C total methane , co2 and heat production
      sumch4=0.
      sumco2=0.
      sumght=0.
C
      ITER=0
      IHLV=0
      KIT=0
      IQIT=0
c*###################################################################  EOS3
      CALL EOS
c*################################################################## BALLAN
         CALL BALLAN
C
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C&@@@@@@@@@@@@@@@@@@@@@@@@@@ MARCHING IN TIME @@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   30 KCYC=KCYC+1
C--------------------------------------------------------------------------
C-----COME HERE FOR NEW TIME STEP.               KON=2
C--------------------------------------------------------------------------
      KC=KC+1
C mop(2)=0
      IF(MOP(2).NE.0) PRINT 29,KON,KC,KCYC,DELT,SUMTIM,TIMAX,MSEC,
     1                         MCYC
   29 FORMAT('CYCIT:  KON,KC =',2I4,3x,'KCYC =',I4,3x,'DELT = ',E1
     A2.6,3x,'SUMTIM = ',E12.6,3x,'TIMAX=',E12.6,3x,'MSEC=',I6,
     B3x,'MCYC=',I9,5X,/)
c go to 10 = the end
      IF(IQIT.NE.0) GOTO 10
      IF(TIMAX.NE.0..AND.SUMTIM.EQ.TIMAX) GOTO10
      IF(KC.GT.MCYC) GOTO10
c 7 fev 05     CALL SECOND(TCUR)
      CALL SECOND(TCUR)
      IF(MSEC.NE.0.AND.TCUR-TZERO.GT.MSEC) GOTO10
C
      IHALVE=0
C
   52 ITER=0
      NOW=0
C-----PRINTOUT OCCURS FOR NOW=1
C
      NOWTIM=0
      IF(ITI.NE.0) CALL TSTEP
C go zapisuva dT vo DELTEX zasto ako ne ispadne iteracijata go reducira dT
      DELTEX=DELT
      KON=1
C
    3 CONTINUE
C-------------------------------------------------------------------------
C-----COME HERE FOR NEXT ITERATION.              KON=1
C-------------------------------------------------------------------------
      IF(ITER.NE.0) GOTO19
      DO20 N=1,NEL
      NLOC=(N-1)*NK1
      DO20 K=1,NK1
      DX(NLOC+K)=0.
   20 CONTINUE
C
   19 CONTINUE
C
      ITER=ITER+1
      ITERC=ITERC+1
C
      FORD=FOR*DELTEX
C
C-----COMPUTE ACCUMULATION-, SOURCE-, AND FLOW-TERMS.
C#################################################################### MULTI
      CALL MULTI
      IF(IGOOD.EQ.3) GOTO 50
C
C-----CHECK FOR CONVERGENCE.
      IF(RERM.LE.RE1) GOTO51
C mop(1)=1
      IF(MOP(1).NE.0) PRINT 22,KC,KCYC,ITER,DELTEX,RERM,ELEM(NER),KER
   22 FORMAT('.....ITERATING AT [KC,KCYC,ITER=',3(I4),'] --- DELTEX = ',
     AE12.6,'   MAX. RES. = ',E12.6,'  AT ELEMENT ',A5,'  EQUATION ',I3)
      IF(MOP(1).LT.2) GOTO 23
      NIT=NER
      IF(NST.NE.0) NIT=NST
      NIT1=(NIT-1)*NK1
      NIT2=(NIT-1)*NEQ1*NSEC
      PNIT=X(NIT1+1)+DX(NIT1+1)
      PRINT 24,ELEM(NIT),DX(NIT1+1),DX(NIT1+2),
     XPAR(NIT2+NSEC-1),PNIT,PAR(NIT2+1)
   24 FORMAT(31X,'     AT *',A5,'* ...   DX1= ',
     XE12.6,6H DX2= ,E12.6,5H T = ,F7.3,5H P = ,F9.0,5H S = ,E12.6)
   23 CONTINUE
C
      IF(ITER.GT.NOITE) GOTO50
C#################################################################### LINEQ
      CALL LINEQ
      IF(IGOOD.NE.0) GOTO50
C##################################################################### EOS3
      CALL EOS
      IF(IGOOD.EQ.0) GOTO54
C if it does not succeed by 8 iteration it decreases the time step
c dT=dT/2 in CYCIT. However if it succeeds in  LE.2  iterations it 
C increases the time step dT=1.3dT in CONVER
   50 DELT=DELTEX/REDLT
C-----REDUCE TIME STEP.
      IGOOD=0
C     COME HERE TO CHECK ON NUMBER OF PREVIOUS TIME STEPS THAT
C     CONVERGED ON ITER=1. mop(16)=2
      IF(KIT.LT.2.OR.MOP(16).EQ.0) GOTO 25
      PRINT 26,KCYC,DELTEX
   26 FORMAT(/' +++++++++ CONVERGENCE FAILURE ON TIME STEP #',I4,
     X' WITH DT = ',E12.6,
     X' SECONDS, FOLLOWING TWO STEPS THAT CONVERGED ON ITER = 1'/
     X'           STOP EXECUTION AFTER NEXT TIME STEP')
      IQIT=1
C
   25 CONTINUE
      PRINT 53,KCYC,ITER,DELT
   53 FORMAT(32H +++++++++ REDUCE TIME STEP AT (,I4,1H,,I3,32H) ++++++++
     A++++++++   NEW DELT = ,E12.6)
C
      IHALVE=IHALVE+1
      IF (IHALVE.LE.25) GO TO 11
C-----STOP AFTER 25 ATTEMPTS TO REDUCE TIME STEP.
      PRINT 12,DELTEX
   12 FORMAT(//' NO CONVERGENCE AFTER 25 REDUCTIONS OF TIME STEP SIZE'/
     1  ' LAST TIME STEP WAS ',E12.6,' SECONDS '//
     2  ' !!!!!!!!!! STOP EXECUTION !!!!!!!!!!')
      STOP
C
   11 ITER=0
      IHLV=1
C##################################################################### EOS3
      CALL EOS
      IHLV=0
      GOTO52
C
C-----COME HERE FOR CONVERGENCE.
   51 KON=2
C
C za mene bi trebalo mop(15)=0
C      IF(MOP(15).GE.1) CALL QLOSS
C                                                     *** BEG MODIFICATIONS
C
C     COULD COMPUTE (IF(MOP(15).GE.1)) THE CUMULATIVE HEAT LOSS
C     TO CONFINING LAYERS SINCE BEGINNING OF SIMULATION:
C     HTLC=HTLC+HTL
C
C     NOTE: IT WOULD REQUIRE PUTTING THE COMMON BLOCK QQ IN THIS
C           MODULE.  THE HEAT LOSS DURING THE TIME STEP (HTL) AND
C           HTLC COULD THEN BE PRINTED IF DESIRED IN SUBROUTINE OUT.
C
C-----LGM COMPUTATIONS PERFORMED IF MOP(24) > 3 :
C
      if(mop(24).eq.0) go to 1000
C
C UPDATES CH4, CO2 & HEAT PRODUCTION RATE AFTER CONVERGENCE KON=2.
C TOTAL SUMCO2=NELA*P0CO2*DM(N)*EVOL(N); SUMCH4=NELA*P0CH4*DM(N)*EVOL(N)
C toa e vkupno proizvodstvo vo (kg) za site elementi. CYCIT-CCHPR DOES
C NOT CHANGE THE RESIDUALS
C#################################################################### cchpr
      call cchpr
      sumch4=sumch4+sgch4
      sumco2=sumco2+sgco2
      sumght=sumght+sght
      smtm=sumtim/(3600.*24.)
1000  continue
c
C#################################################################### lgmbal
      if(mop(23).ge.8)  CALL LGMBAL
C                                                     *** END MODIFICATIONS
C################################################################### CONVER
C
       CALL CONVER
C mop(22)=1   mislam za crtanje                       *** BEG MODIFICATIONS
C      IF(MOP(22).GT.0) CALL TRACE
C
C                                                     *** END MODIFICATIONS
C
C     COUNT NUMBER OF CONSECUTIVE TIME STEPS THAT CONVERGE ON
C      ITER = 1.
      IF(ITER.GT.1) KIT=0
      IF(ITER.EQ.1) KIT=KIT+1
      IF(KIT.LT.10) GOTO 55
      PRINT 56
   56 FORMAT(' +++++++++ FOR 10 CONSECUTIVE TIME STEPS HAVE',
     X' CONVERGENCE ON ITER = 1'/
     X'           WRITE OUT CURRENT DATA, THEN STOP EXECUTION')
      IQIT=1
   55 CONTINUE
C
      smtmday=SUMTIM/(60.*60.*24.)
      nyear=SUMTIM/(365.25*60.*60.*24.)
      smtmday=smtmday-nyear*365.25
      IF(KON.EQ.2.AND.NST.NE.0)PRINT 49,ELEM(NST),KCYC,ITER,SUMTIM,
cc     aDELTEX,DX(NSTL+1),DX(NSTL+2),
     anyear,smtmday,DELTEX,
     bPAR(NSTL2+NSEC-1),X(NSTL+1),PAR(NSTL2+1),SUMCH4,SUMCO2
      IF(KON.EQ.2.AND.NST.EQ.0)PRINT 49,ELEM(NER),KCYC,ITER,SUMTIM,
cc     aDELTEX,DX((NER-1)*NK1+1),DX((NER-1)*NK1+2),
     anyear,smtmday,DELTEX,
     bPAR((NER-1)*NEQ1*NSEC+NSEC-1),
     cX((NER-1)*NK1+1),PAR((NER-1)*NEQ1*NSEC+1),SUMCH4,SUMCO2
cc49    FORMAT(A5,1H(,I4,1H,,I2,1H),9H  SumTim=T,E12.6,4H DT=,E10.4,
cc    a6H  DX1=,E10.4,5H DX2=,E10.4,2x,7H Temp.=,F6.2,7H Pres.=,F8.1,
cc     b6H Gsat=,F4.3,8H   Sch4=,E10.5,7H  Sco2=,E10.5,3h kg)
49    FORMAT(A5,'(',I4,',',I2,')  SumTim=',E13.8,' s [ ',I4,
     a' y ',f7.2,' d]  dT=',E11.5,' s  T=',F5.2,' (oC)  P=',F8.1,
     b' (Pa)  Sg=',F5.3,' %  CH4=',E10.5,'  CO2=',E10.5,' (kg)')
C
C-----DETERMINE WHETHER PRINTOUT IS REQUIRED.
      IF(IQIT.NE.0) NOW=1
      IF(NOWTIM.EQ.1.AND.KON.EQ.2) NOW=1
      IF(KON.EQ.2.AND.MOD(KCYC,MCYPR).EQ.0) NOW=1
   54 IF(KDATA.GE.10.AND.MOD(KCYC,MCYPR).EQ.0) NOW=1
      IF(NOW.EQ.0) GOTO33
C###################################################################   OUT
         CALL OUT
C###################################################################  PLOT
C         if(mop(23).gt.0) call plot            PlOT subroutine eliminated  JPF 03 2005
C################################################################## BALLAN
         IF(KON.EQ.2) CALL BALLAN
   33 CONTINUE
C
C-------------------------------------------------------------------------
      GOTO(3,30),KON
C-------------------------------------------------------------------------
C
C@@@@@@@@@@@@@@@@@@@@@@@ MARCHING IN TIME FINISHED @@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C
C-----WRITE RESULTS ON FILE *SAVE*.
C################################################################### WRIFI
   10 CALL WRIFI
C
      RETURN
      END
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 4xxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE MULTI
      implicit real*8 (a-h,o-z)
C-----SET UP THE COUPLED LINEAR EQUATIONS ARISING AT EACH ITERATION
C     STEP.
C
C***** N O T A T I O N ********************
C
C     NEQ IS THE NUMBER OF EQUATIONS, AND THE NUMBER OF PRIMARY
C         DEPENDENT VARIABLES (PER ELEMENT). =5
C     NPH IS THE NUMBER OF PHASES WHICH CAN BE PRESENT. =2
C     NK  IS THE NUMBER OF COMPONENTS (SPECIES). =4
C     NB  IS THE NUMBER OF PHASE-DEPENDENT SECONDARY VARIABLES OTHER
C         THAN COMPONENT MASS FRACTIONS. =6
C     NBK = NB+NK IS THE TOTAL NUMBER OF PHASE-DEPENDENT SECONDARY
C         VARIABLES. =10
C     NSEC = NPH*NBK+2 IS THE TOTAL NUMBER OF SECONDARY VARIABLES. =22
C
C******************************************
C
C$$$$$$$$$ COMMON BLOCKS FOR ELEMENTS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL (= NUMBER OF ELEMENTS)
C
      COMMON/E1/ELEM(20000)
      COMMON/E2/MATX(20000)
      COMMON/E3/EVOL(20000)
      COMMON/E4/PHI(20000)
      COMMON/E5/P(20000)
      COMMON/E6/T(20000)
C
C$$$$$$$$$ COMMON BLOCKS FOR PRIMARY VARIABLES $$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL*NK1
C
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
      COMMON/P3/DELX(100000)
C
C$$$$$$$$$ COMMON BLOCKS FOR RESIDUALS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL*NEQ
C
      COMMON/P4/R(100000)
      COMMON/P5/DOLD(100000)
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/C1/NEX1(44000)
      COMMON/C2/NEX2(44000)
      COMMON/C3/DEL1(44000)
      COMMON/C4/DEL2(44000)
      COMMON/C5/AREA(44000)
      COMMON/C6/BETA(44000)
      COMMON/C7/ISOX(44000)
      COMMON/C8/GLO(44000)
      COMMON/C9/ELEM1(44000)
      COMMON/C10/ELEM2(44000)
C                                                 *** BEG MODIFICATIONS
      common/C11/fvd(44000)
      common/c12/fvt(44000)
      common/c13/fn2d(44000)
      common/c14/fn2t(44000)
      common/c15/fch4d(44000)
      common/c16/fch4t(44000)
      common/c17/fco2d(44000)
      common/c18/fco2t(44000)
      common/c19/fhd(44000)
      common/c20/fht(44000)
      common/c21/fhc(44000)
      common/c22/fha(44000)
      common/c23/fn2l(44000)
      common/c24/fch4l(44000)
      common/c25/fco2l(44000)
      common/c26/fh2o(44000)
      COMMON/DIFFUSION/D0H2O,D0N2,D0CH4,D0CO2
C                                                 *** END MODIFICATIONS
C
C$$$$$$$$$ COMMON BLOCK FOR SECONDARY VARIABLES $$$$$$$$$$$$$$$$$$$$
C
      COMMON/SECPAR/PAR(2640000)
C
C$$$$$$$$$ COMMON BLOCKS FOR FLOW RATES AND VELOCITIES $$$$$$$$$$$$$$$$$
C
      COMMON/COMPO/FLO(88000)
      COMMON/PORVEL/VEL(88000)
C
C$$$$$$$$$ COMMON BLOCKS FOR ROCK PROPERTIES $$$$$$$$$$$$$$$$$$$$$$$$$$
C
c      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)           JPF 03 2005
      COMMON/SOLID/DM(27),POR(27),PER(3,27),CWET(27),SH(27),NM
      COMMON/SOCH/MAT(27)
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
C
C$$$$$$$$$ COMMON BLOCKS FOR LINEAR EQUATIONS $$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/L1/IRN(5400000)
      COMMON/L2/ICN(10800000)
      COMMON/L3/CO(10800000)
      COMMON/L4/WKAREA(100000)
      COMMON/L5/IKEEP(500000)
      COMMON/L6/IW(800000)
      COMMON/L7/JVECT(2700000)
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                   JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
C      COMMON/KONIT/KON,DELT,IGOOD                                          JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      COMMON/DG/WUP,WNR
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/PATCH/SING
      COMMON/BIND/DIFF0,TEXP,BE
C                                                 *** BEG MODIFICATIONS
C AMH2O=18.016; AMN2=28.96; AMCH4=16.05; and AMCO2=44. MOLECULAR MASS
      COMMON/GASLAW/RGAS,AMH2O,AMN2,AMCH4,AMCO2,CVAIR
C
C                                                 *** END MODIFICATIONS
      COMMON/DFM/TIMAX,REDLT
      COMMON/BC/NELA
      CHARACTER*5 ELEM,ELEM1,ELEM2,MAT
C
C######### LOCAL ARRAYS ###############################################
      DIMENSION D(11,12),F(11,23)
C######################################################################
C
      SAVE ICALL,M11
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
c      PRINT *,'ZAPOCNUVA SUBROUTINE MULTI'
C                                                 *** BEG  MODIFICATIONS
  899 FORMAT(/6X,'MULTI    1.2       4 OCTOBER   1993',6X,
     X       'ASSEMBLE ALL ACCUMULATION AND FLOW TERMS')
C                                                 *** END MODIFICATIONS
C mop(11)=0
      IF(ICALL.EQ.1) M11=MOD(MOP(11),2)
C$$$$$FOR IAB=0 COLUMN INDICES WILL BE STORED IN ICN, OTHERWISE IN JVECT
C-----INITIALIZE COUNTER FOR MATRIX ELEMENTS.
      NZ=0
C mop(3)=0
      IF(MOP(3).GE.1) PRINT 201,KCYC,ITER
  201 FORMAT(/39H @@@@@@@@@@ SUBROUTINE MULTI @@@@@@@@@@,24H   ---   [KC
     AYC,ITER] = [,I4,1H,,I3,1H]/)
C
      IF(MOP(3).GE.3) PRINT 204
  204 FORMAT(' ===== ACCUMULATION TERMS =====   MASS BALANCES FIRST,',
     X       ' ENERGY BALANCE LAST'/)
C#########################################################################
C############################################### LOOP OVER ACTIVE ELEMENTS 
C#########################################################################
C-----COMPUTE ALL QUANTITIES WHICH DEPEND ONLY UPON  
C     VARIABLES  PERTAINING  TO  ONE VOLUME ELEMENT
C--------------------------------------------------------------------------
      DO 100 N=1,NELA
C----------------------------------------------------IDENTIFY MATRIX BLOCK.
      NLOC=(N-1)*NEQ
      NLOCP=(N-1)*NK1
C     IDENTIFY THE START OF SECONDARY VARIABLES (132) FOR ELEMENT N.
      NLOC2=(N-1)*NSEC*NEQ1
C
C-----ZERO OUT RESIDUALS FOR ELEMENT N.
      DO 20 K=1,NEQ
   20 R(NLOC+K)=0.
C
C
C     ASSIGN QUANTITIES WHICH DEPEND ONLY UPON ELEMENT INDEX N.
C PHI: porosity; MATX: material identifier; SH: rock spec. heat; DM: dens.
      PHIN=PHI(N)
      NMAT=MATX(N)
      CD=SH(NMAT)*DM(NMAT)*(1.-POR(NMAT))
C
C-----LOOP THROUGH INCREMENTS FOR DERIVATIVES.
      DO 101 M=1,NEQ1
C     IDENTIFY BEGINNING OF SECONDARY VARIABLES CORRESPONDING
C     TO INCREMENTING PRIMARY VARIABLE (M-1).
      NLM2=NLOC2+(M-1)*NSEC
C
C-----ZERO OUT BLOCK (N,N).
      DO 1011 K=1,NK1
 1011 D(K,M)=0.
C
C-----COMPUTE CHANGE IN POROSITY.
      DPRES=0.
      IF(M.EQ.2) DPRES=DELX(NLOCP+1)
      PRES=X(NLOCP+1)+DX(NLOCP+1)+DPRES
      DPHI=PHIN*(COM(NMAT)*(PRES-P(N))+EXPAN(NMAT)*(PAR(NLM2+NSEC-1)
     A     -T(N)))
      PHINN=PHIN+DPHI
C  NPH e broj na phasi (dva)
      DO 102 NP=1,NPH
C  IDENTIFY BEGINNING OF SECONDARY VARIABLES FOR PHASE NP.
      NL2NP=NLM2+(NP-1)*NBK
C  PHASE SATURATION (liquid or gaseous phase)
      SNP=PAR(NL2NP+1)
      IF(SNP.EQ.0.) GOTO 102
C  PHASE DENSITY (liquid or gaseous phase)
      RHONP=PAR(NL2NP+4)
C  PHASE MASS: PHISRO=porosityPHINN*phase sat.SNP*phase densityRHONP
      PHISRO=PHINN*SNP*RHONP
C-----SUM OVER COMPONENTS NK IN EACH PHASE.
      DO 103 K=1,NK
      XNPKM=PAR(NL2NP+NB+K)
  103 D(K,M)=D(K,M)+XNPKM*PHISRO
C
C
C     INTERNAL ENERGY IN PHASE NP. NK1=NK+1=4+1=5
C          n*Si*ro*enthalpy   -   n*Si*Pression
      ENNP=PHISRO*PAR(NL2NP+5)-PHINN*SNP*PRES
      D(NK1,M)=D(NK1,M)+ENNP
C
  102 CONTINUE
C
C     ADD ROCK ENERGY.
      ENR=CD*PAR(NLM2+NSEC-1)
      D(NK1,M)=D(NK1,M)+ENR
C
  101 CONTINUE
C
c      IF(MOP(3).GE.3) PRINT 200,ELEM(N),(D(K,1),K=1,NEQ)
  200 FORMAT('       AT ELEMENT *',A5,'*   ',8(1X,E12.6))
C
C
C+++++ASSIGN ALL ONE-ELEMENT TERMS++++++++++++++++++++++++++++++
C
      DO 105 K=1,NEQ
C     K IS THE ROW INDEX IN MATRIX BLOCK (N,N).
C-----INITIALIZE RESIDUALS.
C     DOLD(NLOC+K) HOLDS THE QUANTITY OF COMPONENT K IN ELEMENT N AT THE
C     END OF LAST COMPLETED TIME STEP.
C
C-----AT BEGINNING OF TIME STEP, ACCUMULATION TERM CONTRIBUTION TO
C     RESIDUAL IS ZERO.
      IF(ITER.NE.1)
     A   R(NLOC+K)=R(NLOC+K)+D(K,1)-DOLD(NLOC+K)
C
C-----SET DOLD(NLOC+K) AT BEGINNING OF TIME STEP.
      IF(ITER.EQ.1) DOLD(NLOC+K)=D(K,1)
C
      DO 106 L=1,NEQ
C     L IS THE COLUMN INDEX IN MATRIX BLOCK (N,N).
C-----COMPUTE MATRIX ELEMENT ARISING FROM DEPENDENCE OF COMPONENT K
C     UPON PRIMARY VARIABLE L.
      IRN(NZ+1)=NLOC+K
      IF(IAB.EQ.0) ICN(NZ+1)=NLOC+L
      IF(IAB.NE.0) JVECT(NZ+1)=NLOC+L
      CO(NZ+1)=-(D(K,L+1)-D(K,1))/DELX(NLOCP+L)
      IF(PHINN.EQ.0..AND.K.EQ.L.AND.K.NE.NK1) CO(NZ+1)=1.
  106 NZ=NZ+1
  105 CONTINUE
C
C+++++END OF ASSIGNMENT OF ONE-ELEMENT TERMS++++++++++++++++++++
C
C
  100 CONTINUE
C
C                                                     *** BEG MODIFICATIONS
C***** CH4; CO2 and HEAT SOURCE **************************************** JPT
C CHANGE OF THE RESIDUALS
c     IF(MOP(24).gt.1) CALL CCHPR
	IF(MOP(24).eq.4) CALL CCHPR
C                                                     *** END MODIFICATIONS
C***** SINKS AND SOURCES ***********************************************
C sto nie gi generirame const ili vo tabelite F
      IF(NOGN.NE.0) CALL QU
      IF(IGOOD.EQ.3) RETURN
C heat exchange with confining layers za mene mop(15)=0
C      IF(MOP(15).GE.1) CALL QLOSS
C+++++ END OF SINKS/SOURCES ++++++++++++++++++++++++++++++++++++++++++++
C
C
C#########################################################################
C################################################### LOOP OVER CONNECTIONS 
C#########################################################################
C
C-----COMPUTE ALL QUANTITIES WHICH DEPEND UPON VARIABLES FOR TWO VOLUME
C     ELEMENTS ("INTERFACE QUANTITIES").
C
C     AT EACH INTERFACE HAVE ONE FLUX FOR EACH COMPONENT, WHICH IS A SUM
C     OF CONTRIBUTIONS FROM ALL PHASES. 
C     VRLO VAZNO: FOR PURPOSES OF COMPUTING
C     {DERIVATIVES, NEED ALSO THE FLUXES OBTAINED BY INCREMENTING ANY
C     OF THE NEQ VARIABLES FOR THE "FIRST" ELEMENT, AND ANY OF THE
C     NEQ VARIABLES FOR THE "SECOND" ELEMENT}
C
C-----INITIALIZE TOTAL NUMBER OF FLUX TERMS.
C
C main do=loop throughout all the connections
C ------------------------------------------------------------------
      DO1 N=1,NCON
C ------------------------------------------------------------------
      N1=NEX1(N)
      N2=NEX2(N)
      IF(N1.EQ.0.OR.N2.EQ.0) GOTO 1
      IF(N1.GT.NELA.AND.N2.GT.NELA) GOTO 1
C-----IDENTIFY LOCATION OF MATRIX BLOCKS.
      N1LOC=(N1-1)*NEQ
      N2LOC=(N2-1)*NEQ
      N1LOCP=(N1-1)*NK1
      N2LOCP=(N2-1)*NK1
C
C-----IDENTIFY START OF SECONDARY VARIABLES FOR ELEMENTS N1 AND N2.
      N1LOC2=(N1-1)*NSEC*NEQ1
      N2LOC2=(N2-1)*NSEC*NEQ1
C
C-----OBTAIN SOME QUANTITIES PERTAINING TO CONNECTION.
C
      D1=DEL1(N)
      D2=DEL2(N)
C     SPATIAL INTERPOLATION FACTORS.
      WT1=D2/(D1+D2)
      WT2=1.-WT1
C
      NMAT1=MATX(N1)
      NMAT2=MATX(N2)
      PO1=POR(NMAT1)
      PO2=POR(NMAT2)
      ISO=ISOX(N)
      GX=BETA(N)*GF
      AX=AREA(N)
      PER1=PER(ISO,NMAT1)
      PER2=PER(ISO,NMAT2)
      DPERI=WT1*PER1+WT2*PER2
      PERI=0.
      IF(DPERI.NE.0.) PERI=PER1*PER2/DPERI
      IF(D1.EQ.0.) PERI=PER2
      IF(D2.EQ.0.) PERI=PER1
C
C-----ASSIGN FACTORS FOR FLUX TERMS.
      IF(N1.LE.NELA) FAC1=FORD/EVOL(N1)
      IF(N2.LE.NELA) FAC2=FORD/EVOL(N2)
C
      DPX0=X(N2LOCP+1)+DX(N2LOCP+1)-X(N1LOCP+1)-DX(N1LOCP+1)
      DPX0=DPX0/(D1+D2)
C mop(3)=0
      IF(MOP(3).GE.4) PRINT 199,N,ELEM(N1),ELEM(N2),FAC1,FAC2
  199 FORMAT(/' ***** CONNECTION #',I5,'   ELEMENTS (',A5,',',A5,')',
     X'  ***  (DELT/V1) = ',E12.6,'   (DELT/V2) = ',E12.6)
C
C-----NOW GET ALL FLUX TERMS.
C     M=1 IS THE FLUX FOR LATEST UPDATED VARIABLES.
C     M=2, ... ,NEQ+1 ARE THE NEQ FLUXES CORRESPONDING TO INCREMENTING
C     THE VARIABLES FOR ELEMENT N1.
C
C     M=NEQ+2, ... ,2*NEQ+1 ARE THE NEQ FLUXES CORRESPONDING TO
C     INCREMENTING THE VARIABLES FOR ELEMENT N2.
C --------------------------------------------------------------
      DO 2 M=1,NFLUX
C --------------------------------------------- NFLUX=2*NEQ+1=11
c m=1 heat
c m=2 pression
      DPRES1=0.
      DPRES2=0.
      IF(M.EQ.2) DPRES1=DELX(N1LOCP+1)
      IF(M.EQ.NEQ+2) DPRES2=DELX(N2LOCP+1)
      PRES1=X(N1LOCP+1)+DX(N1LOCP+1)+DPRES1
      PRES2=X(N2LOCP+1)+DX(N2LOCP+1)+DPRES2
      DPX=(PRES2-PRES1)/(D1+D2)
C
C-----IDENTIFY LOCATION OF SECONDARY VARIABLES.
C
C     DEFINE AUXILIARY QUANTITY M2 TO BE 0 FOR M=1, TO BE 1 FOR
C     SUBSEQUENT GROUP OF NEQ FLUXES INCREMENTED AT N1, TO BE 2 FOR
C     SUBSEQUENT GROUP OF NEQ FLUXES INCREMENTED AT N2.
C
      M2=2
      IF(M.EQ.1) M2=0
      IF(2.LE.M.AND.M.LE.NEQ1) M2=1
C MOD e nekoj kurac za smaluvanje na time step pa se menuva pozicijata
      N1LM2=N1LOC2+MOD(M2,2)*(M-1)*NSEC
      N2LM2=N2LOC2+(M2/2)*(M-NEQ1)*NSEC
C
      DO 21 K=1,NK1
   21 F(K,M)=0.
C--------------------------------------------------------------------------
C---------------------------------------------------------HEAT CONDUCTIVITY.
C S1 & S2 se water saturation, CWET: heat conductivity of the formation under
C fully liquid-saturated conditions, CDRY: heat conductivity of the formation
C under desaturated conditions dry conditions (default is CDRY=CWET)
          S1=PAR(N1LM2+NBK+1)
          S2=PAR(N2LM2+NBK+1)
C mop(10)=0; choses type of the interpolation formula
      IF(MOP(10).NE.0) GOTO 22
      S1X=MAX(S1,0.)
      S2X=MAX(S2,0.)
          CON1=CDRY(NMAT1)+SQRT(S1X)*(CWET(NMAT1)-CDRY(NMAT1))
          CON2=CDRY(NMAT2)+SQRT(S2X)*(CWET(NMAT2)-CDRY(NMAT2))
      GOTO23
C
   22 CONTINUE
          CON1=CDRY(NMAT1)+S1*(CWET(NMAT1)-CDRY(NMAT1))
          CON2=CDRY(NMAT2)+S2*(CWET(NMAT2)-CDRY(NMAT2))
   23 CONTINUE
      DCONI=WT1*CON1+WT2*CON2
      CONI=0.
      IF(DCONI.NE.0.) CONI=CON1*CON2/DCONI
      IF(D1.EQ.0.) CONI=CON2
      IF(D2.EQ.0.) CONI=CON1
C
C-----TEMPERATURE GRADIENT.
      T1=PAR(N1LM2+NSEC-1)
      T2=PAR(N2LM2+NSEC-1)
      DTX=(T2-T1)/(D1+D2)
C
C-----CONDUCTIVE HEAT FLOW.
      F(NK1,M)=CONI*DTX*AX
C                                                     *** BEG MODIFICATIONS
      if(m.eq.1) fhc(n)=coni*dtx*ax
C                                                     *** END MODIFICATIONS
C
      IF((MOP(3).GE.6.AND.M.EQ.1).OR.MOP(3).GE.8)
     XPRINT 198,M,N1LM2,N2LM2,PRES1,PRES2,DPX
  198 FORMAT(/'   *** FLUX NO.',I3,'  SECONDARY INDICES (',I5,',',I5,
     X')',15X,'      P1 = ',E12.6,'  P2 = ',E12.6,'    DPX = ',E12.6)
      IF((MOP(3).GE.6.AND.M.EQ.1).OR.MOP(3).GE.8)
     XPRINT 197,CONI,PAR(N1LM2+NSEC-1),
     APAR(N2LM2+NSEC-1),DTX,F(NK1,M)
  197 FORMAT(21X,'       CONI = ',E12.6,'    T1 = ',E12.6,
     A'    T2 = ',E12.6,' DTX = ',E12.6,'   FNK1 = ',E12.6)
C
C-----OBTAIN FLUX FOR EACH PHASE. NP=1 GAS PHASE; NP=2 LIQUID PHASE
C---------------------------------------------------------------------
      DO 3 NP=1,NPH
C---------------------------------------------------------------------
C permeabilities for given phase NP
      PER1=PER(ISO,NMAT1)
      PER2=PER(ISO,NMAT2)
      IF(NP.NE.1) GOTO 30
C-----COME HERE TO ASSIGN KLINKENBERG FACTORS gaseous phase only
      FK1=GK(NMAT1)/PRES1
      FK2=GK(NMAT2)/PRES2
      PER1=PER1*(1.+FK1)
      PER2=PER2*(1.+FK2)
   30 CONTINUE
C
      DPERI=WT1*PER1+WT2*PER2
      PERI=0.
      IF(DPERI.NE.0.) PERI=PER1*PER2/DPERI
C
C-----IDENTIFY BEGINNING OF SECONDARY VARIABLES FOR PHASE NP.
C
      N1L2NP=N1LM2+(NP-1)*NBK
      N2L2NP=N2LM2+(NP-1)*NBK
C
      IF(M.EQ.1) VEL((N-1)*NPH+NP)=0.
C relative permeabilities PAR(2) for phase
      REL1=PAR(N1L2NP+2)
      REL2=PAR(N2L2NP+2)
C
      FNPM=0.
C-----CHECK WHETHER PHASE NP IS MOBILE.
      IF(REL1.EQ.0..AND.REL2.EQ.0.) GOTO31
C FOR PHASE NP: S1=satur.=PAR(1); VIS=visc.=PAR(3); RHO=dens.PAR(4); 
C H1=spec.enth.=PAR(5); PCAP=cap.press.=PAR(6)
C                                                first element in the link:
      S1=PAR(N1L2NP+1)
      VIS1=PAR(N1L2NP+3)
      RHO1=PAR(N1L2NP+4)
      RHO10=PAR(N1LOC2+(NP-1)*NBK+4)
      H1=PAR(N1L2NP+5)
      PCAP1=PAR(N1L2NP+6)
      PCAP10=PAR(N1LOC2+(NP-1)*NBK+6)
C                                               second element in the link:
      S2=PAR(N2L2NP+1)
      VIS2=PAR(N2L2NP+3)
      RHO2=PAR(N2L2NP+4)
      RHO20=PAR(N2LOC2+(NP-1)*NBK+4)
      H2=PAR(N2L2NP+5)
      PCAP2=PAR(N2L2NP+6)
      PCAP20=PAR(N2LOC2+(NP-1)*NBK+6)
C
C-----OBTAIN WEIGHTED INTERFACE DENSITY.
      W1=0.5
      IF(RHO1.EQ.0.) W1=0.
      IF(RHO2.EQ.0.) W1=1.
      W2=1.-W1
      RHOX=W1*RHO1+W2*RHO2
      RHOX0=W1*RHO10+W2*RHO20
C
C-----EFFECTIVE PRESSURE GRADIENT.
      DR=DPX+(PCAP2-PCAP1)/(D1+D2)-RHOX*GX
      DR0=DPX0+(PCAP20-PCAP10)/(D1+D2)-RHOX0*GX
C
C
C-----PERFORM APPROPRIATE UPSTREAM WEIGHTING FOR MOBILITIES.
C
      IF(DR0.GT.0..AND.S2.EQ.0.) GOTO 31
      IF(DR0.LT.0..AND.S1.EQ.0.) GOTO 31
      IF(M11.GE.1) WM1=W1
      IF(M11.EQ.0.AND.DR0.GT.0.) WM1=1.-WUP
      IF(M11.EQ.0.AND.DR0.LE.0.) WM1=WUP
C
      IF(RHO1.EQ.0.) WM1=0.
      IF(RHO2.EQ.0.) WM1=1.
C
      WM2=1.-WM1
C
C-----IF A NODAL POINT FALLS RIGHT ON THE INTERFACE (I.E., NODAL
C     DISTANCE = 0), USE RELATIVE PERMEABILITY OF THE OTHER BLOCK.
      IF(D1.EQ.0..AND.REL1.EQ.0.) REL1=REL2
      IF(D2.EQ.0..AND.REL2.EQ.0.) REL2=REL1
C
C-----USE UPSTREAM WEIGHTING FOR ABSOLUTE PERMEABILITY FOR
C     MOP(11) = 0 OR 1; KEEP HARMONIC WEIGHTING FOR MOP(11) .GT. 1.
C mop(11)=0, mobilities are upstream
      IF(MOP(11).LE.1.AND.DR0.GT.0.) PERI=PER2
      IF(MOP(11).LE.1.AND.DR0.LE.0.) PERI=PER1
C     EXCEPTION: WHEN A NODAL DISTANCE IS ZERO, USE ABSOLUTE
C     PERMEABILITY OF THE OTHER BLOCK.
      IF(D1.EQ.0.) PERI=PER2
      IF(D2.EQ.0.) PERI=PER1
C
C-----INTERFACE MOBILITY.
      IF(MOP(11).EQ.4) THEN
      XM1=PER1*REL1/VIS1
      XM2=PER2*REL2/VIS2
      DEN=WT1*XM1+WT2*XM2
      DMOBI=0.
      IF(DEN.NE.0.) DMOBI=XM1*XM2/DEN
C
      ELSE
      DMOBI=(WM1*REL1/VIS1+WM2*REL2/VIS2)*PERI
      ENDIF
C M is counter from do-loop M=1,NFLUX
      IF(M.NE.1) GOTO 7
C
C
C-----COME HERE TO COMPUTE PORE VELOCITIES FOR GASEOUS AND LIQUID
C     PHASES, RESPECTIVELY (FOR EACH OF THE N CONNECTIONS). 
C     N1 left element N2 right element
C
      PHIS=WM1*PHI(N1)*S1+WM2*PHI(N2)*S2
      IF(PHIS.NE.0.)
     xVEL((N-1)*NPH+NP)=DMOBI*DR/PHIS
CDISP      IF(PHIS.NE.0. .and. np.eq.1) VLCTY=DMOBI*DR/PHIS
    7 CONTINUE
C
C
C-----REDEFINE INTERFACE DENSITY WITH UPSTREAM WEIGHTING. mop(18)=0
      IF(MOP(18).EQ.0.OR.S1.EQ.0..OR.S2.EQ.0.)
     XRHOX=WM1*RHO1+WM2*RHO2
C-----FLUX IN PHASE NP.
      FNPM=DMOBI*RHOX*DR*AX
C----------------------------------------------------------------
c  k=1 water   k=2 air   k=3 CH4   k=4 CO2   nk1=5 heat
      DO 4 K=1,NK
C----------------------------------------------------------------
      XNPMK=WM1*PAR(N1L2NP+NB+K)+WM2*PAR(N2L2NP+NB+K)
C
      F(K,M)=F(K,M)+XNPMK*FNPM
4     continue
C
      HNPM=WM1*H1+WM2*H2
C
C-----HEAT FLUX IN PHASE NP. 
      F(NK1,M)=F(NK1,M)+HNPM*FNPM
C
C-----convective/advective heat flux in gas phase
      if(m.eq.1 .and. np.eq.1) fha(n)=hnpm*fnpm
C
   31 CONTINUE
c#######################################################################
C##################  BINARY DIFFUSION SECTION  #########################
C#######################################################################
C DIFFUSION DRIVEN BY VAPOR OR GAS DENSITY GRADIENTS(gaseous phase only)
C {{ MULTICOMPONENT MOLECULAR DIFFUSION - Bird et al. (1960) pp.604
C DIFFUSION COEFFICIENTS FOR BINARY GAS SYSTEMS - Reid pp.587 (FULLER)}}
C NP=1; if TORT=0 or GasSatur.=PAR(1)=0 or Porosity=PHI=0 go to 32 skokaj
C D1/D2 nodal dist. to interface; S1/2 gas saturation Sg=PAR(N1LOC2+1);
C PHI porosity; AX=area; RHOAV=0.5*(RHO1+RHO2) av. density in gas phase
       IF (DIFF0.eq.0.) GO TO 32
       IF (NP.NE.1) GO TO 32
       IF (TORT(NMAT1).EQ.0..OR.TORT(NMAT2).EQ.0.) GO TO 32
       IF (PAR(N1LOC2+1).LE.0..OR.PAR(N2LOC2+1).LE.0.) GO TO 32
       IF (PHI(N1).EQ.0..OR.PHI(N2).EQ.0.) GO TO 32
C AVERAGE DENSITY of gaseous phase for the two elemenTS
            RHOAV=0.5*(RHO1+RHO2)
C      TORT1=TORT(NMAT1)
C      TORT2=TORT(NMAT2)
c      PRINT *,'ORIGINAL TORT1/2:  ',TORT1,TORT2
C TORTUOSITY LAI (1976): TORT=(n*Sg)**5/3
       if (DIFF0.lt.0.) go to 388
         TORT1=(PHI(N1)*S1)**1.667
         TORT2=(PHI(N2)*S2)**1.667
       go to 389
C TORTUOSITY MILLINGTON&QUIRK (1961): TORT=[(n*Sg)**10/3]/n**2
388     TORT1=((PHI(N1)*S1)**3.333)/PHI(N1)**2.
        TORT2=((PHI(N2)*S2)**3.333)/PHI(N1)**2.
c-----------------------------------------------------------
c      PHIS=SQRT(PHI(N1)*S1*TORT1*PHI(N2)*S2*TORT2)
389    TORTAV=SQRT(TORT1*TORT2)
C mass fractions for computing diff.coeff: D and conc.grad: delx
C  X11&X12 water mass fraction in gas phase       =(1-x2)        H2O     
            X11=PAR(N1L2NP+NB+1)
            X12=PAR(N2L2NP+NB+1)
            delx1=x12-x11
C  X21&X22 nytrogen mass fractions in gas phase   =[x2*(1-x3-x4)] N2
            X21=PAR(N1L2NP+NB+2)
            X22=PAR(N2L2NP+NB+2)
            delx2=x22-x21
C  X31&X32 methane mass fractions in gas phase    =(x2*x3)       CH4
            X31=PAR(N1L2NP+NB+3)
            X32=PAR(N2L2NP+NB+3)
            delx3=x32-x31
C  X41&X42 carbon dioxide mass frac. in gas phase =(x2*x4)       CO2
            X41=PAR(N1L2NP+NB+4)
            X42=PAR(N2L2NP+NB+4)
            delx4=x42-x41
C  COMPUTE DIFFUSION COEFFICIENT. PRES1&PRES2=Pressions
      CALL DIFF(T1,PRES1,X11,X21,X31,X41,
     x        DF11,DF21,DF31,DF41)
      CALL DIFF(T2,PRES2,X12,X22,X32,X42,
     x        DF12,DF22,DF32,DF42)
C  EFFECTIVE DIFFUSION COEFFICIENT AT INTERFACE
      EDIFH2O=0.5*(DF11+DF12)*TORTAV
      EDIFN2= 0.5*(DF21+DF22)*TORTAV
      EDIFCH4=0.5*(DF31+DF32)*TORTAV
      EDIFCO2=0.5*(DF41+DF42)*TORTAV
C IF ONE OF THE DISTANCES TO THE INTERFACE IS ZERO
            IF (D1.EQ.0.) EDIFH2O=DF12*TORT2
            IF (D2.EQ.0.) EDIFH2O=DF11*TORT1
            IF (D1.EQ.0.) EDIFN2= DF22*TORT2
            IF (D2.EQ.0.) EDIFN2= DF21*TORT1
            IF (D1.EQ.0.) EDIFCH4=DF32*TORT2
            IF (D2.EQ.0.) EDIFCH4=DF31*TORT1
            IF (D1.EQ.0.) EDIFCO2=DF42*TORT2
            IF (D2.EQ.0.) EDIFCO2=DF41*TORT1
      IF(BE.EQ.0.) GOTO 33
C
C-----COME HERE TO ASSIGN ENHANCED VAPOR DIFFUSION COEFFICIENT
       EDIFH2O=0.5*(DF11+DF12)*BE
       IF(D1.EQ.0.) EDIFH2O=DF12*BE
       IF(D2.EQ.0.) EDIFH2O=DF11*BE
   33 CONTINUE
C
C-----------------------------------------------
C     GRADIENT OF GAS DENSITY       WATER VAPOR |
            DXR=RHOAV*delx1/(D1+D2)
C     DIFFUSIVE FLUX OF GAS (H2O)               |
            FDIFH2O=AX*EDIFH2O*DXR
            F(1,M)=F(1,M)+FDIFH2O
C-----------------------------------------------
C     GRADIENT OF GAS DENSITY                N2 |
            DXR=RHOAV*delx2/(D1+D2)
C     DIFFUSIVE FLUX OF GAS (N2).               |
            FDIFN2=AX*EDIFN2*DXR
            F(2,M)=F(2,M)+FDIFN2
C-----------------------------------------------
C     GRADIENT OF GAS DENSITY           METHANE |
            DXR=RHOAV*delx3/(D1+D2)
C     DIFFUSIVE FLUX OF GAS (METHANE).          |
            FDIFCH4=AX*EDIFCH4*DXR
c           FDIFCH4=1.5*AX*EDIFCH4*DXR
            F(3,M)=F(3,M)+FDIFCH4
C-----------------------------------------------
C     GRADIENT OF GAS DENSITY    CARBON DIOXIDE |
            DXR=RHOAV*delx4/(D1+D2)
C     DIFFUSIVE FLUX OF GAS (CARBON DIOXIDE).   |
c           FDIFCO2=DIFF0*AX*EDIFCO2*DXR
            FDIFCO2=AX*EDIFCO2*DXR
            F(4,M)=F(4,M)+FDIFCO2
c-----------------------------------------------I
c     if(m.eq.1 .and. n.le.18) print 4532,FDIFH2O,FDIFN2,
c    xFDIFCH4,FDIFCO2
c4532   format('Dh2o',f12.10,'  Dn2',f12.10,'  Dch4',f12.10,
c    x'  Dco2',f12.10)
C                      
C     NOW FOR THE ENTHALPIES.                       
            TAV=0.5*(T1+T2)
            DGASI=0.5*((X21+X31+X41)*RHO1+(X22+X32+X42)*RHO2) 
C                                      
       AMM1=(X21+X31+X41)/((X21/AMN2)+(X31/AMCH4)+(X41/AMCO2))
       AMM2=(X22+X32+X42)/((X22/AMN2)+(X32/AMCH4)+(X42/AMCO2))
       AMM=0.5*(AMM1+AMM2)
       PGASID=RGAS*(TAV+273.15)/AMM
            PGASI=PGASID*DGASI
            PVI=MAX(1.,0.5*(PRES1+PRES2)-PGASI)
            CALL SUPST(TAV,PVI,DVAP,UVAP)
C
C     NOW FOR THE DIFFUSIVE FLUX OF HEAT.           HEAT DIFFUSION:
c because of the elevated energy of water vapor, vapor diffusive flux
c will dominate the diffusive heat transfer
            HVI=UVAP+PVI/DVAP
            CALL SHEAT(TAV,SHN2,SHCH4,SHCO2)
c            HGASI=CVAIR*TAV+PGASID
c            if(m.eq.1 .and. n.eq.1) print 2309,t,hgasi,hvi
c            if(m.eq.1 .and. n.eq.1) print 2309,fdifn2,fdifch4,
c     x                                  fdifco2,fdifh2o
c2309        format(4f20.10)
C            GDIF1=(FDIFN2+FDIFCH4+FDIFCO2)*HGASI+FDIFH2O*HVI
            gdif=fdifn2*(shn2*tav+pgasid)+
     x           fdifch4*(shch4*tav+pgasid)+
     x           fdifco2*(shco2*tav+pgasid)+
     x           fdifh2o*hvi
C
C-----------------------------------------------
C     TOTAL HEAT FLUX                           |
            F(NK1,M)=F(NK1,M)+GDIF
C-----------------------------------------------
C     DIFFUSIVE VAPOR FLUX                      |
      IF(M.EQ.1) FVD(N)=FDIFH2O
C-----------------------------------------------
C     DIFFUSIVE N2 FLUX                         |
      IF(M.EQ.1) FN2D(N)=FDIFN2
C-----------------------------------------------
C     DIFFUSIVE METHANE FLUX                    |
      IF(M.EQ.1) FCH4D(N)=FDIFCH4
C-----------------------------------------------
C     DIFFUSIVE CARBON DIOXIDE FLUX             |
      IF(M.EQ.1) FCO2D(N)=FDIFCO2
C-----------------------------------------------
C     DIFFUSIVE HEAT FLUX                       |
      IF(M.EQ.1) fhd(N)=GDIF
C
C                                                     *** END MODIFICATIONS
C------------------------------------------END OF BINARY DIFFUSION SECTION.
C
   32 CONTINUE
C##################################################################
C##############  TOTAL FLUXES ADVECTION + DIFFUSION ###############
C     *** BEGINNING MODIFICATIONS ***
C
C     TOTAL VAPOR FLUX IN GAS PHASE
      if(m.eq.1 .and. np.eq.1) fvt(n)=f(1,m)
C
C     TOTAL N2 fLUX IN GAS PHASE
      if(m.eq.1 .and. np.eq.1) fn2t(n)=f(2,m)
C
C     TOTAL mETHANE fLUX IN GAS PHASE
      if(m.eq.1 .and. np.eq.1) fch4t(n)=f(3,m)
C
C     TOTAL CARBON DIOXIDE fLUX IN GAS PHASE
      if(m.eq.1 .and. np.eq.1) fco2t(n)=f(4,m)
C gi odzemam total gas flux zasto f se sumiraat
C     CONVECTIVE WATER fLUX IN LIQUID PHASE
      if(m.eq.1 .and. np.eq.2) fh2o(n)=f(1,m)-fvt(n)
C
C     CONVECTIVE N2 fLUX IN LIQUID PHASE
      if(m.eq.1 .and. np.eq.2) fn2l(n)=f(2,m)-fn2t(n)
C
C     CONVECTIVE mETHANE fLUX IN LIQUID PHASE
      if(m.eq.1 .and. np.eq.2) fch4l(n)=f(3,m)-fch4t(n)
C
C     CONVECTIVE CARBON DIOXIDE fLUX IN LIQUID PHASE
      if(m.eq.1 .and. np.eq.2) fco2l(n)=f(4,m)-fco2t(n)
     
C     *** END MODIFICATIONS ***
C
      IF(MOP(3).LT.7) GOTO 190
      IF(MOP(3).LT.9.AND.M.GT.1) GOTO 190
      PRINT 196,NP,N1L2NP,N2L2NP,PER1,PER2,PERI
  196 FORMAT(9X,'NP =',I2,'     SECONDARY INDICES (',I5,1H,,I5,1H),
     X15X,'    PER1 = ',E12.6,' PER2= ',E12.6,'   PERI = ',E12.6)
      PRINT 195,S1,REL1,VIS1,RHO1,H1,PCAP1
  195 FORMAT('       S1   = ',E12.6,'  REL1 = ',E12.6,'  VIS1 = ',E12.6,
     A'  RHO1 = ',E12.6,'  H1 = ',E12.6,'  PCAP1 = ',E12.6)
      PRINT 194,S2,REL2,VIS2,RHO2,H2,PCAP2
  194 FORMAT('       S2   = ',E12.6,'  REL2 = ',E12.6,'  VIS2 = ',E12.6,
     A'  RHO2 = ',E12.6,'  H2 = ',E12.6,'  PCAP2 = ',E12.6)
      PRINT 193,DMOBI,RHOX,DR,FNPM,HNPM
  193 FORMAT('       DMOBI= ',E12.6,'  RHOX = ',E12.6,'    DR = ',E12.6,
     X'  FNPM = ',E12.6,' HNPM= ',E12.6)
C
      PRINT 192,(F(K,M),K=1,NK1)
  192 FORMAT('     * FLOW TERMS',8(1X,E12.6))
  190 CONTINUE
C
C
      IF(M.NE.1) GOTO 3
C-----STORE FLUXES IN EACH PHASE AND HEAT FLUX.
      FLO((N-1)*NPH+NP)=FNPM
C                                   DO 3 NP=1,NPH
    3 CONTINUE
C                                   DO 2 M=1,NFLUX
    2 CONTINUE
C-----total heat flux 
      fht(n)=f(nk1,1)
C
C
C+++++ASSIGN ALL INTERFACE TERMS+++++++++++++++++++++++++++++++++
C
      DO 5 K=1,NEQ
C     K IS THE ROW INDEX WITHIN A BLOCK PERTAINING TO ELEMENT N1 OR N2.
C
C-----COMPUTE FLUX CONTRIBUTIONS TO RESIDUALS.
      IF(N1.LE.NELA)
     XR(N1LOC+K)=R(N1LOC+K)-FAC1*F(K,1)
      IF(N2.LE.NELA)
     XR(N2LOC+K)=R(N2LOC+K)+FAC2*F(K,1)
C
      DO 6 L=1,NEQ
C     L IS THE COLUMN INDEX WITHIN A BLOCK.
      IF(N1.GT.NELA .OR. N2.GT.NELA) GOTO 61
C-----OFF-DIAGONAL TERM IN EQUATION FOR ELEMENT N1, ARISING FROM
C     DEPENDENCE OF COMPONENT K-FLUX UPON VARIABLE L IN ELEMENT N2.
      IRN(NZ+1)=N1LOC+K
      IF(IAB.EQ.0) ICN(NZ+1)=N2LOC+L
      IF(IAB.NE.0) JVECT(NZ+1)=N2LOC+L
      CO(NZ+1)=FAC1*(F(K,L+1+NEQ)-F(K,1))/DELX(N2LOCP+L)
      NZ=NZ+1
C
C-----OFF-DIAGONAL TERM IN EQUATION FOR ELEMENT N2, ARISING FROM
C     DEPENDENCE OF COMPONENT K-FLUX UPON VARIABLE L IN ELEMENT N1.
      IRN(NZ+1)=N2LOC+K
      IF(IAB.EQ.0) ICN(NZ+1)=N1LOC+L
      IF(IAB.NE.0) JVECT(NZ+1)=N1LOC+L
      CO(NZ+1)=-FAC2*(F(K,L+1)-F(K,1))/DELX(N1LOCP+L)
      NZ=NZ+1
   61 CONTINUE
C
C
C-----DIAGONAL TERM IN EQUATION FOR ELEMENT N1, ARISING FROM
C     DEPENDENCE OF COMPONENT K-FLUX UPON VARIABLE L IN ELEMENT N1.
C
C     NOTE THAT EACH CONNECTION INVOLVING N1 WILL GENERATE A TERM
C     AT THE SAME MATRIX LOCATION.
      N1KL=(N1-1)*NEQ*NEQ+(K-1)*NEQ+L
      IF(N1.LE.NELA)
     XCO(N1KL)=CO(N1KL)+FAC1*(F(K,L+1)-F(K,1))/DELX(N1LOCP+L)
C
C-----DIAGONAL TERM IN EQUATION FOR ELEMENT N2, ARISING FROM
C     DEPENDENCE OF COMPONENT K-FLUX UPON VARIABLE L IN ELEMENT N2.
      N2KL=(N2-1)*NEQ*NEQ+(K-1)*NEQ+L
      IF(N2.LE.NELA)
     XCO(N2KL)=CO(N2KL)-FAC2*(F(K,L+1+NEQ)-F(K,1))/DELX(N2LOCP+L)
C
C
    6 CONTINUE
    5 CONTINUE
C
C+++++END OF ASSIGNMENT OF INTERFACE TERMS+++++++++++++++++++++++
C
    1 CONTINUE
C
C
C-----TEST FOR CONVERGENCE----------------------------------------------
C
      RERM=0.
      IF(MOP(3).GE.2) PRINT 203
  203 FORMAT(/' ===== RESIDUALS =====   MASS BALANCES FIRST,',
     X' ENERGY BALANCE LAST'/)
      DO10 N=1,NELA
      NLOC=(N-1)*NEQ
C
c      IF(MOP(3).GE.2) PRINT 202,ELEM(N),(R(NLOC+K),K=1,NEQ)
  202 FORMAT('       AT ELEMENT *',A5,'*   ',8(1X,E12.6))
C
      DO10 K=1,NEQ
      NLM=NLOC+K
      DOA=ABS(DOLD(NLM))
      IF(DOA.LT.RE2) RER=R(NLM)/RE2
      IF(DOA.GE.RE2) RER=R(NLM)/DOLD(NLM)
      IF(ABS(RER).LE.RERM) GOTO 10
      RERM=ABS(RER)
      NER=N
      KER=K
   10 CONTINUE
C
C-----------------------------------------------------------------------
C
C
      RETURN
      END
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
      SUBROUTINE QU
      implicit real*8 (a-h,o-z)
C-----THIS SUBROUTINE COMPUTES ALL TERMS ARISING FROM SINKS AND
C     SOURCES.
C
C
      COMMON/L3/CO(10800000)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/SECPAR/PAR(2640000)
      COMMON/SVZ/NOITE,MOP(24)
C      COMMON/ BCIJT /    IHD,    MAX,    MID,    SET                    JPF 03 2005
      COMMON/ BCIJT /    SET,    IHD,    MAX,    MID
C      COMMON/KONIT/KON,DELT,IGOOD                                       JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G1/F1(25000)
      COMMON/G2/F2(25000)
      COMMON/G3/F3(25000)
      COMMON/G4/ELEG(10000)
      COMMON/G5/SOURCE(10000)
      COMMON/G6/LTABG(10000)
      COMMON/G7/G(10000)
      COMMON/G8/EG(10000)
      COMMON/G9/NEXG(10000)
      COMMON/G10/ITABG(10000)
      COMMON/G11/NGIND(10000)
      COMMON/G12/LCOM(10000)
      COMMON/G13/PI(10000)
      COMMON/G14/PWB(10000)
      COMMON/G15/HG(10000)
      COMMON/G16/GPO(10000)
      COMMON/G17/SDENS(10000)
      COMMON/G18/SSAT(10000)
      COMMON/G19/GVOL(10000)
      COMMON/G20/HL(10000)
      COMMON/G21/HS(10000)
      COMMON/G22/QVGC(10000)
      COMMON/G23/QVWC(10000)
      COMMON/G24/QVOC(10000)
      COMMON/G25/GRAD(10000)
      COMMON/G26/FF(10000)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/E3/EVOL(20000)
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
      COMMON/P3/DELX(100000)
      COMMON/P4/R(100000)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT               JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/DFM/TIMAX,REDLT
C      COMMON/TIMES/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX         JPF 03 2005
      COMMON/BC/NELA
      CHARACTER ELEG*5,SOURCE*5,ITABG*1,ITABA*1
C
      DIMENSION D(11,12)
      DIMENSION ANUM(10),ADEN(10)
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'QU       1.0      22 JANUARY   1990',6X,
     X'ASSEMBLE ALL SOURCE AND SINK TERMS')
C
      IF(MOP(4).GE.1) PRINT 1,KCYC,ITER
    1 FORMAT(/46H QQQQQQQQQQ SUBROUTINE QU QQQQQQQQQQQQQQQQQQQQ,21H  ---
     A [KCYC,ITER] = [,I4,1H,,I3,1H])
      IC=MOP(9)+1
      IHD=16
C
      SETI=SUMTIM
      SETF=SUMTIM+DELTEX
C
C NOGN total number of wells
C NEXG(N) is number of the element (from 1 to NELA) containing well-s/s
      DO3901 N=1,NOGN
C
      J=NEXG(N)
      IF(J.EQ.0.OR.J.GT.NELA) GOTO3900
c ford=deltex
      FAC=FORD/EVOL(J)
C
      JLOC=(J-1)*NEQ
      JLOCP=(J-1)*NK1
C
C WATE: LCOM(N)=      1
c MASS: LCOM(N)=      1
C AIR : LCOM(N)=      2
C DELV: LCOM(N)=NK1+1=6
C VOL.: LCOM(N)=NK1+2=7
      MN=LCOM(N)
C-----IGNORE HEAT SINKS/SOURCES WHEN NOT SOLVING ENERGY EQUATION.
      IF(MN.EQ.NK1.AND.NEQ.NE.NK1) GOTO 3900
C
      JMN=JLOC+MN
      JNK1=JLOC+NK1
      J2LOC=(J-1)*NSEC*NEQ1
C
      LTABA=IABS(LTABG(N))
      ITABA=ITABG(N)
C                               ********** FOR VOL. & DELV GO TO 21
      IF(MN.GE.NK1+1) GOTO 21
      IF(LTABA.LE.1) GOTO 3100
C
C-----COME HERE IF A TABLE OF TIME-DEPENDENT GENERATION DATA IS
C     SPECIFIED.
C
      MAX=LTABA
      SET=SETI
C
      NBG=NGIND(N)
      CALL FINDL(NBG)
      IF(IHD.EQ.0) GOTO4000
C
      IF(MOP(12).EQ.1) Q1=F2(NBG+MID)
      IF(MOP(12).EQ.0) CALL QINTER(Q1,NBG)
C-----Q1 IS THE GENERATION RATE AT BEGINNING OF CURRENT TIME STEP.
C
      IF(ITABA.NE.' '.AND.MOP(12).EQ.1) R1=F3(NBG+MID)
      IF(ITABA.NE.' '.AND.MOP(12).EQ.0) CALL HINTER(R1,NBG)
C
      SET=SETF
      MAX=LTABA
      CALL FINDL(NBG)
      IF(IHD.EQ.0) GOTO4000
C
      IF(MOP(12).EQ.1) Q2=F2(NBG+MID)
      IF(MOP(12).EQ.0) CALL QINTER(Q2,NBG)
C-----Q2 IS THE GENERATION RATE AT END OF CURRENT TIME STEP.
C
      IF(ITABA.NE.' '.AND.MOP(12).EQ.1) R2=F3(NBG+MID)
      IF(ITABA.NE.' '.AND.MOP(12).EQ.0) CALL HINTER(R2,NBG)
C
C      G(N)=(Q1+Q2)/2.
c to have a real step function and not a trapesoid
      g(n)=Q1
c      print 732,n,q1,q2,g(n)
732   format(1i5,5f15.10)
C-----GENERATION RATE IS ASSUMED TO BE THE AVERAGE OF THE VALUES AT
C     BEGINNING AND END OF TIME STEP, RESPECTIVELY.
C
      IF(ITABA.NE.' ') EG(N)=(R1+R2)/2.
C-----FOR (ITABA.NE.' ') A VALUE FOR (PRODUCED OR INJECTED) ENTHALPY
C     IS ASSIGNED.
C
 3100 CONTINUE
C
      GN=G(N)
      GPO(N)=G(N)
C
      IF(MN.NE.NK1) GOTO 20
C-----COME HERE FOR HEAT SOURCE/SINK.
C
      R(JMN)=R(JMN)-FAC*G(N)
      GOTO 3900
C
   20 CONTINUE
C-----COME HERE FOR MASS SOURCE/SINK.
C
      IF(G(N).GE.0.) GOTO 40
C-----FOR INJECTION NEED AN EXPLICIT VALUE FOR SPECIFIC ENTHALPY.
C
      IF(ITABA.NE.' ') GOTO 40
C-----AN EXPLICIT VALUE FOR SPECIFIC ENTHALPY MAY BE SPECIFIED IN
C     THE CASE OF PRODUCTION, IN WHICH CASE IT WILL NOT BE COMPUTED
C     INTERNALLY.
C
C-----COME HERE TO DETERMINE PHASE COMPOSITION AND SPECIFIC ENTHALPY
C     OF SOURCE FLUID.
C                                             *********** for VOL & DELV
   21 CONTINUE
C
C-----LOOP OVER NEQ1 SETS OF PARAMETERS.
      DO 101 M=1,NEQ1
      JLM2=J2LOC+(M-1)*NSEC
C
C-----ZERO OUT BLOCK (J,J).
      DO 1011 K=1,NK1
 1011 D(K,M)=0.
C
      FFS=0.
C-----FFS IS THE SUM OVER MOBILITY*DENSITY IN ALL PHASES.
      VOLR=0.
C-----VOLR IS THE SUM OVER MOBILITIES IN ALL PHASES.
C     FFS/VOLR IS THE PRODUCED DENSITY OF FLUID.
C                                           ********** if VOL.  go to 28
      IF(MN.NE.NK1+1) GOTO 28
      G(N)=0.
      IF(ITER.NE.1.OR.M.NE.1) GOTO 28
      IF(LTABA.LE.1) GOTO 28
C
C=====COME HERE FOR WELL ON DELIVERABILITY, AND COMPUTE GRAVITY
C     CORRECTION TO FLOWING BOTTOMHOLE PRESSURE.========================
C     CODING CAN HANDLE UP TO THREE PHASES FLOWING SIMULTANEOUSLY
C
C     COMPUTE THIS CORRECTION ONCE AND FOR ALL, AT BEGINNING OF EACH
C     TIME STEP, USING NON-INCREMENTED VARIABLES.
C
C     THE GRAVITY-CORRECTIONS FOR FLOWING WELLBORE PRESSURE ARE
C     COMPUTED FOR ALL LAYERS IN WHICH A WELL ON DELIVERABILITY
C     IS OPEN. THE COMPUTATION IS MADE WHEN THE SOURCE CORRESPONDING
C     TO THE BOTTOM LAYER IS ENCOUNTERED. THIS IS SINGLED OUT BY
C     HAVING "TYPE = DELV" AND "LTAB = NUMBER OF LAYERS".
C
      NLTAB1=N+LTABA-1
C
      ANUM(1)=0.
      ADEN(1)=0.
C
C-----LOOP OVER ALL LAYERS FOR THIS WELL. (actually I give one well/layer)
      DO 50 ND=N,NLTAB1
C     ND IS THE GENERATION INDEX.
      JD=NEXG(ND)
      JDLOC=(JD-1)*NEQ
      JDLOCP=(JD-1)*NK1
      JDLOC2=(JD-1)*NSEC*NEQ1
C     COMPUTE PHASE MOBILITIES.
      AMG=0.
      IF(PAR(JDLOC2+3).NE.0.) AMG=PAR(JDLOC2+2)/PAR(JDLOC2+3)
      AMW=0.
      IF(NPH.GE.2.AND.
     XPAR(JDLOC2+3+NBK).NE.0.) AMW=PAR(JDLOC2+2+NBK)/PAR(JDLOC2+3+NBK)
      AMO=0.
      IF(NPH.GE.3.AND.
     XPAR(JDLOC2+3+2*NBK).NE.0.) AMO=PAR(JDLOC2+2+2*NBK)/PAR(JDLOC2
     A+3+2*NBK)
C
C*****COMPUTE VOLUMETRIC PRODUCTION RATES FOR INDIVIDUAL PHASES.
      QVG=PI(ND)*AMG*X(JDLOCP+1)
      QVW=PI(ND)*AMW*X(JDLOCP+1)
      QVO=PI(ND)*AMO*X(JDLOCP+1)
C*****ARRAYS FOR CUMULATIVE PHASE RATES.
      IF(ND.EQ.N) GOTO 52
      QVGC(ND)=QVGC(ND-1)+QVG
      QVWC(ND)=QVWC(ND-1)+QVW
      QVOC(ND)=QVOC(ND-1)+QVO
      GOTO 53
   52 QVGC(ND)=QVG
      QVWC(ND)=QVW
      QVOC(ND)=QVO
   53 CONTINUE
C*****ARRAY OF GRADIENTS.
      GRAD(ND)=(QVGC(ND)*PAR(JDLOC2+4)+QVWC(ND)*PAR(JDLOC2+NBK+4)
     A+QVOC(ND)*PAR(JDLOC2+2*NBK+4))*9.80665
     B/(QVGC(ND)+QVWC(ND)+QVOC(ND))
   50 CONTINUE
C
C-----NOW PERFORM LOOP GOING FROM TOP LAYER DOWN.
C
      N1=N+1
      DO51 ND=N1,NLTAB1
      NDD=NLTAB1+N-ND
C     NDD IS THE RUNNING SOURCE INDEX, GOING FROM THE LAYER JUST BELOW
C     THE TOP (INDEX N+LTABA-2) DOWN TO THE BOTTOM LAYER (INDEX N).
C
      PWB(NDD)=PWB(NDD+1)+(HG(NDD+1)*GRAD(NDD+1)+HG(NDD)*GRAD(NDD))/2.
   51 CONTINUE
C                                            *********************** VOL.
   28 CONTINUE
C
C=====END OF DELIVERABILITY SECTION.====================================
C
C
C-----NOW COMPUTE FRACTIONAL FLOWS (FF) IN THE VARIOUS PHASES.
C
      DO 30 NP=1,NPH
      J2LNP=JLM2+(NP-1)*NBK
      FF((N-1)*NPH+NP)=0.
      IF(IC.EQ.1.AND.PAR(J2LNP+3).NE.0.)
     AFF((N-1)*NPH+NP)=PAR(J2LNP+2)*PAR(J2LNP+4)/PAR(J2LNP+3)
      IF(IC.EQ.2) FF((N-1)*NPH+NP)=PAR(J2LNP+1)*PAR(J2LNP+4)
      IF(MN.NE.NK1+1) GOTO 29
C
C=====COMPUTE FLOW RATES FOR EACH PHASE AND TOTAL RATE FOR WELL ON
C     DELIVERABILITY.===================================================
C
      DPRES=0.
      IF(M.EQ.2) DPRES=DELX(JLOCP+1)
      DELP=X(JLOCP+1)+DX(JLOCP+1)+DPRES+PAR(J2LNP+6)-PWB(N)
      IF(DELP.LE.0.) DELP=0.
C-----COMPUTE MASS FLOW RATE.
      FF((N-1)*NPH+NP)=FF((N-1)*NPH+NP)*PI(N)*DELP
      G(N)=G(N)-FF((N-1)*NPH+NP)
   29 CONTINUE
C
C=====END OF DELIVERABILITY SECTION.====================================
C
      IF(PAR(J2LNP+4).NE.0.) VOLR=VOLR+FF((N-1)*NPH+NP)/PAR(J2LNP+4)
      FFS=FFS+FF((N-1)*NPH+NP)
      IF(MOP(4).GE.5) PRINT 3,M,NP,PAR(J2LNP+2),PAR(J2LNP+3),
     APAR(J2LNP+4),FF((N-1)*NPH+NP),VOLR,FFS,DELP
    3 FORMAT(3H M=,I2,4H NP=,I1,3H K=,E12.6,5H VIS=,E12.6,3H D=,E12.6,4H
     A FF=,E12.6,6H VOLR=,E12.6,5H FFS=,E12.6,6H DELP=,E12.6)
C
   30 CONTINUE
C IF VOLUMETRIC WELL: pretvara GVOL m3/s vo G kg/s,
      IF(MN.EQ.NK1+2.AND.VOLR.NE.0.)
     AG(N)=GVOL(N)*FFS/VOLR
      if(g(n).lt.0.) EG(N)=0.
C
C-----RENORMALIZE FRACTIONAL FLOWS TO 1, AND COMPUTE CONTRIBUTIONS
C     OF COMPONENTS IN PHASES.
C
      DO 31 NP=1,NPH
      J2LNP=JLM2+(NP-1)*NBK
      IF(FFS.NE.0.)
     AFF((N-1)*NPH+NP)=FF((N-1)*NPH+NP)/FFS
      if(g(n).lt.0.) EG(N)=EG(N)+FF((N-1)*NPH+NP)*PAR(J2LNP+5)
C
      GN=G(N)
C
      DO 32 K=1,NK
      D(K,M)=D(K,M)-FAC*GN*FF((N-1)*NPH+NP)*PAR(J2LNP+NB+K)
   32 CONTINUE
c
   31 CONTINUE
C
      D(NK1,M)=D(NK1,M)-FAC*GN*EG(N)
C
  101 CONTINUE
C 
      GPO(N)=GN
C
      IF(MOP(4).GE.7) PRINT 4,AMG,AMW,AMO,PWB(N)
    4 FORMAT(10H    AMG = ,E12.6,9H   AMW = ,E12.6,8H  AMO = ,E12.6,
     B9H  PWBN = ,E12.6)
C
C
C-----ASSIGN TERMS FOR LINEAR EQUATIONS.
C
      DO 105 K=1,NEQ
      R(JLOC+K)=R(JLOC+K)+D(K,1)
C
      DO 106 L=1,NEQ
      JKL=(J-1)*NEQ*NEQ+(K-1)*NEQ+L
      CO(JKL)=CO(JKL)-(D(K,L+1)-D(K,1))/DELX(JLOCP+L)
  106 CONTINUE
C
  105 CONTINUE
C
      GOTO 3900
C
   40 CONTINUE
C-----COME HERE FOR MASS INJECTION.
C
      IF(NEQ.EQ.NK1) R(JNK1)=R(JNK1)-FAC*G(N)*EG(N)
      R(JMN)=R(JMN)-FAC*G(N)
c if WATER injection
      if(lcom(n).eq.1) ff((n-1)*nph+1)=0.
      if(lcom(n).eq.1) ff((n-1)*nph+2)=1.
c if AIR (N2) injection
      if(lcom(n).eq.2) ff((n-1)*nph+1)=1.
      if(lcom(n).eq.2) ff((n-1)*nph+2)=0.
C
 3900 CONTINUE
C
C
      IF(MOP(4).GE.2) PRINT 2,ELEG(N),SOURCE(N),G(N),EG(N),
     A(FF((N-1)*NPH+NP),NP=1,NPH)
    2 FORMAT(9H ELEMENT ,A5,10H   SOURCE ,A5,21H   ---   FLOW RATE = ,E1
     A2.6,23H   SPECIFIC ENTHALPY = ,E12.6/8H FNP1 = ,E12.6,10H   FNP2 =
     B ,E12.6,10H   FNP3 = ,E12.6,10H   FNP4 = ,E12.6)
C
C
 3901 CONTINUE
      RETURN
C
 4000 IGOOD=3
      PRINT 4001,KCYC,ELEG(N),SOURCE(N)
 4001 FORMAT(11H AT KCYC = ,I4,41H EXCEED GENERATION TIME TABLE AT ELEME
     ANT ,A5,9H (SOURCE ,A5,22H) -- WILL REDUCE DELTE)
      RETURN
C
      END
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 6xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx    no changes    xx
      SUBROUTINE LINEQ
      implicit real*8 (a-h,o-z)
C-----THIS SUBROUTINE CALLS THE LINEAR EQUATION SOLVER.
C     IT HAS LOGIC TO HANDLE FAILURES IN LINEAR EQUATION SOLUTION.
C
C     AFTER SOLUTION, LATEST UPDATED ITERATES ARE OBTAINED FOR
C     ALL PRIMARY DEPENDENT VARIABLES.
C
C
C
C$$$$$$$$$ COMMON BLOCKS FOR LINEAR EQUATIONS $$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/L1/IRN(5400000)
      COMMON/L2/ICN(10800000)
      COMMON/L3/CO(10800000)
      COMMON/L4/WKAREA(100000)
      COMMON/L5/IKEEP(500000)
      COMMON/L6/IW(800000)
      COMMON/L7/JVECT(2700000)
      COMMON/L8/W19A(500000)
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/E1/ELEM(20000)
      COMMON/P2/DX(100000)
      COMMON/P4/R(100000)
      COMMON/P6/ROW(100000)
      COMMON/P7/COL(100000)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
C      COMMON/KONIT/KON,DELT,IGOOD                                                        JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      COMMON/DG/WUP,WNR
C      COMMON/MA28F/EPS,RMIN,RESID,IRNCP,ICNCP,MINIRN,MINICN,IRANK,                       JPF 03 2005
C     AABORT1,ABORT2
      COMMON/MA28F/ABORT1,ABORT2,EPS,RMIN,RESID,IRNCP,ICNCP,MINIRN,
     AMINICN,IRANK
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                                   JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/LDIM/LICN,LIRN
      COMMON/BC/NELA
      CHARACTER*5 ELEM
C
      SAVE ICALL,N
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'LINEQ    1.0      22 JANUARY   1990',6X,
     X'INTERFACE FOR THE LINEAR EQUATION SOLVER MA28')
C
      ISCALE=0
      IF(MOP(17).GE.7) ISCALE=1
      IF(MOP(17).GE.5.AND.ITER.EQ.1) ISCALE=1
      MA=MA+1
      IF(MA.EQ.1) N=NEQ*NELA
      IF(ISCALE.EQ.0) GOTO 300
C
  128 CONTINUE
C-----COME HERE TO COMPUTE SCALING FACTORS.
      IF(IAB.EQ.0) CALL MC19A(N,NZ,CO,IRN,ICN,ROW,COL,W19A)
      IF(IAB.NE.0) CALL MC19A(N,NZ,CO,IRN,JVECT,ROW,COL,W19A)
      DO301 I=1,N
      ROW(I)=EXP(ROW(I))
      COL(I)=EXP(COL(I))
  301 CONTINUE
  300 CONTINUE
C
C
C-----FOR IAB.EQ.0 CALL MA28A, OTHERWISE CALL MA28B.
C
      INUM=0
      IGOOD=0
C-----MA COUNTS CALLS TO LINEQ.
C
      IF(MOP(6).NE.0) PRINT 109,MA,N,NZ
  109 FORMAT(/51H !!!!! L I N E Q !!!!!     !!!!!     !!!!!     MA =,I4,
     A'   N =',I4,'   NZ =',I6)
C
  117 CONTINUE
C
      IF(ISCALE.EQ.0) GOTO 200
C
C-----COME HERE TO SCALE MATRIX.
      DO 201 II=1,NZ
      I=IRN(II)
      IF(IAB.EQ.0) J=ICN(II)
      IF(IAB.NE.0) J=JVECT(II)
  201 CO(II)=CO(II)*ROW(I)*COL(J)
  200 CONTINUE
C
C
      IF(MOP(6).LT.7) GOTO99
      PRINT 102
  102 FORMAT(/23H MATRIX OF COEFFICIENTS/)
      IF(IAB.NE.0) PRINT 100,(IRN(NN),JVECT(NN),CO(NN),NN=1,NZ)
      IF(IAB.EQ.0) PRINT 100,(IRN(NN),ICN(NN),CO(NN),NN=1,NZ)
  100 FORMAT(5(1X,2I5,E14.6))
   99 CONTINUE
C
C
C
      IF(MOP(6).NE.0) CALL THYME(0,TS,TT)
C
      IF(IAB.EQ.0)
     ACALL MA28A(N,NZ,CO,LICN,IRN,LIRN,ICN,U,IKEEP,IW,WKAREA,IFLAG)
C
C
      IF(IAB.NE.0)
     ACALL MA28B(N,NZ,CO,LICN,IRN,JVECT,ICN,IKEEP,IW,WKAREA,IFLAG)
C
      IAB=1
C
      IF(IFLAG.EQ.-2) GOTO 2
      INUM=0
C-----IF NO NUMERICAL SINGULARITY IS ENCOUNTERED, SET INUM=0.
      IF(IFLAG.LE.0) GOTO115
C
C=====PIVOT FAILURE=====PIVOT FAILURE=====PIVOT FAILURE=====
C
      IPIV=IPIV+1
C
      IF(MOP(17).EQ.2.OR. MOP(17).EQ.3) GOTO 137
      GOTO 138
  137 ISCALE=1
      IF(MOP(6).NE.0)
     XPRINT 140,IFLAG,IPIV
  140 FORMAT(21H PIVOT FAILURE IN ROW,I4,19H --- RESCALE MATRIX,13H   ==
     A= IPIV =,I5)
      GOTO 128
  138 CONTINUE
C
      IF(MOP(14).EQ.0) GOTO1
C-----COME HERE TO PROCEED AFTER PIVOT FAILURE WITHOUT MAKING NEW
C     DECOMPOSITION.
      IF(MOP(6).NE.0)
     XPRINT 118,IFLAG,IPIV
  118 FORMAT(24H VERY SMALL PIVOT IN ROW,I4,13H  ===  IPIV =,I5)
C     FOR MOP(14)  > 0 IGNORE PIVOT PROBLEM AND PROCEED.
      GOTO 115
C
C
C-----COME HERE FOR PIVOT FAILURE IN MA28B, AND PROCEED TO CALL
C     MA28A TO PERFORM A NEW DECOMPOSITION.
C
    1 IF(MOP(6).NE.0) PRINT 116,IFLAG,IPIV
  116 FORMAT(24H VERY SMALL PIVOT IN ROW,I4,18H  ---  PERFORM NEW,14H DE
     ACOMPOSITION,12H  === IPIV =,I5)
      GOTO 3
C
    2 CONTINUE
      IF(MOP(17).EQ.1.OR. MOP(17).EQ.3) GOTO 127
      GOTO 20
  127 ISCALE=1
      IF(MOP(6).NE.0) PRINT 141
  141 FORMAT(74H MATRIX NUMERICALLY SINGULAR ---------------------------
     A   PERFORM SCALING)
      GOTO 128
C
   20 PRINT 4
    4 FORMAT(84H MATRIX NUMERICALLY SINGULAR ---------------------------
     A   PERFORM NEW DECOMPOSITION)
C
      INUM=INUM+1
C-----COUNT THE NUMBER OF SINGULARITIES THAT ARISE WITHOUT
C     INTERRUPTION BY A NON-SINGULAR DECOMPOSITION.
C
      IAB=0
      IF(INUM.EQ.2) GOTO 107
C-----IF TWO SUBSEQUENT DECOMPOSITIONS ENCOUNTER A SINGUL@RITY,
C     REDUCE THE TIME STEP.
C
    3 IAB=0
C
C-----RECOMPUTE MATRIX----------
      CALL MULTI
C
      IF(IGOOD.EQ.0) GOTO117
      PRINT 119
  119 FORMAT(54H ++++++++++++++++++++ MULTI FAILS ++++++++++++++++++++)
      RETURN
C
  115 CONTINUE
C
C
      IF(MOP(6).EQ.0) GOTO110
      CALL THYME(1,TD,TT)
      PRINT 111,TD,IFLAG
  111 FORMAT(22H DECOMPOSITION TIME = ,E12.4,9H  SECONDS,21H   *********
     A* IFLAG =,I4)
  110 CONTINUE
C
C
      IF(IFLAG.LT.0.AND.IFLAG.GE.-12) GOTO107
C
C
      IF(MOP(6).NE.0) CALL THYME(0,TS,TT)
C
      IF(ISCALE.EQ.0) GOTO 210
C
C-----COME HERE TO SCALE RIGHT HAND SIDE VECTOR.
      DO211 II=1,N
  211 R(II)=R(II)*ROW(II)
  210 CONTINUE
C
      CALL MA28C(N,CO,LICN,ICN,IKEEP,R,WKAREA,1)
C
      IF(ISCALE.EQ.0) GOTO 220
      DO 221 II=1,N
  221 R(II)=R(II)*COL(II)
  220 CONTINUE
C
      IF(MOP(6).EQ.0) GOTO112
      CALL THYME(1,TSS,TT)
      PRINT 113,TSS
  113 FORMAT(22H      SOLUTION TIME = ,E12.4,9H  SECONDS)
  112 CONTINUE
C
      IF(MOP(6).GE.2) PRINT 114,EPS,RMIN,RESID,IRNCP,ICNCP,MINIRN,
     AMINICN,IRANK
  114 FORMAT(7H EPS = ,E10.4,9H  RMIN = ,E10.4,10H  RESID = ,E10.4,9H  I
     ARNCP =,I6,9H  ICNCP =,I6,10H  MINIRN =,I6,10H  MINICN =,I6,9H  IRA
     BNK =,I4)
C
      GOTO103
C
  107 IGOOD=1
      IAB=0
  108 CONTINUE
C-----COME HERE FOR FAILURE IN SOLVING LINEAR EQUATION.
      PRINT 104,KCYC,ITER,IFLAG
      IF(IFLAG.GE.0) RETURN
  104 FORMAT(17H LINEQ FAILS AT (,I4,1H,,I3,14H)      IFLAG =,I4,16HWILL
     A REDUCE DELT)
C
      IFL=-IFLAG
      GOTO(402,202,203,204,205,206,207,208,209,410,411,212),IFL
  402 PRINT 401
      RETURN
  202 PRINT 302
      RETURN
  203 PRINT 303,MINIRN
      RETURN
  204 PRINT 304
      RETURN
  205 PRINT 305,MINICN
      RETURN
  206 PRINT 306,MINIRN,MINICN
      RETURN
  207 PRINT 307
      RETURN
  208 PRINT 308,NZ
      RETURN
  209 PRINT 309,NZ
      RETURN
  410 PRINT 310,NZ
      RETURN
  411 PRINT 311
      RETURN
  212 PRINT 312
      RETURN
  401 FORMAT(32H MATRIX IS STRUCTURALLY SINGULAR)
  302 FORMAT(31H MATRIX IS NUMERICALLY SINGULAR)
  303 FORMAT(45H INCREASE DIMENSION OF ARRAY IRN TO AT LEAST ,I6,9H ELEM
     AENTS)
  304 FORMAT(51H DIMENSIONS OF ARRAYS ICN AND CO ARE MUCH TOO SMALL)
  305 FORMAT(53H INCREASE DIMENSION OF ARRAYS ICN AND CO TO AT LEAST ,I7
     A,9H ELEMENTS)
  306 FORMAT(47H INSUFFICIENT DIMENSIONS; NEED AT LEAST  IRN - ,I6,27H E
     ALEMENTS;   ICN AND CO  - ,I7,9H ELEMENTS)
  307 FORMAT(33H ERROR IN BLOCK TRIANGULARIZATION)
  308 FORMAT(28H MAKE LIRN LARGER THAN NZ = ,I7)
  309 FORMAT(28H MAKE LICN LARGER THAN NZ = ,I7)
  310 FORMAT(41H NUMBER OF NON-ZERO MATRIX ELEMENTS NZ = ,I7,25H MUST BE
     A LARGER THAN ZERO)
  311 FORMAT(44H ORDER OF MATRIX MUST BE BETWEEN 1 AND 32767)
  312 FORMAT(46H ROW OR COLUMN INDEX IS OUT OF RANGE, INCREASE,17H ARRAY
     A DIMENSIONS)
C
  103 CONTINUE
C
C-----UPDATE CHANGES.
C
      IF(MOP(6).GE.5) PRINT  31
   31 FORMAT(/' ===== INCREMENTS ====   MASS BALANCES FIRST,',
     X' ENERGY BALANCE LAST'/)
      DO5 NN=1,NELA
      NLOC=(NN-1)*NEQ
      NLOCP=(NN-1)*NK1
C
      IF(MOP(6).GE.5) PRINT 30,ELEM(NN),(R(NLOC+K),K=1,NEQ)
   30 FORMAT('       AT ELEMENT *',A5,'*   ',8(1X,E12.6))
C
      DO5 K=1,NEQ
      DX(NLOCP+K)=DX(NLOCP+K)+WNR*R(NLOC+K)
    5 CONTINUE
C
C
      ISCALE=0
      RETURN
      END
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 7xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE CONVER
      implicit real*8 (a-h,o-z)
c
C-----THIS SUBROUTINE IS CALLED AFTER SUCCESFULL COMPLETION OF
C     A TIME STEP.
C     IT UPDATES PRIMARY VARIABLES, AND DEFINES THE NEXT TIME STEP.
C
C
      COMMON/SECPAR/PAR(2640000)
      COMMON/E2/MATX(20000)
      COMMON/E4/PHI(20000)
      COMMON/E5/P(20000)
      COMMON/E6/T(20000)
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
C
      COMMON/G7/G(10000)
      COMMON/G12/LCOM(10000)
      COMMON/G15/HG(10000)
C
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/KC/KC
      COMMON/DFM/TIMAX,REDLT
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
C      COMMON/DLT/NDLT,DLT(100)                                        JPF 03 2005
      COMMON/DLT/DLT(100),NDLT
c      COMMON/KONIT/KON,DELT,IGOOD                                     JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
C      COMMON/TIMES/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX          JPF 03 2005
      COMMON/TIMES/DELAF,ITI,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
      COMMON/BC/NELA
C
      DIMENSION DXM(10)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'CONVER   1.0       4 MARCH     1991',6X,
     X'UPDATE PRIMARY VARIABLES AFTER CONVERGENCE IS ACHIEVED')
C
      DO10 K=1,NK1
   10 DXM(K)=0.
C
      DO3 N=1,NELA
      NLOC=(N-1)*NK1
      NLOC2=(N-1)*NSEC*NEQ1
C-----COMPUTE CHANGES IN POROSITY.
      PHIN=PHI(N)
      NMAT=MATX(N)
      DPHI=PHIN*(COM(NMAT)*DX(NLOC+1)+EXPAN(NMAT)*(PAR(NLOC2+NSEC-1)
     A-T(N)))
      PHI(N)=PHIN+DPHI
C
C-----UPDATE ELEMENT PRESSURES AND TEMPERATURES.
      P(N)=X(NLOC+1)+DX(NLOC+1)
      T(N)=PAR(NLOC2+NSEC-1)
C
C-----INCREMENT PRIMARY VARIABLES.
      DO3 M=1,NEQ
      NLM=NLOC+M
      X(NLM)=X(NLM)+DX(NLM)
      DXM(M)=MAX(DXM(M),ABS(DX(NLM)))
c      IF(M.eq.1) print *,'CONVER:  P(N),T(N),X(NLM),DX(NLM)    ',
c     1                    P(N),T(N),X(NLM),DX(NLM)
    3 CONTINUE
C
C-----FOR PERCENTAGE INJECTION, ASSIGN INJECTION RATE FOR NEXT
C     TIME STEP.
C%    DO 30 N=1,NOGN
C%    IF(LCOM(N).NE.NEQ+4) GOTO 30
C%    G(N)=-HG(N)*G(N-1)
C% 30 CONTINUE
C
C
      SUMTIM=SUMTIM+DELTEX
      IF(TIMAX.NE.0..AND.TIMAX.EQ.SUMTIM) NOWTIM=1
C-----AFTER CONVERGENCE UPDATE TOTAL TIME AND ASSIGN NEW TIME STEP.
      IF(NDLT.EQ.0) GOTO20
      IF(KC+1.GT.8*NDLT.OR.DLT(KC+1).EQ.0.) GOTO20
C-----IF NO FURTHER TIME STEP INSTRUCTIONS ARE PROVIDED, KEEP
C     GOING WITH LAST TIME STEP.
C-----COME HERE FOR NEW TIME STEP ASSIGNMENT.
      DELT=DLT(KC+1)
      GOTO 22
   20 DELT=DELTEX
C                                             *** BEG MODIFICATIONS ***
C if it does not succeed by 8 iteration it decreases the time step
c dT=dT/2 in CYCIT. However if it succeeds in  LE.2  iterations it 
C increases the time step dT=1.3dT in CONVER
      IF(ITER.LE.MOP(16)) DELT=1.3*DELTEX
C                                             *** END MODIFICATIONS ***
   22 IF(TIMAX.NE.0.) DELT=MIN(DELT,TIMAX-SUMTIM)
      IF(DELTMX.NE.0.) DELT=MIN(DELT,DELTMX)
C
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 8xxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no changes  xxxx
      SUBROUTINE WRIFI
      implicit real*8 (a-h,o-z)
C-----THIS SUBROUTINE IS CALLED AT THE COMPLETION OF A TOUGH2-RUN.
C     IT GENERATES A FILE *SAVE* TO BE USED FOR RESTARTING.
C
C
      COMMON/E1/ELEM(20000)
      COMMON/E4/PHI(20000)
      COMMON/P1/X(100000)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT                   JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
c      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)            JPF 03 2005
      COMMON/SOLID/DM(27),POR(27),PER(3,27),CWET(27),SH(27),NM
      COMMON/SOCH/MAT(27)
      COMMON/BALA/TFM0(27),TVM0(27),TFE0(27),TVE0(27),TSE0(27)
      COMMON/POV6/TSTART
      CHARACTER*5 ELEM,MAT
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
      PRINT *,'TU AS FAIT UN BON TRAVAIL MON PIPO PREFERE'
  899 FORMAT(6X,'WRIFI    1.0      22 JANUARY   1990',6X,
     X'AT THE COMPLETION OF A TOUGH2 RUN, WRITE PRIMARY VARIABLES',
     X' ON FILE *SAVE*')
C
      KCYC=KCYC-1
C
      REWIND 7
      PRINT 5,KCYC,SUMTIM
    5 FORMAT(/' WRITE FILE *SAVE* AFTER',I5,' TIME STEPS  ---  THE',
     X' TIME IS ',E12.6,' SECONDS'/)
      WRITE(7,1) NEL,SUMTIM
    1 FORMAT(31HINCON -- INITIAL CONDITIONS FOR,I5,17H ELEMENTS AT TIME,
     AE14.6)
      DO 10 N=1,NEL
      NLOC=(N-1)*NK1
C
      WRITE(7,2) ELEM(N),PHI(N),(X(NLOC+I),I=1,NK1)
    2 FORMAT(A5,10X,E15.8/(5E20.13))
   10 CONTINUE
C
      WRITE(7,3)
    3 FORMAT(5H+++  )
C
      WRITE(7,4) KCYC,ITERC,NM,TSTART,SUMTIM
    4 FORMAT(3I5,2E15.8)
C
      ENDFILE 7
C
      RETURN
C
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 9xxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no changes xxxx
      SUBROUTINE FINDER(VECT,N)
      implicit real*8 (a-h,o-z)
C-----FINDER LOCATES GIVEN VALUE AMONG ENTRIES OF GIVEN VECTOR.         FINDER.4
C                                                                       FINDER.5
C      COMMON/ BCIJT /    IHD,    MAX,    MID,    SET                   JPF 03 2005
      COMMON/ BCIJT /    SET,    IHD,    MAX,    MID
      COMMON/  CDT7 /     HC
C                                                                       FINDER.9
      DIMENSION VECT(N)
C                                                                       FINDER.1
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'FINDER   1.0      22 JANUARY   1990',6X,
     X'INTERPOLATE FROM A TABLE OF TIME-DEPENDENT DATA')
C
      MIN = 1
      IF(SET    .LT. VECT(MIN) .OR. SET .GT. VECT(MAX)) GO TO 5
 1    MID = (MIN + MAX)/2
      IF(SET - VECT(MID)) 2, 4, 3
 2    MAX = MID
      IF(MAX - 2) 4, 1, 1
 3    MIN = MID
      IF(MAX - MIN .GE. 2) GO TO 1
 4    RETURN
    5 CONTINUE
      PRINT 5000,SET,VECT(MIN),VECT(MAX)
      IHD    = 0
      RETURN
C                                                                       FINDER.2
 5000 FORMAT(1X,E12.6,24H IS OUTSIDE THE RANGE  (,E11.4,1H,,E11.4,1H))
C                                                                       FINDER.3
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 10xxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx   no changes   xxxxx
      SUBROUTINE FINDL(NB)
      implicit real*8 (a-h,o-z)
C                                                                       FINDER.3
C-----FINDER LOCATES GIVEN VALUE AMONG ENTRIES OF GIVEN VECTOR.         FINDER.4
C                                                                       FINDER.5
      COMMON/G1/F1(25000)
C      COMMON/ BCIJT /    IHD,    MAX,    MID,    SET                   JPF 03 2005
      COMMON/ BCIJT /    SET,    IHD,    MAX,    MID
      COMMON/  CDT7 /     HC
C                                                                       FINDER.9
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'FINDL    1.0      22 JANUARY   1990',6X,
     X'INTERPOLATE FROM A TABLE OF TIME-DEPENDENT DATA')
C                                                                       FINDER.1
      MIN = 1
      IF(SET    .LT. F1(NB+MIN) .OR. SET .GT. F1(NB+MAX)) GO TO 5
 1    MID = (MIN + MAX)/2
      IF(SET - F1(NB+MID)) 2, 4, 3
 2    MAX = MID
      IF(MAX - 2) 4, 1, 1
 3    MIN = MID
      IF(MAX - MIN .GE. 2) GO TO 1
 4    RETURN
    5 CONTINUE
      PRINT 5000,SET,F1(NB+MIN),F1(NB+MAX)
      IHD    = 0
      RETURN
C                                                                       FINDER.2
 5000 FORMAT(1X,E12.6,24H IS OUTSIDE THE RANGE  (,E11.4,1H,,E11.4,1H))
C                                                                       FINDER.3
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 11xxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no changes  xxxxxxx
      SUBROUTINE QINTER(Q,NBG)
      implicit real*8 (a-h,o-z)
C*****THIS ROUTINE USES LINEAR INTERPOLATION TO COMPUTE GENERATION
C     RATE AT TIME T=SET.
C
C
      COMMON/G1/F1(25000)
      COMMON/G2/F2(25000)
C      COMMON/BCIJT/IHD,MAX,MID,SET                                       JPF 03 2005
      COMMON/ BCIJT /    SET,    IHD,    MAX,    MID
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'QINTER   1.0      22 JANUARY   1990',6X,
     X'PERFORM LINEAR INTERPOLATION')
C
      Q=F2(NBG+MID)+(SET-F1(NBG+MID))*(F2(NBG+MID+1)-F2(NBG+MID))
     A                               /(F1(NBG+MID+1)-F1(NBG+MID))
C
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 12xxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no changes  xxxxxxx
      SUBROUTINE HINTER(Q,NBG)
      implicit real*8 (a-h,o-z)
C*****THIS ROUTINE USES LINEAR INTERPOLATION TO COMPUTE FLOWING
C     ENTHALPY AT TIME T=SET.
C
C
      COMMON/G1/F1(25000)
      COMMON/G3/F3(25000)
C      COMMON/BCIJT/IHD,MAX,MID,SET                                   JPF 03 2005
      COMMON/ BCIJT /    SET,    IHD,    MAX,    MID
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'HINTER   1.0      22 JANUARY   1990',6X,
     X'PERFORM LINEAR INTERPOLATION')
      Q=F3(NBG+MID)+(SET-F1(NBG+MID))*(F3(NBG+MID+1)-F3(NBG+MID))
     A                               /(F1(NBG+MID+1)-F1(NBG+MID))
C
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 13xxxxxxxxxxx  no changes  xxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE THYME(IFLAG,DT,TOTAL)                                  WHV.95
      implicit real*8 (a-h,o-z)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'THYME    1.0      22 JANUARY   1990',6X,
     X'CALCULATE ELAPSED CPU TIME')
C 7 fev 05     CALL SECOND(T2)
      CALL SECOND(T2)
      IF(IFLAG.NE.0) GO TO 1
      T1=T2
      RETURN
    1 DT=T2-T1
      TOTAL=T2
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 14xxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE TSTEP
      implicit real*8 (a-h,o-z)
C-----THIS ROUTINE MODIFIES TIME STEPS TO COINCIDE WITH USER-
C     INPUT VALUES AT WHICH PRINTOUT IS DESIRED.
C
C      COMMON/TIMES/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX         JPF 03 2005
      COMMON/TIMES/DELAF,ITI,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT               JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
C      COMMON/KONIT/KON,DELT,IGOOD                                     JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
cmn-start
      common/dm/delten,deltex,for,ford
      common/dfm/timax,redlt
      common/g6/ltabg(10000)
      common/g1/f1(25000)
      common/nn/nel,ncon,nogn,nk,neq,nph,nb,nk1,neq1,nbk,nsec,nflux
cmn-end
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'TSTEP    1.0          MARCH    1995',6X,
     X'ADJUST TIME STEPS TO COINCIDE WITH USER-DEFINED TARGET',
     X' TIMES')
C SUMTIM  is the elapsed time from the begining   
C SUMTIM=TSTART najprvin pa posledovatelno  SUMTIM=SUMTIM+DELTEX
      IF(TIS(ITI).LE.SUMTIM) GOTO 100
c
cmn-start
c adjusting the time step to coincide with the user input first well 
c time table (times at which pumping/injection rate change value)
c It is assumed that well data are given in the order that the wells 
c with time tables precede other wells, and that time tables coincide.
c It is also assumed that wells time tables coincide with output times.
      if(nogn.eq.0) go to 11
      if(ltabg(1).le.1) go to 11
      do 12 m=1,ltabg(1)
      if(f1(m).eq.sumtim) go to 11
      if(f1(m).lt.sumtim) go to 12
      go to 13
12    continue
13    if(sumtim+delt.lt.f1(m)) go to 11
      delt=f1(m)-sumtim
cmn-end
C
C-----FIND TIME FOLLOWING SUMTIM.
11    DO1 I=1,ITI
      IF(TIS(I).EQ.SUMTIM) GOTO 4
      IF(TIS(I).LT.SUMTIM) GOTO 1
      GOTO 2
    1 CONTINUE
C
    2 IF(SUMTIM+DELT.LT.TIS(I)) GOTO 10
C
C-----COME HERE TO ADJUST DELT.
      DELT=TIS(I)-SUMTIM
      NOWTIM=1
      GOTO 10
C
C-----COME HERE AFTER NEXT TIME HAS BEEN REACHED.
    4 ITCO=I
C
      IF(DELAF.GT.0.) DELT=MIN(DELT,DELAF)
C
C                                                    *** BEG MODIFICATIONS
      IF(DELAF.LT.0.AND.DELT.NE.(DELTEX/REDLT))
     1               DELT=MAX(DELT,ABS(DELAF))
C                                                     *** END MODIFICATIONS
C
      IF(TIS(I+1).GT.TIS(I)+DELT) RETURN
      DELT=MIN(DELT,TIS(I+1)-TIS(I))
      NOWTIM=1
C
      RETURN
   10 ITCO=I-1
      RETURN
C
  100 ITCO=ITI
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 15xxxxxxxxxxxxx no changes xxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE MC19A (N,NA,A,IRN,ICN,R,C,W)                           MC19A.2
C     DOUBLE PRECISION A(NA)                                         ID/MC19A.3
      implicit real*8 (a-h,o-z)
C     REAL A(NA)                                                        JPF 03 2005
      REAL*8 A(NA)
C     INTEGER   IRN(NA),ICN(NA)                                      DS/MC19A.5
      INTEGER   IRN(NA),ICN(NA)
C      IRN(K) GIVES ROW NUMBER OF ELEMENT IN A(K).                      MC19A.7
C      ICN(K) GIVES COL NUMBER OF ELEMENT IN A(K).                      MC19A.8
c     REAL R(N),C(N),W(N,5)                                             JPF 03 20005                                         
      REAL*8 R(N),C(N),W(N,5)                                               
C      R(I) IS USED TO RETURN LOG(SCALING FACTOR FOR ROW I).            MC19A.10
C      C(J) IS USED TO RETURN LOG(SCALING FACTOR FOR COL J).            MC19A.11
C      W(I,1),  W(I,2) HOLD ROW, COL NON-ZERO COUNTS.                   MC19A.12
C      W(J,3) HOLDS - COL J LOG DURING EXECUTION.                       MC19A.13
C      W(J,4) HOLDS 2-ITERATION CHANGE IN W(J,3).                       MC19A.14
C      W(I,5) IS USED TO SAVE AVERAGE ELEMENT LOG FOR ROW I.            MC19A.15
C     COMMON/MC19BD/LP,IFAIL                                         ID/MC19A.16
      COMMON/MC19B /LP,IFAIL
      SAVE ICALL
      DATA MAXIT/100/,SMIN/0.1/
C MAXIT IS THE MAXIMAL PERMITTED NUMBER OF ITERATIONS                   MC19A.19
C     SMIN IS USED IN A CONVERGENCE TEST ON (RESIDUAL NORM)**2          MC19A.20
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'MC19A                              ',6X,
     X'HARWELL SUBROUTINE FOR SCALING MATRIX')
C                                                                       MC19A.21
C CHECK SCALAR DATA                                                     MC19A.22
      IFAIL=1
      IF(N.LT.1)GO TO 230
      IFAIL=2
C     IFAIL=2                                                    DS/    MC19A.26
      IF(N.GT.32767)GO TO 230
      IFAIL=0
C                                                                       MC19A.29
C     INITIALISE FOR ACCUMULATION OF SUMS AND PRODUCTS                  MC19A.30
      DO 5 I=1,N
      C(I)=0.
      R(I)=0.
5     CONTINUE
      DO 10 L=1,4
      DO 10 I=1,N
      W(I,L)=0.
10    CONTINUE
      IF(NA.LE.0)GO TO 250
      DO 30 K=1,NA
C     U=DABS(A(K))                                                   ID/MC19A.41
      U= ABS(A(K))
      IF(U.EQ.0.)GO TO 30
      U=LOG(U)
      I1=IRN(K)
      I2=ICN(K)
      IF(I1.GE.1 .AND. I1.LE.N .AND. I2.GE.1 .AND. I2.LE.N)GO TO 20
      IF(LP.GT.0)WRITE(LP,15)K,I1,I2
15    FORMAT(20H MC19 ERROR. ELEMENT,I5,10H IS IN ROW,I5,8H AND COL,I5)
      IFAIL=3
      GO TO 30
C     COUNT ROW/COL NON-ZEROS, AND COMPUTE RHS VECTORS.                 MC19A.53
20    W(I1,1)=W(I1,1)+1.
      W(I2,2)=W(I2,2)+1.
      R(I1)=R(I1)+U
      W(I2,3)=W(I2,3)+U
   30 CONTINUE
      IF(IFAIL.EQ.3)GO TO 230
C                                                                       MC19A.60
C     DIVIDE RHS BY DIAG MATRICES                                       MC19A.61
      DO 70 I=1,N
      IF(W(I,1).EQ.0.)W(I,1)=1.
      R(I)=R(I)/W(I,1)
C     SAVE R(I) FOR USE AT END.                                         MC19A.65
      W(I,5)=R(I)
      IF(W(I,2).EQ.0.)W(I,2)=1.
      W(I,3)=W(I,3)/W(I,2)
70    CONTINUE
      SM=SMIN*FLOAT(NA)
C     SWEEP TO COMPUTE INITIAL RESIDUAL VECTOR                          MC19A.71
      DO 80 K=1,NA
      IF(A(K).EQ.0.0 )GO TO 80
      I=IRN(K)
      J=ICN(K)
      R(I)=R(I)-W(J,3)/W(I,1)
   80 CONTINUE
C                                                                       MC19A.78
C     INITIALISE ITERATION                                              MC19A.79
      E=0.
      Q=1.
      S=0.
      DO 100 I=1,N
      S=S+W(I,1)*R(I)**2
100   CONTINUE
      IF(S.LE.SM)GO TO 186
C                                                                       MC19A.87
C     ITERATION LOOP                                                    MC19A.88
      DO 185 ITER=1,MAXIT
C    SWEEP THROUGH MATRIX TO UPDATE RESIDUAL VECTOR                     MC19A.90
      DO 130 K=1,NA
      IF(A(K).EQ.0.)GO TO 130
      I=ICN(K)
      J=IRN(K)
      C(I)=C(I)+R(J)
  130 CONTINUE
      S1=S
      S=0.
      DO 140 I=1,N
      V=-C(I)/Q
      C(I)=V/W(I,2)
      S=S+V*C(I)
140   CONTINUE
      E1=E
      E=Q*S/S1
      Q=1.-E
      IF(S.LE.SM)E=0.
C     UPDATE RESIDUAL.                                                  MC19A.10
      DO 150 I=1,N
      R(I)=R(I)*E*W(I,1)
150   CONTINUE
      IF(S.LE.SM)GO TO 190
      EM=E*E1
C    SWEEP THROUGH MATRIX TO UPDATE RESIDUAL VECTOR                     MC19A.11
      DO 152 K=1,NA
      IF(A(K).EQ.0.0 ) GO TO 152
      I=IRN(K)
      J=ICN(K)
      R(I)=R(I)+C(J)
152   CONTINUE
      S1=S
      S=0.
      DO 155 I=1,N
      V=-R(I)/Q
      R(I)=V/W(I,1)
      S=S+V*R(I)
155   CONTINUE
      E1=E
      E=Q*S/S1
      Q1=Q
      Q=1.-E
C     SPECIAL FIXUP FOR LAST ITERATION.                                 MC19A.13
      IF(S.LE.SM)Q=1.
C     UPDATE COL. SCALING POWERS                                        MC19A.13
      QM=Q*Q1
      DO 160 I=1,N
      W(I,4)=(EM*W(I,4)+C(I))/QM
      W(I,3)=W(I,3)+W(I,4)
160   CONTINUE
      IF(S.LE.SM)GO TO 186
C     UPDATE RESIDUAL.                                                  MC19A.14
      DO 180 I=1,N
      C(I)=C(I)*E*W(I,2)
180   CONTINUE
185   CONTINUE
186   DO 188 I=1,N
      R(I)=R(I)*W(I,1)
188   CONTINUE
C                                                                       MC19A.14
C      SWEEP THROUGH MATRIX TO PREPARE TO GET ROW SCALING POWERS        MC19A.15
190   DO 200 K=1,NA
      IF(A(K).EQ.0.0 )GO TO 200
      I=IRN(K)
      J=ICN(K)
      R(I)=R(I)+W(J,3)
  200 CONTINUE
C                                                                       MC19A.15
C      FINAL CONVERSION TO OUTPUT VALUES.                               MC19A.15
      DO 220 I=1,N
      R(I)=R(I)/W(I,1)-W(I,5)
      C(I)=-W(I,3)
220   CONTINUE
      GO TO 250
230   IF(LP.GT.0)WRITE(LP,240)IFAIL
240   FORMAT(//13H ERROR RETURN,I2,10H FROM MC19)
250   RETURN
      END
C contents: nadole se 16 i 17 relative permeability and capillary pressure
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 16xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      SUBROUTINE RELP(SG,REPL,REPG,NMAT,K,NLOC,SG0)
      implicit real*8 (a-h,o-z)
C-----THIS ROUTINE COMPUTES RELATIVE PERMEABILITIES FOR LIQUID
C     AND GASEOUS PHASES.
C
      COMMON/P3/DELX(100000)
C      COMMON/RPCAP/IRP(27),RP(7,27),ICP(27),CP(7,27),IRPD,RPD(7),           JPF 03 2005
C     XICPD,CPD(7)
      COMMON/RPCAP/RP(7,27),RPD(7),IRP(27),ICP(27),CP(7,27),IRPD,
     XICPD,CPD(7)
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C     *** BEGINNING MODIFICATIONS ***
  899 FORMAT(6X,'RELP     1.1         MARCH     1995',6X,
     X'LIQUID AND GAS PHASE RELATIVE PERMEABILITIES AS FUNCTIONS',
     X' OF SATURATION ')
C     *** END MODIFICATIONS ***
C
      SL=1.-SG
C     *** BEGINNING MODIFICATIONS ***
      GOTO(10,11,12,12,13,14,15,16,17),IRP(NMAT)
C     *** END MODIFICATIONS ***
   10 CONTINUE
C*************************************************************************
C--   1)   ---LINEAR FUNCTIONS.
C*************************************************************************
C     CHECK IF INCREMENT NEEDS TO BE ADJUSTED AT LOWER LIQUID CUTOFF.
      IF(K.NE.3) GOTO 20
      IF((SL-RP(1,NMAT))*(1.-SG0-RP(1,NMAT)).GE.0.) GOTO 20
C     ADJUST INCREMENT.
      DELX(NLOC+2)=-DELX(NLOC+2)
      SG=SG0+DELX(NLOC+2)
      SL=1.-SG
   20 CONTINUE
C
      REPL=(SL-RP(1,NMAT))/(RP(3,NMAT)-RP(1,NMAT))
      IF(SL.GE.RP(3,NMAT)) REPL=1.
      IF(SL.LE.RP(1,NMAT)) REPL=0.
      REPG=(SG-RP(2,NMAT))/(RP(4,NMAT)-RP(2,NMAT))
      IF(SG.GE.RP(4,NMAT)) REPG=1.
      IF(SG.LE.RP(2,NMAT)) REPG=0.
C
      RETURN
C
   11 CONTINUE
C*************************************************************************
C--   2)  ---RELATIVE PERMEABILITY OF PICKENS ET AL.
C*************************************************************************
C
      REPG=1.
      REPL=(1.-SG)**RP(1,NMAT)
C
      RETURN
C
   12 CONTINUE
C*************************************************************************
C--   3) & 4)   ---COREY@S OR GRANT@S CURVES.
C*************************************************************************
C
      SSTAR=(SL-RP(1,NMAT))/(1.-RP(1,NMAT)-RP(2,NMAT))
      REPL=SSTAR**4
      REPG=(1.-SSTAR**2)*(1.-SSTAR)**2
      IF(SG.GE.RP(2,NMAT)) GOTO 50
      REPG=0.
      REPL=1.
      GOTO 102
   50 IF(SG.LT.(1.-RP(1,NMAT))) GOTO 102
      REPL=0.
      REPG=1.
  102 CONTINUE
      IF(IRP(NMAT).EQ.4) REPG=1.-REPL
      RETURN
C
   13 CONTINUE
C*************************************************************************
C--   5)   ---BOTH PHASES ARE PERFECTLY MOBILE.
C*************************************************************************
C
      REPL=1.
      REPG=1.
C
   14 CONTINUE
C*************************************************************************
C--   6)   ---RELATIVE PERMEABILITIES OF FATT AND KLIKOFF (1959), AS REPORTED
C     BY K. UDELL (BERKELEY, 1982).
C*************************************************************************
C
      SS=0.
      IF(SL.GT.RP(1,NMAT)) SS=(SL-RP(1,NMAT))/(1.-RP(1,NMAT))
      REPL=SS**3
      REPG=(1.-SS)**3
      RETURN
C
   15 CONTINUE
C*************************************************************************
C-   7)   ----RELATIVE PERMEABILITY OF VAN GENUCHTEN, SOIL SCI. SOC. AM. J. 44,
C     PP. 892-898, 1980.
C*************************************************************************
C
      IF(SL.GE.RP(3,NMAT)) GOTO 150
      SS=(SL-RP(2,NMAT))/(RP(3,NMAT)-RP(2,NMAT))
      REPL=0.
      IF(SS.GT.0.)
     XREPL=SQRT(SS)*(1.-(1.-SS**(1./RP(1,NMAT)))**RP(1,NMAT))**2
      REPG=1.-REPL
      RETURN
C
  150 REPL=1.
      REPG=0.
      RETURN
C
   16 CONTINUE
C*************************************************************************
C--   8)   ---RELATIVE PERMEABILITIES AS MEASURED BY VERMA ET AL. IN
C     LABORATORY FLOW EXPERIMENTS FOR STEAM-WATER MIXTURES
C*************************************************************************
C
      SS=(SL-RP(1,NMAT))/(RP(2,NMAT)-RP(1,NMAT))
      IF(SS.GT.1.) SS=1.
      IF(SS.LT.0.) SS=0.
      REPL=SS**3
      REPG=RP(3,NMAT)+RP(4,NMAT)*SS+RP(5,NMAT)*SS*SS
      IF(REPG.GT.1.) REPG=1.
      IF(REPG.LT.0.) REPG=0.
      RETURN
C
C     *** BEGINNING MODIFICATIONS ***
   17 CONTINUE
C*************************************************************************
C--   9)   ---RELATIVE PERMEABILITY OF VAN GENUCHTEN, SOIL SCI. SOC. AM. J. 44,
C     PP. 892-898, 1980.
C*************************************************************************
C
C     RELATIVE PERMEABILITY TO GAS MODIFIED ACCORDING TO J.C. PARKER, 
C     REV. IN GEOPH. 27 NO.3, PP. 311-328, 1989.
C     (RENE LEFEBVRE, LAVAL UNIVERSITY, APRIL 1993)
C
      IF(SL.GE.RP(3,NMAT)) GOTO 170
      SS=(SL-RP(2,NMAT))/(RP(3,NMAT)-RP(2,NMAT))
      REPL=0.
      REPG=1.
      IF(SS.GT.0.)
     XREPL=SQRT(SS)*(1.-(1.-SS**(1./RP(1,NMAT)))**RP(1,NMAT))**2
      IF(SS.GT.0.)
     XREPG=SQRT(1.-SS)*(1.-SS**(1./RP(1,NMAT)))**(2.*RP(1,NMAT))
      RETURN
C
  170 REPL=1.
      REPG=0.
      RETURN
C     *** END MODIFICATIONS ***
C
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 17xxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no change   xxxxx
      SUBROUTINE PCAP(SL,T,PC,NMAT)
      implicit real*8 (a-h,o-z)
C-----THIS ROUTINE COMPUTES CAPILLARY PRESSURE AS FUNCTION OF LIQUID
C     SATURATION SL AND TEMPERATURE T.
C
C      COMMON/RPCAP/IRP(27),RP(7,27),ICP(27),CP(7,27),IRPD,RPD(7),               JPF 03 2005
C     AICPD,CPD(7)
      COMMON/RPCAP/RP(7,27),RPD(7),IRP(27),ICP(27),CP(7,27),IRPD,
     XICPD,CPD(7)
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'PCAP     1.0       4 MARCH     1991',6X,
     X'CAPILLARY PRESSURE AS FUNCTION OF SATURATION')
C
      GOTO(10,11,12,13,14,15,16),ICP(NMAT)
C
   10 CONTINUE
C*************************************************************************
C--   1)   ---LINEAR FUNCTION.
C*************************************************************************
      PC=-CP(1,NMAT)*(CP(3,NMAT)-SL)/(CP(3,NMAT)-CP(2,NMAT))
      IF(SL.GE.CP(3,NMAT)) PC=0.
      IF(SL.LE.CP(2,NMAT)) PC=-CP(1,NMAT)
      RETURN
   11 CONTINUE
C*************************************************************************
C--   2)   ---CAPILLARY PRESSURE FUNCTION OF PICKENS ET AL, AS GIVEN IN
C     J. HYDROLOGY 40, 243-264, 1979.
C*************************************************************************
C
      SLX=MAX(SL,1.001*CP(2,NMAT))
      IF(SLX.GT..999*CP(3,NMAT)) SLX=.999*CP(3,NMAT)
      A=(1.+SLX/CP(3,NMAT))*(CP(3,NMAT)-CP(2,NMAT))/
     A(CP(3,NMAT)+CP(2,NMAT))
      B=(1.-SLX/CP(3,NMAT))
      PC=-CP(1,NMAT)*LOG(A*(1.+SQRT(1.-B*B/(A*A)))/B)**
     A(1./CP(4,NMAT))
      IF(SL.GT..999*CP(3,NMAT)) PC=PC*(1.-SL)/.001
      RETURN
C
C
   12 CONTINUE
C*************************************************************************
C--   3)   ---CAPILLARY PRESSURE FUNCTION AS USED IN THE TRUST-PROGRAM, WHICH
C     WAS DEVELOPED BY T.N. NARASIMHAN AT LAWRENCE BERKELEY LABORATORY.
C*************************************************************************
C
      IF(SL.NE.1) GOTO 120
      PC=0.
      RETURN
C
  120 SLX=SL
      IF(CP(5,NMAT).EQ.0.)SLX=MAX(SL,1.001*CP(2,NMAT))
      PC=-ABS(CP(5,NMAT))
      IF(SLX.GT.CP(2,NMAT))
     APC=-CP(4,NMAT)-CP(1,NMAT)*((1.-SLX)/(SLX-CP(2,NMAT)))
     B**(1./CP(3,NMAT))
      IF(CP(5,NMAT).NE.0.)PC=MAX(PC,-ABS(CP(5,NMAT)))
      IF(SL.GT..999) PC=PC*(1.-SL)/.001
      RETURN
C
   13 CONTINUE
C*************************************************************************
C--   4)   ---CAPILLARY PRESSURE OF YOLO CLAY AFTER CHRIS MILLY,
C     WATER RES. RES., VOL. 18 NO.3 (JUNE 1982), PP. 489-498.
C*************************************************************************
C
      IF(SL-CP(1,NMAT).GE..371) GOTO 130
      SLX=MAX(SL,1.001*CP(1,NMAT))
      EX=(0.371/(SLX-CP(1,NMAT))-1.)**.25
      EX=2.26*EX-2.
      PC=-9.7783E3*10.**EX
      RETURN
C
  130 PC=-97.783
      RETURN
   14 CONTINUE
   15 CONTINUE
C*************************************************************************
C--   5) & 6)   ---LEVERETT@S J-FUNCTION.
C*************************************************************************
C
     SS=0.
      IF(SL.GT.CP(2,NMAT)) SS=(SL-CP(2,NMAT))/(1.-CP(2,NMAT))
      OSS=1.-SS
      F=1.417*OSS-2.120*OSS**2+1.263*OSS**3
      CALL SIGMA(T,ST)
      PC=-CP(1,NMAT)*ST*F
      RETURN
   16 CONTINUE
C*************************************************************************
C--   7)   ---CAPILLARY FUNCTION OF VAN GENUCHTEN, SOIL SCI. SOC. AM. J. 44,
C     PP.892-898, 1980.
C*************************************************************************
C
      IF(SL.NE.1.)GO TO 160
      PC=0.
      RETURN
C
  160 SLX=SL
      IF(SLX.GE.CP(5,NMAT)) GOTO 161
      IF(CP(4,NMAT).EQ.0.)SLX=MAX(SL,1.001*CP(2,NMAT))
      PC=-ABS(CP(4,NMAT))
      IF(SLX.GT.CP(2,NMAT))
     APC=-1./ABS(CP(3,NMAT))*(((SL-CP(2,NMAT))/(CP(5,NMAT)-CP(2,NMAT)))
     B**(-1./CP(1,NMAT))-1.)**(1.-CP(1,NMAT))
      IF(CP(4,NMAT).NE.0.) PC=MAX(PC,-ABS(CP(4,NMAT)))
      IF(SL.GT..999) PC=PC*(1.-SL)/.001
      RETURN
  161 PC=0.
      RETURN
C
      END
C contents: nadole od 18 do 26 se water i water vapor characteristics
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 18xxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx   no change  xxxxxx
C calculates saturated water vapor pressure P(exit) for different temp. T(input)
C RESULTS:  T(oC)   0      5      10      15      20      25      30      40      50 
C           P(Pa) 610.8  871.8  1226.9  1703.9  2336.5  3165.9  4241.5  7375.0 12335.3 
C           T(oC)  60       70        80       90       100
C           P(Pa)19920.2  31162.0  47359.9  70108.8  101325.2     1Pa=1N/m2
      SUBROUTINE SAT(T,P)
      implicit real*8 (a-h,o-z)
C      COMMON/KONIT/KON,DELT,IGOOD                                       JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      DIMENSION A(9)
      SAVE ICALL,A
      DATA A/
     A-7.691234564,-2.608023696E1,-1.681706546E2,6.423285504E1,
     B-1.189646225E2,4.167117320,2.097506760E1,1.E9,6./
C
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'SAT      1.0      22 JANUARY   1990',6X,
     X'STEAM TABLE EQUATION: SATURATION PRESSURE AS FUNCTION OF',
     X' TEMPERATURE')
C
      IF(T.LT.1..OR.T.GT.500.) GOTO 10
      TC=(T+273.15)/647.3
      SC=0.
      DO 5 J=1,5
    5 SC=SC+A(J)*(1.-TC)**J
      PC=EXP(SC/(TC*(1.+A(6)*(1.-TC)+A(7)*(1.-TC)**2))-(1.-TC)/(A(8)*
     A(1.-TC)**2+A(9)))
      P=PC*2.212E7
      RETURN
   10 IGOOD=2
      PRINT 1,T
    1 FORMAT(15H TEMPERATURE = ,E12.6,22H  OUT OF RANGE IN SAT )
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 19xxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx   no change   xxxxxx
C calculates saturation temp. TS (oC) for known water vapor press.  PX (Pa)
C I did not check it there were some problems
      SUBROUTINE TSAT(PX,TX00,TS)
      implicit real*8 (a-h,o-z)
C      COMMON/KONIT/KON,DELT,IGOOD                             JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
C
C-----FIND SATURATION TEMPERATURE TS AT PRESSURE PX.
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'TSAT     1.0      14 MARCH     1991',6X,
     X'SATURATION TEMPERATURE AS FUNCTION OF PRESSURE')
C
C     TX0 IS STARTING TEMPERATURE FOR ITERATION.
C
      TX0=TX00
      IF(TX0.NE.0.) GOTO 2
C
C-----COME HERE TO OBTAIN ROUGH STARTING VALUE FOR ITERATION.
      TX0=4606./(24.02-LOG(PX)) - 273.15
      TX0=MAX(TX0,5.)
C
    2 CONTINUE
      TS=TX0
      DT=TS*1.E-8
      TSD=TS+DT
C
    1 CONTINUE
C
      CALL SAT(TS,PS)
      IF(IGOOD.NE.0) RETURN
      IF(ABS((PX-PS)/PX).LE.1.E-10) RETURN
C
      TSD=TS+DT
      CALL SAT(TSD,PSD)
      TS=TS+(PX-PS)*DT/(PSD-PS)
      GOTO 1
C
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 20xxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no change xxxx
C calculates liquid water density D (kg/m3) and internal energy U both-exit 
C for known pressure PP (Pa) and temperature TF (oC) both input
C RESULTS:  T(oC)    0         5        15       20       25      30       40       50      
C         D(kg/m3) 998.826  1000.019  999.213  998.323  997.16  995.753  992.291  988.069
C             U   21017.4  42023.4    63095.6  83954.4  104856  125808   167524   209341
C           T(oC)   60        70       80       90       100
C         D(kg/m3) 983.180  977.692  971.660  965.126  958.122 
C             U               251229   293004   335015   377078   418810
C   pression was keeped at 101325.2 (Pa)    ;     1Pa=1N/m2      
      SUBROUTINE COWAT(TF,PP,D,U)
      implicit real*8 (a-h,o-z)
C      COMMON/KONIT/KON,DELT,IGOOD                                    JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      DIMENSION A(23),SA(12)
      SAVE ICALL,A,SA
      DATA A/
     A6.824687741E3,-5.422063673E2,-2.096666205E4,3.941286787E4,
     B-6.733277739E4,9.902381028E4,-1.093911774E5,8.590841667E4,
     C-4.511168742E4,1.418138926E4,-2.017271113E3,7.982692717E0,
     D-2.616571843E-2,1.522411790E-3,2.284279054E-2,2.421647003E2,
     E1.269716088E-10,2.074838328E-7,2.174020350E-8,1.105710498E-9,
     F1.293441934E1,1.308119072E-5,6.047626338E-14/
      DATA SA/
     A8.438375405E-1,5.362162162E-4,1.720000000E0,7.342278489E-2,
     B4.975858870E-2,6.537154300E-1,1.150E-6,1.51080E-5,
     C1.41880E-1,7.002753165E0,2.995284926E-4,2.040E-1   /
C
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'COWAT    1.0      22 JANUARY   1990',6X,
     X'LIQUID WATER DENSITY AND INT.',
     X' ENERGY AS FUNCTION OF TEMPERATURE AND PRESSURE')
C
      TKR=(TF+273.15)/647.3
      PNMR=PP/2.212E7
      Y=1.-SA(1)*TKR*TKR-SA(2)/TKR**6
      ZP=(SA(3)*Y*Y-2.*SA(4)*TKR+2.*SA(5)*PNMR)
      IF(ZP.LT.0.) GOTO 1
      Z=Y+SQRT(ZP)
      PAR1=A(12)*SA(5)/Z**(5./17.)
      PAR2=A(13)+A(14)*TKR+A(15)*TKR*TKR+A(16)*(SA(6)-TKR)**10+A(17)/
     A(SA(7)+TKR**19)
      PAR3=(A(18)+2.*A(19)*PNMR+3.*A(20)*PNMR*PNMR)/(SA(8)+TKR**11)
      PAR4=A(21)*TKR**18*(SA(9)+TKR*TKR)*(-3./(SA(10)+PNMR)**4+SA(11))
      PAR5=3.*A(22)*(SA(12)-TKR)*PNMR*PNMR+4.*A(23)/TKR**20*PNMR**3
      VMKR=PAR1+PAR2-PAR3-PAR4+PAR5
      V=VMKR*3.17E-3
      D=1./V
      YD=-2.*SA(1)*TKR+6.*SA(2)/TKR**7
      SNUM=0.
      DO 10 I=1,10
   10 SNUM=SNUM+(I-2)*A(I+1)*TKR**(I-1)
      PRT1=A(12)*(Z*(17.*(Z/29.-Y/12.)+5.*TKR*YD/12.)+SA(4)*TKR-
     A(SA(3)-1.)*TKR*Y*YD)/Z**(5./17.)
      PRT2=PNMR*(A(13)-A(15)*TKR*TKR+A(16)*(9.*TKR+SA(6))*(SA(6)-TKR)**9
     A+A(17)*(20.*TKR**19+SA(7))/(SA(7)+TKR**19)**2)
      PRT3=(12.*TKR**11+SA(8))/(SA(8)+TKR**11)**2*(A(18)*PNMR+A(19)*
     APNMR*PNMR+A(20)*PNMR*PNMR*PNMR)
      PRT4=A(21)*TKR**18*(17.*SA(9)+19.*TKR*TKR)*(1./(SA(10)+PNMR)**3+
     ASA(11)*PNMR)
      PRT5=A(22)*SA(12)*PNMR**3+21.*A(23)/TKR**20*PNMR**4
      ENTR=A(1)*TKR-SNUM+PRT1+PRT2-PRT3+PRT4+PRT5
      H=ENTR*70120.4
      U=H-PP*V
      RETURN
    1 IGOOD=2
      PRINT 2,TF
    2 FORMAT(15H TEMPERATURE = ,E12.6,23H  OUT OF RANGE IN COWAT)
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 21xxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no change  xxxx
C calculates water vapor density D (kg/m3) and its internal energy U for 
C given temperature T (oC) and vapor partial pressure P (Pa)
C results: T (oC)      0       5      10      15      20      30      40
C        D (kg/m3) .865792 .844866 .825223 .806730 .789281 .757120 .728093 
C          U       2344610 2353280 2361870 2370380 2378810 2395460 2411870
C          T (oC)     50      60      70      80      90     100             
C        D (kg/m3) .701696 .677534 .655289 .634707 .615578 .597730    
C          U       2428050 2444030 2459850 2475520 2491060 2506490
C pression P was keeped at 101325.2 (Pa)
      SUBROUTINE SUPST(T,P,D,U)
      implicit real*8 (a-h,o-z)
      INTEGER Z
      DIMENSION A(6),B(3,8),C(7),SB(5),Z(3,8),S(3),R(3)
      SAVE ICALL,A,B,C,SB,Z
      DATA A/
     A16.83599274,28.56067796,-54.38923329,0.4330662834,-0.6547711697,
     B8.565182058E-2/
      DATA B/6.670375918E-2,1.388983801,0.,8.390104328E-2,
     1   2.614670893E-2,-3.373439453E-2,4.520918904E-1,1.069036614E-1,
     2    0.,-5.975336707E-1,-8.847535804E-2,0.,5.958051609E-1,
     3  -5.159303373E-1,2.075021122E-1,1.190610271E-1,-9.867174132E-2,
     4    0.,1.683998803E-1,-5.809438001E-2,0.,6.552390126E-3,
     5   5.710218649E-4,0./
      DATA C/
     A1.936587558E2,-1.388522425E3,4.126607219E3,-6.508211677E3,
     B5.745984054E3,-2.693088365E3,5.235718623E2/
      DATA SB/
     A7.633333333E-1,4.006073948E-1,8.636081627E-2,-8.532322921E-1,
     B3.460208861E-1/
      DATA Z/
     A13,3,0,18,2,1,18,10,0,25,14,0,32,28,24,12,11,0,
     B24,18,0,24,14,0/
C
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'SUPST    1.0      29 JANUARY   1990',6X,
     X'VAPOR DENSITY AND INTERNAL ENERGY AS FUNCTION OF',
     X' TEMPERATURE AND PRESSURE')
C
      TK=(T+273.15)/647.3
      PK=P/2.212E7
      EL1=4.260321148
      X=EXP(SB(1)*(1.-TK))
      BEL=15.74373327-34.17061978*TK+19.31380707*TK**2
      BELP=-34.17061978+2.*19.31380707*TK
      SUM=EL1*TK/PK
      DO 30 J=1,5
      SC=0.
      DO 20 K=1,3
   20 SC=B(K,J)*X**Z(K,J)+SC
   30 SUM=SUM-J*PK**(J-1)*SC
      S(1)=(1./PK**4+SB(2)*X**14)
      S(2)=(1./PK**5+SB(3)*X**19)
      S(3)=(1./PK**6+SB(4)*X**54+SB(5)*X**27)
      DO 50 J=6,8
      S2=0.
      DO 40 K=1,3
   40 S2=S2+B(K,J)*X**Z(K,J)
   50 SUM=SUM-S2/S(J-5)**2*(J-2)*PK**(1-J)
      S2=0.
      DO 60 K=1,7
   60 S2=C(K)*X**(K-1)+S2
      SUM=SUM+11.*(PK/BEL)**10*S2
      V=SUM*0.00317
      D=1./V
      HK=0.0
      HK=A(1)*TK
      DO 70 K=1,5
   70 HK=HK-A(K+1)*(K-2)*TK**(K-1)
      DO 90 J=1,5
      S2=0.
      DO 80 K=1,3
   80 S2=S2+B(K,J)*(1.+Z(K,J)*SB(1)*TK)*X**Z(K,J)
   90 HK=HK-PK**J*S2
      R(1)=14.*SB(2)*X**14
      R(2)=19.*SB(3)*X**19
      R(3)=54.*SB(4)*X**54+27.*SB(5)*X**27
      DO 100 J=6,8
      S2=0.
      DO 95 K=1,3
   95 S2=S2+B(K,J)*X**Z(K,J)*(1.+Z(K,J)*SB(1)*TK-SB(1)*TK*R(J-5)/S(J-5))
     A/S(J-5)
  100 HK=HK-S2
      S2=0.
      DO 110 K=1,7
  110 S2=S2+(1.+TK*(10.*BELP/BEL+(K-1)*SB(1)))*C(K)*X**(K-1)
      HK=HK+PK*(PK/BEL)**10*S2
      H=HK*70120.4
      U=H-P*V
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 22xxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no change xxxx
C calculates water surface tension SIGMA (N/m) for given temperature T (oC) 
C results: T:    0       5       10      15      20       30      40       50      
C      SIGMA: .07566  .07495  .07423   0.0735  .07275  .07121  .069612  .067961  
C          T:    60      70      80       90         100
C      SIGMA: .066256  0.0645  .062693  .0608376  .0589345
      SUBROUTINE SIGMA(T,ST)
      implicit real*8 (a-h,o-z)
C-----COMPUTE SURFACE TENSION OF WATER, USING THE
C     "INTERNATIONAL REPRESENTATION OF THE SURFACE TENSION OF
C                                               WATER SUBSTANCE" (1975).
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'SIGMA    1.0      19 OCTOBER   1990',6X,
     X'SURFACE TENSION OF WATER AS FUNCTION OF TEMPERATURE')
C
      IF(T.GE.374.15) GOTO 1
      ST=1.-0.625*(374.15-T)/647.3
      ST=ST*.2358*((374.15-T)/647.3)**1.256
      RETURN
C
    1 CONTINUE
      ST=0.
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 23xxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx no change  xxxx
C includes subroutines: VISW water dynamic visc VW as funct of Temp. and Press.
C                       VISS water vapor viscosity VS as function of  T and P
C PS: pression of water vapor at saturation;  D density of water vapor input
C results: T (oC)     0        5        10      15       20       30       40
C         PS (Pa)   610.8    871.8    1227.   1703.9   2336.6    4241.5   7375
C    VW (Nsec/m2) 1.753E-3 1.501E-3 1.299E-3 1.135E-3 1.0017E-3 .797E-3 .651E-3
C       D (kg/m3) .865792  .844866  .825223  .806730  .789281   .757120 .728093 
C           VS(?) 7.879E-6 8.089E-6 8.298E-6 8.5045E-6 8.717E-6 9.134E-6 9.55E-6
C          T         50       60       70       80       90           100
C         PS       12335    19920    31162    47360     70109       101325.2
C    VW (Nsec/m2).5442E-3 .4631E-3 .40044E-3 .351E-3  .31132E-3    .27898E-3
C       D (kg/m3).701696   .677534    .655289   .634707  .615578    .597730    
C        VS(?)   9.965E-6 10.038E-6 10.794E-6 11.208E-6 11.621E-6 12.034E-6
C pression P was keeped at 101325.2 (Pa)
      SUBROUTINE VIS(T,P,D,VW,VS,PS)
      implicit real*8 (a-h,o-z)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'VIS      1.0      22 JANUARY   1990',6X,
     X'VISCOSITY OF LIQUID WATER AND VAPOR AS FUNCTION OF',
     X' TEMPERATURE AND PRESSURE')
C
      EX=247.8/(T+133.15)
      PHI=1.0467*(T-31.85)
      AM=1.+PHI*(P-PS)*1.E-11
      VW=1.E-7*AM*241.4*10.**EX
C
      V1=.407*T+80.4
      IF(T.LE.350.) VS=1.E-7*(V1-D*(1858.-5.9*T)*1.E-3)
      IF(T.GT.350.) VS=1.E-7*(V1+.353*D+676.5E-6*D**2+102.1E-9*D**3)
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 24xxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx   no change  xxxx
C calculates liquid water dynamic viscosity VW as function of Temp. and
C  Press. and PS: pression of water vapor at saturation
C T  (oC)         0        5       10        15       20       30       40
C PS (Pa)       610.8    871.8    1227.    1703.9   2336.6   4241.5    7375
C VW (Nsec/m2) 1.753E-3 1.501E-3 1.299E-3 1.135E-3 1.0017E-3 .797E-3 .651E-3
C T                50        60        70         80       90         100
C PS             12335     19920     31162      47360    70109     101325.2
C VW (Nsec/m2) .5442E-3  .4631E-3  .40044E-3  .351E-3  .31132E-3  .27898E-3
C pression P was keeped at 101325.2 (Pa)
      SUBROUTINE VISW(T,P,PS,VW)
      implicit real*8 (a-h,o-z)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'VISW     1.0      22 JANUARY   1990',6X,
     X'VISCOSITY OF LIQUID WATER AS FUNCTION OF',
     X' TEMPERATURE AND PRESSURE')
C
      EX=247.8/(T+133.15)
      PHI=1.0467*(T-31.85)
      AM=1.+PHI*(P-PS)*1.E-11
      VW=1.E-7*AM*241.4*10.**EX
C
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 25xxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  no change  xxxxx
C calculates water vapor viscosity VS as function of Temp. and  Pression
C and D density of water vapor 
C results: T  (oC)    0        5       10       15         20        30       40
C        D (kg/m3) .865792  .844866 .825223   .806730    .789281   .757120  .728093 
C            VS(? 7.879E-6 8.089E-6 8.298E-6  8.5045E-6  8.717E-6  9.134E-6  9.55E-6
C             T       50         60       70        80        90        100
C        D (kg/m3)  .701696   .677534   .655289   .634707   .615578   .597730    
C             VS(?) 9.965E-6 10.038E-6 10.794E-6 11.208E-6 11.621E-6 12.034E-6
C pression P was keeped at 101325.2 (Pa)
      SUBROUTINE VISS(T,P,D,VS)
      implicit real*8 (a-h,o-z)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'VISS     1.0      22 JANUARY   1990',6X,
     X'VISCOSITY OF VAPOR AS FUNCTION OF',
     X' TEMPERATURE AND PRESSURE')
C
      V1=.407*T+80.4
      IF(T.LE.350.) VS=1.E-7*(V1-D*(1858.-5.9*T)*1.E-3)
      IF(T.GT.350.) VS=1.E-7*(V1+.353*D+676.5E-6*D**2+102.1E-9*D**3)
C
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 26xxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx no change  xxxxxx
      SUBROUTINE THERC(T,P,D,CONW,CONS,PS)
      implicit real*8 (a-h,o-z)
      SAVE ICALL,A0,A1,A2,A3,A4,B0,B1,B2,B3,C0,C1,C2,C3,T0
      DATA A0,A1,A2,A3,A4/-922.47,2839.5,-1800.7,525.77,-73.440/
      DATA B0,B1,B2,B3/-.94730,2.5186,-2.0012,.51536/
      DATA C0,C1,C2,C3/1.6563E-3,-3.8929E-3,2.9323E-3,-7.1693E-4/
      DATA T0/273.15/
C
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'THERC    1.0       4 MARCH     1991',6X,
     X'THERMAL CONDUCTIVITY OF WATER AND VAPOR AS FUNCTION OF',
     X' TEMPERATURE AND PRESSURE')
C
      T1=(T+273.15)/T0
      T2=T1*T1
      T3=T2*T1
      T4=T3*T1
C
C     IF(P-PS.LT.0.) GOTO1
      CON1=A0+A1*T1+A2*T2+A3*T3+A4*T4
      CON2=(P-PS)*(B0+B1*T1+B2*T2+B3*T3)*1.E-5
      CON3=(P-PS)*(P-PS)*(C0+C1*T1+C2*T2+C3*T3)*1.E-10
      CONW=(CON1+CON2+CON3)*1.E-3
      CON1=17.6+5.87E-2*T
      CON2=1.04E-4*T*T
      CON3=4.51E-8*T**3
      CONS1=1.E-3*(CON1+CON2-CON3)
      CONS=CONS1+1.E-6*(103.51+.4198*T-2.771E-5*T*T)*D
     A+1.E-9*D*D*2.1482E14/T**4.2
C
C     PRINT 10,T,P,PS,CON
   10 FORMAT(5H T = ,E12.6,5H P = ,E12.6,6H PS = ,E12.6,7H CON = ,E12.6)
C
      RETURN
    1 CONTINUE
C     PRINT 2,T,P,PS
    2 FORMAT(8H AT T = ,E12.6,5H P = ,E12.6,19H IS LESS THAN PS = ,E12.6
     A)
      RETURN
      END
C
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 27xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C
C  SUBROUTINES FOR LGM COMPUTATIONS OF: CH4, CO2 AND HEAT PRODUCTION RATES 
C  CALLED FROM MULTI TO CALCULATE CH4, CO2 & HEAT SOURCE CHANGES RESIDUALS
C  CALLED FROM CYCIT TO UPDATE THE TOTAL QUANTITY OF CH4&CO2 PRODUCED ONLY
C               TDEPOS .LE. -10. NO PRODUCTION
C               TDEPOS .EQ.   O. CONSTANT PRODUCTION CH4 & CO2
C               TDEPOS .LT.   0. CONSTANT PRODUCTION CO2 ONLY
C
      SUBROUTINE CCHPR
      implicit real*8 (a-h,o-z)
C     USES A FIRST ORDER KINETIC MODEL WITH RESPECT TO TIME
C     A) CCHPR MAY BE CALLED WITH NEQ=NK1: THEN CH4, CO2 & HEAT PRODUCTION
C     B) CCHPR MAY BE CALLED WITH NEQ=NK : THEN ONLY CH4 & CO2  PRODUCTION
C
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/P1/X(100000)
      COMMON/P2/DX(100000)
      COMMON/P3/DELX(100000)
      COMMON/P4/R(100000)
      COMMON/E2/MATX(20000)
      COMMON/E3/EVOL(20000)
      COMMON/E4/PHI(20000)
      COMMON/E5/P(20000)
      COMMON/E6/T(20000)
      COMMON/L3/CO(10800000)
C      COMMON/KONIT/KON,DELT,IGOOD                                    JPF 03 2005
      COMMON/KONIT/DELT,KON,IGOOD
      COMMON/SECPAR/PAR(2640000)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
C      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT              JPF 03 2005
      COMMON/CYC/TIMIN,SUMTIM,GF,TIMOUT,KCYC,ITER,ITERC
      COMMON/BC/NELA
c      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)       JPF 03 2005
      COMMON/SOLID/DM(27),POR(27),PER(3,27),CWET(27),SH(27),NM
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
C
C     ***** NEW COMMON BLOCKS REQUIRED *****
C
      COMMON/E7/GCH4(20000)
      COMMON/E8/GCO2(20000)
      COMMON/E9/TDEPOS(20000)
      COMMON sgch4,sgco2,sght
C     LOCAL ARRAY: FIRST ORDER KINETIC RATE CONSTANT XKI 
C                                                      *** BEG MODIFICATIONS
      DIMENSION XKI(3),FRAC(3)
C
C     PRINT VERSION NUMBER ON FIRST CALL TO SUBROUTINE
      SAVE ICALL
C coefficients are     XKI=ln(0.5)/t=0.6931/t
c half lives  t= 1   y   Xki=0.69315
c             t= 5   y   Xki=0.13863
c             t=10   y   Xki=0.06931
c             t=12.6 y   Xki=0.055
c             t=15   y   Xki=0.04621
c             t=20   y   Xki=0.03466
c             t=30   y   Xki=0.02310
c             t=40   y   Xki=0.01733
c
      DATA XKI/0.055,0.0,0.0/
c only one fraction 
      DATA FRAC/1.00,0.0,0.0/
C                                                      *** END MODIFICATIONS
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'CCHPR    1.0         MARCH     1995',6X,
     X'*LGM SUBROUTINE: ',
     X'CH4, CO2 and HEAT PRODUCTION RATES')
C-----CCHPR TO BE RUN EITHER FOR:
C     1) (NEQ = NK1 = 5)   NON ISOTHERMAL SIMULATIONS 
C     2) (NEQ = NK  = 4)   ISOTHERMAL SIMULATIONS 
C                          BUT VARIABLE CH4 & CO2 CONCENTRATION        
C
      IF(NEQ.GE.NK) GOTO 3
      PRINT 4
    4 FORMAT(' ERRONEOUS CALL TO SUBROUTINE CCHPR',/,
     X' LGM COMPUTATIONS (MOP(24) > 0) ARE ONLY AVAILABLE ',
     X'FOR EITHER NEQ = NK1 = 5, OR FOR NEQ = NK = 4,'/,
     X' RESET MOP(24) TO ZERO AND CONTINUE WITHOUT',
     X' LGM COMPUTATIONS')
      MOP(24)=0
      RETURN
    3 CONTINUE
C
C-----STOICHIOMETRIC CONSTANT (DEPENDING ON THE ENTHALPY OF LANDFILL 
C     PRODUCTION RATE (JOULES PRODUCED PER KG CH4 PRODUCED).
C     IF THE VALUE OF CFHT IS MODIFIED IN THIS SUBROUTINE, IT ALSO
C     HAS TO BE CHANGED IN SUBROUTINE "OUT" IN MODULE "LGM-EOS3".
C     Rees 1980: heat= 632000 J produced for each 0.25 kg of CH4, or
C                     2528000 J                   1.00 kg
      cfht= 1528000.
C time for activation of biodegradation =6mois(15.552.000 s 0.5YEARS)
      TPRIME=0.0
      RG=8.31456
C Initial potential of BIOGAS production: 172.73 m3 per ton wet waste 
C                  55% (95 m3/t) CH4 & 45% (77.73m3/t) CO2 
C CH4: 1mol=16.05gr =22.414liters; 1m3=0.716 kg; 95.00m3= 60.23kg CH4/t waste
C CO2: 1mol=44.00gr =22.414liters; 1m3=1.963 kg; 77.73m3=135.25kg CO2/t waste
      P0CH4=60.23
      P0CO2=135.25
c****  constant production of 3m3 biogas/t  if -10<TDEPOS<0  ***********
c if  production is cst. =   3m3 landfill gas/t/y (55% CH4 & 44% CO2)
c                            PCH4C = 1.65 m3/t/y = 1.0 kg CH4/t waste/y
c                            PCO2C = 1.35 m3/t/y = 2.3 kg CO2/t waste/y
      PCH4C=3.17E-08
      PCO2C=7.29E-08
c***********************************************************************
      sgch4=0.
      sgco2=0.
      sght =0.
C
C     CONDITIONAL PRINTOUT OF RESULTS
c      IF(KON.GE.2.AND.MOP(24).GE.4) PRINT 2102
 2102 FORMAT(/,' ELEM  DEPOSED BEFORE   ELAPSED TIME   DELTEX         ',
     1' CH4          CO2        HEAT                  DELTEN',
     2'        KON',/)
C#################################################################
      DO 1000 N=1,NELA
C#################################################################
      IF(TDEPOS(N).LE.-10.) GO TO 1000
c ako e pomalku ili ednakvo na -10 skokaj sve nema proizvodnja
      NMAT=MATX(N)
      FAC=FORD/EVOL(N)
C FACtor = FORD(=FOR*DELTEX)/VOLUME = dT/VOLUME
C     CH4 or CO2 SOURCE AND HEAT SOURCE SET TO ZERO BY DEFAULT
      GCH4(N)=0.
      GCO2(N)=0.
      GHT=0.
c
C-----LOCATION OF DATA BLOCKS:
C     NLOCP: LOCATION OF PRIMARY PARAMETERS IN ARRAY X(N)
C     NLOC:  LOCATION OF MATRIX TERMS CONTROLLED BY THE
C            NUMBER (NEQ) OF EQUATIONS TO BE SOLVED FOR,
      NLOC=(N-1)*NEQ
      NLOCP=(N-1)*NK1
C     BEGINNING OF SECONDARY PARAMETERS IN ARRAY PAR(N)
      NLOC2=(N-1)*NSEC*NEQ1
C
C-----METHANE PRODUCTION RATE (SOURCE) IN ELEMENT (KG/S):
C  TDEPOS : elapsed time since deposition
c  SUMTIM : elapsed time since start of the simulation
c  TPRIME : activation time for biodegradation (6 mois) 
c  XKI(I) : biodegradation coefficient (k)
c  FRAC(I): mass fractions according to biodegradability
c           readily, moderate & inert 
c
      IF(TDEPOS(N).LE.0.) GO TO 60
c ako e pogolemo od -10 no pomalku ili ednakvo na 0 constantna proizvodnja
      TIMESTART=TDEPOS(N)-TPRIME+SUMTIM/(60*60*24*365)
      IF(TIMESTART.LT.0.) GO TO 1000
      TIMEEND=TIMESTART+DELTEX/(60*60*24*365)
      SUMSTART=0.
      SUMEND=0.
c   counter I is the number of fractions (in this case 1)
      DO 22 I=1,1
c     SUMSTART=SUMSTART+FRAC(I)*XKI(I)*EXP(-XKI(I)*TIMESTART)
c     SUMEND=SUMEND+FRAC(I)*XKI(I)*EXP(-XKI(I)*TIMEEND)
      SUMSTART=SUMSTART+XKI(I)*EXP(-XKI(I)*TIMESTART)
      SUMEND=SUMEND+XKI(I)*EXP(-XKI(I)*TIMEEND)
   22 continue
c
C AVERAGE PRODUCTION RATE OF THE VALUES AT START AND END OF TIME STEP
      SUM=0.5*(SUMSTART+SUMEND)
c baca 12 july 97
      summax=60*3600*24*365.
      if(sumstart.gt.summax) sum=0.
c
C AVERAGE PRODUCTION RATE IN kg/t/y; waste density DM=650-850 kg/m3
      GCH4(N)=P0CH4*SUM*EVOL(N)*DM(NMAT)/1000.
      GCO2(N)=P0CO2*SUM*EVOL(N)*DM(NMAT)/1000.
c     PRINT 2011,sumstart,sumend,P0CH4,P0CO2,Evol(n),DM(nmat),xki(1)
c2011 FORMAT(7F10.4)
c   
C AVERAGE PRODUCTION RATE IN KG FOR GIVEN VOLUME PER SECOND
      GCH4(N)=GCH4(N)/(365.*60.*60.*24.)
      GCO2(N)=GCO2(N)/(365.*60.*60.*24.)
      go to 40
C************************ constant production ***************************
60    continue
c only if tdepos=0. in addition to CO2 there is also CH4 production 
c if tdepos < 0 (e.g. -2) than only CO2 is produced. This is for the upper most waste layer
c which is in contact with atm. air and because of presence of Oxygen, production of CH4=0
c temperatura here is not of our interest
      IF(TDEPOS(N).EQ.0.) GCH4(N)=PCH4C*EVOL(N)*DM(NMAT)/1000.
      GCO2(N)=PCO2C*EVOL(N)*DM(NMAT)/1000.
C
C-----HEAT SOURCE IN ELEMENT (W/S):   for NEQ = NK1.
40    IF(NEQ.LT.NK1) GOTO 5
      GHT=CFHT*GCH4(N)
    5 CONTINUE
C
C      CONDITIONNAL PRINTOUT OF CCHPR RESULTS
C
      IF(KON.LT.2) GOTO 2000
      IF(MOP(24).LT.4) GOTO 2000
c     PRINT 2001,N,TDEPOS(N),SUMTIM,DELTEX,GCH4(N),GCO2(N),GHT,
c    1           DELTEN,kon
 2001 FORMAT(I5,4X,F6.1,8x,5G13.5,10X,1G13.5,i6)
 2000 CONTINUE
C
C***** SKIP NEXT SECTION IN CASE OF CONVERGENCE (KON=2)
C
      IF(KON.EQ.2) SGCH4=SGCH4+DELTEX*GCH4(N)
      IF(KON.EQ.2) SGCO2=SGCO2+DELTEX*GCO2(N)
      IF(KON.EQ.2) SGHT=SGHT+DELTEX*GHT
      IF(KON.EQ.2) GOTO 1000
C
C-----ADD MASS CH4 & CO2 PRODUCED AND HEAT PRODUCED TO CORRESPONDING 
C     RESIDUAL (KG PER TIME STEP (DELTEX) PER UNIT VOLUME).
      R(NLOC+3)=R(NLOC+3)-DELTEX*GCH4(N)/EVOL(N)
      R(NLOC+4)=R(NLOC+4)-DELTEX*GCO2(N)/EVOL(N)
      IF(NEQ.LT.NK1) GOTO 6
      R(NLOC+NK1)=R(NLOC+NK1)-DELTEX*GHT/EVOL(N)
    6 CONTINUE
c
C
C##########################################################################
1000  CONTINUE
C
      RETURN
      END                             
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 30xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C calculates CO2 mass.fract in liquid phase for given CO2 partial pressure 
C PCX and temp. TX
C ************ DIREKTNO ZA MASS FRACTIONS *********************
C jas so mojata henryc racunam mole fractions
C  T:     0         5         10        15        20        30         40
C X1M -1.716E-3 -2.753E-3 -6.474E-3  2.279E-2  4.307E-3  1.724E-3  1.121E-3
C  T:     50        60       70        80       90        100
C X1M  8.541E-4  7.05E-4  6.107E-4  5.466E-4  5.009E-4  4.673E-4
C PCX was keeped constant at 101325.2 Pa
      SUBROUTINE HENRY(X1M,TX,PCX)
C     IMPLICIT REAL*8 (A-H,O-Z)
      implicit real*8 (a-h,o-z)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'HENRY    1.0      26 JANUARY   1990',6X,
     X'USE HEN RYS LAW TO CALCULATE CO2 MASS FRACTION IN LIQUID')
C
      T=0.01*(TX-170.)
      T2=T*T
      T4=T2*T2
      IF(T.GE.0.0) CA=6400.-2077.78*T2+311.11*T4
      IF(T.LT.0.0) CA=6400.-2149.14*T2-195.43*T4
C tuka Ca mu e Henry's constant kako kaj Tchoban no se slaga samo okolu
C 50oC i verojatno nagore. Za pod 10oC duri dava negativni vrednosti. 
C Go deli CO2 partial pressure so 1.E+5 da dobie press.vo atmosph.
C X1m mu e direktno CO2 mass fraction in water/CO2 solution in gr CO2 per
C gr solution (kg/kg). Formulata e P=XMOLE*HC
      X1M=(PCX*1.0E-05/CA)*44./18.
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 31xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C calculates enthalpy HSOL of CO2 dissolution in liquid (water)
C   T:    0        5        10       15       20       30       40       50         
CHSOL: -73696  -100174  -123339  -143391  -160523  -186768  -203490  -212005
C   T:    60       70       80       90       100
CHSOL: -213526  -209164  -199928  -186725  -170357
      SUBROUTINE SOLUT(TX,HSOL)
C     IMPLICIT REAL*8 (A-H,O-Z)
      implicit real*8 (a-h,o-z)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'SOLUT    1.0      26 JANUARY   1990',6X,
     X'CALCULATE ENTHALPY OF CO2 DISSOLUTION IN LIQUID')
C
      T=0.01*TX
      T2=T*T
      T3=T*T2
      T4=T*T3
      HSOL=-0.073696 - 0.56405*T+0.70363*T2-0.27882*T3+0.042579*T4
      HSOL=HSOL*1000000.
      RETURN
      END
C 
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 32xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C enthalpy HC and density DC of CO2 gas given temp. TX and part.press. PCX
C TX     0       5      10      15     20       30       40        50
C HC  -916.2  3328.8  7583.7  11850  16129.1  24730.7  33396.5  42132.9
CVAIR*T  0    3665.0  7330.0  10995  14660.0  21990.0  29320.0  36650.0
C DC   1.98   1.94    1.90   1.87087   1.84    1.78     1.72     1.66
C TX    60      70       80      90       100
C HC 50945.1  59835.2  68806.8  77860.8  86998.6
CVAIR*T       51310.0  58640.0  65970.0  73300.0
C DC  1.61     1.57     1.52    1.48     1.44 
C CO2 part.press. PCX was keeped constant at 101325.2 Pa 
C TchobanoglusTchoban for 0oC Dco2=1.9768 for 0oC Dco2=1.9768
      SUBROUTINE CO2(TX,PCX,HC,DC)
C     IMPLICIT REAL*8 (A-H,O-Z)
      implicit real*8 (a-h,o-z)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'CO2      1.0      26 JANUARY   1990',6X,
     X'SPECIFIC ENTHALPY AND DENSITY OF CO2 GAS')
C
      PC=PCX*0.000001
      TA=TX+273.15
      TB=0.01*TA
      TC=TB**3.3333333333
      HCI=1.667 + 0.001542*TA - 0.7948*LOG10(TA)-41.35/TA
      HC=HCI-0.3571*PC*(1.0+0.07576*PC)/TC
      HC=HC*1000000.
      VC1=0.00018882*TA
      VC2=-PC*(0.0824 +0.01249*PC)/TC
      DC=PC/(VC1+VC2)
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 33xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C calcul visc. VC of pure CO2 gas for given CO2 part.press. PCX and temp.TX
C mene mi treba gas mixture WILKE formula for gas mixtures at low pressures
C TX      0         5        10        15        20         30       40
C VC  1.384E-5  1.406E-5  1.429E-5  1.451E-5  1.474E-5  1.519E-5  1.563E-5
C TX     50        60        70        80        90         100
C VC  1.608E-5  1.652E-5  1.696E-5  1.740E-5  1.784E-5  1.827E-5
C PCX was keeped constant at 101325.2 Pa
C
C      SUBROUTINE VISCO(VC,PCX,TX)
C      DIMENSION A(5),B(5),C(5)
C      SAVE ICALL,A,B
C      DATA A/1357.8, 4.9227, -2.9661E-03, 2.8529E-06, -2.1829E-09/
C      DATA B/3918.9, -35.984, 2.5825E-01, -7.1178E-04, 6.9578E-07/
C
C      DATA ICALL/0/
C      ICALL=ICALL+1
C     IF(ICALL.EQ.1) WRITE(11,899)
C  899 FORMAT(6X,'VISCO    1.0      26 JANUARY   1990',6X,
C     X'VISCOSIT OF GASEOUS CO2')
C
CC MODIFICATION OF S**3 FORMULA - ONLY GOOD FOR P<100BAR
C      PBAR=PCX*1.0E-05
C      XX=0.01*PBAR
C      IF(XX.GT.1.0) XX=1.0
C      DO 5 I=1,5
C    5 C(I)=A(I)+XX*(B(I)-A(I))
C      T2=TX*TX
C      T3=TX*T2
C      T4=TX*T3
C      VC=1.0E-08*(C(1)+C(2)*TX+C(3)*T2+C(4)*T3+C(5)*T4)
C      RETURN
C      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 37xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C diffusion coefficients for binary gas mixtures
C at low pressures ~101325.2 Pa as a function of
C the temperature in oK. According Fuller et al.
C cited by Reid, Prausnitz and Poling, pp.587
C diffusion coefficients D0--- are in cm2/sec
      SUBROUTINE DIFF(T,P,XH2O,XN2,XCH4,XCO2,D0H2O,D0N2,D0CH4,D0CO2)
      implicit real*8 (a-h,o-z)
      DIMENSION AM(4),S(4),XMSL(4)
C
      SAVE ICALL
      DATA ICALL/0/
      DATA AM/18.016,28.96,16.05,44./
      DATA S/13.1,19.7,25.14,26.9/
C
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'DIFF     1.0      22  MARCH    1995',6X,
     X'DIFFUSION COEFFICIENTS FOR BINARY GAS MIXTURES AS',
     X' FUNCTION OF TEMPERATURE')
C
      Ta=T+273.15
c izmena 12 sept.96
      XMSL(1)=XH2O
      XMSL(2)=XN2
      XMSL(3)=XCH4
      XMSL(4)=XCO2
c izmena 12 sept.96
c     dole=xh2o/am(1)+xn2/am(2)+
c    1xch4/am(3)+xco2/am(4)
c     XMSL(1)=(xh2o/am(1))/dole
c     XMSL(2)=(xn2/am(2))/dole
c     XMSL(3)=(xch4/am(3))/dole
c     XMSL(4)=(xco2/am(4))/dole
      PO=1./3.
c
      DO 1 I=1,4
      XM=0.
      DO 2 J=1,4
      IF(I.EQ.J) GO TO 2
      AMAV=2./((1./AM(I))+(1./AM(J)))
C P/100000. is pressure in bars; 
C delam so (100)**2=10000 za prelaz od cm2/sec vo m2/sec
      DAB=(0.00143*Ta**1.75)/(10000.*(P/100000.)*
     1    (SQRT(AMAV))*(S(I)**PO+S(J)**PO)**2)
CC      print 35,i,j,dab
35    format(2i5,1f15.8)
      XM=XM+XMSL(J)/DAB
2     CONTINUE
      DZERO=(1.-XMSL(I))/XM
      IF(I.EQ.1)D0H2O=DZERO
      IF(I.EQ.2)D0N2 =DZERO
      IF(I.EQ.3)D0CH4=DZERO
      IF(I.EQ.4)D0CO2=DZERO
1     CONTINUE
      RETURN
      END
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxx NEW SUBROUTINE 36xxxxxxxxxxxxxxxxxxxxxxxxxxxx
Cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
c This is a subroutine for calculating Henry's law constants HC. Gas
C partial pressure is given by P=Xmol/Hc (Pa). The mole fraction  of
C gas dissolved in water (solvent) is dorectly proportional  to  the
C partial pressure of that gas above the  water  surface.  DATA  are 
C taken from Tchobanoglus pp.938
C 
c      SUBROUTINE HENRYC(T,HCN2,HCCH4,HCCO2)
c      DIMENSION N2(7),CH4(7),CO2(7),TEMP(7)
c      DATA N2 /4.320,5.49,6.64,7.71,8.70,9.46,10.1/
c      DATA CH4/2.240,2.97,3.76,4.49,5.20,5.77,6.26/
c      DATA CO2/.0728,.104,.142,.186,.233,.283,.341/
c      DATA TEMP/0.,10.,20.,30.,40.,50.,60./
C
c      IF(0.0.LE.T .AND. 10..GT.T) NB=1
c      IF(10..LE.T .AND. 20..GT.T) NB=2
c      IF(20..LE.T .AND. 30..GT.T) NB=3
c      IF(30..LE.T .AND. 40..GT.T) NB=4
c      IF(40..LE.T .AND. 50..GT.T) NB=5
c      IF(50..LE.T .AND. 60..GT.T) NB=6
C
c      DO 1 I=1,7
c      IF(I.NE.NB) GO TO 1
c      DO 1 J=1,3
c      IF(J.EQ.1)A1=N2 (NB)
c      IF(J.EQ.1)A2=N2 (NB+1)
c      IF(J.EQ.2)A1=CH4(NB)
c      IF(J.EQ.2)A2=CH4(NB+1)
c      IF(J.EQ.3)A1=CO2(NB)
c      IF(J.EQ.3)A2=CO2(NB+1)
c      T1=TEMP(NB)
c      HC=((A2-A1)*(T-T1)/10.+A1)*1.E+09
c      IF(J.EQ.1) HCN2 =1./HC
c      IF(J.EQ.2) HCCH4=1./HC
c      IF(J.EQ.3) HCCO2=1./HC
c1     CONTINUE
c      HC=1.E-10
c      HCN2=HC
C
c      RETURN
c      END
C
      SUBROUTINE HENRYC(TX,HCAIR,HCCH4,HCCO2)
C     IMPLICIT DOUBLE PRECISION (A-H,O-Z)                               
      implicit real*8 (a-h,o-z)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
  899 FORMAT(6X,'HENRY    1.0      25 FEBRUARY  1993',6X,
     X'HENRYS LAW TO CALCULATE NCG BUBBLING PRESSURE',
     X' OR DISSOLVED NCG MASS FRACTION')
C
      T2=TX*TX
      T3=T2*TX
      T4=T2*T2
C     AIR:D'AMORE AND TRUESDELL (1988), CRAMER (1982), CYGAN (1991)
    1    RKHW=(51372.6E0+1586.03E0*TX-5.93780E0*T2-6.98282E-2*T3+
     X      5.10330E-4*T4-1.21388E-6*T2*T3+1.00041E-9*T3*T3)*1.01325E5
         RKHO2=(26234.0E0+610.628E0*TX+7.00732E0*T2-.139299E0*T3+
     X         7.13850E-4*T4-1.54216E-6*T2*T3+1.23190E-9*T3*T3)*1.E5
         RKH=0.79E0*RKHW+0.21E0*RKHO2
      hcair=1./rkh
C
C     CO2: CRAMER, 1982.
         RKHW=(783.666E0+19.6025E0*TX+0.820574E0*T2-7.40674E-03*T3+
     X       2.18380E-05*T4-2.20999E-08*T2*T3)*1.E5
         RKH=RKHW
      hcco2=1./rkh
C
C     CH4: CRAMER, 1982.
         RKHW=(24582.4E0+671.091E0*TX+6.87067E0*T2-.173079E0*T3+
     X       1.09652E-03*T4-3.19599E-06*T2*T3+4.46172E-9*T3*T3-
     X       2.40294E-12*T4*T3)*1.E5
         RKH=RKHW
       hcch4=1./rkh
C
C     H2: D'AMORE AND TRUESDELL (1988)
cc         RKHW=(57106.3E0+761.981E0*TX-8.55167E0*T2+1.20762E-3*T3+
cc     X       2.85520E-04*T4-1.42066E-06*T2*T3+2.81250E-9*T3*T3-
cc     X       2.04787E-12*T4*T3)*1.01325E5
cc         RKH=RKHW
cc      hh2=1./rkh
cc         GOTO 100
C
C     N2: D'AMORE AND TRUESDELL (1988),  CYGAN (1991)
cc         RKHW=51372.6E0+1586.03E0*TX-5.93780E0*T2-6.98282E-2*T3+
cc     X         5.10330E-4*T4-1.21388E-6*T2*T3+1.00041E-9*T3*T3
cc         RKHW=RKHW*1.01325E5
cc         RKH=RKH
cc      hn2=1./rkh
cc         GOTO 100
C
100   continue
c      if(icall.eq.1) print *,'t=',tx,' HCN2=',hcair,
c     x ' HCCH4=',hcch4,' HCCO2=',hcco2
      RETURN
      END
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
                  
     
